/* SavePov.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include "giram.h"

/*****************************************************************************
*  SavePovGlobalSettings
******************************************************************************/
void SavePovGlobalSettings(FILE *OutFile, GlobalSetting Global)
{
  fprintf(OutFile, "global_settings\n{\n");
  fprintf(OutFile, "  assumed_gamma %g\n", Global.AssumedGamma);
  if (Global.HfGray16)
  {
    fprintf(OutFile, "  hf_gray_16\n");
  }
  fprintf(OutFile, "  ambient_light rgb <%g, %g, %g>\n",
                   Global.AmbientLight[0],
                   Global.AmbientLight[1],
                   Global.AmbientLight[2]);
  fprintf(OutFile, "  number_of_waves %d\n", Global.NumberOfWaves);
  fprintf(OutFile, "  adc_bailout %g\n", Global.ADCBailout);
  fprintf(OutFile, "  irid_wavelength rgb <%g, %g, %g>\n",
                   Global.IridWavelength[0],
                   Global.IridWavelength[1],
                   Global.IridWavelength[2]);
  fprintf(OutFile, "  max_intersections %d\n", Global.MaxIntersections);
  fprintf(OutFile, "  max_trace_level %d\n", Global.MaxTraceLevel);
  fprintf(OutFile, "  radiosity\n  {\n");
  fprintf(OutFile, "    brightness %g\n", Global.Radiosity.Brightness);
  fprintf(OutFile, "    count %d\n", Global.Radiosity.Count);
  fprintf(OutFile, "    distance_maximum %g\n", Global.Radiosity.DistanceMaximum);
  fprintf(OutFile, "    error_bound %g\n", Global.Radiosity.ErrorBound);
  fprintf(OutFile, "    gray_threshold %g\n", Global.Radiosity.GrayThreshold);
  fprintf(OutFile, "    low_error_factor %g\n", Global.Radiosity.LowErrorFactor);
  fprintf(OutFile, "    minimum_reuse %g\n", Global.Radiosity.MinimumReuse);
  fprintf(OutFile, "    nearest_count %d\n", Global.Radiosity.NearestCount);
  fprintf(OutFile, "    recursion_limit %d\n", Global.Radiosity.RecursionLimit);
  fprintf(OutFile, "  }\n}\n\n");
}

/*****************************************************************************
*  SavePovColorMap
******************************************************************************/
void SavePovColorMap(FILE *OutFile, ColorMapStruct *ColorMap, int indentation)
{
  int i;
  char *s;
  
  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%scolor_map\n%s{\n", s, s);
  for (i=0 ; i<ColorMap->NbMapEntries ; i++)
    fprintf(OutFile, "%s  [%g color <%g, %g, %g,  %g, %g>]\n",
            s, ColorMap->Entries[i].Value,
            ColorMap->Entries[i].Color[0],
            ColorMap->Entries[i].Color[1],
            ColorMap->Entries[i].Color[2],
            ColorMap->Entries[i].Color[3],
            ColorMap->Entries[i].Color[4]);
  fprintf(OutFile, "%s}\n", s);
}

/*****************************************************************************
*  SavePovTrans
******************************************************************************/
void SavePovTrans(FILE *OutFile, TransformStruct *Trans, int indentation)
{
  int i;
  char *s;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%smatrix  <%g, %g, %g,\n", s,
          Trans->Direct[0][0], Trans->Direct[0][1], Trans->Direct[0][2]);
  fprintf(OutFile, "%s   %g, %g, %g,\n", s,
          Trans->Direct[1][0], Trans->Direct[1][1], Trans->Direct[1][2]);
  fprintf(OutFile, "%s   %g, %g, %g,\n", s,
          Trans->Direct[2][0], Trans->Direct[2][1], Trans->Direct[2][2]);
  fprintf(OutFile, "%s   %g, %g, %g>\n", s,
          Trans->Direct[3][0], Trans->Direct[3][1], Trans->Direct[3][2]);
}

/*****************************************************************************
*  SavePovPigment
******************************************************************************/
void SavePovPigment(FILE *OutFile, PigmentStruct *Pigment, int indentation)
{
  int i;
  char *s;
  
  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%spigment\n%s{\n", s, s);
  switch (Pigment->Type)
  {
    case PAT_SOLID_COLOR:
      fprintf(OutFile, "%s  color rgbft <%g, %g, %g,  %g, %g>\n", s,
              Pigment->Color[0], Pigment->Color[1], Pigment->Color[2],
              Pigment->Color[3], Pigment->Color[4]);
      break;
    case PAT_BRICK:
      /* FIXME */
      break;
    case PAT_CHECKER:
      /* FIXME */
      break;
    case PAT_HEXAGON:
      /* FIXME */
      break;
    case PAT_AGATE:
      fprintf(OutFile, "%s  agate\n", s);
      fprintf(OutFile, "%s  agate_turb %g\n", s, Pigment->AgateTurb);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_AVERAGE:
      break;
    case PAT_BOZO:
      fprintf(OutFile, "%s  bozo\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_BUMPS:
      fprintf(OutFile, "%s  bumps\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_CRACKLE:
      fprintf(OutFile, "%s  crackle\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_DENTS:
      fprintf(OutFile, "%s  dents\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_GRADIENT:
      fprintf(OutFile, "%s  gradient <%g, %g, %g>\n", s, 
              Pigment->Gradient[0], Pigment->Gradient[1], Pigment->Gradient[2]);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_GRANITE:
      fprintf(OutFile, "%s  granite\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_LEOPARD:
      fprintf(OutFile, "%s  leopard\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_MANDEL:
      fprintf(OutFile, "%s  mandel %d\n", s, Pigment->Mandel);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_MARBLE:
      fprintf(OutFile, "%s  marble\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_ONION:
      fprintf(OutFile, "%s  onion\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_QUILTED:
      fprintf(OutFile, "%s  quilted\n", s);
      fprintf(OutFile, "%s  control0 %g\n", s, Pigment->Control0);
      fprintf(OutFile, "%s  control1 %g\n", s, Pigment->Control1);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_RADIAL:
      fprintf(OutFile, "%s  radial\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_RIPPLES:
      fprintf(OutFile, "%s  ripples\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_SPIRAL1:
      fprintf(OutFile, "%s  spiral1\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_SPIRAL2:
      fprintf(OutFile, "%s  spiral2\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_SPOTTED:
      fprintf(OutFile, "%s  spotted\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_WAVES:
      fprintf(OutFile, "%s  waves\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_WOOD:
      fprintf(OutFile, "%s  wood\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
    case PAT_WRINKLES:
      fprintf(OutFile, "%s  wrinkles\n", s);
      SavePovColorMap(OutFile, Pigment->ColorMap, indentation+1);
      break;
  }
  if (Pigment->Trans)
    SavePovTrans(OutFile, Pigment->Trans, indentation+1);
  fprintf(OutFile, "%s}\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovNormal
******************************************************************************/
void SavePovNormal(FILE *OutFile, NormalStruct *Normal, int indentation)
{
  int i;
  char *s;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2]=0;
  fprintf(OutFile, "%snormal\n%s{\n", s, s);
  /* FIXME */
  if (Normal->Trans)
    SavePovTrans(OutFile, Normal->Trans, indentation+1);
  fprintf(OutFile, "%s}\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovFinish
******************************************************************************/
void SavePovFinish(FILE *OutFile, FinishStruct *Finish, int indentation)
{
  int i;
  char *s;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2]=0;
  fprintf(OutFile, "%sfinish\n%s{\n", s, s);
  fprintf(OutFile, "%s  ambient <%g, %g, %g>\n", s, Finish->Ambient[0],
                                                    Finish->Ambient[1],
                                                    Finish->Ambient[2]);
  fprintf(OutFile, "%s  diffuse %g\n", s, Finish->Diffuse);
  fprintf(OutFile, "%s  brilliance %g\n", s, Finish->Brilliance);
  fprintf(OutFile, "%s  phong %g\n", s, Finish->Phong);
  fprintf(OutFile, "%s  phong_size %g\n", s, Finish->PhongSize);
  fprintf(OutFile, "%s  specular %g\n", s, Finish->Specular);
  fprintf(OutFile, "%s  roughness %g\n", s, Finish->Roughness);
  fprintf(OutFile, "%s  metallic %g\n", s, Finish->Metallic);
  fprintf(OutFile, "%s  reflection <%g, %g, %g>\n", s, Finish->Reflection[0],
                                                       Finish->Reflection[1],
                                                       Finish->Reflection[2]);
  fprintf(OutFile, "%s  refraction %g\n", s, Finish->Refraction);
  fprintf(OutFile, "%s  ior %g\n", s, Finish->Ior);
  fprintf(OutFile, "%s  caustics %g\n", s, Finish->Caustics);
  fprintf(OutFile, "%s  fade_distance %g\n", s, Finish->FadeDistance);
  fprintf(OutFile, "%s  fade_power %g\n", s, Finish->FadePower);
  /* FIXME : irid */
  fprintf(OutFile, "%s  crand %g\n", s, Finish->Crand);
  fprintf(OutFile, "%s}\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovHalo
******************************************************************************/
void SavePovHalo(FILE *OutFile, HaloStruct *Halo, int indentation)
{
  int i;
  char *s;
  
  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%shalo\n%s{\n", s, s);
  /* FIXME */
  if (Halo->Trans)
    SavePovTrans(OutFile, Halo->Trans, indentation+1);
  fprintf(OutFile, "%s}\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovTexture
******************************************************************************/
void SavePovTexture(FILE *OutFile, TextureStruct *Texture, int indentation)
{
  int i;
  char *s;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%stexture\n%s{\n", s, s);
  if (Texture->Pigment)
  {
    SavePovPigment(OutFile, Texture->Pigment, indentation+1);
  }
  if (Texture->Normal)
  {
    SavePovNormal(OutFile, Texture->Normal, indentation+1);
  }
  if (Texture->Finish)
  {
    SavePovFinish(OutFile, Texture->Finish, indentation+1);
  }
  if (Texture->Halo)
  {
    SavePovHalo(OutFile, Texture->Halo, indentation+1);
  }
  if (Texture->Trans)
    SavePovTrans(OutFile, Texture->Trans, indentation+1);
  fprintf(OutFile, "%s}\n", s);
  if (Texture->NextLayer)
  {
    SavePovTexture(OutFile, Texture->NextLayer, indentation);
  }
  g_free(s);
}

/*****************************************************************************
*  SavePovSkySphere
******************************************************************************/
void SavePovSkySphere(FILE *OutFile, SkySphereStruct *SkySphere)
{
  int i;
  fprintf(OutFile, "sky_sphere\n{\n");
  for (i=0 ; i<SkySphere->NbPigment ; i++)
  {
    SavePovPigment(OutFile, SkySphere->pPigmentList[i], 1);
  }
  if (SkySphere->Trans)
    SavePovTrans(OutFile, SkySphere->Trans, 1);
  fprintf(OutFile, "}\n\n");
}

/*****************************************************************************
*  SavePovBicubicPatch
******************************************************************************/
void SavePovBicubicPatch(FILE *OutFile, ObjectStruct *BicubicPatch, int indentation)
{
  int i;
  char *s;
/*  BicubicPatchStruct *BBicubicPatch = (BicubicPatchStruct *)BicubicPatch;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sbicubic_patch\n%s{\n%s To be fixed \n", s, s, s);
  if (BicubicPatch->Texture)
  {
    SavePovTexture(OutFile, BicubicPatch->Texture, indentation+1);
  }
  if (BicubicPatch->Trans)
    SavePovTrans(OutFile, BicubicPatch->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovBlob
******************************************************************************/
void SavePovBlob(FILE *OutFile, ObjectStruct *Blob, int indentation)
{
  int i;
  char *s;
/*  BlobStruct *BBlob = (BlobStruct *)Blob;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sblob\n%s{\n%s  To be fixed \n", s, s, s);
  if (Blob->Texture)
  {
    SavePovTexture(OutFile, Blob->Texture, indentation+1);
  }
  if (Blob->Trans)
    SavePovTrans(OutFile, Blob->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovBox
******************************************************************************/
void SavePovBox(FILE *OutFile, ObjectStruct *Box, int indentation)
{
  int i;
  char *s;
  BoxStruct *BBox = (BoxStruct *)Box;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sbox\n%s{\n%s  <%g, %g, %g>, <%g, %g, %g>\n", s, s, s,
                   BBox->MinCorner[0], BBox->MinCorner[1], BBox->MinCorner[2],
                   BBox->MaxCorner[0], BBox->MaxCorner[1], BBox->MaxCorner[2]);
  if (Box->Texture)
  {
    SavePovTexture(OutFile, Box->Texture, indentation+1);
  }
  if (Box->Trans)
    SavePovTrans(OutFile, Box->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovCone
******************************************************************************/
void SavePovCone(FILE *OutFile, ObjectStruct *Cone, int indentation)
{
  int i;
  char *s;
  ConeStruct *CCone = (ConeStruct *)Cone;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%scone\n%s{\n%s  <0, %g, 0>, %g, <0, %g, 0>, %g\n",
          s, s, s, CCone->Radius1, CCone->Radius1,
                   CCone->Radius2, CCone->Radius2);
  if (CCone->Open)
    fprintf(OutFile, "%s  open\n",s);
  if (Cone->Texture)
  {
    SavePovTexture(OutFile, Cone->Texture, indentation+1);
  }
  if (Cone->Trans)
    SavePovTrans(OutFile, Cone->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovCubic
******************************************************************************/
void SavePovCubic(FILE *OutFile, ObjectStruct *Cubic, int indentation)
{
  int i;
  char *s;
  CubicStruct *CCubic = (CubicStruct *)Cubic;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%scubic\n%s{\n%s  <", s, s, s);
  for (i=0 ; i<19 ; i++)
    fprintf(OutFile, "%g, ", CCubic->T[i]);
  fprintf(OutFile, "%g>", CCubic->T[i]);
  if (Cubic->Texture)
  {
    SavePovTexture(OutFile, Cubic->Texture, indentation+1);
  }
  if (Cubic->Trans)
    SavePovTrans(OutFile, Cubic->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovCylinder
******************************************************************************/
void SavePovCylinder(FILE *OutFile, ObjectStruct *Cylinder, int indentation)
{
  int i;
  char *s;
  CylinderStruct *CCylinder = (CylinderStruct *)Cylinder;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%scylinder\n%s{\n%s  <%g, %g, %g>, <%g, %g, %g>, %g\n", s, s, s,
                   0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0);
  if (CCylinder->Open)
    fprintf(OutFile, "%s  open\n",s);
  if (Cylinder->Texture)
  {
    SavePovTexture(OutFile, Cylinder->Texture, indentation+1);
  }
  if (Cylinder->Trans)
    SavePovTrans(OutFile, Cylinder->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovDisc
******************************************************************************/
void SavePovDisc(FILE *OutFile, ObjectStruct *Disc, int indentation)
{
  int i;
  char *s;
  DiscStruct *DDisc = (DiscStruct *)Disc;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sdisc\n%s{\n%s  <0, 0, 0>, <0, 1, 0>, %g", 
          s, s, s, DDisc->Radius);
  if (DDisc->HoleRadius != 0.0)
    fprintf(OutFile, ", %g\n", DDisc->HoleRadius);
  else
    fprintf(OutFile, "\n");
  if (Disc->Texture)
  {
    SavePovTexture(OutFile, Disc->Texture, indentation+1);
  }
  if (Disc->Trans)
    SavePovTrans(OutFile, Disc->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovHeightField
******************************************************************************/
void SavePovHeightField(FILE *OutFile, ObjectStruct *HeightField, int indentation)
{
  int i;
  char *s;
/*  HeightFieldStruct *HHeightField = (HeightFieldStruct *)HeightField;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sheight_field\n%s{\n%s  To be fixed \n", s, s, s);
  if (HeightField->Texture)
  {
    SavePovTexture(OutFile, HeightField->Texture, indentation+1);
  }
  if (HeightField->Trans)
    SavePovTrans(OutFile, HeightField->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovJuliaFractal
******************************************************************************/
void SavePovJuliaFractal(FILE *OutFile, ObjectStruct *JuliaFractal, int indentation)
{
  int i;
  char *s;
/*  JuliaFractalStruct *JJuliaFractal = (JuliaFractalStruct *)JuliaFractal;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sjulia_fractal\n%s{\n%s  To be fixed \n", s, s, s);
  if (JuliaFractal->Texture)
  {
    SavePovTexture(OutFile, JuliaFractal->Texture, indentation+1);
  }
  if (JuliaFractal->Trans)
    SavePovTrans(OutFile, JuliaFractal->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovLathe
******************************************************************************/
void SavePovLathe(FILE *OutFile, ObjectStruct *Lathe, int indentation)
{
  int i;
  char *s;
/*  LatheStruct *LLathe = (LatheStruct *)Lathe;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%slathe\n%s{\n%s  To be fixed \n", s, s, s);
  if (Lathe->Texture)
  {
    SavePovTexture(OutFile, Lathe->Texture, indentation+1);
  }
  if (Lathe->Trans)
    SavePovTrans(OutFile, Lathe->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovMesh
******************************************************************************/
void SavePovMesh(FILE *OutFile, ObjectStruct *Mesh, int indentation)
{
  int i;
  char *s;
  MeshStruct *MMesh = (MeshStruct *)Mesh;
  TriangleListStruct *TmpTri;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%smesh\n%s{\n", s, s);
  for (TmpTri = MMesh->FirstTriangle ; TmpTri ; TmpTri=TmpTri->Next)
  {
    fprintf(OutFile, "%s  smooth_triangle <%g, %g, %g>, <%g, %g, %g>,\n", s,
            TmpTri->P1[0], TmpTri->P1[1], TmpTri->P1[2],
            TmpTri->N1[0], TmpTri->N1[1], TmpTri->N1[2]);
    fprintf(OutFile, "%s                  <%g, %g, %g>, <%g, %g, %g>,\n", s,
            TmpTri->P2[0], TmpTri->P2[1], TmpTri->P2[2],
            TmpTri->N2[0], TmpTri->N2[1], TmpTri->N2[2]);
    fprintf(OutFile, "%s                  <%g, %g, %g>, <%g, %g, %g>\n", s,
            TmpTri->P3[0], TmpTri->P3[1], TmpTri->P3[2],
            TmpTri->N3[0], TmpTri->N3[1], TmpTri->N3[2]);
    
  }
  if (Mesh->Texture)
  {
    SavePovTexture(OutFile, Mesh->Texture, indentation+1);
  }
  if (Mesh->Trans)
    SavePovTrans(OutFile, Mesh->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovPlane
******************************************************************************/
void SavePovPlane(FILE *OutFile, ObjectStruct *Plane, int indentation)
{
  int i;
  char *s;
  PlaneStruct *PPlane = (PlaneStruct *)Plane;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%splane\n%s{\n%s  <%g, %g, %g>, %g\n", s, s, s,
                   PPlane->Normal[0], PPlane->Normal[1], PPlane->Normal[2],
                   PPlane->Distance);
  if (Plane->Texture)
  {
    SavePovTexture(OutFile, Plane->Texture, indentation+1);
  }
  if (Plane->Trans)
    SavePovTrans(OutFile, Plane->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovPoly
******************************************************************************/
void SavePovPoly(FILE *OutFile, ObjectStruct *Poly, int indentation)
{
  int i;
  char *s;
/*  PolyStruct *PPoly = (PolyStruct *)Poly;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%spoly\n%s{\n%s  To be fixed \n", s, s, s);
  if (Poly->Texture)
  {
    SavePovTexture(OutFile, Poly->Texture, indentation+1);
  }
  if (Poly->Trans)
    SavePovTrans(OutFile, Poly->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovPolygon
******************************************************************************/
void SavePovPolygon(FILE *OutFile, ObjectStruct *Polygon, int indentation)
{
  int i;
  char *s;
/*  PolygonStruct *PPolygon = (PolygonStruct *)Polygon;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%spolygon\n%s{\n%s  To be fixed \n", s, s, s);
  if (Polygon->Texture)
  {
    SavePovTexture(OutFile, Polygon->Texture, indentation+1);
  }
  if (Polygon->Trans)
    SavePovTrans(OutFile, Polygon->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovPrism
******************************************************************************/
void SavePovPrism(FILE *OutFile, ObjectStruct *Prism, int indentation)
{
  int i;
  char *s;
/*  PrismStruct *PPrism = (PrismStruct *)Prism;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%sprism\n%s{\n%s  To be fixed \n", s, s, s);
  if (Prism->Texture)
  {
    SavePovTexture(OutFile, Prism->Texture, indentation+1);
  }
  if (Prism->Trans)
    SavePovTrans(OutFile, Prism->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovQuadric
******************************************************************************/
void SavePovQuadric(FILE *OutFile, ObjectStruct *Quadric, int indentation)
{
  int i;
  char *s;
  QuadricStruct *QQuadric = (QuadricStruct *)Quadric;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%squadric\n%s{\n", s, s);
  fprintf(OutFile, "%s  <%g, %g, %g>,\n", s, QQuadric->A, QQuadric->B, QQuadric->C);
  fprintf(OutFile, "%s  <%g, %g, %g>,\n", s, QQuadric->D, QQuadric->E, QQuadric->F);
  fprintf(OutFile, "%s  <%g, %g, %g>,\n", s, QQuadric->G, QQuadric->H, QQuadric->I);
  fprintf(OutFile, "%s  %g\n", s, QQuadric->J);
  if (Quadric->Texture)
  {
    SavePovTexture(OutFile, Quadric->Texture, indentation+1);
  }
  if (Quadric->Trans)
    SavePovTrans(OutFile, Quadric->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovQuartic
******************************************************************************/
void SavePovQuartic(FILE *OutFile, ObjectStruct *Quartic, int indentation)
{
  int i;
  char *s;
  QuarticStruct *QQuartic = (QuarticStruct *)Quartic;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%squartic\n%s{\n%s  To be fixed \n", s, s, s);
  fprintf(OutFile, "%squartic\n%s{\n%s  <", s, s, s);
  for (i=0 ; i<34 ; i++)
    fprintf(OutFile, "%g, ", QQuartic->T[i]);
  fprintf(OutFile, "%g>", QQuartic->T[i]);
  if (Quartic->Texture)
  {
    SavePovTexture(OutFile, Quartic->Texture, indentation+1);
  }
  if (Quartic->Trans)
    SavePovTrans(OutFile, Quartic->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovSor
******************************************************************************/
void SavePovSor(FILE *OutFile, ObjectStruct *Sor, int indentation)
{
  int i;
  char *s;
  SorStruct *SSor = (SorStruct *)Sor;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%ssor\n%s{\n%s  %d,\n", s, s, s, SSor->NumberOfPoints);
  for (i=0 ; i<SSor->NumberOfPoints-1 ; i++)
    fprintf(OutFile, "%s  <%g, %g>,\n", s, SSor->Point[i][0], SSor->Point[i][1]);
  fprintf(OutFile, "%s  <%g, %g>\n", s, SSor->Point[i][0], SSor->Point[i][1]);
  if (Sor->Texture)
  {
    SavePovTexture(OutFile, Sor->Texture, indentation+1);
  }
  if (Sor->Trans)
    SavePovTrans(OutFile, Sor->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovSphere
******************************************************************************/
void SavePovSphere(FILE *OutFile, ObjectStruct *Sphere, int indentation)
{
  int i;
  char *s;
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%ssphere\n%s{\n%s  <%g, %g, %g>, %g\n", s, s, s,
                   SSphere->Center[0], SSphere->Center[1], SSphere->Center[2],
                   SSphere->Radius);
  if (Sphere->Texture)
  {
    SavePovTexture(OutFile, Sphere->Texture, indentation+1);
  }
  if (Sphere->Trans)
    SavePovTrans(OutFile, Sphere->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovSuperEllipsoid
******************************************************************************/
void SavePovSuperEllipsoid(FILE *OutFile, ObjectStruct *SuperEllipsoid, int indentation)
{
  int i;
  char *s;
  SuperEllipsoidStruct *SSuperEllipsoid = (SuperEllipsoidStruct *)SuperEllipsoid;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%ssuperellipsoid\n%s{\n%s  %g, %g\n", s, s, s,
          SSuperEllipsoid->E, SSuperEllipsoid->N);
  if (SuperEllipsoid->Texture)
  {
    SavePovTexture(OutFile, SuperEllipsoid->Texture, indentation+1);
  }
  if (SuperEllipsoid->Trans)
    SavePovTrans(OutFile, SuperEllipsoid->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovText
******************************************************************************/
void SavePovText(FILE *OutFile, ObjectStruct *Text, int indentation)
{
  int i;
  char *s;
/*  TextStruct *TText = (TextStruct *)Text;*/

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%stext\n%s{\n%s  To be fixed \n", s, s, s);
  if (Text->Texture)
  {
    SavePovTexture(OutFile, Text->Texture, indentation+1);
  }
  if (Text->Trans)
    SavePovTrans(OutFile, Text->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovTorus
******************************************************************************/
void SavePovTorus(FILE *OutFile, ObjectStruct *Torus, int indentation)
{
  int i;
  char *s;
  TorusStruct *TTorus = (TorusStruct *)Torus;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%storus\n%s{\n%s  %g, %g\n", s, s, s,
                   TTorus->Major, TTorus->Minor);
  if (Torus->Texture)
  {
    SavePovTexture(OutFile, Torus->Texture, indentation+1);
  }
  if (Torus->Trans)
    SavePovTrans(OutFile, Torus->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovTriangle
******************************************************************************/
void SavePovTriangle(FILE *OutFile, ObjectStruct *Triangle, int indentation)
{
  int i;
  char *s;
  TriangleStruct *TTriangle = (TriangleStruct *)Triangle;

  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i] = ' ';
  s[indentation*2] = 0;
  fprintf(OutFile, "%ssmooth_triangle\n%s{\n", s, s);
  fprintf(OutFile, "%s  <%g, %g, %g>, <%g, %g, %g>,\n", s,
          TTriangle->P1[0], TTriangle->P1[1], TTriangle->P1[2],
          TTriangle->N1[0], TTriangle->N1[1], TTriangle->N1[2]);
  fprintf(OutFile, "%s  <%g, %g, %g>, <%g, %g, %g>,\n", s,
          TTriangle->P2[0], TTriangle->P2[1], TTriangle->P2[2],
          TTriangle->N2[0], TTriangle->N2[1], TTriangle->N2[2]);
  fprintf(OutFile, "%s  <%g, %g, %g>, <%g, %g, %g>\n", s,
          TTriangle->P3[0], TTriangle->P3[1], TTriangle->P3[2],
          TTriangle->N3[0], TTriangle->N3[1], TTriangle->N3[2]);
  if (Triangle->Texture)
  {
    SavePovTexture(OutFile, Triangle->Texture, indentation+1);
  }
  if (Triangle->Trans)
    SavePovTrans(OutFile, Triangle->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovCSG
******************************************************************************/
void SavePovCSG(FILE *OutFile, ObjectStruct *CSG, int indentation)
{
  char *s;
  int i;
  ObjectStruct *TmpObject;
  CSGStruct *CCSG = (CSGStruct *)CSG;
  
  s = g_malloc(indentation*2+1);
  for (i=0 ; i<indentation*2 ; i++)
    s[i]=' ';
  s[indentation*2] = 0;
  switch (CCSG->Type)
  {
    case CSG_DIFFERENCE:
      fprintf(OutFile, "%sdifference\n%s{\n", s, s);
      break;
    case CSG_INTERSECTION:
      fprintf(OutFile, "%sintersection\n%s{\n", s, s);
      break;
    case CSG_MERGE:
      fprintf(OutFile, "%smerge\n%s{\n", s, s);
      break;
    case CSG_UNION:
      fprintf(OutFile, "%sunion\n%s{\n", s, s);
      break;
  }
  for (TmpObject = CCSG->FirstObject ; TmpObject ; TmpObject = TmpObject->Next)
  {
    switch (TmpObject->Type)
    {
      case BICUBIC_PATCH_OBJECT:
        SavePovBicubicPatch(OutFile, TmpObject, indentation+1);
        break;
      case BLOB_OBJECT:
        SavePovBlob(OutFile, TmpObject, indentation+1);
        break;
      case BOX_OBJECT:
        SavePovBox(OutFile, TmpObject, indentation+1);
        break;
      case CONE_OBJECT:
        SavePovCone(OutFile, TmpObject, indentation+1);
        break;
      case CSG_OBJECT:
        SavePovCSG(OutFile, TmpObject, indentation+1);
        break;
      case CUBIC_OBJECT:
        SavePovCubic(OutFile, TmpObject, indentation+1);
        break;
      case CYLINDER_OBJECT:
        SavePovCylinder(OutFile, TmpObject, indentation+1);
        break;
      case DISC_OBJECT:
        SavePovDisc(OutFile, TmpObject, indentation+1);
        break;
      case HEIGHT_FIELD_OBJECT:
        SavePovHeightField(OutFile, TmpObject, indentation+1);
        break;
      case JULIA_FRACTAL_OBJECT:
        SavePovJuliaFractal(OutFile, TmpObject, indentation+1);
        break;
      case LATHE_OBJECT:
        SavePovLathe(OutFile, TmpObject, indentation+1);
        break;
      case MESH_OBJECT:
        SavePovMesh(OutFile, TmpObject, indentation+1);
        break;
      case PLANE_OBJECT:
        SavePovPlane(OutFile, TmpObject, indentation+1);
        break;
      case POLY_OBJECT:
        SavePovPoly(OutFile, TmpObject, indentation+1);
        break;
      case POLYGON_OBJECT:
        SavePovPolygon(OutFile, TmpObject, indentation+1);
        break;
      case PRISM_OBJECT:
        SavePovPrism(OutFile, TmpObject, indentation+1);
        break;
      case QUADRIC_OBJECT:
        SavePovQuadric(OutFile, TmpObject, indentation+1);
        break;
      case QUARTIC_OBJECT:
        SavePovQuartic(OutFile, TmpObject, indentation+1);
        break;
      case SOR_OBJECT:
        SavePovSor(OutFile, TmpObject, indentation+1);
        break;
      case SPHERE_OBJECT:
        SavePovSphere(OutFile, TmpObject, indentation+1);
        break;
      case SUPERELLIPSOID_OBJECT:
        SavePovSuperEllipsoid(OutFile, TmpObject, indentation+1);
        break;
      case TEXT_OBJECT:
        SavePovText(OutFile, TmpObject, indentation+1);
        break;
      case TORUS_OBJECT:
        SavePovTorus(OutFile, TmpObject, indentation+1);
        break;
      case TRIANGLE_OBJECT:
        SavePovTriangle(OutFile, TmpObject, indentation+1);
        break;
    }
  }
  if (CSG->Texture)
  {
    SavePovTexture(OutFile, CSG->Texture, indentation+1);
  }
  if (CSG->Trans)
    SavePovTrans(OutFile, CSG->Trans, indentation+1);
  fprintf(OutFile, "%s}\n\n", s);
  g_free(s);
}

/*****************************************************************************
*  SavePovLightSource
******************************************************************************/
void SavePovLightSource(FILE *OutFile, LightSourceStruct *FirstLS)
{ /* FIXME: far from complete... */
  LightSourceStruct *TmpLS;

  for (TmpLS=FirstLS ; TmpLS ; TmpLS=TmpLS->Next)
  {
    fprintf(OutFile, "light_source\n{\n");
    fprintf(OutFile, "  <%g, %g, %g>\n", TmpLS->Location[0],
                                         TmpLS->Location[1],
                                         TmpLS->Location[2]);
    fprintf(OutFile, "  color rgb <%g, %g, %g>\n", TmpLS->Color[0],
                                               TmpLS->Color[1],
                                               TmpLS->Color[2]);
    fprintf(OutFile, "}\n");
  }
  fprintf(OutFile, "\n");
}

/*****************************************************************************
*  SavePovCamera
******************************************************************************/
void SavePovCamera(FILE *OutFile, CameraStruct *Camera)
{

  if (Camera == NULL)
    return;
  fprintf(OutFile, "camera\n{\n");
  switch (Camera->Type)
  {
    case CAMERA_PERSPECTIVE:
      fprintf(OutFile, "  perspective\n");
      break;
    case CAMERA_ORTHOGRAPHIC:
      fprintf(OutFile, "  orthographic\n");
      break;
    case CAMERA_FISHEYE:
      fprintf(OutFile, "  fisheye\n");
      break;
    case CAMERA_ULTRA_WIDE_ANGLE:
      fprintf(OutFile, "  ultra_wide_angle\n");
      break;
    case CAMERA_OMNIMAX:
      fprintf(OutFile, "  omnimax\n");
      break;
    case CAMERA_PANORAMIC:
      fprintf(OutFile, "  panoramic\n");
      break;
    case CAMERA_CYLINDER:
      fprintf(OutFile, "  cylinder %d\n", Camera->CylinderType);
    default:
      /* panic */
  }
  fprintf(OutFile, "  location <%g, %g, %g>\n", Camera->Location[0],
                                                Camera->Location[1],
                                                Camera->Location[2]);
  fprintf(OutFile, "  right <%g, %g, %g>\n", Camera->Right[0],
                                             Camera->Right[1],
                                             Camera->Right[2]);
  fprintf(OutFile, "  up <%g, %g, %g>\n", Camera->Up[0],
                                          Camera->Up[1],
                                          Camera->Up[2]);
  fprintf(OutFile, "  direction <%g, %g, %g>\n", Camera->Direction[0],
                                                 Camera->Direction[1],
                                                 Camera->Direction[2]);
  fprintf(OutFile, "  sky <%g, %g, %g>\n", Camera->Sky[0],
                                           Camera->Sky[1],
                                           Camera->Sky[2]);
  if (Camera->BlurSamples)
  {
    fprintf(OutFile, "  blur_samples %d\n", Camera->BlurSamples);
    fprintf(OutFile, "  aperture %g\n", Camera->Aperture);
    fprintf(OutFile, "  focal_point <%g, %g, %g>\n", Camera->FocalPoint[0],
                                                     Camera->FocalPoint[1],
                                                     Camera->FocalPoint[2]);
    fprintf(OutFile, "  confidence %g\n", Camera->Confidence);
    fprintf(OutFile, "  variance %g\n", Camera->Variance);
  }
  if (Camera->Normal)
  {
    SavePovNormal(OutFile, Camera->Normal, 1);
  }
  fprintf(OutFile, "}\n\n");
}

/*****************************************************************************
*  SavePovByName
******************************************************************************/
void SavePovByName(FrameStruct *LocalFrame, char *FileName)
{
  ObjectStruct *TmpObject;
  FILE *OutFile;

  OutFile = fopen(FileName, "w");

  fprintf(OutFile, _("// Persistence of Vision 3.0 File.\n"));
  fprintf(OutFile, _("// This file has been generated by giram.\n"));
  fprintf(OutFile, _("// If you found any error, please contact David.Odin@bigfoot.com\n\n"));
  fprintf(OutFile, _("// The Global Settings:\n"));
  SavePovGlobalSettings(OutFile, LocalFrame->Global);
  fprintf(OutFile, _("// The Camera:\n"));
  SavePovCamera(OutFile, LocalFrame->Camera);
  if (LocalFrame->SkySphere)
  {
    fprintf(OutFile, _("// The SkySphere:\n"));
    SavePovSkySphere(OutFile, LocalFrame->SkySphere);
  }
  fprintf(OutFile, _("// The Scene Light Sources:\n"));
  SavePovLightSource(OutFile, LocalFrame->FirstLightSource);
  fprintf(OutFile, _("// The Scene Objects:\n"));
  for (TmpObject = LocalFrame->FirstObject ; TmpObject ;
       TmpObject = TmpObject->Next)
  {
    switch (TmpObject->Type)
    {
      case BICUBIC_PATCH_OBJECT:
        SavePovBicubicPatch(OutFile, TmpObject, 0);
        break;
      case BLOB_OBJECT:
        SavePovBlob(OutFile, TmpObject, 0);
        break;
      case BOX_OBJECT:
        SavePovBox(OutFile, TmpObject, 0);
        break;
      case CONE_OBJECT:
        SavePovCone(OutFile, TmpObject, 0);
        break;
      case CSG_OBJECT:
        SavePovCSG(OutFile, TmpObject, 0);
        break;
      case CUBIC_OBJECT:
        SavePovCubic(OutFile, TmpObject, 0);
        break;
      case CYLINDER_OBJECT:
        SavePovCylinder(OutFile, TmpObject, 0);
        break;
      case DISC_OBJECT:
        SavePovDisc(OutFile, TmpObject, 0);
        break;
      case HEIGHT_FIELD_OBJECT:
        SavePovHeightField(OutFile, TmpObject, 0);
        break;
      case JULIA_FRACTAL_OBJECT:
        SavePovJuliaFractal(OutFile, TmpObject, 0);
        break;
      case LATHE_OBJECT:
        SavePovLathe(OutFile, TmpObject, 0);
        break;
      case MESH_OBJECT:
        SavePovMesh(OutFile, TmpObject, 0);
        break;
      case PLANE_OBJECT:
        SavePovPlane(OutFile, TmpObject, 0);
        break;
      case POLY_OBJECT:
        SavePovPoly(OutFile, TmpObject, 0);
        break;
      case POLYGON_OBJECT:
        SavePovPolygon(OutFile, TmpObject, 0);
        break;
      case PRISM_OBJECT:
        SavePovPrism(OutFile, TmpObject, 0);
        break;
      case QUADRIC_OBJECT:
        SavePovQuadric(OutFile, TmpObject, 0);
        break;
      case QUARTIC_OBJECT:
        SavePovQuartic(OutFile, TmpObject, 0);
        break;
      case SOR_OBJECT:
        SavePovSor(OutFile, TmpObject, 0);
        break;
      case SPHERE_OBJECT:
        SavePovSphere(OutFile, TmpObject, 0);
        break;
      case SUPERELLIPSOID_OBJECT:
        SavePovSuperEllipsoid(OutFile, TmpObject, 0);
        break;
      case TEXT_OBJECT:
        SavePovText(OutFile, TmpObject, 0);
        break;
      case TORUS_OBJECT:
        SavePovTorus(OutFile, TmpObject, 0);
        break;
      case TRIANGLE_OBJECT:
        SavePovTriangle(OutFile, TmpObject, 0);
        break;
    }
  }
  fclose(OutFile);
}

/*****************************************************************************
*  SavePov
******************************************************************************/
void SavePov(GtkWidget *w, GtkFileSelection *FS)
{
  FrameStruct *LocalFrame;
  char *FileName;

  FileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FS));
  LocalFrame = (FrameStruct *)gtk_object_get_data(GTK_OBJECT(FS), "frame");
  SavePovByName(LocalFrame, FileName);
}

