/* about_dialog.c 
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Note: this file is heavily inspired by the app/about_dialog.c of the
 * Gimp Package. So it is copyrighted:
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * I hope they forgive me for using it there :)
 */

#include <stdlib.h>
#include "giram.h"
#include "about_dialog.h"

static GtkWidget *AboutDialog;
static GtkWidget *ScrollArea = NULL;
static GtkWidget *LogoArea = NULL;
static GdkPixmap *ScrollPixmap = NULL;

static char *ContribList[] =
{
  "Philippe P.E. David",
  "Noah Davis",
  "Steeve Devergne",
  "Vik Olliver",
  "Ben Smith",
  "Joseph Turian",
};

static int NbContrib = sizeof (ContribList) / sizeof (ContribList[0]);
static int Widths[100];
static int CurScrollText = 0;
static int CurScrollIndex = 0;
static int ShuffleArray[ sizeof(ContribList) / sizeof(ContribList[0]) ];

static int LogoWidth = 0;
static int LogoHeight = 0;
static int do_animation = FALSE;
static int do_scrolling = FALSE;
static int ScrollState = 0;
static int frame = 0;
static int offset = 0;
static int timer = 0;

/*****************************************************************************
*  about_dialog_unmap
******************************************************************************/
static void UnMapAboutDialog()
{
  if (timer)
  {
    gtk_timeout_remove(timer);
    timer = 0;
  }
}

/*****************************************************************************
*  DestroyAboutDialog
******************************************************************************/
static void DestroyAboutDialog()
{
  AboutDialog = NULL;
  UnMapAboutDialog();
}

/*****************************************************************************
*  AboutDialogDelete
******************************************************************************/
static int AboutDialogDelete(GtkWidget *widget, GdkEventButton *event)
{
  if (timer)
    gtk_timeout_remove(timer);
  timer = 0;
  frame = 0;

  gtk_widget_hide(AboutDialog);

  return TRUE;
}

/*****************************************************************************
*  AboutDialogButton
******************************************************************************/
static int AboutDialogButton(GtkWidget *widget, GdkEventButton *event)
{
  if (timer)
    gtk_timeout_remove(timer);
  timer = 0;
  frame = 0;

  gtk_widget_hide(AboutDialog);

  return FALSE;
}

/*****************************************************************************
*  AboutDialogTimer
******************************************************************************/
static int AboutDialogTimer(gpointer data)
{
/*  gint i, j, k;*/
  gint return_val;

  return_val = TRUE;

  if (do_animation)
  {
/*    if (LogoArea->allocation.width != 1)
    {
      for (i = 0, k = 0; i < dissolve_height; i++)
        for (j = 0; j < dissolve_width; j++, k++)
          if (frame == dissolve_map[k])
          {
            gdk_draw_pixmap(logo_area->window, logo_area->style->black_gc,
                            logo_pixmap, j * ANIMATION_SIZE, i * ANIMATION_SIZE,
                            j * ANIMATION_SIZE, i * ANIMATION_SIZE,
                            ANIMATION_SIZE, ANIMATION_SIZE);
          }

      frame += 1;
      if (frame == ANIMATION_STEPS)
      {*/
        do_animation = FALSE;
        do_scrolling = TRUE;
        frame = 0;
        timer = gtk_timeout_add (75, AboutDialogTimer, NULL);
        return FALSE;
/*      }
    }*/
  }

  if (do_scrolling)
  {
    if (!ScrollPixmap)
      ScrollPixmap = gdk_pixmap_new(ScrollArea->window,
                                    ScrollArea->allocation.width,
                                    ScrollArea->allocation.height, -1);
    switch (ScrollState)
    {
      case 1:
        ScrollState = 2;
        timer = gtk_timeout_add(700, AboutDialogTimer, NULL);
        return_val = FALSE;
        break;
      case 2:
        ScrollState = 3;
        timer = gtk_timeout_add(75, AboutDialogTimer, NULL);
        return_val = FALSE;
        break;
    }

    if (offset > (Widths[CurScrollText] + ScrollArea->allocation.width))
    {
      ScrollState = 0;
      CurScrollIndex += 1;
      if (CurScrollIndex == NbContrib)
        CurScrollIndex = 0;
      CurScrollText = ShuffleArray[CurScrollIndex];
      offset = 0;
    }

    gdk_draw_rectangle(ScrollPixmap, ScrollArea->style->white_gc,
                       TRUE, 0, 0,
                       ScrollArea->allocation.width,
                       ScrollArea->allocation.height);
    gdk_draw_string(ScrollPixmap, ScrollArea->style->font,
                    ScrollArea->style->black_gc,
                    ScrollArea->allocation.width - offset,
                    ScrollArea->style->font->ascent,
                    ContribList[CurScrollText]);
    gdk_draw_pixmap(ScrollArea->window, ScrollArea->style->black_gc,
                    ScrollPixmap, 0, 0, 0, 0,
                    ScrollArea->allocation.width,
                    ScrollArea->allocation.height);

    offset += 15;
    if (ScrollState == 0)
    {
      if (offset > ((ScrollArea->allocation.width + Widths[CurScrollText]) / 2))
      {
        ScrollState = 1;
        offset = (ScrollArea->allocation.width + Widths[CurScrollText]) / 2;
      }
    }
  }
  return return_val;
}

/*****************************************************************************
*  AboutDialogLogoExpose
******************************************************************************/
int AboutDialogLogoExpose(GtkWidget *widget, GdkEventExpose *event)
{
  if (do_animation)
  {
    if (!timer)
    {
      AboutDialogTimer(widget);
      timer = gtk_timeout_add (75, AboutDialogTimer, NULL);
    }
  }
  else
  {
    /* If we draw beyond the boundaries of the pixmap, then X
     * will generate an expose area for those areas, starting
     * an infinite cycle. We now set allow_grow = FALSE, so
     * the drawing area can never be bigger than the preview.
     * Otherwise, it would be necessary to intersect event->area
     * with the pixmap boundary rectangle.
     */
/* FIXME     gdk_draw_pixmap(widget->window, widget->style->black_gc, logo_pixmap,
                     event->area.x, event->area.y,
                     event->area.x, event->area.y,
                     event->area.width, event->area.height);*/
  }
  return FALSE;
}

/*****************************************************************************
*  CreateAboutDialog
******************************************************************************/
void CreateAboutDialog()
{
  GtkWidget *VBox;
  GtkWidget *Label;
  GtkStyle *style;
  gchar *LabelText;
  GtkWidget *Alignment;
  GtkWidget *Frame;
  gint MaxWidth;
  int i;

  if (!AboutDialog)
  {
    AboutDialog = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_wmclass(GTK_WINDOW(AboutDialog), "about_dialog", "Giram");
    gtk_window_set_title(GTK_WINDOW(AboutDialog), _("About Giram"));
    gtk_window_set_policy(GTK_WINDOW(AboutDialog), FALSE, FALSE, FALSE);
    gtk_window_position(GTK_WINDOW(AboutDialog), GTK_WIN_POS_CENTER);
    gtk_signal_connect(GTK_OBJECT(AboutDialog), "destroy",
                       (GtkSignalFunc)DestroyAboutDialog, NULL);
    gtk_signal_connect(GTK_OBJECT(AboutDialog), "unmap_event",
                       (GtkSignalFunc)UnMapAboutDialog, NULL);
    gtk_signal_connect(GTK_OBJECT(AboutDialog), "button_press_event",
                       (GtkSignalFunc)AboutDialogButton, NULL);
    gtk_signal_connect(GTK_OBJECT(AboutDialog), "delete_event",
                       (GtkSignalFunc)AboutDialogDelete, NULL);
    gtk_widget_set_events(AboutDialog, GDK_BUTTON_PRESS_MASK);
                                    
/*    if (!AboutDialogLoadLogo(about_dialog))
    {
      gtk_widget_destroy(AboutDialog);
      AboutDialog = NULL;
      return;
    }*/

    VBox = gtk_vbox_new(FALSE, 1);
    gtk_container_border_width(GTK_CONTAINER(VBox), 1);
    gtk_container_add(GTK_CONTAINER(AboutDialog), VBox);
    gtk_widget_show(VBox);

    Frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(Frame), GTK_SHADOW_IN);
    gtk_container_border_width(GTK_CONTAINER(Frame), 0);
    gtk_box_pack_start(GTK_BOX(VBox), Frame, TRUE, TRUE, 0);
    gtk_widget_show(Frame);

    LogoArea = gtk_drawing_area_new();
    gtk_signal_connect(GTK_OBJECT(LogoArea), "expose_event",
                       (GtkSignalFunc)AboutDialogLogoExpose, NULL);
    gtk_drawing_area_size(GTK_DRAWING_AREA(LogoArea), LogoWidth, LogoHeight);
    gtk_widget_set_events(LogoArea, GDK_EXPOSURE_MASK);
    gtk_container_add(GTK_CONTAINER(Frame), LogoArea);
    gtk_widget_show(LogoArea);

    gtk_widget_realize(LogoArea);
    gdk_window_set_background(LogoArea->window, &LogoArea->style->black);
    
    Label = gtk_label_new(_("Giram"));
    gtk_box_pack_start(GTK_BOX(VBox), Label, TRUE, TRUE, 0);
    gtk_widget_show(Label);

    style = gtk_style_new();
    gdk_font_unref(style->font);
    style->font = gdk_font_load(_("-Adobe-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*"));
    gtk_widget_push_style(style);

    LabelText = g_strdup_printf("%s%s%s", _("Version "), VERSION,
                                _(" brought to you by"));
    Label = gtk_label_new(LabelText);
    g_free(LabelText);
    LabelText=NULL;
    gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, TRUE, 0);
    gtk_widget_show(Label);

    Label = gtk_label_new(_("David Odin (aka DindinX) David.Odin@bigfoot.com"));
    gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, TRUE, 0);
    gtk_widget_show(Label);

    gtk_widget_pop_style();

    Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
    gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
    gtk_widget_show(Alignment);

    Frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(Frame), GTK_SHADOW_IN);
    gtk_container_border_width(GTK_CONTAINER(Frame), 0);
    gtk_container_add(GTK_CONTAINER(Alignment), Frame);
    gtk_widget_show(Frame);

    MaxWidth = 0;
    for (i = 0; i < sizeof (ContribList) / sizeof (ContribList[0]) ; i++)
    {
      Widths[i] = gdk_string_width(Frame->style->font, ContribList[i]);
      MaxWidth = MAX(MaxWidth, Widths[i]);
    }

    ScrollArea = gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(ScrollArea), MaxWidth + 10,
                          Frame->style->font->ascent + Frame->style->font->descent);
    gtk_widget_set_events(ScrollArea, GDK_BUTTON_PRESS_MASK);
    gtk_container_add(GTK_CONTAINER(Frame), ScrollArea);
    gtk_widget_show(ScrollArea);

    Label = gtk_label_new(_("Please visit http://www.minet.net/giram/ for more info"));
    gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, TRUE, 0);
    gtk_widget_show(Label);

    gtk_widget_realize(ScrollArea);
    gdk_window_set_background(ScrollArea->window, &ScrollArea->style->white);
  }

  if (!GTK_WIDGET_VISIBLE(AboutDialog))
  {
    gtk_widget_show(AboutDialog);

    do_animation = TRUE;
    do_scrolling = FALSE;
    ScrollState = 0;
    frame = 0;
    offset = 0;

    for (i=0; i<NbContrib ; i++)
    {
      ShuffleArray[i] = i;
    }

    for (i=0; i<NbContrib ; i++)
    {
      int j, k;
 
      j = rand() % NbContrib;
      k = rand() % NbContrib;
      if (j != k)
      {
        int t;

        t = ShuffleArray[j];
        ShuffleArray[j] = ShuffleArray[k];
        ShuffleArray[k] = t;
      }
    }
  }
  else
  {
    gdk_window_raise(AboutDialog->window);
  }
}

