/* bicubic_patch.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "csgtree.h"
#include "object.h"
#include "bicubic_patch.h"

/*****************************************************************************
*  InsideBicubicPatch
******************************************************************************/
int InsideBicubicPatch(ObjectStruct *BicubicPatch, double x, double y, double z)
{
  return 0; /* We cannot be "Inside" a bicubic_patch */
}

/*****************************************************************************
*  FindIntersectBicubicPatchSegment
******************************************************************************/
int FindIntersectionBicubicPatchSegment(Vector InterPoint, Vector InterNorm,
                                        Vector InPoint, Vector OutPoint,
                                        ObjectStruct *BicubicPatch)
{
  return 0;
}

/*****************************************************************************
*  ComputeBicubicPatchTriangleMesh FIXME
******************************************************************************/
void ComputeBicubicPatchTriangleMesh(ObjectStruct *BicubicPatch)
{ /* XXX: clipped_by */
}

/*****************************************************************************
*  TranslateBicubicPatch
******************************************************************************/
void TranslateBicubicPatch(ObjectStruct *BicubicPatch, Vector Vect)
{
  TransformStruct Trans;

  if (BicubicPatch->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(BicubicPatch->Trans, &Trans);
  } else
  {
    BicubicPatch->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(BicubicPatch->Trans, Vect);
  }
  if (BicubicPatch->ClippedBy)
    TranslateObject(BicubicPatch->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateBicubicPatch
******************************************************************************/
void RotateBicubicPatch(ObjectStruct *BicubicPatch, Vector Vect)
{
  TransformStruct Trans;

  if (BicubicPatch->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(BicubicPatch->Trans, &Trans);
  } else
  {
    BicubicPatch->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(BicubicPatch->Trans, Vect);
  }
  if (BicubicPatch->ClippedBy)
    RotateObject(BicubicPatch->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleBicubicPatch
******************************************************************************/
void ScaleBicubicPatch(ObjectStruct *BicubicPatch, Vector Vect)
{
  TransformStruct Trans;

  if (BicubicPatch->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(BicubicPatch->Trans, &Trans);
  } else
  {
    BicubicPatch->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(BicubicPatch->Trans, Vect);
  }
  if (BicubicPatch->ClippedBy)
    ScaleObject(BicubicPatch->ClippedBy, Vect);
}

/*****************************************************************************
*  IsBicubicPatchIntersection FIXME
******************************************************************************/
int IsBicubicPatchIntersection(ObjectStruct *BicubicPatch, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;

  if (BicubicPatch->Trans)
  {
    MInverseEvaluatePoint(Org, BicubicPatch->Trans, Origin);
    MInverseEvaluateVector(Dir, BicubicPatch->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInBicubicPatchXY
******************************************************************************/
int ClickInBicubicPatchXY(ObjectStruct *BicubicPatch, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsBicubicPatchIntersection(BicubicPatch, Origin, Direction);
}

/*****************************************************************************
*  ClickInBicubicPatchXZ
******************************************************************************/
int ClickInBicubicPatchXZ(ObjectStruct *BicubicPatch, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsBicubicPatchIntersection(BicubicPatch, Origin, Direction);
}

/*****************************************************************************
*  ClickInBicubicPatchZY
******************************************************************************/
int ClickInBicubicPatchZY(ObjectStruct *BicubicPatch, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsBicubicPatchIntersection(BicubicPatch, Origin, Direction);
}

/*****************************************************************************
*  ToolNewBicubicPatch FIXME
******************************************************************************/
void ToolNewBicubicPatch(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Label;

  Dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New Bicubic Patch"));

  Label = gtk_label_new(_("Sorry, the New Bicubic Patch function"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);
  Label = gtk_label_new(_("hasn't been implemented yet"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);

  Button = gtk_button_new_with_label("Close");
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button,
                     TRUE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  gtk_widget_show(Dialog);
}
