/* blob.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "csgtree.h"
#include "object.h"
#include "blob.h"

/*****************************************************************************
*  InsideBlob FIXME
******************************************************************************/
int InsideBlob(ObjectStruct *Blob, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Blob->Trans)
  {
    MInverseEvaluatePoint(Vect, Blob->Trans, Vect);
  }
  return 0;
}

/*****************************************************************************
*  FindIntersectBlobSegment FIXME
******************************************************************************/
int FindIntersectionBlobSegment(Vector InterPoint, Vector InterNorm,
                                Vector InPoint, Vector OutPoint,
                                ObjectStruct *Blob)
{
  return 0;
}

/*****************************************************************************
*  ComputeBlobTriangleMesh FIXME
******************************************************************************/
void ComputeBlobTriangleMesh(ObjectStruct *Blob)
{ /* XXX: clipped_by */
}

/*****************************************************************************
*  TranslateBlob
******************************************************************************/
void TranslateBlob(ObjectStruct *Blob, Vector Vect)
{
  TransformStruct Trans;

  if (Blob->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Blob->Trans, &Trans);
  } else
  {
    Blob->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Blob->Trans, Vect);
  }
  if (Blob->ClippedBy)
    TranslateObject(Blob->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateBlob
******************************************************************************/
void RotateBlob(ObjectStruct *Blob, Vector Vect)
{
  TransformStruct Trans;

  if (Blob->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Blob->Trans, &Trans);
  } else
  {
    Blob->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Blob->Trans, Vect);
  }
  if (Blob->ClippedBy)
    RotateObject(Blob->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleBlob
******************************************************************************/
void ScaleBlob(ObjectStruct *Blob, Vector Vect)
{
  TransformStruct Trans;

  if (Blob->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Blob->Trans, &Trans);
  } else
  {
    Blob->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Blob->Trans, Vect);
  }
  if (Blob->ClippedBy)
    ScaleObject(Blob->ClippedBy, Vect);
}

/*****************************************************************************
*  IsBlobIntersection FIXME
******************************************************************************/
int IsBlobIntersection(ObjectStruct *Blob, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;

  if (Blob->Trans)
  {
    MInverseEvaluatePoint(Org, Blob->Trans, Origin);
    MInverseEvaluateVector(Dir, Blob->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInBlobXY
******************************************************************************/
int ClickInBlobXY(ObjectStruct *Blob, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsBlobIntersection(Blob, Origin, Direction);
}

/****************************************************************************
*  ClickInBlobXZ
*****************************************************************************/
int ClickInBlobXZ(ObjectStruct *Blob, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsBlobIntersection(Blob, Origin, Direction);
}

/****************************************************************************
*  ClickInBlobZY
*****************************************************************************/
int ClickInBlobZY(ObjectStruct *Blob, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsBlobIntersection(Blob, Origin, Direction);
}

/*****************************************************************************
*  ToolNewBlob FIXME
******************************************************************************/
void ToolNewBlob(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Label;

  Dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New Blob"));

  Label = gtk_label_new(_("Sorry, the New Blob function"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);
  Label = gtk_label_new(_("hasn't been implemented yet"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);

  Button = gtk_button_new_with_label("Close");
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button,
                     TRUE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  gtk_widget_show(Dialog);
}
