/* box.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "snap.h"
#include "box.h"

/*****************************************************************************
*  InsideBox
******************************************************************************/
int InsideBox(ObjectStruct *Box, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;
  BoxStruct *BBox = (BoxStruct *)Box;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Box->Trans)
  {
    MInverseEvaluatePoint(Vect, Box->Trans, Vect);
  }
  if ((Vect[0] < BBox->MinCorner[0]) || (Vect[0] > BBox->MaxCorner[0]) ||
      (Vect[1] < BBox->MinCorner[1]) || (Vect[1] > BBox->MaxCorner[1]) ||
      (Vect[2] < BBox->MinCorner[2]) || (Vect[2] > BBox->MaxCorner[2]))
    return Box->Inverse;
  else
    return !(Box->Inverse);
}

/*****************************************************************************
*  FindIntersectBoxSegment FIXME
******************************************************************************/
int FindIntersectionBoxSegment(Vector InterPoint, Vector InterNorm,
                               Vector InPoint, Vector OutPoint,
                               ObjectStruct *Box)
{
  return 0;
}

/*****************************************************************************
*  ComputeBoxTriangleMesh
******************************************************************************/
void ComputeBoxTriangleMesh(ObjectStruct *Box)
{ /* XXX: clipped_by */ /* XXX: should be scalable: use detaillevel in a way */
  Vector P[8];
  Vector N[6];
  BoxStruct *BBox = (BoxStruct *)Box;

  if (Box->FirstTriangle)
    DestroyObjectTriangleMesh(Box);
  P[0][0] = BBox->MinCorner[0];
  P[0][1] = BBox->MinCorner[1];
  P[0][2] = BBox->MinCorner[2];
  P[1][0] = BBox->MinCorner[0];
  P[1][1] = BBox->MinCorner[1];
  P[1][2] = BBox->MaxCorner[2];
  P[2][0] = BBox->MinCorner[0];
  P[2][1] = BBox->MaxCorner[1];
  P[2][2] = BBox->MinCorner[2];
  P[3][0] = BBox->MinCorner[0];
  P[3][1] = BBox->MaxCorner[1];
  P[3][2] = BBox->MaxCorner[2];
  P[4][0] = BBox->MaxCorner[0];
  P[4][1] = BBox->MinCorner[1];
  P[4][2] = BBox->MinCorner[2];
  P[5][0] = BBox->MaxCorner[0];
  P[5][1] = BBox->MinCorner[1];
  P[5][2] = BBox->MaxCorner[2];
  P[6][0] = BBox->MaxCorner[0];
  P[6][1] = BBox->MaxCorner[1];
  P[6][2] = BBox->MinCorner[2];
  P[7][0] = BBox->MaxCorner[0];
  P[7][1] = BBox->MaxCorner[1];
  P[7][2] = BBox->MaxCorner[2];
  N[0][0] =  0.0; N[0][1] = -1.0; N[0][2] =  0.0;
  N[1][0] =  0.0; N[1][1] =  1.0; N[1][2] =  0.0;
  N[2][0] =  0.0; N[2][1] =  0.0; N[2][2] = -1.0;
  N[3][0] =  0.0; N[3][1] =  0.0; N[3][2] =  1.0;
  N[4][0] = -1.0; N[4][1] =  0.0; N[4][2] =  0.0;
  N[5][0] =  1.0; N[5][1] =  0.0; N[5][2] =  0.0;

  if (Box->Inverse)
  {
    N[0][1] = 1.0; N[1][1] = -1.0;
    N[2][2] = 1.0; N[3][2] = -1.0;
    N[4][0] = 1.0; N[5][0] = -1.0;
  }
  
  if (Box->Trans)
  {
    MEvaluatePoint(P[0], Box->Trans, P[0]);
    MEvaluatePoint(P[1], Box->Trans, P[1]);
    MEvaluatePoint(P[2], Box->Trans, P[2]);
    MEvaluatePoint(P[3], Box->Trans, P[3]);
    MEvaluatePoint(P[4], Box->Trans, P[4]);
    MEvaluatePoint(P[5], Box->Trans, P[5]);
    MEvaluatePoint(P[6], Box->Trans, P[6]);
    MEvaluatePoint(P[7], Box->Trans, P[7]);
    MEvaluateVector(N[0], Box->Trans, N[0]);
    MEvaluateVector(N[1], Box->Trans, N[1]);
    MEvaluateVector(N[2], Box->Trans, N[2]);
    MEvaluateVector(N[3], Box->Trans, N[3]);
    MEvaluateVector(N[4], Box->Trans, N[4]);
    MEvaluateVector(N[5], Box->Trans, N[5]);
  }
    
  AddTriangleToObjectMesh(Box, P[0],P[1],P[4], N[0], N[0], N[0]);
  AddTriangleToObjectMesh(Box, P[5],P[1],P[4], N[0], N[0], N[0]);
  AddTriangleToObjectMesh(Box, P[2],P[3],P[6], N[1], N[1], N[1]);
  AddTriangleToObjectMesh(Box, P[7],P[3],P[6], N[1], N[1], N[1]);
  AddTriangleToObjectMesh(Box, P[0],P[2],P[4], N[2], N[2], N[2]);
  AddTriangleToObjectMesh(Box, P[2],P[6],P[4], N[2], N[2], N[2]);
  AddTriangleToObjectMesh(Box, P[1],P[3],P[5], N[3], N[3], N[3]);
  AddTriangleToObjectMesh(Box, P[7],P[3],P[5], N[3], N[3], N[3]);
  AddTriangleToObjectMesh(Box, P[0],P[1],P[2], N[4], N[4], N[4]);
  AddTriangleToObjectMesh(Box, P[1],P[2],P[3], N[4], N[4], N[4]);
  AddTriangleToObjectMesh(Box, P[4],P[5],P[6], N[5], N[5], N[5]);
  AddTriangleToObjectMesh(Box, P[5],P[6],P[7], N[5], N[5], N[5]);
}

/*****************************************************************************
*  TranslateBox
******************************************************************************/
void TranslateBox(ObjectStruct *Box, Vector Vect)
{
  TransformStruct Trans;
  BoxStruct *BBox = (BoxStruct *)Box;

  if (Box->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Box->Trans, &Trans);
  } else
  {
    BBox->MinCorner[0] += Vect[0];
    BBox->MinCorner[1] += Vect[1];
    BBox->MinCorner[2] += Vect[2];
    BBox->MaxCorner[0] += Vect[0];
    BBox->MaxCorner[1] += Vect[1];
    BBox->MaxCorner[2] += Vect[2];
  }
  if (Box->ClippedBy)
    TranslateObject(Box->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateBox
******************************************************************************/
void RotateBox(ObjectStruct *Box, Vector Vect)
{
  TransformStruct Trans;

  if (Box->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Box->Trans, &Trans);
  } else
  {
    Box->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Box->Trans, Vect);
  }
  if (Box->ClippedBy)
    RotateObject(Box->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleBox
******************************************************************************/
void ScaleBox(ObjectStruct *Box, Vector Vect)
{
  BoxStruct *BBox = (BoxStruct *)Box;
  TransformStruct Trans;
  double Tmp;

  if (Box->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Box->Trans, &Trans);
  } else
  {
    BBox->MinCorner[0] *= Vect[0];
    BBox->MaxCorner[0] *= Vect[0];
    BBox->MinCorner[1] *= Vect[1];
    BBox->MaxCorner[1] *= Vect[1];
    BBox->MinCorner[2] *= Vect[2];
    BBox->MaxCorner[2] *= Vect[2];
    if (BBox->MinCorner[0] > BBox->MaxCorner[0])
    {
      Tmp = BBox->MinCorner[0];
      BBox->MinCorner[0] = BBox->MaxCorner[0];
      BBox->MaxCorner[0] = Tmp;
    }
    if (BBox->MinCorner[1] > BBox->MaxCorner[1])
    {
      Tmp = BBox->MinCorner[1];
      BBox->MinCorner[1] = BBox->MaxCorner[1];
      BBox->MaxCorner[1] = Tmp;
    }
    if (BBox->MinCorner[2] > BBox->MaxCorner[2])
    {
      Tmp = BBox->MinCorner[2];
      BBox->MinCorner[2] = BBox->MaxCorner[2];
      BBox->MaxCorner[2] = Tmp;
    }
  }
  if (Box->ClippedBy)
    ScaleObject(Box->ClippedBy, Vect);
}

/*****************************************************************************
*  IsBoxIntersection
******************************************************************************/
int IsBoxIntersection(ObjectStruct *Box, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
  double tnear, tfar;
  double t1, t2, tmp;
  BoxStruct *BBox = (BoxStruct *)Box;

  if (Box->Trans)
  {
    MInverseEvaluatePoint(Org, Box->Trans, Origin);
    MInverseEvaluateVector(Dir, Box->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  
  tnear = -G_MAXDOUBLE; tfar = G_MAXDOUBLE;

  if (fabs(Dir[0]) > 10e-6)
  {
    t1 = (BBox->MinCorner[0]-Org[0]) / Dir[0];
    t2 = (BBox->MaxCorner[0]-Org[0]) / Dir[0];
    if ( t1 > t2 ) 
    {
      tmp = t1; t1 = t2 ; t2 = tmp;
    }
    if (t1 > tnear)
      tnear = t1;
    if (t2 < tfar)
      tfar = t2;
    if (tnear > tfar)
      return 0;
  } else
  {
    if (Org[0] < BBox->MinCorner[0] || 
        Org[0] > BBox->MaxCorner[0])
      return 0;
  }
  if (fabs(Dir[1]) > 10e-6)
  {
    t1 = (BBox->MinCorner[1]-Org[1]) / Dir[1];
    t2 = (BBox->MaxCorner[1]-Org[1]) / Dir[1];
    if ( t1 > t2 )
    {
      tmp = t1; t1 = t2 ; t2 = tmp;
    }
    if (t1 > tnear)
      tnear = t1;
    if (t2 < tfar)
      tfar = t2;
    if (tnear > tfar)
      return 0;
  } else
  {
    if (Org[1] < BBox->MinCorner[1] ||
        Org[1] > BBox->MaxCorner[1])
      return 0;
  }
  if (fabs(Dir[2]) > 10e-6)
  {
    t1 = (BBox->MinCorner[2]-Org[2]) / Dir[2];
    t2 = (BBox->MaxCorner[2]-Org[2]) / Dir[2];
    if ( t1 > t2 )
    {
      tmp = t1; t1 = t2 ; t2 = tmp;
    }
    if (t1 > tnear)
      tnear = t1;
    if (t2 < tfar)
      tfar = t2;
    if (tnear > tfar)
      return 0;
  } else
  {
    if (Org[2] < BBox->MinCorner[2] ||
        Org[2] > BBox->MaxCorner[2])
      return 0;
  }
  return 1;
}

/*****************************************************************************
*  ClickInBoxXY
******************************************************************************/
int ClickInBoxXY(ObjectStruct *Box, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};
  BoxStruct *BBox = (BoxStruct *)Box;

  if (Box->Trans)
  {
    Origin[0] = x;
    Origin[1] = y;
    return IsBoxIntersection(Box, Origin, Direction);
  } else if ( (BBox->MinCorner[0] < x) && (BBox->MaxCorner[0] > x) &&
              (BBox->MinCorner[1] < y) && (BBox->MaxCorner[1] > y))
    return !(Box->Inverse);
  else
    return Box->Inverse;
}

/****************************************************************************
*  ClickInBoxXZ
*****************************************************************************/
int ClickInBoxXZ(ObjectStruct *Box, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};
  BoxStruct *BBox = (BoxStruct *)Box;

  if (Box->Trans)
  {
    Origin[0] = x;
    Origin[2] = y;
    return IsBoxIntersection(Box, Origin, Direction);
  } else if ( (BBox->MinCorner[0] < x) && (BBox->MaxCorner[0] > x) &&
              (BBox->MinCorner[2] < y) && (BBox->MaxCorner[2] > y))
    return !(Box->Inverse);
  else
    return Box->Inverse;
}

/****************************************************************************
*  ClickInBoxZY
*****************************************************************************/
int ClickInBoxZY(ObjectStruct *Box, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};
  BoxStruct *BBox = (BoxStruct *)Box;

  if (Box->Trans)
  {
    Origin[2] = x;
    Origin[1] = y;
    return IsBoxIntersection(Box, Origin, Direction);
  } else if ( (BBox->MinCorner[2] < x) && (BBox->MaxCorner[2] > x) &&
              (BBox->MinCorner[1] < y) && (BBox->MaxCorner[1] > y))
    return !(Box->Inverse);
  else
    return Box->Inverse;
}

double Xorg, Yorg, Xold, Yold;
GdkColormap *colormap;
GdkGC *gcPurple;
GdkColor Purple;
guint id;

#define NONE   1
#define MOVING 2
int BOX_STATE = NONE;

/*****************************************************************************
*  ToolInitNewBox
******************************************************************************/
void ToolInitNewBox(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ViewUserDataStruct *UserData;
  GdkColormap *colormap;
  GtkWidget *StatusBar;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  id = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "new box");
  MouseToReal(bevent->x, bevent->y, DrawingArea, &Xorg, &Yorg);
  colormap = gdk_window_get_colormap(DrawingArea->window);
  gcPurple = gdk_gc_new(DrawingArea->window);
  Purple.red = 65535; Purple.green = 0; Purple.blue = 65535;
  /* XXX: the color should be settable... */
  gdk_color_alloc(colormap, &Purple);
  gdk_gc_set_foreground(gcPurple, &Purple);
  gdk_gc_set_function(gcPurple, GDK_XOR);

  Xold = Xorg; Yold = Yorg;

  BOX_STATE = MOVING;
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, _("Box: "));
  gdk_pointer_grab(DrawingArea->window, FALSE,
                   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                   NULL, NULL, bevent->time);
}

/*****************************************************************************
*  ToolMotionNewBox
******************************************************************************/
void ToolMotionNewBox(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventMotion *Mev = (GdkEventMotion *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  char Message[50];
  double x1, y1, x2, y2;
  Vector MinCorner, MaxCorner;

  if (BOX_STATE != MOVING)
    return;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);
      
  x1 = MIN(Xorg, Xold); x2 = MAX(Xorg, Xold);
  y1 = MIN(Yorg, Yold); y2 = MAX(Yorg, Yold);
  gdk_draw_rectangle(DrawingArea->window, gcPurple, FALSE,
                     DrawingArea->allocation.width/2+(x1-Xoff)*Zoom,
                     DrawingArea->allocation.height/2-(y2-Yoff)*Zoom,
                     (x2-x1)*Zoom, (y2-y1)*Zoom);
  MouseToReal(Mev->x, Mev->y, DrawingArea, &Xold, &Yold);
  x1 = MIN(Xorg, Xold); x2 = MAX(Xorg, Xold);
  y1 = MIN(Yorg, Yold); y2 = MAX(Yorg, Yold);
  gdk_draw_rectangle(DrawingArea->window, gcPurple, FALSE,
                     DrawingArea->allocation.width/2+(x1-Xoff)*Zoom,
                     DrawingArea->allocation.height/2-(y2-Yoff)*Zoom,
                     (x2-x1)*Zoom, (y2-y1)*Zoom);
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      MinCorner[0] = x1;
      MinCorner[1] = y1;
      MinCorner[2] = 0.0;
      MaxCorner[0] = x2;
      MaxCorner[1] = y2;
      MaxCorner[2] = 1.0;
      break;
    case ORTHO_XZ_CAMERA:
      MinCorner[0] = x1;
      MinCorner[1] = 0.0;
      MinCorner[2] = y1;
      MaxCorner[0] = x2;
      MaxCorner[1] = 1.0;
      MaxCorner[2] = y2;
      break;
    case ORTHO_ZY_CAMERA:
      MinCorner[0] = 0.0;
      MinCorner[1] = y1;
      MinCorner[2] = x1;
      MaxCorner[0] = 1.0;
      MaxCorner[1] = y2;
      MaxCorner[2] = x2;
      break;
  }
  sprintf(Message, _("Box: <%g, %g, %g>, <%g, %g, %g>"),
          MinCorner[0], MinCorner[1], MinCorner[2],
          MaxCorner[0], MaxCorner[1], MaxCorner[2]);
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
}

/*****************************************************************************
*  ToolReleaseNewBox
******************************************************************************/
void ToolReleaseNewBox(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventButton *Bev = (GdkEventButton *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  FrameViewStruct *TmpView;
  ObjectStruct *Box;
  BoxStruct *BBox;
  double gx1, gy1, gx2, gy2;

  if (BOX_STATE != MOVING)
    return;
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;

  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

  BBox = g_malloc(sizeof(BoxStruct));
  Box = (ObjectStruct *)BBox;
  InitObject(Box);
  Box->Type = BOX_OBJECT;
  MouseToReal(Bev->x, Bev->y, DrawingArea, &Xold, &Yold);
  gx1 = MIN(Xorg, Xold);
  gx2 = MAX(Xorg, Xold);
  gy1 = MIN(Yorg, Yold);
  gy2 = MAX(Yorg, Yold);
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      BBox->MinCorner[0] = gx1;
      BBox->MinCorner[1] = gy1;
      BBox->MinCorner[2] = 0.0;
      BBox->MaxCorner[0] = gx2;
      BBox->MaxCorner[1] = gy2;
      BBox->MaxCorner[2] = 1.0;
      break;
    case ORTHO_XZ_CAMERA:
      BBox->MinCorner[0] = gx1;
      BBox->MinCorner[1] = 0.0;
      BBox->MinCorner[2] = gy1;
      BBox->MaxCorner[0] = gx2;
      BBox->MaxCorner[1] = 1.0;
      BBox->MaxCorner[2] = gy2;
      break;
    case ORTHO_ZY_CAMERA:
      BBox->MinCorner[0] = 0.0;
      BBox->MinCorner[1] = gy1;
      BBox->MinCorner[2] = gx1;
      BBox->MaxCorner[0] = 1.0;
      BBox->MaxCorner[1] = gy2;
      BBox->MaxCorner[2] = gx2;
      break;
  }
  Box->Next = UserData->FrameView->FirstObject;
  UserData->FrameView->FirstObject = Box;
  ComputeBoxTriangleMesh(Box);
  BOX_STATE = NONE;

  gdk_gc_destroy(gcPurple);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
  gdk_pointer_ungrab(Bev->time);
}

/*****************************************************************************
*  PDBCreateBox
******************************************************************************/
int PDBCreateBox(int *Id, Vector *MinCoord, Vector *MaxCoord)
{
  ObjectStruct *Box;
  BoxStruct *BBox;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  BBox = g_malloc(sizeof(BoxStruct));
  Box = (ObjectStruct *)BBox;
  InitObject(Box);
  Box->Type = BOX_OBJECT;
  BBox->MinCorner[0] = (*MinCoord)[0];
  BBox->MinCorner[1] = (*MinCoord)[1];
  BBox->MinCorner[2] = (*MinCoord)[2];
  BBox->MaxCorner[0] = (*MaxCoord)[0];
  BBox->MaxCorner[1] = (*MaxCoord)[1];
  BBox->MaxCorner[2] = (*MaxCoord)[2];

  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
        ;
  Box->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Box;
  ComputeBoxTriangleMesh(Box);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Box->Id;
}

