/* cone.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "cone.h"

int ConeFunnelFlag=0;
int ConeOpenFlag=0;

/*****************************************************************************
*  InsideCone
******************************************************************************/
int InsideCone(ObjectStruct *Cone, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;
  ConeStruct *CCone = (ConeStruct *)Cone;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Cone->Trans)
  {
    MInverseEvaluatePoint(Vect, Cone->Trans, Vect);
  }
  if ( (CCone->Radius1 > Vect[1]) || (CCone->Radius2 < Vect[1]) )
    return Cone->Inverse;
  if (Vect[0]*Vect[0]+Vect[2]*Vect[2] < Vect[1]*Vect[1])
    return !(Cone->Inverse);
  else
    return Cone->Inverse;
}

/*****************************************************************************
*  FindIntersectConeSegment
******************************************************************************/
int FindIntersectionConeSegment(Vector InterPoint,Vector InterNorm,
                                Vector InPoint, Vector OutPoint,
                                ObjectStruct *Cone)
{
  int i;
  Vector InP, OutP;
  double A, B, C, Delta, t;
  Vector Tmp;
  ConeStruct *CCone = (ConeStruct *)Cone;

  if (Cone->Trans)
  {
    MInverseEvaluatePoint(InP, Cone->Trans, InPoint);
    MInverseEvaluatePoint(OutP, Cone->Trans, OutPoint);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      InP[i] = InPoint[i];
      OutP[i] = OutPoint[i];
    }
  }
  A =   (OutP[0]-InP[0])*(OutP[0]-InP[0])
      - (OutP[1]-InP[1])*(OutP[1]-InP[1])
      + (OutP[2]-InP[2])*(OutP[2]-InP[2]);
  B = 2.0* ( InP[0]*(OutP[0]-InP[0]) - InP[1]*(OutP[1]-InP[1]) + InP[2]*(OutP[2]-InP[2]) );
  C = InP[0]*InP[0] - InP[1]*InP[1] + InP[2]*InP[2];
  Delta = B*B - 4*A*C;
  if (fabs(A) < 10e-6)
  {
    /* FIXME */
    return 0;
  }
  if (Delta < 0.0)
    return 0;
  t = (-B-sqrt(Delta)) / (2.0*A);
  if ((t >= 0.0) && (t <= 1.0))
  {
    Tmp[0] = InP[0] + (OutP[0] - InP[0]) * t;
    Tmp[1] = InP[1] + (OutP[1] - InP[1]) * t;
    Tmp[2] = InP[2] + (OutP[2] - InP[2]) * t;
    if ( (Tmp[1] > CCone->Radius1) && (Tmp[1] < CCone->Radius2) )
    {
      if (Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2] > 10e-6)
      {
        InterPoint[0] = Tmp[0]; InterPoint[1] = Tmp[1]; InterPoint[2] = Tmp[2];
        InterNorm[0] = sqrt(0.5) * Tmp[0] / sqrt(Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2]);
        if (Tmp[1] > 0.0)
          InterNorm[1] = -sqrt(1.0 / 2.0);
        else
          InterNorm[1] = sqrt(1.0 / 2.0);
        InterNorm[2] = sqrt(0.5) * Tmp[2] / sqrt(Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2]);
      } else
      {
        InterNorm[0] = 0.0; InterNorm[1] = 1.0; InterNorm[2] = 0.0;
      }
      if (Cone->Trans)
      {
        MEvaluatePoint(InterPoint, Cone->Trans, InterPoint);
        MEvaluateVector(InterNorm, Cone->Trans, InterNorm);
      }
      return 1;
    }
  }
  t = (-B + sqrt(Delta)) / (2.0 * A);
  if ((t >= 0.0) && (t <= 1.0))
  {
    Tmp[0] = InP[0] + (OutP[0] - InP[0]) * t;
    Tmp[1] = InP[1] + (OutP[1] - InP[1]) * t;
    Tmp[2] = InP[2] + (OutP[2] - InP[2]) * t;
    if ( (Tmp[1] > CCone->Radius1) && (Tmp[1] < CCone->Radius2) )
    {
      InterPoint[0] = Tmp[0]; InterPoint[1] = Tmp[1]; InterPoint[2] = Tmp[2];
      if (Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2] > 10e-6)
      {
        InterPoint[0] = Tmp[0]; InterPoint[1] = Tmp[1]; InterPoint[2] = Tmp[2];
        InterNorm[0] = sqrt(0.5) * Tmp[0] / sqrt(Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2]);
        if (Tmp[1] > 0.0)
          InterNorm[1] = -sqrt(1.0 / 2.0);
        else
          InterNorm[1] = sqrt(1.0 / 2.0);
        InterNorm[2] = sqrt(0.5) * Tmp[2] / sqrt(Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2]);
      } else
      {
        InterNorm[0] = 0.0; InterNorm[1] = 1.0; InterNorm[2] = 0.0;
      }
      if (Cone->Trans)
      {
        MEvaluatePoint(InterPoint, Cone->Trans, InterPoint);
        MEvaluateVector(InterNorm, Cone->Trans, InterNorm);
      }
      return 1;
    }
  }
  if (fabs(OutP[1]-InP[1]) < 10e-6)
    return 0;
  /* Check for the bottom */
  t = (CCone->Radius1-InP[1]) / (OutP[1]-InP[1]);
  Tmp[0] = InP[0] + (OutP[0] - InP[0]) * t;
  Tmp[1] = CCone->Radius1;
  Tmp[2] = InP[2] + (OutP[2] - InP[2]) * t;
  if (Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2] < Tmp[1]*Tmp[1])
  {
    InterPoint[0] = Tmp[0]; InterPoint[1] = Tmp[1]; InterPoint[2] = Tmp[2];
    InterNorm[0] = 0.0; InterNorm[1] = -1.0; InterNorm[0] = 0.0;
    if (Cone->Trans)
    {
      MEvaluatePoint(InterPoint, Cone->Trans, InterPoint);
      MEvaluateVector(InterNorm, Cone->Trans, InterNorm);
    }
    return 1;
  }
  /* Check for the top */
  t = (CCone->Radius2-InP[1]) / (OutP[1]-InP[1]);
  Tmp[0] = InP[0] + (OutP[0] - InP[0]) * t;
  Tmp[1] = CCone->Radius2;
  Tmp[2] = InP[2] + (OutP[2] - InP[2]) * t;
  if (Tmp[0]*Tmp[0]+Tmp[2]*Tmp[2] < Tmp[1]*Tmp[1])
  {
    InterPoint[0] = Tmp[0]; InterPoint[1] = Tmp[1]; InterPoint[2] = Tmp[2];
    InterNorm[0] = 0.0; InterNorm[1] = 1.0; InterNorm[0] = 0.0;
    if (Cone->Trans)
    {
      MEvaluatePoint(InterPoint, Cone->Trans, InterPoint);
      MEvaluateVector(InterNorm, Cone->Trans, InterNorm);
    }
    return 1;
  }
  return 0;
}

/*****************************************************************************
*  ComputeConeTriangleMesh
******************************************************************************/
void ComputeConeTriangleMesh(ObjectStruct *Cone)
{ /* XXX: clipped_by */
  Vector P1, P2, P3, P4;
  Vector N1, N2, N3, N4;
  Vector NO1, NO2;
  Vector PO1, PO2;
  int DetailLevel;
  int i;
  ConeStruct *CCone = (ConeStruct *)Cone;

  DetailLevel = 16; /* XXX: Should be a private field */

  if (Cone->FirstTriangle)
    DestroyObjectTriangleMesh(Cone);

  NO1[0] = 0.0; NO1[1] = -1.0; NO1[2] = 0.0;
  NO2[0] = 0.0; NO2[1] =  1.0; NO2[2] = 0.0;
  PO1[0] = 0.0; PO1[1] =  CCone->Radius1; PO1[2] = 0.0;
  PO2[0] = 0.0; PO2[1] =  CCone->Radius2; PO2[2] = 0.0;
  if (Cone->Trans)
  {
    MEvaluatePoint(PO1, Cone->Trans, PO1);
    MEvaluateVector(NO1, Cone->Trans, NO1);
    MEvaluatePoint(PO2, Cone->Trans, PO2);
    MEvaluateVector(NO2, Cone->Trans, NO2);
  }
  for (i=0 ; i<DetailLevel ; i++)
  {
    P1[0] = CCone->Radius1*cos((i*2.0)*M_PI/DetailLevel);
    P1[1] = CCone->Radius1;
    P1[2] = CCone->Radius1*sin((i*2.0)*M_PI/DetailLevel);
    N1[0] = sqrt(1.0/2.0)*cos((i*2.0)*M_PI/DetailLevel);
    if (CCone->Radius1 > 0.0)
      N1[1] = -sqrt(1.0/2.0);
    else
      N1[1] = sqrt(1.0/2.0);
    N1[2] = sqrt(1.0/2.0)*sin((i*2.0)*M_PI/DetailLevel);
    
    P2[0] = CCone->Radius2*cos((i*2.0+1.0)*M_PI/DetailLevel);
    P2[1] = CCone->Radius2;
    P2[2] = CCone->Radius2*sin((i*2.0+1.0)*M_PI/DetailLevel);
    N2[0] = sqrt(1.0/2.0)*cos((i*2.0+1.0)*M_PI/DetailLevel);
    if (CCone->Radius1 > 0.0)
      N2[1] = -sqrt(1.0/2.0);
    else
      N2[1] = sqrt(1.0/2.0);
    N2[2] = sqrt(1.0/2.0)*sin((i*2.0+1.0)*M_PI/DetailLevel);
    
    P3[0] = CCone->Radius1*cos((i+1)*2.0*M_PI/DetailLevel);
    P3[1] = CCone->Radius1;
    P3[2] = CCone->Radius1*sin((i+1)*2.0*M_PI/DetailLevel);
    N3[0] = sqrt(1.0/2.0)*cos(((i+1)*2.0)*M_PI/DetailLevel);
    if (CCone->Radius1 > 0.0)
      N3[1] = -sqrt(1.0/2.0);
    else
      N3[1] = sqrt(1.0/2.0);
    N3[2] = sqrt(1.0/2.0)*sin(((i+1)*2.0)*M_PI/DetailLevel);
    
    P4[0] = CCone->Radius2*cos(((i+1)*2.0+1.0)*M_PI/DetailLevel);
    P4[1] = CCone->Radius2;
    P4[2] = CCone->Radius2*sin(((i+1)*2.0+1.0)*M_PI/DetailLevel);
    N4[0] = sqrt(1.0/2.0)*cos(((i+1)*2.0+1.0)*M_PI/DetailLevel);
    if (CCone->Radius1 > 0.0)
      N4[1] = -sqrt(1.0/2.0);
    else
      N4[1] = sqrt(1.0/2.0);
    N4[2] = sqrt(1.0/2.0)*sin(((i+1)*2.0+1.0)*M_PI/DetailLevel);
    if (Cone->Trans)
    {
      MEvaluatePoint(P1, Cone->Trans, P1);
      MEvaluateVector(N1, Cone->Trans, N1);
      MEvaluatePoint(P2, Cone->Trans, P2);
      MEvaluateVector(N2, Cone->Trans, N2);
      MEvaluatePoint(P3, Cone->Trans, P3);
      MEvaluateVector(N3, Cone->Trans, N3);
      MEvaluatePoint(P4, Cone->Trans, P4);
      MEvaluateVector(N4, Cone->Trans, N4);
    }
    AddTriangleToObjectMesh(Cone, P1,P2,P3, N1,N2,N3);
    AddTriangleToObjectMesh(Cone, P2,P3,P4, N2,N3,N4);
    if (!(CCone->Open))
    {
      if (CCone->Radius1)
        AddTriangleToObjectMesh(Cone, PO1,P1,P3, NO1, NO1, NO1);
      if (CCone->Radius2)
        AddTriangleToObjectMesh(Cone, PO2,P2,P4, NO2, NO2, NO2);
    }
  }
}

/*****************************************************************************
*  TranslateCone
******************************************************************************/
void TranslateCone(ObjectStruct *Cone, Vector Vect)
{
  TransformStruct Trans;

  if (Cone->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Cone->Trans, &Trans);
  } else
  {
    Cone->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Cone->Trans, Vect);
  }
  if (Cone->ClippedBy)
    TranslateObject(Cone->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateCone
******************************************************************************/
void RotateCone(ObjectStruct *Cone, Vector Vect)
{
  TransformStruct Trans;

  if (Cone->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Cone->Trans, &Trans);
  } else
  {
    Cone->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Cone->Trans, Vect);
  }
  if (Cone->ClippedBy)
    RotateObject(Cone->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleCone
******************************************************************************/
void ScaleCone(ObjectStruct *Cone, Vector Vect)
{
  TransformStruct Trans;

  if (Cone->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Cone->Trans, &Trans);
  } else
  {
    Cone->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Cone->Trans, Vect);
  }
  if (Cone->ClippedBy)
    ScaleObject(Cone->ClippedBy, Vect);
}

/*****************************************************************************
*  IsConeIntersection
******************************************************************************/
int IsConeIntersection(ObjectStruct *Cone, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
  double A, B, C, Delta;
  double t;
  double Alt;
  ConeStruct *CCone = (ConeStruct *)Cone;

  if (Cone->Trans)
  {
    MInverseEvaluatePoint(Org, Cone->Trans, Origin);
    MInverseEvaluateVector(Dir, Cone->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  A = Dir[0]*Dir[0]-Dir[1]*Dir[1]+Dir[2]*Dir[2];
  B = 2.0 * (Org[0]*Dir[0]-Org[1]*Dir[1]+Org[2]*Dir[2]);
  C = Org[0]*Org[0]-Org[1]*Org[1]+Org[2]*Org[2];
  if (fabs(A) < 10e-6)
  { /* The Ray is Parallel to the generatory line */
    if (fabs(B) < 10e-6)
      return 0;
    t = -C / B;
    Alt = Org[1] + t*Dir[1];
    if ( (Alt > CCone->Radius1) && (Alt < CCone->Radius2) )
      return 1;
    else return 0;
  }
  Delta = B*B - 4*A*C;
  if (Delta < 0.0)
    return 0;
  t = (-B-sqrt(Delta)) / (2.0 * A);
  Alt = Org[1] + t*Dir[1];
  if ( (Alt > CCone->Radius1) && (Alt < CCone->Radius2) )
    return 1;
  t = (-B+sqrt(Delta)) / (2.0 * A);
  Alt = Org[1] + t*Dir[1];
  if ( (Alt > CCone->Radius1) && (Alt < CCone->Radius2) )
    return 1;
  return 0;
}

/*****************************************************************************
*  ClickInConeXY
******************************************************************************/
int ClickInConeXY(ObjectStruct *Cone, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsConeIntersection(Cone, Origin, Direction);
}

/****************************************************************************
*  ClickInConeXZ
*****************************************************************************/
int ClickInConeXZ(ObjectStruct *Cone, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsConeIntersection(Cone, Origin, Direction);
}

/****************************************************************************
*  ClickInConeZY
*****************************************************************************/
int ClickInConeZY(ObjectStruct *Cone, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsConeIntersection(Cone, Origin, Direction);
}

/*****************************************************************************/
double Xorg, Yorg, Xold, Yold;
double Radius1, Radius2, ConeLength;
GdkColormap *colormap;
GdkGC *gcPurple;
GdkColor Purple;
guint id;

#define NONE           1
#define MOVING         2
#define BETWEEN_RADIUS 3
#define SECOND_RADIUS  4

int CONE_STATE = NONE;

/*****************************************************************************
*  ToolInitNewCone
******************************************************************************/
void ToolInitNewCone(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ViewUserDataStruct *UserData;
  GdkColormap *colormap;
  GtkWidget *StatusBar;

  if (CONE_STATE == BETWEEN_RADIUS)
  {
    CONE_STATE = SECOND_RADIUS;
    return;
  }

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  id = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "new cone");
  Xorg = bevent->x; Yorg = bevent->y;

  colormap = gdk_window_get_colormap(DrawingArea->window);
  gcPurple = gdk_gc_new(DrawingArea->window);
  Purple.red = 65535; Purple.green = 0; Purple.blue = 65535;
  /* XXX: the color should be settable... */
  gdk_color_alloc(colormap, &Purple);
  gdk_gc_set_foreground(gcPurple, &Purple);
  gdk_gc_set_function(gcPurple, GDK_XOR);

  Xold = Xorg; Yold = Yorg;

  CONE_STATE = MOVING;
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, _("Cone: "));
  gdk_pointer_grab(DrawingArea->window, FALSE,
                   GDK_BUTTON_PRESS_MASK |
                   GDK_POINTER_MOTION_MASK | 
                   GDK_BUTTON_RELEASE_MASK,
                   NULL, NULL, bevent->time);
}

/*****************************************************************************
*  ToolMotionNewCone
******************************************************************************/
void ToolMotionNewCone(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventMotion *Mev = (GdkEventMotion *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  char Message[100];
  double x1, x2;
  double gx1, gy1;
  Vector From;
  double Length, Radius;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  if (CONE_STATE == MOVING)
  {
    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

    if (Xorg < Xold)
    {
      x1 = Xorg-(Xold-Xorg); x2 = Xold;
    } else
    {
      x1 = Xold; x2 = Xorg+Xorg-Xold;
    }
    gdk_draw_line(DrawingArea->window, gcPurple, x1,Yorg, x2,Yorg);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg,Yold, x2,Yorg);
    gdk_draw_line(DrawingArea->window, gcPurple, x1,Yorg, Xorg,Yold);

    Xold = Mev->x ; Yold = Mev->y;
    if (Xorg < Xold)
    {
      x1 = Xorg-(Xold-Xorg); x2 = Xold;
    } else
    {
      x1 = Xold; x2 = Xorg+Xorg-Xold;
    }
    gdk_draw_line(DrawingArea->window, gcPurple, x1,Yorg, x2,Yorg);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg,Yold, x2,Yorg);
    gdk_draw_line(DrawingArea->window, gcPurple, x1,Yorg, Xorg,Yold);

    gx1 = Xoff+(Xorg-DrawingArea->allocation.width /2)/Zoom;
    gy1 = Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom;
    Radius = fabs(Xorg-Mev->x)/Zoom;
    Length = fabs(Yorg-Mev->y)/Zoom;
    switch (UserData->CameraStyle)
    {
      case ORTHO_XY_CAMERA:
        From[0] = gx1;
        From[1] = gy1;
        From[2] = 0.0;
        break;
      case ORTHO_XZ_CAMERA:
        From[0] = gx1;
        From[1] = 0.0;
        From[2] = gy1;
        break;
      case ORTHO_ZY_CAMERA:
        From[0] = 0.0;
        From[1] = gy1;
        From[2] = gx1;
        break;
    }
    sprintf(Message, _("Cone: From <%g, %g, %g>, Length: %g, Radius: %g"),
                       From[0], From[1], From[2], Length, Radius);
    gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
  } else if (!ConeFunnelFlag && 
             ((CONE_STATE == SECOND_RADIUS) || (CONE_STATE == BETWEEN_RADIUS)))
  {
    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

    gdk_draw_line(DrawingArea->window, gcPurple, Xorg-Radius1, Yorg, Xorg+Radius1, Yorg);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg-Radius1, Yorg, Xorg-Radius2, Yorg-ConeLength);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg+Radius1, Yorg, Xorg+Radius2, Yorg-ConeLength);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg-Radius2, Yorg-ConeLength, Xorg+Radius2, Yorg-ConeLength);
    Radius2 = Mev->x-Xorg;
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg-Radius1, Yorg, Xorg+Radius1, Yorg);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg-Radius1, Yorg, Xorg-Radius2, Yorg-ConeLength);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg+Radius1, Yorg, Xorg+Radius2, Yorg-ConeLength);
    gdk_draw_line(DrawingArea->window, gcPurple, Xorg-Radius2, Yorg-ConeLength, Xorg+Radius2, Yorg-ConeLength);
    sprintf(Message, _("Cone: Length: %g, Radius 1: %g, Radius 2: %g"),
            ConeLength / Zoom, Radius1 / Zoom, Radius2 / Zoom);
    gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
  }
}

/*****************************************************************************
*  ToolReleaseNewCone
******************************************************************************/
void ToolReleaseNewCone(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventButton *Bev = (GdkEventButton *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  FrameViewStruct *TmpView;
  ObjectStruct *Cone;
  ConeStruct *CCone;
  double gx1, gy1, gx2, gy2;
  double Radius, Length, ULength;
  Vector Scale, Translate, From, To, N, U1, U2;
  TransformStruct Transform;

  if (CONE_STATE == MOVING)
  {
    UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
    StatusBar = UserData->StatusBar;
    Zoom = UserData->ZoomFactor;
    Xoff = UserData->Xoffset;
    Yoff = UserData->Yoffset;

    if (ConeFunnelFlag)
    {
      gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);
      CCone = g_malloc(sizeof(ConeStruct));
      Cone = (ObjectStruct *)CCone;
      InitObject(Cone);
      Cone->Type = CONE_OBJECT;
      CCone->Open = ConeOpenFlag;
      gx1 = Xoff+(Xorg-DrawingArea->allocation.width /2)/Zoom;
      gy1 = Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom;
      gx2 = Xoff+(Bev->x-DrawingArea->allocation.width /2)/Zoom;
      gy2 = Yoff-(Bev->y-DrawingArea->allocation.height/2)/Zoom;
      Radius = fabs(Xorg-Bev->x)/Zoom;
      switch (UserData->CameraStyle)
      {
        case ORTHO_XY_CAMERA:
          From[0] = gx1;
          From[1] = gy2;
          From[2] = 0.0;
          To[0] = gx1;
          To[1] = gy1;
          To[2] = 0.0;
          break;
        case ORTHO_XZ_CAMERA:
          From[0] = gx1;
          From[1] = 0.0;
          From[2] = gy2;
          To[0] = gx1;
          To[1] = 0.0;
          To[2] = gy1;
          break;
        case ORTHO_ZY_CAMERA:
          From[0] = 0.0;
          From[1] = gy2;
          From[2] = gx1;
          To[0] = 0.0;
          To[1] = gy1;
          To[2] = gx1;
          break;
      }
      CCone->Radius1 = 0.0;
      CCone->Radius2 = Radius;
 
      N[0] = To[0] - From[0];
      N[1] = To[1] - From[1];
      N[2] = To[2] - From[2];
      Length = VLength(N);
      Scale[0] = 1.0;
      Scale[1] = Length / CCone->Radius2;
      Scale[2] = 1.0;
      ScaleCone(Cone, Scale);

      N[0] /= Length;
      N[1] /= Length;
      N[2] /= Length;
      if ( (fabs(N[0])>fabs(N[1])) && (fabs(N[0])>fabs(N[2])) )
      { /* The cone axis is mainly upon the X axis */
        U1[0] = -N[1]/N[0];
        U1[1] = 1;
        U1[2] = 0;
      } else if ( (fabs(N[1])>fabs(N[0])) && (fabs(N[1])>fabs(N[2])) )
      { /* The cone axis is mainly upon the Y axis */
        U1[0] = 1;
        U1[1] = -N[0]/N[1];
        U1[2] = 0;
      } else
      { /* The cone axis is mainly upon the Z axis */
        U1[0] = 0;
        U1[1] = 1;
        U1[2] = -N[1]/N[2];
      }
      ULength = VLength(U1);
      U1[0]/=ULength; U1[1]/=ULength; U1[2]/=ULength;
      VCross(U2, N, U1);
      ULength = VLength(U2);
      U2[0]/=ULength; U2[1]/=ULength; U2[2]/=ULength;
      Transform.Direct[0][0] = U1[0];
      Transform.Direct[0][1] = U1[1];
      Transform.Direct[0][2] = U1[2];
      Transform.Direct[0][3] = 0.0;
      Transform.Direct[1][0] = N[0];
      Transform.Direct[1][1] = N[1];
      Transform.Direct[1][2] = N[2];
      Transform.Direct[1][3] = 0.0;
      Transform.Direct[2][0] = U2[0];
      Transform.Direct[2][1] = U2[1];
      Transform.Direct[2][2] = U2[2];
      Transform.Direct[2][3] = 0.0;
      Transform.Direct[3][0] = 0.0;
      Transform.Direct[3][1] = 0.0;
      Transform.Direct[3][2] = 0.0;
      Transform.Direct[3][3] = 1.0;
      MInverse(Transform.Inverse, Transform.Direct);

      ComposeTrans(Cone->Trans, &Transform);
          
      TranslateCone(Cone, From);
        
      Cone->Next = UserData->FrameView->FirstObject;
      UserData->FrameView->FirstObject = Cone;
      ComputeConeTriangleMesh(Cone);
      CONE_STATE = NONE;

      gdk_gc_destroy(gcPurple);
      for (TmpView = UserData->FrameView->FirstView ;
           TmpView ;
           TmpView = TmpView->Next)
      gtk_widget_queue_draw(TmpView->DrawingArea);
      RebuildCSGTree(NULL, UserData->FrameView);
      gdk_pointer_ungrab(Bev->time);
    } else
    {
      CONE_STATE = BETWEEN_RADIUS;
      Radius1 = fabs(Xorg-Bev->x);
      Radius2 = 0.0;
      ConeLength = Yorg-Bev->y;
    }
  } else if ((!ConeFunnelFlag) && (CONE_STATE == SECOND_RADIUS))
  {
    UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
    StatusBar = UserData->StatusBar;
    Zoom = UserData->ZoomFactor;
    Xoff = UserData->Xoffset;
    Yoff = UserData->Yoffset;

    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);



    CCone = g_malloc(sizeof(ConeStruct));
    Cone = (ObjectStruct *)CCone;
    InitObject(Cone);
    Cone->Type = CONE_OBJECT;
    CCone->Open = ConeOpenFlag;
    gx1 = Xoff+(Xorg-DrawingArea->allocation.width /2)/Zoom;
    gy1 = Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom;
    gx2 = Xoff+(Bev->x-DrawingArea->allocation.width /2)/Zoom;
    gy2 = Yoff-(Yorg-ConeLength-DrawingArea->allocation.height/2)/Zoom;
    Radius = fabs(Xorg-Bev->x)/Zoom;
    switch (UserData->CameraStyle)
    {
      case ORTHO_XY_CAMERA:
        From[0] = gx1;
        From[1] = gy1;
        From[2] = 0.0;
        To[0] = gx1;
        To[1] = gy2;
        To[2] = 0.0;
        break;
      case ORTHO_XZ_CAMERA:
        From[0] = gx1;
        From[1] = 0.0;
        From[2] = gy1;
        To[0] = gx1;
        To[1] = 0.0;
        To[2] = gy2;
        break;
      case ORTHO_ZY_CAMERA:
        From[0] = 0.0;
        From[1] = gy1;
        From[2] = gx1;
        To[0] = 0.0;
        To[1] = gy2;
        To[2] = gx1;
        break;
    }
    CCone->Radius1 = Radius1 / Zoom;
    CCone->Radius2 = Radius2 / Zoom;
 
    if (CCone->Radius1 > CCone->Radius2)
    { /* We want Radius1 < Radius2, no matter what sign */
      double TmpRad;
      Vector TmpVect;
      int i;

      TmpRad = CCone->Radius1;
      CCone->Radius1 = CCone->Radius2;
      CCone->Radius2 = TmpRad;
      for (i=0 ; i<3 ; i++)
      {
        TmpVect[i] = From[i];
        From[i] = To[i];
        To[i] = TmpVect[i];
      }
    }
     
    N[0] = To[0] - From[0];
    N[1] = To[1] - From[1];
    N[2] = To[2] - From[2];
    Length = VLength(N);
    Scale[0] = 1.0;
    Scale[1] = Length / (CCone->Radius2-CCone->Radius1);
    Scale[2] = 1.0;
    ScaleCone(Cone, Scale);

    N[0] /= Length;
    N[1] /= Length;
    N[2] /= Length;
    if ( (fabs(N[0])>fabs(N[1])) && (fabs(N[0])>fabs(N[2])) )
    { /* The cone axis is mainly upon the X axis */
      U1[0] = -N[1]/N[0];
      U1[1] = 1;
      U1[2] = 0;
    } else if ( (fabs(N[1])>fabs(N[0])) && (fabs(N[1])>fabs(N[2])) )
    { /* The cone axis is mainly upon the Y axis */
      U1[0] = 1;
      U1[1] = -N[0]/N[1];
      U1[2] = 0;
    } else
    { /* The cone axis is mainly upon the Z axis */
      U1[0] = 0;
      U1[1] = 1;
      U1[2] = -N[1]/N[2];
    }
    ULength = VLength(U1);
    U1[0]/=ULength; U1[1]/=ULength; U1[2]/=ULength;
    VCross(U2, N, U1);
    ULength = VLength(U2);
    U2[0]/=ULength; U2[1]/=ULength; U2[2]/=ULength;
    Transform.Direct[0][0] = U1[0];
    Transform.Direct[0][1] = U1[1];
    Transform.Direct[0][2] = U1[2];
    Transform.Direct[0][3] = 0.0;
    Transform.Direct[1][0] = N[0];
    Transform.Direct[1][1] = N[1];
    Transform.Direct[1][2] = N[2];
    Transform.Direct[1][3] = 0.0;
    Transform.Direct[2][0] = U2[0];
    Transform.Direct[2][1] = U2[1];
    Transform.Direct[2][2] = U2[2];
    Transform.Direct[2][3] = 0.0;
    Transform.Direct[3][0] = 0.0;
    Transform.Direct[3][1] = 0.0;
    Transform.Direct[3][2] = 0.0;
    Transform.Direct[3][3] = 1.0;
    MInverse(Transform.Inverse, Transform.Direct);

    ComposeTrans(Cone->Trans, &Transform);
    Translate[0] = From[0]+(To[0]-From[0])*CCone->Radius1/(CCone->Radius1-CCone->Radius2);
    Translate[1] = From[1]+(To[1]-From[1])*CCone->Radius1/(CCone->Radius1-CCone->Radius2);
    Translate[2] = From[2]+(To[2]-From[2])*CCone->Radius1/(CCone->Radius1-CCone->Radius2);
          
    TranslateCone(Cone, Translate);
      
    Cone->Next = UserData->FrameView->FirstObject;
    UserData->FrameView->FirstObject = Cone;
    ComputeConeTriangleMesh(Cone);



      
    
    CONE_STATE = NONE;
    gdk_gc_destroy(gcPurple);
    for (TmpView = UserData->FrameView->FirstView ;
         TmpView ;
         TmpView = TmpView->Next)
      gtk_widget_queue_draw(TmpView->DrawingArea);
    RebuildCSGTree(NULL, UserData->FrameView);
    gdk_pointer_ungrab(Bev->time);
  }
}

/*****************************************************************************
*  ToggleConeFunnel
******************************************************************************/
void ToggleConeFunnel(GtkWidget *Check)
{
    ConeFunnelFlag = !ConeFunnelFlag;
}

/*****************************************************************************
*  ToggleConeOpen
******************************************************************************/
void ToggleConeOpen(GtkWidget *Check)
{
    ConeOpenFlag = !ConeOpenFlag;
}

/*****************************************************************************
*  BuildConeOptions
******************************************************************************/
GtkWidget *BuildConeOptions(GtkWidget *VBox)
{
  GtkWidget *vbox;
  GtkWidget *Check;
  GtkWidget *label;

  /*  the main vbox  */
  vbox = gtk_vbox_new (FALSE, 1);
  /*  the main label  */
  label = gtk_label_new(_("Cone"));
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
  /* The Funnel Check Button */
  Check = gtk_check_button_new_with_label(_("Funnel Only"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Check), ConeFunnelFlag);
  gtk_signal_connect_object(GTK_OBJECT(Check), "toggled",
                            GTK_SIGNAL_FUNC(ToggleConeFunnel), NULL);
  gtk_box_pack_start(GTK_BOX(vbox), Check, FALSE, FALSE, 0);
  gtk_widget_show(Check);
  /* The Open Check Button */
  Check = gtk_check_button_new_with_label(_("Open"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Check), ConeOpenFlag);
  gtk_signal_connect_object(GTK_OBJECT(Check), "toggled",
                            GTK_SIGNAL_FUNC(ToggleConeOpen), NULL);
  gtk_box_pack_start(GTK_BOX(vbox), Check, FALSE, FALSE, 0);
  gtk_widget_show(Check);

  gtk_box_pack_start(GTK_BOX(VBox), vbox, TRUE, TRUE, 0);
  gtk_widget_show(vbox);
  return vbox;
}

/*****************************************************************************
*  CreateCone
******************************************************************************/
ObjectStruct *CreateCone(Vector Base, double BaseRadius, 
                         Vector Apex, double ApexRadius)
{
  ConeStruct *CCone;
  ObjectStruct *Cone;
  Vector LocalVector1, LocalVector2, TmpVect;
  double TmpRad, Length, ULength;
  int i;
  Vector N, U1, U2;
  Vector Scale, Translate;
  TransformStruct Transform;

  CCone = g_malloc(sizeof(ConeStruct));
  Cone = (ObjectStruct *)CCone;
  InitObject(Cone);
  Cone->Type = CONE_OBJECT;

  CCone->Radius1 = BaseRadius;
  CCone->Radius2 = ApexRadius;
  LocalVector1[0] = Base[0];
  LocalVector1[1] = Base[1];
  LocalVector1[2] = Base[2];
  LocalVector2[0] = Apex[0];
  LocalVector2[1] = Apex[1];
  LocalVector2[2] = Apex[2];

  if (CCone->Radius1 > CCone->Radius2)
  { /* We want Radius1 < Radius2, no matter what sign */
    TmpRad = CCone->Radius1;
    CCone->Radius1 = CCone->Radius2;
    CCone->Radius2 = TmpRad;
    for (i=0 ; i<3 ; i++)
    {
      TmpVect[i] = LocalVector1[i];
      LocalVector1[i] = LocalVector2[i];
      LocalVector2[i] = TmpVect[i];
    }
  }
  N[0] = LocalVector2[0] - LocalVector1[0];
  N[1] = LocalVector2[1] - LocalVector1[1];
  N[2] = LocalVector2[2] - LocalVector1[2];
  Length = VLength(N);
  if (Length == 0.0)
  { /* FIXME */
  }
  if (CCone->Radius2 - CCone->Radius1 < 10e-6)
  { /* FIXME */
  }
  Scale[0] = 1.0;
  Scale[1] = Length / (CCone->Radius2-CCone->Radius1);
  Scale[2] = 1.0;
  ScaleCone(Cone, Scale);
  N[0] /= Length;
  N[1] /= Length;
  N[2] /= Length;
  if ( (fabs(N[0])>fabs(N[1])) && (fabs(N[0])>fabs(N[2])) )
  { /* The cone axis is mainly upon the X axis */
    U1[0] = -N[1]/N[0];
    U1[1] = 1.0;
    U1[2] = 0.0;
  } else if ( (fabs(N[1])>fabs(N[0])) && (fabs(N[1])>fabs(N[2])) )
  { /* The cone axis is mainly upon the Y axis */
    U1[0] = 1.0;
    U1[1] = -N[0]/N[1];
    U1[2] = 0.0;
  } else
  { /* The cone axis is mainly upon the Z axis */
    U1[0] = 0.0;
    U1[1] = 1.0;
    U1[2] = -N[1]/N[2];
  }
  ULength = VLength(U1);
  U1[0]/=ULength; U1[1]/=ULength; U1[2]/=ULength;
  VCross(U2, N, U1);
  ULength = VLength(U2);
  U2[0]/=ULength; U2[1]/=ULength; U2[2]/=ULength;
  Transform.Direct[0][0] = U1[0];
  Transform.Direct[0][1] = U1[1];
  Transform.Direct[0][2] = U1[2];
  Transform.Direct[0][3] = 0.0;
  Transform.Direct[1][0] = N[0];
  Transform.Direct[1][1] = N[1];
  Transform.Direct[1][2] = N[2];
  Transform.Direct[1][3] = 0.0;
  Transform.Direct[2][0] = U2[0];
  Transform.Direct[2][1] = U2[1];
  Transform.Direct[2][2] = U2[2];
  Transform.Direct[2][3] = 0.0;
  Transform.Direct[3][0] = 0.0;
  Transform.Direct[3][1] = 0.0;
  Transform.Direct[3][2] = 0.0;
  Transform.Direct[3][3] = 1.0;
  MInverse(Transform.Inverse, Transform.Direct);

  ComposeTrans(Cone->Trans, &Transform);
  Translate[0] = LocalVector1[0]+(LocalVector2[0]-LocalVector1[0])*
                                 CCone->Radius1/(CCone->Radius1-CCone->Radius2);
  Translate[1] = LocalVector1[1]+(LocalVector2[1]-LocalVector1[1])*
                                 CCone->Radius1/(CCone->Radius1-CCone->Radius2);
  Translate[2] = LocalVector1[2]+(LocalVector2[2]-LocalVector1[2])*
                                 CCone->Radius1/(CCone->Radius1-CCone->Radius2);
  TranslateCone(Cone, Translate);

  return Cone;
}

/*****************************************************************************
*  PDBCreateCone
******************************************************************************/
int PDBCreateCone(int *Id, Vector *Base, double *BaseRadius,
                           Vector *Apex, double *ApexRadius)
{
  ObjectStruct *Cone;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  Cone = CreateCone(*Base, *BaseRadius, *Apex, *ApexRadius);
  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
    ;
  Cone->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Cone;
  ComputeConeTriangleMesh(Cone);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
  gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Cone->Id;
}

