/* csg.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "object.h"
#include "marching_cube.h"
#include "csg.h"

/*****************************************************************************
*  InsideCSG
******************************************************************************/
int InsideCSG(ObjectStruct *CSG, double x, double y, double z)
{
  ObjectStruct *TmpObject;
  CSGStruct *CCSG = (CSGStruct *)CSG;
  Vector Vect;
  /* XXX: clipped_by */
  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (CSG->Trans)
  {
    MInverseEvaluatePoint(Vect, CSG->Trans, Vect);
  }
  switch (CCSG->Type)
  {
    case CSG_DIFFERENCE:
      if (!InsideObject(CCSG->FirstObject, Vect[0],Vect[1],Vect[2]))
        return CSG->Inverse;
      for (TmpObject = CCSG->FirstObject->Next ;
           TmpObject ;
           TmpObject = TmpObject->Next)
      {
        if (InsideObject(TmpObject, Vect[0],Vect[1],Vect[2]))
          return CSG->Inverse;
      }
      return !(CSG->Inverse);
    case CSG_INTERSECTION:
      for (TmpObject = CCSG->FirstObject ; TmpObject ; TmpObject = TmpObject->Next)
      {
        if (!InsideObject(TmpObject, Vect[0],Vect[1],Vect[2]))
          return CSG->Inverse;
      }
      return !(CSG->Inverse);
    case CSG_MERGE:
    case CSG_UNION:
      for (TmpObject = CCSG->FirstObject ; TmpObject ; TmpObject = TmpObject->Next)
      {
        if (InsideObject(TmpObject, Vect[0],Vect[1],Vect[2]))
          return !(CSG->Inverse);
      }
      return CSG->Inverse;
  }
  return 0; /* keep the compiler happy ;) */
}

/*****************************************************************************
*  ComputeCSGTriangleMesh
******************************************************************************/
void ComputeCSGTriangleMesh(ObjectStruct *CSG)
{
  CSGStruct *CCSG = (CSGStruct *)CSG;
  ObjectStruct *TmpObject;
  double sizex, sizey, sizez;
  TriangleListStruct *TmpTri;
  int DetailLevel;
  int i,j,k;
  Vector Cube[8];
  Vector MaxExt, MinExt;

  if (CSG->FirstTriangle)
        DestroyObjectTriangleMesh(CSG);
  MinExt[0] = MinExt[1] = MinExt[2] = 1e100;
  MaxExt[0] = MaxExt[1] = MaxExt[2] = -1e100;
  for (TmpObject = CCSG->FirstObject;
       TmpObject;
       TmpObject = TmpObject->Next)
  {
    ComputeObjectTriangleMesh(TmpObject);
    /* Find the extents */
    for (TmpTri = TmpObject->FirstTriangle ; TmpTri ; TmpTri=TmpTri->Next)
    {
      if (MinExt[0] > TmpTri->P1[0]) MinExt[0] = TmpTri->P1[0];
      if (MinExt[0] > TmpTri->P2[0]) MinExt[0] = TmpTri->P2[0];
      if (MinExt[0] > TmpTri->P3[0]) MinExt[0] = TmpTri->P3[0];
      if (MinExt[1] > TmpTri->P1[1]) MinExt[1] = TmpTri->P1[1];
      if (MinExt[1] > TmpTri->P2[1]) MinExt[1] = TmpTri->P2[1];
      if (MinExt[1] > TmpTri->P3[1]) MinExt[1] = TmpTri->P3[1];
      if (MinExt[2] > TmpTri->P1[2]) MinExt[2] = TmpTri->P1[2];
      if (MinExt[2] > TmpTri->P2[2]) MinExt[2] = TmpTri->P2[2];
      if (MinExt[2] > TmpTri->P3[2]) MinExt[2] = TmpTri->P3[2];

      if (MaxExt[0] < TmpTri->P1[0]) MaxExt[0] = TmpTri->P1[0];
      if (MaxExt[0] < TmpTri->P2[0]) MaxExt[0] = TmpTri->P2[0];
      if (MaxExt[0] < TmpTri->P3[0]) MaxExt[0] = TmpTri->P3[0];
      if (MaxExt[1] < TmpTri->P1[1]) MaxExt[1] = TmpTri->P1[1];
      if (MaxExt[1] < TmpTri->P2[1]) MaxExt[1] = TmpTri->P2[1];
      if (MaxExt[1] < TmpTri->P3[1]) MaxExt[1] = TmpTri->P3[1];
      if (MaxExt[2] < TmpTri->P1[2]) MaxExt[2] = TmpTri->P1[2];
      if (MaxExt[2] < TmpTri->P2[2]) MaxExt[2] = TmpTri->P2[2];
      if (MaxExt[2] < TmpTri->P3[2]) MaxExt[2] = TmpTri->P3[2];
    }
  }
  DetailLevel = 16; /* XXX should be settable */
  sizex = (MaxExt[0] - MinExt[0]) / DetailLevel;
  sizey = (MaxExt[1] - MinExt[1]) / DetailLevel;
  sizez = (MaxExt[2] - MinExt[2]) / DetailLevel;
  /* Compute triangles for each cube of the zone */
  for (i=0 ; i<DetailLevel ; i++)
    for (j=0 ; j<DetailLevel ; j++)
      for (k=0 ; k<DetailLevel ; k++)
      {
        Cube[0][0] = MinExt[0]+(i+0)*sizex;
        Cube[0][1] = MinExt[1]+(j+0)*sizey;
        Cube[0][2] = MinExt[2]+(k+0)*sizez;
        
        Cube[1][0] = MinExt[0]+(i+1)*sizex;
        Cube[1][1] = MinExt[1]+(j+0)*sizey;
        Cube[1][2] = MinExt[2]+(k+0)*sizez;
        
        Cube[2][0] = MinExt[0]+(i+0)*sizex;
        Cube[2][1] = MinExt[1]+(j+1)*sizey;
        Cube[2][2] = MinExt[2]+(k+0)*sizez;
        
        Cube[3][0] = MinExt[0]+(i+1)*sizex;
        Cube[3][1] = MinExt[1]+(j+1)*sizey;
        Cube[3][2] = MinExt[2]+(k+0)*sizez;
        
        Cube[4][0] = MinExt[0]+(i+0)*sizex;
        Cube[4][1] = MinExt[1]+(j+0)*sizey;
        Cube[4][2] = MinExt[2]+(k+1)*sizez;
        
        Cube[5][0] = MinExt[0]+(i+1)*sizex;
        Cube[5][1] = MinExt[1]+(j+0)*sizey;
        Cube[5][2] = MinExt[2]+(k+1)*sizez;
        
        Cube[6][0] = MinExt[0]+(i+0)*sizex;
        Cube[6][1] = MinExt[1]+(j+1)*sizey;
        Cube[6][2] = MinExt[2]+(k+1)*sizez;
        
        Cube[7][0] = MinExt[0]+(i+1)*sizex;
        Cube[7][1] = MinExt[1]+(j+1)*sizey;
        Cube[7][2] = MinExt[2]+(k+1)*sizez;
        MarchingCube(CSG, Cube);
      }
}

/*****************************************************************************
*  TranslateCSG
******************************************************************************/
void TranslateCSG(ObjectStruct *CSG, Vector Vect)
{
  TransformStruct Trans;

  if (CSG->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(CSG->Trans, &Trans);
  } else
  {
    CSG->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(CSG->Trans, Vect);
  }
  if (CSG->ClippedBy)
    TranslateObject(CSG->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateCSG
******************************************************************************/
void RotateCSG(ObjectStruct *CSG, Vector Vect)
{
  TransformStruct Trans;

  if (CSG->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(CSG->Trans, &Trans);
  } else
  {
    CSG->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(CSG->Trans, Vect);
  }
  if (CSG->ClippedBy)
    RotateObject(CSG->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleCSG
******************************************************************************/
void ScaleCSG(ObjectStruct *CSG, Vector Vect)
{ /* XXX: clipped_by */
  TransformStruct Trans;

  if (CSG->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(CSG->Trans, &Trans);
  } else
  {
    CSG->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(CSG->Trans, Vect);
  }
  if (CSG->ClippedBy)
    ScaleObject(CSG->ClippedBy, Vect);
}

/*****************************************************************************
*  IsCSGIntersection
******************************************************************************/
int IsCSGIntersection(ObjectStruct *CSG, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  ObjectStruct *TmpObject;
  CSGStruct *CCSG = (CSGStruct *)CSG;
  int i;

  if (CSG->Trans)
  {
    MInverseEvaluatePoint(Org, CSG->Trans, Origin);
    MInverseEvaluateVector(Dir, CSG->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  switch (CCSG->Type)
  {
    case CSG_DIFFERENCE:
      if (!IsObjectIntersection(CCSG->FirstObject, Org, Dir))
        return CSG->Inverse;
      for (TmpObject = CCSG->FirstObject->Next ;
           TmpObject ;
           TmpObject = TmpObject->Next)
      {
        if (IsObjectIntersection(TmpObject, Org, Dir))
          return CSG->Inverse;
      }
      return !(CSG->Inverse);
    case CSG_INTERSECTION:
      for (TmpObject = CCSG->FirstObject ;
           TmpObject ;
           TmpObject = TmpObject->Next)
      {
        if (!IsObjectIntersection(TmpObject, Org, Dir))
          return CSG->Inverse;
      }
      return !(CSG->Inverse);
    case CSG_MERGE:
    case CSG_UNION:
      for (TmpObject = CCSG->FirstObject ;
           TmpObject ;
           TmpObject = TmpObject->Next)
      {
        if (IsObjectIntersection(TmpObject, Org, Dir))
          return !(CSG->Inverse);
      }
      return CSG->Inverse;
  }
  return 0; /* keep the compiler happy ;) */
}

/*****************************************************************************
*  ClickInCSGXY
******************************************************************************/
int ClickInCSGXY(ObjectStruct *CSG, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  ObjectStruct *Object;
  CSGStruct *CCSG = (CSGStruct *)CSG;
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  if (CSG->Trans)
  {
    Origin[0] = x;
    Origin[1] = y;
    return IsCSGIntersection(CSG, Origin, Direction);
  } else 
  {
    switch (CCSG->Type)
    {
      case CSG_DIFFERENCE:
        if (!ClickInObjectXY(CCSG->FirstObject, x, y))
          return CSG->Inverse;
        for (Object = CCSG->FirstObject->Next ;
             Object ;
             Object = Object->Next)
          if (ClickInObjectXY(Object, x, y))
            return CSG->Inverse;
        return !(CSG->Inverse);
      case CSG_INTERSECTION:
        for (Object = CCSG->FirstObject ;
             Object ;
             Object = Object->Next)
          if (!ClickInObjectXY(Object, x, y))
            return CSG->Inverse;
        return !(CSG->Inverse);
      case CSG_MERGE:
      case CSG_UNION:
        for (Object = CCSG->FirstObject ;
             Object ;
             Object = Object->Next)
          if (ClickInObjectXY(Object, x, y))
            return !(CSG->Inverse);
        return CSG->Inverse;
    }
  }
  return 0; /* keep the compiler happy ;) */
}

/*****************************************************************************
*  ClickInCSGXZ
******************************************************************************/
int ClickInCSGXZ(ObjectStruct *CSG, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  ObjectStruct *Object;
  CSGStruct *CCSG = (CSGStruct *)CSG;
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  if (CSG->Trans)
  {
    Origin[0] = x;
    Origin[2] = y;
    return IsCSGIntersection(CSG, Origin, Direction);
  } else 
  {
    switch (CCSG->Type)
    {
      case CSG_DIFFERENCE:
        if (!ClickInObjectXZ(CCSG->FirstObject, x, y))
          return CSG->Inverse;
        for (Object = CCSG->FirstObject->Next ;
             Object ;
             Object = Object->Next)
          if (ClickInObjectXZ(Object, x, y))
            return CSG->Inverse;
        return !(CSG->Inverse);
      case CSG_INTERSECTION:
        for (Object = CCSG->FirstObject ;
             Object ;
             Object = Object->Next)
          if (!ClickInObjectXZ(Object, x, y))
            return CSG->Inverse;
        return !(CSG->Inverse);
      case CSG_MERGE:
      case CSG_UNION:
        for (Object = CCSG->FirstObject ;
             Object ;
             Object = Object->Next)
          if (ClickInObjectXZ(Object, x, y))
            return !(CSG->Inverse);
        return CSG->Inverse;
    }
  }
  return 0; /* keep the compiler happy ;) */
}

/*****************************************************************************
*  ClickInCSGZY
******************************************************************************/
int ClickInCSGZY(ObjectStruct *CSG, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  ObjectStruct *Object;
  CSGStruct *CCSG = (CSGStruct *)CSG;
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  if (CSG->Trans)
  {
    Origin[2] = x;
    Origin[1] = y;
    return IsCSGIntersection(CSG, Origin, Direction);
  } else 
  {
    switch (CCSG->Type)
    {
      case CSG_DIFFERENCE:
        if (!ClickInObjectZY(CCSG->FirstObject, x, y))
          return CSG->Inverse;
        for (Object = CCSG->FirstObject->Next ;
             Object ;
             Object = Object->Next)
          if (ClickInObjectZY(Object, x, y))
            return CSG->Inverse;
        return !(CSG->Inverse);
      case CSG_INTERSECTION:
        for (Object = CCSG->FirstObject ;
             Object ;
             Object = Object->Next)
          if (!ClickInObjectZY(Object, x, y))
            return CSG->Inverse;
        return !(CSG->Inverse);
      case CSG_MERGE:
      case CSG_UNION:
        for (Object = CCSG->FirstObject ;
             Object ;
             Object = Object->Next)
          if (ClickInObjectZY(Object, x, y))
            return !(CSG->Inverse);
        return CSG->Inverse;
    }
  }
  return 0; /* keep the compiler happy ;) */
}
/*****************************************************************************
*  FindIntersectionCSGSegment
******************************************************************************/
int FindIntersectionCSGSegment(Vector InterPoint, Vector InterNorm,
                               Vector InPoint, Vector OutPoint,
                               ObjectStruct *CSG)
{
  CSGStruct *CCSG = (CSGStruct *)CSG;
  ObjectStruct *TmpObject;

  switch (CCSG->Type)
  {
    case CSG_DIFFERENCE:
      MidPoint(InterPoint, InPoint, OutPoint);
      break;
    case CSG_INTERSECTION:
      MidPoint(InterPoint, InPoint, OutPoint);
      break;
    case CSG_MERGE:
    case CSG_UNION:
      for (TmpObject = CCSG->FirstObject ; 
           TmpObject ; 
           TmpObject = TmpObject->Next)
      {
        if (FindIntersectionObjectSegment(InterPoint, InterNorm, 
                                          InPoint, OutPoint, TmpObject))
          return 1;
      }
      MidPoint(InterPoint, InPoint, OutPoint);
      break;
  }
  return 1;
}

