/* cubic.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "csgtree.h"
#include "object.h"
#include "cubic.h"

/*****************************************************************************
*  InsideCubic
******************************************************************************/
int InsideCubic(ObjectStruct *Cubic, double x, double y, double z)
{
  return 0; /* FIXME */
}

/*****************************************************************************
*  FindIntersectCubicSegment FIXME
******************************************************************************/
int FindIntersectionCubicSegment(Vector InterPoint, Vector InterNorm,
                                 Vector InPoint, Vector OutPoint,
                                 ObjectStruct *Cubic)
{
  return 0;
}

/*****************************************************************************
*  ComputeCubicTriangleMesh FIXME
******************************************************************************/
void ComputeCubicTriangleMesh(ObjectStruct *Cubic)
{ /* XXX: clipped_by */
}

/*****************************************************************************
*  TranslateCubic
******************************************************************************/
void TranslateCubic(ObjectStruct *Cubic, Vector Vect)
{
  TransformStruct Trans;

  if (Cubic->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Cubic->Trans, &Trans);
  } else
  {
    Cubic->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Cubic->Trans, Vect);
  }
  if (Cubic->ClippedBy)
    TranslateObject(Cubic->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateCubic
******************************************************************************/
void RotateCubic(ObjectStruct *Cubic, Vector Vect)
{
  TransformStruct Trans;

  if (Cubic->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Cubic->Trans, &Trans);
  } else
  {
    Cubic->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Cubic->Trans, Vect);
  }
  if (Cubic->ClippedBy)
    RotateObject(Cubic->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleCubic
******************************************************************************/
void ScaleCubic(ObjectStruct *Cubic, Vector Vect)
{
  TransformStruct Trans;

  if (Cubic->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Cubic->Trans, &Trans);
  } else
  {
    Cubic->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Cubic->Trans, Vect);
  }
  if (Cubic->ClippedBy)
    ScaleObject(Cubic->ClippedBy, Vect);
}

/*****************************************************************************
*  IsCubicIntersection FIXME
******************************************************************************/
int IsCubicIntersection(ObjectStruct *Cubic, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;

  if (Cubic->Trans)
  {
    MInverseEvaluatePoint(Org, Cubic->Trans, Origin);
    MInverseEvaluateVector(Dir, Cubic->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInCubicXY
******************************************************************************/
int ClickInCubicXY(ObjectStruct *Cubic, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsCubicIntersection(Cubic, Origin, Direction);
}

/****************************************************************************
*  ClickInCubicXZ
*****************************************************************************/
int ClickInCubicXZ(ObjectStruct *Cubic, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsCubicIntersection(Cubic, Origin, Direction);
}

/****************************************************************************
*  ClickInCubicZY
*****************************************************************************/
int ClickInCubicZY(ObjectStruct *Cubic, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsCubicIntersection(Cubic, Origin, Direction);
}

/*****************************************************************************
*  ToolNewCubic FIXME
******************************************************************************/
void ToolNewCubic(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Label;

  Dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New Cubic"));

  Label = gtk_label_new(_("Sorry, the New Cubic function"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);
  Label = gtk_label_new(_("hasn't been implemented yet"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);

  Button = gtk_button_new_with_label("Close");
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button,
                     TRUE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  gtk_widget_show(Dialog);
}
