/* cylinder.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "cylinder.h"

int CylinderOpenFlag=0;

/*****************************************************************************
*  InsideCylinder
******************************************************************************/
int InsideCylinder(ObjectStruct *Cylinder, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Cylinder->Trans)
  {
    MInverseEvaluatePoint(Vect, Cylinder->Trans, Vect);
  }
  if (Vect[1] < 0.0 || Vect[1] > 1.0)
    return Cylinder->Inverse;
  if (Vect[0]*Vect[0]+Vect[2]*Vect[2] > 1.0)
    return Cylinder->Inverse;
  else
    return !(Cylinder->Inverse);
}

/*****************************************************************************
*  FindIntersectCylinderSegment
******************************************************************************/
int FindIntersectionCylinderSegment(Vector InterPoint, Vector InterNorm,
                                    Vector InPoint, Vector OutPoint,
                                    ObjectStruct *Cylinder)
{
  int i;
  Vector InP, OutP;
  double A, B, C, Delta, t;
  Vector Tmp;

  if (Cylinder->Trans)
  {
    MInverseEvaluatePoint(InP, Cylinder->Trans, InPoint);
    MInverseEvaluatePoint(OutP, Cylinder->Trans, OutPoint);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      InP[i] = InPoint[i];
      OutP[i] = OutPoint[i];
    }
  }
  A = (OutP[0]-InP[0])*(OutP[0]-InP[0]) + (OutP[2]-InP[2])*(OutP[2]-InP[2]);
  B = 2.0*(InP[0]*(OutP[0]-InP[0]) + InP[2]*(OutP[2]-InP[2]));
  C = InP[0]*InP[0] + InP[2]*InP[2] - 1;
  Delta = B*B-4*A*C;
  if (Delta < 0.0)
  { /* The Ray miss the cylinder or is parallel to the cylinder axis */
    if (fabs(A) < 10e-6)
    { /* The Ray is parallel to the Cylinder axis */
      if (C < 0.0)
      {
        if ( ( (InP[1] >= 0.0) && (InP[1] <= 1.0) && (OutP[1] >= 1.0) ) ||
             ( (OutP[1] >= 0.0) && (OutP[1] <= 1.0) && (InP[1] >= 1.0) ) )
        { /* The Ray Intersect the Top */
          InterPoint[0] = InP[0]; InterPoint[1] = 1.0; InterPoint[2] = InP[2];
          InterNorm[0] = 0.0; InterNorm[1] = 1.0; InterNorm[2] = 0.0;
          if (Cylinder->Trans)
          {
            MEvaluatePoint(InterPoint, Cylinder->Trans, InterPoint);
            MEvaluateVector(InterNorm, Cylinder->Trans, InterNorm);
          }
          return 1;
        }
        if ( ( (InP[1] >= 0.0) && (InP[1] <= 1.0) && (OutP[1] <= 0.0) ) ||
             ( (OutP[1] >= 0.0) && (OutP[1] <= 1.0) && (InP[1] <= 0.0) ) )
        { /* The Ray Intersect the Bottom */
          InterPoint[0] = InP[0]; InterPoint[1] = 0.0; InterPoint[2] = InP[2];
          InterNorm[0] = 0.0; InterNorm[1] = -1.0; InterNorm[2] = 0.0;
          if (Cylinder->Trans)
          {
            MEvaluatePoint(InterPoint, Cylinder->Trans, InterPoint);
            MEvaluateVector(InterNorm, Cylinder->Trans, InterNorm);
          }
          return 1;
        }
      }
    }
    return 0;
  }
  t = (-B - sqrt(Delta)) / (2.0 * A);
  if ((t >= 0.0) && (t <= 1.0))
  {
    Tmp[0] = InP[0] + (OutP[0] - InP[0]) * t;
    Tmp[1] = InP[1] + (OutP[1] - InP[1]) * t;
    Tmp[2] = InP[2] + (OutP[2] - InP[2]) * t;
  } else
  {
    t = (-B + sqrt(Delta)) / (2.0 * A);
    if ((t >= 0.0) && (t <= 1.0))
    {
      Tmp[0] = InP[0] + (OutP[0] - InP[0]) * t;
      Tmp[1] = InP[1] + (OutP[1] - InP[1]) * t;
      Tmp[2] = InP[2] + (OutP[2] - InP[2]) * t;
    } else
      return 0;
  }
  if ( (Tmp[1] >= 0.0) && (Tmp[1] <= 1.0) )
  {
    InterPoint[0] = Tmp[0]; InterPoint[1] = Tmp[1]; InterPoint[2] = Tmp[2];
    InterNorm[0] = Tmp[0]; InterNorm[1] = 0.0; InterNorm[2] = Tmp[2];
    if (Cylinder->Trans)
    {
      MEvaluatePoint(InterPoint, Cylinder->Trans, InterPoint);
      MEvaluateVector(InterNorm, Cylinder->Trans, InterNorm);
    }
    return 1;
  }
#if 0
  if ( (InsideCylinder(Cylinder, InP[0],InP[1],InP[2]) && (OutP[1] >= 1.0)) ||
       (InsideCylinder(Cylinder, OutP[0],OutP[1],OutP[2]) && (InP[1] >= 1.0)) )
  { /* Check for top */
    t = (1.0 - InP[1]) / (OutP[1] - InP[1]);
    InterPoint[0] = InP[0] + (OutP[0] - InP[0]) * t;
    InterPoint[1] = InP[1] + (OutP[1] - InP[1]) * t;
    InterPoint[2] = InP[2] + (OutP[2] - InP[2]) * t;
    InterNorm[0] = 0.0; InterNorm[1] = 1.0; InterNorm[2] = 0.0;
    if (Cylinder->Trans)
    {
      MEvaluatePoint(InterPoint, Cylinder->Trans, InterPoint);
      MEvaluateVector(InterNorm, Cylinder->Trans, InterNorm);
    }
    return 1;
  }
  if ( (InsideCylinder(Cylinder, InP[0],InP[1],InP[2]) && (OutP[1] <= 0.0)) ||
       (InsideCylinder(Cylinder, OutP[0],OutP[1],OutP[2]) && (InP[1] <= 0.0)) )
  { /* Check for bottom */
    t = (1.0 - InP[1]) / (OutP[1] - InP[1]);
    InterPoint[0] = InP[0] + (OutP[0] - InP[0]) * t;
    InterPoint[1] = InP[1] + (OutP[1] - InP[1]) * t;
    InterPoint[2] = InP[2] + (OutP[2] - InP[2]) * t;
    InterNorm[0] = 0.0; InterNorm[1] = -1.0; InterNorm[2] = 0.0;
    if (Cylinder->Trans)
    {
      MEvaluatePoint(InterPoint, Cylinder->Trans, InterPoint);
      MEvaluateVector(InterNorm, Cylinder->Trans, InterNorm);
    }
    return 1;
  }
#endif /* FIXME */
  return 0;
}

/*****************************************************************************
*  ComputeCylinderTriangleMesh
******************************************************************************/
void ComputeCylinderTriangleMesh(ObjectStruct *Cylinder)
{ /* XXX: clipped_by */
  Vector P1, P2, P3, P4;
  Vector N1, N2, N3, N4;
  Vector NO1, NO2;
  Vector PO1, PO2;
  int DetailLevel;
  int i;
  CylinderStruct *CCylinder = (CylinderStruct *)Cylinder;

  DetailLevel = 16; /* XXX: Should be a private field */
  
  if (Cylinder->FirstTriangle)
    DestroyObjectTriangleMesh(Cylinder);
  
  NO1[0] = 0.0; NO1[1] = -1.0; NO1[2] = 0.0;
  NO2[0] = 0.0; NO2[1] =  1.0; NO2[2] = 0.0;
  PO1[0] = 0.0; PO1[1] =  0.0; PO1[2] = 0.0;
  PO2[0] = 0.0; PO2[1] =  1.0; PO2[2] = 0.0;
  if (Cylinder->Trans)
  {
    MEvaluatePoint(PO1, Cylinder->Trans, PO1);
    MEvaluateVector(NO1, Cylinder->Trans, NO1);
    MEvaluatePoint(PO2, Cylinder->Trans, PO2);
    MEvaluateVector(NO2, Cylinder->Trans, NO2);
  }
  for (i=0 ; i<DetailLevel ; i++)
  {
    N1[0] = P1[0] = cos((i*2.0)*M_PI/DetailLevel);
    N1[1] = P1[1] = 0.0;
    N1[2] = P1[2] = sin((i*2.0)*M_PI/DetailLevel);
    N2[0] = P2[0] = cos((i*2.0+1.0)*M_PI/DetailLevel);
    N2[1] = 0.0; P2[1] = 1.0;
    N2[2] = P2[2] = sin((i*2.0+1.0)*M_PI/DetailLevel);
    N3[0] = P3[0] = cos((i+1)*2.0*M_PI/DetailLevel);
    N3[1] = P3[1] = 0.0;
    N3[2] = P3[2] = sin((i+1)*2.0*M_PI/DetailLevel);
    N4[0] = P4[0] = cos(((i+1)*2.0+1.0)*M_PI/DetailLevel);
    N4[1] = 0.0; P4[1] = 1.0;
    N4[2] = P4[2] = sin(((i+1)*2.0+1.0)*M_PI/DetailLevel);
    if (Cylinder->Trans)
    {
      MEvaluatePoint(P1, Cylinder->Trans, P1);
      MEvaluateVector(N1, Cylinder->Trans, N1);
      MEvaluatePoint(P2, Cylinder->Trans, P2);
      MEvaluateVector(N2, Cylinder->Trans, N2);
      MEvaluatePoint(P3, Cylinder->Trans, P3);
      MEvaluateVector(N3, Cylinder->Trans, N3);
      MEvaluatePoint(P4, Cylinder->Trans, P4);
      MEvaluateVector(N4, Cylinder->Trans, N4);
    }
    AddTriangleToObjectMesh(Cylinder, P1,P2,P3, N1,N2,N3);
    AddTriangleToObjectMesh(Cylinder, P2,P3,P4, N2,N3,N4);
    if (!(CCylinder->Open))
    {
      AddTriangleToObjectMesh(Cylinder, PO1,P1,P3, NO1, NO1, NO1);
      AddTriangleToObjectMesh(Cylinder, PO2,P2,P4, NO2, NO2, NO2);
    }
  }
}

/*****************************************************************************
*  TranslateCylinder
******************************************************************************/
void TranslateCylinder(ObjectStruct *Cylinder, Vector Vect)
{
  TransformStruct Trans;

  if (Cylinder->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Cylinder->Trans, &Trans);
  } else
  {
    Cylinder->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Cylinder->Trans, Vect);
  }
  if (Cylinder->ClippedBy)
    TranslateObject(Cylinder->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateCylinder
******************************************************************************/
void RotateCylinder(ObjectStruct *Cylinder, Vector Vect)
{
  TransformStruct Trans;

  if (Cylinder->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Cylinder->Trans, &Trans);
  } else
  {
    Cylinder->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Cylinder->Trans, Vect);
  }
  if (Cylinder->ClippedBy)
    RotateObject(Cylinder->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleCylinder
******************************************************************************/
void ScaleCylinder(ObjectStruct *Cylinder, Vector Vect)
{
  TransformStruct Trans;

  if (Cylinder->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Cylinder->Trans, &Trans);
  } else
  {
    Cylinder->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Cylinder->Trans, Vect);
  }
  if (Cylinder->ClippedBy)
    ScaleObject(Cylinder->ClippedBy, Vect);
}

/*****************************************************************************
*  IsCylinderIntersection
******************************************************************************/
int IsCylinderIntersection(ObjectStruct *Cylinder, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
  double A, B, C, D;
  double t1, t2, z1, z2;
  CylinderStruct *CCylinder = (CylinderStruct *)Cylinder;

  if (Cylinder->Trans)
  {
    MInverseEvaluatePoint(Org, Cylinder->Trans, Origin);
    MInverseEvaluateVector(Dir, Cylinder->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  if (Dir[0] == 0.0 && Dir[2] == 0.0)
  {
    if (CCylinder->Open)
      return 0;
    else
    {
      if (Org[0]*Org[0]+Org[2]*Org[2] < 1.0)
        return 1;
      else
        return 0;
    }
  }
  A = Dir[0]*Dir[0] + Dir[2]*Dir[2];
  B = 2.0*(Org[0]*Dir[0] + Org[2]*Dir[2]);
  C = Org[0]*Org[0] + Org[2]*Org[2] - 1.0;
  D = B*B - 4.0*A*C;
  if (D < 0.0)
    return 0;
  t1 = (-B-sqrt(D)) / 2.0 / A;
  z1 = Org[1] + t1*Dir[1];
  t2 = (-B+sqrt(D)) / 2.0 / A;
  z2 = Org[1] + t2*Dir[1];
  if ((z1>=0.0 && z1<=1.0) || (z2>=0.0 && z2<=1.0))
    return 1;
  if ((z1>1.0) && (z2>1.0))
    return 0;
  if ((z1<0.0) && (z2<0.0))
    return 0;
  return !(CCylinder->Open);
}

/*****************************************************************************
*  ClickInCylinderXY
******************************************************************************/
int ClickInCylinderXY(ObjectStruct *Cylinder, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  if (Cylinder->Trans)
  {
    Origin[0] = x;
    Origin[1] = y;
    return IsCylinderIntersection(Cylinder, Origin, Direction);
  } else if ( (x>=-1.0) && (x<=1.0) && (y>=0.0) && (y<=1.0) )
    return 1;
  else
    return 0;
}

/****************************************************************************
*  ClickInCylinderXZ
*****************************************************************************/
int ClickInCylinderXZ(ObjectStruct *Cylinder, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  if (Cylinder->Trans)
  {
    Origin[0] = x;
    Origin[2] = y;
    return IsCylinderIntersection(Cylinder, Origin, Direction);
  } else if ( x*x+y*y <= 1.0 )
    return 1;
  else
    return 0;
}

/****************************************************************************
*  ClickInCylinderZY
*****************************************************************************/
int ClickInCylinderZY(ObjectStruct *Cylinder, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  if (Cylinder->Trans)
  {
    Origin[2] = x;
    Origin[1] = y;
    return IsCylinderIntersection(Cylinder, Origin, Direction);
  } else if ( (x>=-1.0) && (x<=1.0) && (y>=0.0) && (y<=1.0) )
    return 1;
  else
    return 0;
}

/****************************************************************************/
double Xorg, Yorg, Xold, Yold;
GdkColormap *colormap;
GdkGC *gcPurple;
GdkColor Purple;
guint id;

#define NONE   1
#define MOVING 2
int CYLINDER_STATE = NONE;

/*****************************************************************************
*  ToolInitNewCylinder
******************************************************************************/
void ToolInitNewCylinder(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ViewUserDataStruct *UserData;
  GdkColormap *colormap;
  GtkWidget *StatusBar;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  id = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "new cylinder");
  Xorg = bevent->x; Yorg = bevent->y;

  colormap = gdk_window_get_colormap(DrawingArea->window);
  gcPurple = gdk_gc_new(DrawingArea->window);
  Purple.red = 65535; Purple.green = 0; Purple.blue = 65535;
  /* XXX: the color should be settable... */
  gdk_color_alloc(colormap, &Purple);
  gdk_gc_set_foreground(gcPurple, &Purple);
  gdk_gc_set_function(gcPurple, GDK_XOR);

  Xold = Xorg; Yold = Yorg;

  CYLINDER_STATE = MOVING;
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, _("Cylinder: "));
  gdk_pointer_grab(DrawingArea->window, FALSE,
                   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                   NULL, NULL, bevent->time);
}

/*****************************************************************************
*  ToolMotionNewCylinder
******************************************************************************/
void ToolMotionNewCylinder(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventMotion *Mev = (GdkEventMotion *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  gchar *Message;
  double x1, y1, x2, y2;
  double gx1, gy1, gy2;
  double Length, Radius;
  Vector From;

  if (CYLINDER_STATE != MOVING)
    return;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

  if (Xorg < Xold)
  {
    x1 = Xorg-(Xold-Xorg); x2 = Xold;
  } else
  {
    x1 = Xold; x2 = Xorg+Xorg-Xold;
  }
  y1 = MIN(Yorg, Yold); y2 = MAX(Yorg, Yold);
  gdk_draw_rectangle(DrawingArea->window, gcPurple, FALSE, x1, y1, x2-x1, y2-y1);
  Xold = Mev->x ; Yold = Mev->y;
  if (Xorg < Xold)
  {
    x1 = Xorg-(Xold-Xorg); x2 = Xold;
  } else
  {
    x1 = Xold; x2 = Xorg+Xorg-Xold;
  }
  y1 = MIN(Yorg, Yold); y2 = MAX(Yorg, Yold);
  gdk_draw_rectangle(DrawingArea->window, gcPurple, FALSE, x1, y1, x2-x1, y2-y1);
                                
  gx1 = Xoff+(Xorg-DrawingArea->allocation.width /2)/Zoom;
  Radius = fabs(Xorg-Mev->x)/Zoom;
      
  gy1 = MIN(Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom, 
            Yoff-(Mev->y-DrawingArea->allocation.height/2)/Zoom);
  gy2 = MAX(Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom, 
                  Yoff-(Mev->y-DrawingArea->allocation.height/2)/Zoom);
  Length = gy2-gy1;
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      From[0] = gx1;
      From[1] = gy1;
      From[2] = 0.0;
      break;
    case ORTHO_XZ_CAMERA:
      From[0] = gx1;
      From[1] = 0.0;
      From[2] = gy1;
      break;
    case ORTHO_ZY_CAMERA:
      From[0] = 0.0;
      From[1] = gy1;
      From[2] = gx1;
      break;
  }
  Message = g_strdup_printf(_("Cylinder: From <%g, %g, %g>, Length: %g, Radius: %g"),
                            From[0], From[1], From[2], Length, Radius);
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
  g_free(Message);
}

/*****************************************************************************
*  ToolReleaseNewCylinder
******************************************************************************/
void ToolReleaseNewCylinder(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventButton *Bev = (GdkEventButton *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  FrameViewStruct *TmpView;
  ObjectStruct *Cylinder;
  CylinderStruct *CCylinder;
  double gx1, gy1, gy2;
  double Radius;
  Vector Translate, Rotate, Scale;

  if (CYLINDER_STATE != MOVING)
    return;
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;

  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

  gx1 = Xoff+(Xorg-DrawingArea->allocation.width /2)/Zoom;
  Radius = fabs(Xorg-Bev->x)/Zoom;
      
  gy1 = MIN(Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom, 
                  Yoff-(Bev->y-DrawingArea->allocation.height/2)/Zoom);
  gy2 = MAX(Yoff-(Yorg-DrawingArea->allocation.height/2)/Zoom, 
                  Yoff-(Bev->y-DrawingArea->allocation.height/2)/Zoom);

  if ((Radius > 10e-6) && (gy2-gy1 > 10e-6))
  {
    CCylinder = g_malloc(sizeof(CylinderStruct));
    Cylinder = (ObjectStruct *)CCylinder;
    InitObject(Cylinder);
    Cylinder->Type = CYLINDER_OBJECT;
    CCylinder->Open = CylinderOpenFlag;
    switch (UserData->CameraStyle)
    {
      case ORTHO_XY_CAMERA:
        Scale[0] = Radius;
        Scale[1] = gy2-gy1;
        Scale[2] = Radius;
        ScaleCylinder(Cylinder, Scale);
        Translate[0] = gx1;
        Translate[1] = gy1;
        Translate[2] = 0.0;
        TranslateCylinder(Cylinder, Translate);
        break;
      case ORTHO_XZ_CAMERA:
        Scale[0] = Radius;
        Scale[1] = gy2-gy1;
        Scale[2] = Radius;
        ScaleCylinder(Cylinder, Scale);
        Rotate[0] = 90.0;
        Rotate[1] = 0.0;
        Rotate[2] = 0.0;
        RotateCylinder(Cylinder, Rotate);
        Translate[0] = gx1;
        Translate[1] = 0.0;
        Translate[2] = gy1;
        TranslateCylinder(Cylinder, Translate);
        break;
      case ORTHO_ZY_CAMERA:
        Scale[0] = Radius;
        Scale[1] = gy2-gy1;
        Scale[2] = Radius;
        ScaleCylinder(Cylinder, Scale);
        Translate[0] = 0.0;
        Translate[1] = gy1;
        Translate[2] = gx1;
        TranslateCylinder(Cylinder, Translate);
        break;
    }
    Cylinder->Next = UserData->FrameView->FirstObject;
    UserData->FrameView->FirstObject = Cylinder;
    ComputeCylinderTriangleMesh(Cylinder);
  }
  CYLINDER_STATE = NONE;

  gdk_gc_destroy(gcPurple);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
  gdk_pointer_ungrab(Bev->time);
}

/*****************************************************************************
*  ToggleCylinderOpen
******************************************************************************/
void ToggleCylinderOpen(GtkWidget *Check)
{
  CylinderOpenFlag = !CylinderOpenFlag;
}
/*****************************************************************************
*  BuildCylinderOptions
******************************************************************************/
GtkWidget *BuildCylinderOptions(GtkWidget *VBox)
{
  GtkWidget *vbox;
  GtkWidget *CheckOpen;
  GtkWidget *label;

  /*  the main vbox  */
  vbox = gtk_vbox_new (FALSE, 1);
  /*  the main label  */
  label = gtk_label_new(_("Cylinder"));
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  CheckOpen = gtk_check_button_new_with_label(_("Open-ended"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CheckOpen), CylinderOpenFlag);
  gtk_signal_connect_object(GTK_OBJECT(CheckOpen), "toggled",
                            GTK_SIGNAL_FUNC(ToggleCylinderOpen), NULL);

  gtk_box_pack_start(GTK_BOX(vbox), CheckOpen, FALSE, FALSE, 0);
  gtk_widget_show(CheckOpen);

  gtk_box_pack_start(GTK_BOX(VBox), vbox, TRUE, TRUE, 0);
  gtk_widget_show(vbox);
  return vbox;
}

/*****************************************************************************
*  CreateCylinder
******************************************************************************/
ObjectStruct *CreateCylinder(Vector Base, Vector Apex, double Radius)
{
  CylinderStruct *CCylinder;
  ObjectStruct *Cylinder;
  Vector LocalVector1, LocalVector2, Translate, Scale, N, U1, U2;
  double Length, ULength;
  TransformStruct Transform;
  
  CCylinder = g_malloc(sizeof(CylinderStruct));
  Cylinder = (ObjectStruct *)CCylinder;
  InitObject(Cylinder);
  Cylinder->Type = CYLINDER_OBJECT;

  LocalVector1[0] = Base[0];
  LocalVector1[1] = Base[1];
  LocalVector1[2] = Base[2];
  LocalVector2[0] = Apex[0];
  LocalVector2[1] = Apex[1];
  LocalVector2[2] = Apex[2];
  Translate[0] = LocalVector1[0];
  Translate[1] = LocalVector1[1];
  Translate[2] = LocalVector1[2];
  LocalVector2[0] -= Translate[0];
  LocalVector2[1] -= Translate[1];
  LocalVector2[2] -= Translate[2];

  Length = VLength(LocalVector2);
  if ((Length == 0.0) || (Radius <= 0.0))
  {
    g_free(CCylinder);
    return NULL;
  }
  Scale[0] = Radius;
  Scale[1] = Length;
  Scale[2] = Radius;
  ScaleCylinder(Cylinder, Scale);

  N[0] = LocalVector2[0] / Length;
  N[1] = LocalVector2[1] / Length;
  N[2] = LocalVector2[2] / Length;
  if ( (fabs(N[0])>fabs(N[1])) && (fabs(N[0])>fabs(N[2])) )
  { /* The cylinder axis is mainly upon the X axis */
    U1[0] = -N[1]/N[0];
    U1[1] = 1;
    U1[2] = 0;
  } else if ( (fabs(N[1])>fabs(N[0])) && (fabs(N[1])>fabs(N[2])) )
  { /* The cylinder axis is mainly upon the Y axis */
    U1[0] = 1;
    U1[1] = -N[0]/N[1];
    U1[2] = 0;
  } else
  { /* The cylinder axis is mainly upon the Z axis */
    U1[0] = 0;
    U1[1] = 1;
    U1[2] = -N[1]/N[2];
  }
  ULength = VLength(U1);
  U1[0]/=ULength; U1[1]/=ULength; U1[2]/=ULength;
  VCross(U2, N, U1);
  ULength = VLength(U2);
  U2[0]/=ULength; U2[1]/=ULength; U2[2]/=ULength;
  Transform.Direct[0][0] = U1[0];
  Transform.Direct[0][1] = U1[1];
  Transform.Direct[0][2] = U1[2];
  Transform.Direct[0][3] = 0.0;
  Transform.Direct[1][0] = N[0];
  Transform.Direct[1][1] = N[1];
  Transform.Direct[1][2] = N[2];
  Transform.Direct[1][3] = 0.0;
  Transform.Direct[2][0] = U2[0];
  Transform.Direct[2][1] = U2[1];
  Transform.Direct[2][2] = U2[2];
  Transform.Direct[2][3] = 0.0;
  Transform.Direct[3][0] = 0.0;
  Transform.Direct[3][1] = 0.0;
  Transform.Direct[3][2] = 0.0;
  Transform.Direct[3][3] = 1.0;
  MInverse(Transform.Inverse, Transform.Direct);
  ComposeTrans(Cylinder->Trans, &Transform);
  TranslateCylinder(Cylinder, Translate);

  ComputeCylinderTriangleMesh(Cylinder);
  return Cylinder;                    
}

/*****************************************************************************
*  PDBCreateCylinder
******************************************************************************/
int PDBCreateCylinder(int *Id, Vector *Base, Vector *Apex, double *Radius)
{
  ObjectStruct *Cylinder;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;
    
  Cylinder = CreateCylinder(*Base, *Apex, *Radius);
  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
    ;
  Cylinder->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Cylinder;
  ComputeCylinderTriangleMesh(Cylinder);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Cylinder->Id;
}

