/* disc.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "snap.h"
#include "disc.h"

int DiscWithHoleFlag;

/*****************************************************************************
*  InsideDisc
******************************************************************************/
int InsideDisc(ObjectStruct *Disc, double x, double y, double z)
{
  return 0; /* We cannot be "Inside" a flat disc... */
}

/*****************************************************************************
*  FindIntersectDiscSegment
******************************************************************************/
int FindIntersectionDiscSegment(Vector InterPoint, Vector InterNorm,
                                Vector InPoint, Vector OutPoint,
                                ObjectStruct *Disc)
{
  return 0;
}

/*****************************************************************************
*  ComputeDiscTriangleMesh
******************************************************************************/
void ComputeDiscTriangleMesh(ObjectStruct *Disc)
{ /* XXX: clipped_by */
  DiscStruct *DDisc = (DiscStruct *)Disc;
  Vector P1, P2, P3, P4;
  Vector N;
  int i;

  Disc->DetailLevel = 16;

  if (Disc->FirstTriangle)
    DestroyObjectTriangleMesh(Disc);
  N[0] = 0.0; N[1] = 1.0; N[2] = 0.0;
  if (Disc->Trans)
  {
    MEvaluateVector(N, Disc->Trans, N);
  }
  for (i=0 ; i<Disc->DetailLevel ; i++)
  {
    P1[0] = DDisc->HoleRadius*cos(i*2.0*M_PI/Disc->DetailLevel);
    P1[1] = 0.0;
    P1[2] = DDisc->HoleRadius*sin(i*2.0*M_PI/Disc->DetailLevel);
    P2[0] = DDisc->Radius*cos((i*2.0+1.0)*M_PI/Disc->DetailLevel);
    P2[1] = 0.0;
    P2[2] = DDisc->Radius*sin((i*2.0+1.0)*M_PI/Disc->DetailLevel);
    P3[0] = DDisc->HoleRadius*cos((i+1)*2.0*M_PI/Disc->DetailLevel);
    P3[1] = 0.0;
    P3[2] = DDisc->HoleRadius*sin((i+1)*2.0*M_PI/Disc->DetailLevel);
    P4[0] = DDisc->Radius*cos(((i+1)*2.0+1.0)*M_PI/Disc->DetailLevel);
    P4[1] = 0.0;
    P4[2] = DDisc->Radius*sin(((i+1)*2.0+1.0)*M_PI/Disc->DetailLevel);
    if (Disc->Trans)
    {
      MEvaluatePoint(P1, Disc->Trans, P1);
      MEvaluatePoint(P2, Disc->Trans, P2);
      MEvaluatePoint(P3, Disc->Trans, P3);
      MEvaluatePoint(P4, Disc->Trans, P4);
    }
    if (DDisc->HoleRadius > 10e-6)
      AddTriangleToObjectMesh(Disc, P1,P2,P3, N,N,N);
    AddTriangleToObjectMesh(Disc, P2,P3,P4, N,N,N);
  }
}

/*****************************************************************************
*  TranslateDisc
******************************************************************************/
void TranslateDisc(ObjectStruct *Disc, Vector Vect)
{
  TransformStruct Trans;

  if (Disc->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Disc->Trans, &Trans);
  } else
  {
    Disc->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Disc->Trans, Vect);
  }
  if (Disc->ClippedBy)
    TranslateObject(Disc->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateDisc
******************************************************************************/
void RotateDisc(ObjectStruct *Disc, Vector Vect)
{
  TransformStruct Trans;

  if (Disc->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Disc->Trans, &Trans);
  } else
  {
    Disc->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Disc->Trans, Vect);
  }
  if (Disc->ClippedBy)
    RotateObject(Disc->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleDisc
******************************************************************************/
void ScaleDisc(ObjectStruct *Disc, Vector Vect)
{
  TransformStruct Trans;

  if (Disc->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Disc->Trans, &Trans);
  } else
  {
    Disc->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Disc->Trans, Vect);
  }
  if (Disc->ClippedBy)
    ScaleObject(Disc->ClippedBy, Vect);
}

/*****************************************************************************
*  IsDiscIntersection FIXME
******************************************************************************/
int IsDiscIntersection(ObjectStruct *Disc, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;

  if (Disc->Trans)
  {
    MInverseEvaluatePoint(Org, Disc->Trans, Origin);
    MInverseEvaluateVector(Dir, Disc->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInDiscXY
******************************************************************************/
int ClickInDiscXY(ObjectStruct *Disc, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsDiscIntersection(Disc, Origin, Direction);
}

/****************************************************************************
*  ClickInDiscXZ
*****************************************************************************/
int ClickInDiscXZ(ObjectStruct *Disc, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsDiscIntersection(Disc, Origin, Direction);
}

/****************************************************************************
*  ClickInDiscZY
*****************************************************************************/
int ClickInDiscZY(ObjectStruct *Disc, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsDiscIntersection(Disc, Origin, Direction);
}

/****************************************************************************/
double Xorg, Yorg, Radius, HoleRadius;
GdkColormap *colormap;
GdkGC *gcPurple;
GdkColor Purple;
guint id;
Vector Center;

#define NONE           1
#define MOVING         2
#define BETWEEN_RADIUS 3
#define HOLE_RADIUS    4

int DISC_STATE = NONE;

/*****************************************************************************
*  ToolInitNewDisc
******************************************************************************/
void ToolInitNewDisc(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ViewUserDataStruct *UserData;
  GdkColormap *colormap;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;

  if (DISC_STATE == BETWEEN_RADIUS)
  {
    DISC_STATE = HOLE_RADIUS;
    return;
  }
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  id = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "new disc");
  MouseToReal(bevent->x, bevent->y, DrawingArea, &Xorg, &Yorg);

  colormap = gdk_window_get_colormap(DrawingArea->window);
  gcPurple = gdk_gc_new(DrawingArea->window);
  Purple.red = 65535; Purple.green = 0; Purple.blue = 65535;
  /* XXX: the color should be settable... */
  gdk_color_alloc(colormap, &Purple);
  gdk_gc_set_foreground(gcPurple, &Purple);
  gdk_gc_set_function(gcPurple, GDK_XOR);

  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      Center[0] = Xorg;
      Center[1] = Yorg;
      Center[2] = 0.0;
      break;
    case ORTHO_XZ_CAMERA:
      Center[0] = Xorg;
      Center[1] = 0.0;
      Center[2] = Yorg;
      break;
    case ORTHO_ZY_CAMERA:
      Center[0] = 0.0;
      Center[1] = Yorg;
      Center[2] = Xorg;
      break;
  }
  Radius = 0.0;

  DISC_STATE = MOVING;
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, _("Disc: "));
  gdk_pointer_grab(DrawingArea->window, FALSE,
                   GDK_BUTTON_PRESS_MASK |
                   GDK_POINTER_MOTION_MASK |
                   GDK_BUTTON_RELEASE_MASK,
                   NULL, NULL, bevent->time);
}

/*****************************************************************************
*  ToolMotionNewDisc
******************************************************************************/
void ToolMotionNewDisc(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventMotion *Mev = (GdkEventMotion *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff, Xold, Yold;
  char *Message;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  if (DISC_STATE == MOVING)
  {
    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-Radius-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+Radius-Yoff)*Zoom,
                 Radius*2.0*Zoom, Radius*2.0*Zoom,  0, 64*360);
    MouseToReal(Mev->x, Mev->y, DrawingArea, &Xold, &Yold);
    Radius = sqrt((Xold-Xorg)*(Xold-Xorg)+(Yold-Yorg)*(Yold-Yorg));
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-Radius-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+Radius-Yoff)*Zoom,
                 Radius*2.0*Zoom, Radius*2.0*Zoom,  0, 64*360);
    Message = g_strdup_printf(_("Disc: Center <%g, %g, %g>, Radius: %g"),
                              Center[0], Center[1], Center[2], Radius);
    gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
    g_free(Message);
  } else if ((DISC_STATE == HOLE_RADIUS) || (DISC_STATE == BETWEEN_RADIUS))
  {
    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-HoleRadius-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+HoleRadius-Yoff)*Zoom,
                 HoleRadius*2.0*Zoom, HoleRadius*2.0*Zoom, 0,64*360);
    MouseToReal(Mev->x, Mev->y, DrawingArea, &Xold, &Yold);
    HoleRadius = sqrt((Xold-Xorg)*(Xold-Xorg)+(Yold-Yorg)*(Yold-Yorg));
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-HoleRadius-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+HoleRadius-Yoff)*Zoom,
                 HoleRadius*2.0*Zoom, HoleRadius*2.0*Zoom, 0,64*360);
    Message = g_strdup_printf(_("Disc: Center <%g, %g, %g>, Radius: %g, Hole Radius: %g"),
            Center[0], Center[1], Center[2], 
            MAX(Radius, HoleRadius),
            MIN(Radius, HoleRadius));
    gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
    g_free(Message);
  }
}

/*****************************************************************************
*  ToolReleaseNewDisc
******************************************************************************/
void ToolReleaseNewDisc(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventButton *Bev = (GdkEventButton *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  FrameViewStruct *TmpView;
  ObjectStruct *Disc;
  DiscStruct *DDisc;
  Vector Vect;

  if (DISC_STATE == MOVING)
  {
    DISC_STATE = BETWEEN_RADIUS;
    HoleRadius = 0.0;
    return;
  }
  if (DISC_STATE != HOLE_RADIUS)
    return;
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);
  DDisc = g_malloc(sizeof(DiscStruct));
  Disc = (ObjectStruct *)DDisc;
  InitObject(Disc);
  Disc->Type = DISC_OBJECT;
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      Vect[0] = 90.0;
      Vect[1] = 0.0;
      Vect[2] = 0.0;
      RotateDisc(Disc, Vect);
      break;
    case ORTHO_ZY_CAMERA:
      Vect[0] = 0.0;
      Vect[1] = 0.0;
      Vect[2] = 90.0;
      RotateDisc(Disc, Vect);
      break;
  }
  TranslateDisc(Disc, Center);
  DDisc->Radius = MAX(Radius, HoleRadius);
  DDisc->HoleRadius = MIN(Radius, HoleRadius);
  Disc->Next = UserData->FrameView->FirstObject;
  UserData->FrameView->FirstObject = Disc;
  ComputeDiscTriangleMesh(Disc);
  DISC_STATE = NONE;
  gdk_gc_destroy(gcPurple);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
  gdk_pointer_ungrab(Bev->time);
}

/*****************************************************************************
*  ToggleDiscWithHole
******************************************************************************/
void ToggleDiscWithHole(GtkWidget *Check)
{
  DiscWithHoleFlag = !DiscWithHoleFlag;
}

/*****************************************************************************
*  BuildDiscOptions
******************************************************************************/
GtkWidget *BuildDiscOptions(GtkWidget *VBox)
{
  GtkWidget *vbox;
  GtkWidget *Check;
  GtkWidget *label;

  /*  the main vbox  */
  vbox = gtk_vbox_new (FALSE, 1);
  /*  the main label  */
  label = gtk_label_new(_("Disc"));
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  Check = gtk_check_button_new_with_label(_("Disc with Hole"));
  DiscWithHoleFlag = TRUE;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Check), DiscWithHoleFlag);
  gtk_signal_connect_object(GTK_OBJECT(Check), "toggled",
                            GTK_SIGNAL_FUNC(ToggleDiscWithHole), NULL);
  gtk_box_pack_start(GTK_BOX(vbox), Check, FALSE, FALSE, 0);
                     gtk_widget_show(Check);

  gtk_box_pack_start(GTK_BOX(VBox), vbox, TRUE, TRUE, 0);
                     gtk_widget_show(vbox);
  return vbox;
}

/*****************************************************************************
*  PDBCreateDisc
******************************************************************************/
int PDBCreateDisc(int *Id, Vector *Center, Vector *pNormal,
                  double *Radius, double *HoleRadius)
{
  ObjectStruct *Disc;
  DiscStruct *DDisc;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;
  Vector Normal, U1, U2;
  double Length, ULength;

  DDisc = g_malloc(sizeof(DiscStruct));
  Disc = (ObjectStruct *)DDisc;
  InitObject(Disc);
  Disc->Type = DISC_OBJECT;

  DDisc->Radius = MAX((*Radius), (*HoleRadius));
  DDisc->HoleRadius = MIN((*Radius), (*HoleRadius));

  Disc->Trans = g_malloc(sizeof(TransformStruct));
  Normal[0] = (*pNormal)[0]; Normal[1] = (*pNormal)[1]; Normal[2] = (*pNormal)[2];
  Length = VLength(Normal);
  Normal[0] /= Length;
  Normal[1] /= Length;
  Normal[2] /= Length;
  if ( (fabs(Normal[0])>fabs(Normal[1])) && (fabs(Normal[0])>fabs(Normal[2])) )
  { /* The normal is mainly upon the X axis */
    U1[0] = -Normal[1]/Normal[0];
    U1[1] = 1;
    U1[2] = 0;
  } else if ( (fabs(Normal[1])>fabs(Normal[0])) && (fabs(Normal[1])>fabs(Normal[2])) )
  { /* The normal is mainly upon the Y axis */
    U1[0] = 1;
    U1[1] = -Normal[0]/Normal[1];
    U1[2] = 0;
  } else
  { /* The normal axis is mainly upon the Z axis */
    U1[0] = 0;
    U1[1] = 1;
    U1[2] = -Normal[1]/Normal[2];
  }
  ULength = VLength(U1);
  U1[0]/=ULength; U1[1]/=ULength; U1[2]/=ULength;
  VCross(U2, Normal, U1);
  ULength = VLength(U2);
  U2[0]/=ULength; U2[1]/=ULength; U2[2]/=ULength;

  Disc->Trans->Direct[0][0] = U1[0];
  Disc->Trans->Direct[0][1] = U1[1];
  Disc->Trans->Direct[0][2] = U1[2];
  Disc->Trans->Direct[0][3] = 0.0;
  Disc->Trans->Direct[1][0] = Normal[0];
  Disc->Trans->Direct[1][1] = Normal[1];
  Disc->Trans->Direct[1][2] = Normal[2];
  Disc->Trans->Direct[1][3] = 0.0;
  Disc->Trans->Direct[2][0] = U2[0];
  Disc->Trans->Direct[2][1] = U2[1];
  Disc->Trans->Direct[2][2] = U2[2];
  Disc->Trans->Direct[2][3] = 0.0;
  Disc->Trans->Direct[3][0] = 0.0;
  Disc->Trans->Direct[3][1] = 0.0;
  Disc->Trans->Direct[3][2] = 0.0;
  Disc->Trans->Direct[3][3] = 1.0;
  MInverse(Disc->Trans->Inverse, Disc->Trans->Direct);
  TranslateDisc(Disc, *Center);

  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
                  ;
  Disc->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Disc;
  ComputeDiscTriangleMesh(Disc);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
  gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Disc->Id;
}

