/* edit.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "giram.h"
#include "object.h"
#include "destroy.h"
#include "copy.h"
#include "csgtree.h"
#include "tool_select.h"
#include "edit.h"

ObjectSelection *CutBuffer = NULL;

/*****************************************************************************
*  DestroyCutBuffer
******************************************************************************/
void DestroyCutBuffer(void)
{
  ObjectSelection *TmpSel1, *TmpSel2;

  TmpSel1 = CutBuffer;
  while (TmpSel1)
  {
    TmpSel2 = TmpSel1->Next;
    DestroyObject(TmpSel1->Object);
    g_free(TmpSel1);
    TmpSel1 = TmpSel2;
  }
  CutBuffer = NULL;
}

/*****************************************************************************
*  AddObjectToCutBuffer
******************************************************************************/
void AddObjectToCutBuffer(ObjectStruct *Object)
{
  ObjectSelection *TmpSel;

  TmpSel = g_malloc(sizeof(ObjectSelection));
  TmpSel->Next = CutBuffer;
  TmpSel->Object = CopyObject(Object);
  CutBuffer = TmpSel;
}

/*****************************************************************************
*  CutSelection
******************************************************************************/
void CutSelection(GtkWidget *DrawingArea)
{
  ViewUserDataStruct *UserData;
  ObjectSelection *TmpSel, *TmpSel1, *TmpSel2;
  ObjectStruct *TmpObject;
  FrameViewStruct *TmpView;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  /* Destroy the current cut buffer */
  DestroyCutBuffer();
  if (UserData->FrameView->Selection == NULL)
    return;
  for (TmpSel = UserData->FrameView->Selection ;
       TmpSel->Next != UserData->FrameView->Selection ;
       TmpSel = TmpSel->Next)
  {
    /* Remove the object from the frame. */
    if (TmpSel->Object == UserData->FrameView->FirstObject)
    {
      UserData->FrameView->FirstObject = UserData->FrameView->FirstObject->Next;
    } else
    {
      for (TmpObject = UserData->FrameView->FirstObject ;
           TmpObject->Next != TmpSel->Object ;
           TmpObject = TmpObject->Next)
             ;
      TmpObject->Next = TmpObject->Next->Next;
    }
    /* Add the object to the current cut buffer. */
    AddObjectToCutBuffer(TmpSel->Object);
    /* Destroy the object */
    DestroyObject(TmpSel->Object);
  }
  /* Remove the object from the frame. */
  if (TmpSel->Object == UserData->FrameView->FirstObject)
  {
    UserData->FrameView->FirstObject = UserData->FrameView->FirstObject->Next;
  } else
  {
    for (TmpObject = UserData->FrameView->FirstObject ;
         TmpObject->Next != TmpSel->Object ;
         TmpObject = TmpObject->Next)
           ;
    TmpObject->Next = TmpObject->Next->Next;
  }
  /* Add the object to the current cut buffer. */
  AddObjectToCutBuffer(TmpSel->Object);
  /* Destroy the object */
  DestroyObject(TmpSel->Object);
  /* Destroy the Current Selection */
  TmpSel1 = UserData->FrameView->Selection;
  while (TmpSel1->Next != UserData->FrameView->Selection)
  {
    TmpSel2 = TmpSel1->Next;
    g_free(TmpSel1);
    TmpSel1 = TmpSel2;
  }
  g_free(TmpSel1);
  UserData->FrameView->Selection = NULL;
  /* Redraw all view plus CSG Tree */
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
}

/*****************************************************************************
*  CopySelection
******************************************************************************/
void CopySelection(GtkWidget *DrawingArea)
{
  ViewUserDataStruct *UserData;
  ObjectSelection *TmpSel;
  FrameViewStruct *TmpView;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  /* Destroy the current cut buffer */
  DestroyCutBuffer();
  if (UserData->FrameView->Selection == NULL)
    return;
  for (TmpSel = UserData->FrameView->Selection ;
       TmpSel->Next != UserData->FrameView->Selection ;
       TmpSel = TmpSel->Next)
  {
    /* Add the object to the current cut buffer. */
    AddObjectToCutBuffer(TmpSel->Object);
  }
  /* Add the object to the current cut buffer. */
  AddObjectToCutBuffer(TmpSel->Object);
  /* Redraw all view plus CSG Tree */
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
}

/*****************************************************************************
*  PasteSelection
******************************************************************************/
void PasteSelection(GtkWidget *DrawingArea)
{
  ViewUserDataStruct *UserData;
  ObjectSelection *TmpSel, *TmpSel1, *TmpSel2;
  ObjectStruct *TmpObject;
  FrameViewStruct *TmpView;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  /* Destroy the Current Selection */
  if (UserData->FrameView->Selection)
  {
    TmpSel1 = UserData->FrameView->Selection;
    while (TmpSel1->Next != UserData->FrameView->Selection)
    {
      TmpSel2 = TmpSel1->Next;
      TmpSel1->Object->Selected = FALSE;
      g_free(TmpSel1);
      TmpSel1 = TmpSel2;
    }
    TmpSel1->Object->Selected = FALSE;
    g_free(TmpSel1);
    UserData->FrameView->Selection = NULL;
  }

  if (CutBuffer)
  {
    for (TmpSel = CutBuffer ; TmpSel ; TmpSel = TmpSel->Next)
    {
      ObjectStruct *Tmp1;

      TmpObject = CopyObject(TmpSel->Object);
      if (UserData->FrameView->FirstObject)
      {
        for (Tmp1 = UserData->FrameView->FirstObject ; 
             Tmp1->Next ; 
             Tmp1 = Tmp1->Next)
          ;
        Tmp1->Next = TmpObject;
        TmpObject->Next = NULL;
      } else
        UserData->FrameView->FirstObject = TmpObject;
      AddObjectToSelection(&(UserData->FrameView->Selection), TmpObject);
    }
  }
  
  /* Redraw all view plus CSG Tree */
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
}

