/* frame.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "giram.h"
#include "camera.h"
#include "frame.h"

/*****************************************************************************
*  InitFrame
******************************************************************************/
FrameStruct *NewFrame(char *Name)
{
  FrameStruct *LocalFrame;
  int Id;

  Id = 0;
  for (LocalFrame = FirstFrame ; LocalFrame ; LocalFrame = LocalFrame->Next)
    if (LocalFrame->Id >= Id)
      Id = LocalFrame->Id+1;
  LocalFrame = g_malloc(sizeof(FrameStruct));
  LocalFrame->Next = FirstFrame;
  FirstFrame = LocalFrame;
  LocalFrame->Name = g_strdup(Name);
  LocalFrame->Id = Id;
  LocalFrame->Global.AssumedGamma = 1.0;
  LocalFrame->Global.HfGray16 = 0;
  LocalFrame->Global.AmbientLight[0] = 1.0;
  LocalFrame->Global.AmbientLight[1] = 1.0;
  LocalFrame->Global.AmbientLight[2] = 1.0;
  LocalFrame->Global.AmbientLight[3] = 0.0;
  LocalFrame->Global.AmbientLight[4] = 0.0;
  LocalFrame->Global.NumberOfWaves = 10;
  LocalFrame->Global.ADCBailout = 1.0 / 255.0;
  LocalFrame->Global.IridWavelength[0] = 0.25;
  LocalFrame->Global.IridWavelength[1] = 0.18;
  LocalFrame->Global.IridWavelength[2] = 0.14;
  LocalFrame->Global.IridWavelength[3] = 0.0;
  LocalFrame->Global.IridWavelength[4] = 0.0;
  LocalFrame->Global.MaxIntersections = 64;
  LocalFrame->Global.MaxTraceLevel = 5;
  LocalFrame->Global.Radiosity.Brightness = 3.3;
  LocalFrame->Global.Radiosity.Count = 100;
  LocalFrame->Global.Radiosity.DistanceMaximum = 0.0;
  LocalFrame->Global.Radiosity.ErrorBound = 0.4;
  LocalFrame->Global.Radiosity.GrayThreshold = 0.5;
  LocalFrame->Global.Radiosity.LowErrorFactor = 0.8;
  LocalFrame->Global.Radiosity.MinimumReuse = 0.015;
  LocalFrame->Global.Radiosity.MinimumReuse = 0.015;
  LocalFrame->Global.Radiosity.NearestCount = 6;
  LocalFrame->Global.Radiosity.RecursionLimit = 1;
  LocalFrame->Camera = g_malloc(sizeof(CameraStruct));
  InitCamera(LocalFrame->Camera);
  ComputeCamera(LocalFrame->Camera);
  LocalFrame->FirstObject = NULL;
  LocalFrame->FirstLightSource = NULL;
  LocalFrame->SkySphere = NULL;
  LocalFrame->FloatConstantList = NULL;
  LocalFrame->VectorConstantList = NULL;
  LocalFrame->ColorConstantList = NULL;
  LocalFrame->StringConstantList = NULL;
  LocalFrame->ObjectConstantList = NULL;
  LocalFrame->TextureConstantList = NULL;
  LocalFrame->PigmentConstantList = NULL;
  LocalFrame->NormalConstantList = NULL;
  LocalFrame->FinishConstantList = NULL;
  LocalFrame->HaloConstantList = NULL;
  LocalFrame->ColorMapConstantList = NULL;
  LocalFrame->PigmentMapConstantList = NULL;
  LocalFrame->SlopeMapConstantList = NULL;
  LocalFrame->NormalMapConstantList = NULL;
  LocalFrame->CameraConstantList = NULL;
  LocalFrame->LightSourceConstantList = NULL;
  LocalFrame->AtmosphereConstantList = NULL;
  LocalFrame->FogConstantList = NULL;
  LocalFrame->RainbowConstantList = NULL;
  LocalFrame->SkySphereConstantList = NULL;
  LocalFrame->TransformConstantList = NULL;
  LocalFrame->FirstView = NULL;
  LocalFrame->Selection = NULL;
  LocalFrame->RotationCenter[0] = 0.0;
  LocalFrame->RotationCenter[1] = 0.0;
  LocalFrame->RotationCenter[2] = 0.0;
  LocalFrame->ScaleCenter[0] = 0.0;
  LocalFrame->ScaleCenter[1] = 0.0;
  LocalFrame->ScaleCenter[2] = 0.0;
  LocalFrame->SnapToGrid = FALSE;
  LocalFrame->ShowGrid = FALSE;
  LocalFrame->SnapGrid[0] = 1.0;
  LocalFrame->SnapGrid[1] = 1.0;
  LocalFrame->SnapGrid[2] = 1.0;
  return LocalFrame;
}

