/* mesh.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "mesh.h"

/*****************************************************************************
*  InsideMesh
******************************************************************************/
int InsideMesh(ObjectStruct *Mesh, double x, double y, double z)
{
  return 0; /* We cannot be "Inside" a mesh */
}

/*****************************************************************************
*  FindIntersectMeshSegment
******************************************************************************/
int FindIntersectionMeshSegment(Vector InterPoint, Vector InterNorm,
                                Vector InPoint, Vector OutPoint,
                                ObjectStruct *Mesh)
{
  return 0;
}

/*****************************************************************************
*  ComputeMeshTriangleMesh
******************************************************************************/
void ComputeMeshTriangleMesh(ObjectStruct *Mesh)
{ /* XXX: clipped_by */
  Vector P1, P2, P3, N1, N2, N3;
  TriangleListStruct *TmpTri;
  MeshStruct *MMesh = (MeshStruct *)Mesh;

  if (Mesh->FirstTriangle)
    DestroyObjectTriangleMesh(Mesh);
  for (TmpTri=MMesh->FirstTriangle ; TmpTri ; TmpTri=TmpTri->Next)
  {
    P1[0] = TmpTri->P1[0]; P1[1] = TmpTri->P1[1]; P1[2] = TmpTri->P1[2];
    P2[0] = TmpTri->P2[0]; P2[1] = TmpTri->P2[1]; P2[2] = TmpTri->P2[2];
    P3[0] = TmpTri->P3[0]; P3[1] = TmpTri->P3[1]; P3[2] = TmpTri->P3[2];
    N1[0] = TmpTri->N1[0]; N1[1] = TmpTri->N1[1]; N1[2] = TmpTri->N1[2];
    N2[0] = TmpTri->N2[0]; N2[1] = TmpTri->N2[1]; N2[2] = TmpTri->N2[2];
    N3[0] = TmpTri->N3[0]; N3[1] = TmpTri->N3[1]; N3[2] = TmpTri->N3[2];
    if (Mesh->Trans)
    {
      MEvaluatePoint(P1, Mesh->Trans, P1);
      MEvaluatePoint(P2, Mesh->Trans, P2);
      MEvaluatePoint(P3, Mesh->Trans, P3);
      MEvaluateVector(N1, Mesh->Trans, N1);
      MEvaluateVector(N2, Mesh->Trans, N2);
      MEvaluateVector(N3, Mesh->Trans, N3);
    }
    AddTriangleToObjectMesh(Mesh, P1,P2,P3, N1,N2,N3);
  }
}

/*****************************************************************************
*  TranslateMesh
******************************************************************************/
void TranslateMesh(ObjectStruct *Mesh, Vector Vect)
{
  TransformStruct Trans;

  if (Mesh->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Mesh->Trans, &Trans);
  } else
  {
    Mesh->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Mesh->Trans, Vect);
  }
  if (Mesh->ClippedBy)
    TranslateObject(Mesh->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateMesh
******************************************************************************/
void RotateMesh(ObjectStruct *Mesh, Vector Vect)
{
  TransformStruct Trans;

  if (Mesh->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Mesh->Trans, &Trans);
  } else
  {
    Mesh->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Mesh->Trans, Vect);
  }
  if (Mesh->ClippedBy)
    RotateObject(Mesh->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleMesh
******************************************************************************/
void ScaleMesh(ObjectStruct *Mesh, Vector Vect)
{
  TransformStruct Trans;

  if (Mesh->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Mesh->Trans, &Trans);
  } else
  {
    Mesh->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Mesh->Trans, Vect);
  }
  if (Mesh->ClippedBy)
    ScaleObject(Mesh->ClippedBy, Vect);
}

/*****************************************************************************
*  IsMeshIntersection
******************************************************************************/
int IsMeshIntersection(ObjectStruct *Mesh, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
  MeshStruct *MMesh = (MeshStruct *)Mesh;
  TriangleListStruct *TmpTri;
  Matrix Mat, InvMat;
  double D, u, v;
  
  if (Mesh->Trans)
  {
    MInverseEvaluatePoint(Org, Mesh->Trans, Origin);
    MInverseEvaluateVector(Dir, Mesh->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  for (TmpTri = MMesh->FirstTriangle ; TmpTri ; TmpTri=TmpTri->Next)
  {
    Mat[0][0] = TmpTri->P2[0] - TmpTri->P1[0];
    Mat[0][1] = TmpTri->P2[1] - TmpTri->P1[1];
    Mat[0][2] = TmpTri->P2[2] - TmpTri->P1[2];
    Mat[1][0] = TmpTri->P3[0] - TmpTri->P1[0];
    Mat[1][1] = TmpTri->P3[1] - TmpTri->P1[1];
    Mat[1][2] = TmpTri->P3[2] - TmpTri->P1[2];
    Mat[2][0] = -Dir[0];
    Mat[2][1] = -Dir[1];
    Mat[2][2] = -Dir[2];
    D = Mat[0][0]*Mat[1][1]*Mat[2][2]+Mat[0][1]*Mat[1][2]*Mat[2][0]+
        Mat[0][2]*Mat[1][0]*Mat[2][1]-Mat[2][0]*Mat[1][1]*Mat[0][2]-
        Mat[2][1]*Mat[1][2]*Mat[0][0]-Mat[2][2]*Mat[1][0]*Mat[0][1];
    if (D != 0.0)
    {
      InvMat[0][0] =  (Mat[1][1]*Mat[2][2]-Mat[2][1]*Mat[1][2])/D;
      InvMat[0][1] = -(Mat[0][1]*Mat[2][2]-Mat[2][1]*Mat[0][2])/D;
      InvMat[0][2] =  (Mat[0][1]*Mat[1][2]-Mat[1][1]*Mat[0][2])/D;
      InvMat[1][0] = -(Mat[1][0]*Mat[2][2]-Mat[2][0]*Mat[1][2])/D;
      InvMat[1][1] =  (Mat[0][0]*Mat[2][2]-Mat[2][0]*Mat[0][2])/D;
      InvMat[1][2] = -(Mat[0][0]*Mat[1][2]-Mat[1][0]*Mat[0][2])/D;
      InvMat[2][0] =  (Mat[1][0]*Mat[2][1]-Mat[2][0]*Mat[1][1])/D;
      InvMat[2][1] = -(Mat[0][0]*Mat[2][1]-Mat[2][0]*Mat[0][1])/D;
      InvMat[2][2] =  (Mat[0][0]*Mat[1][1]-Mat[1][0]*Mat[0][1])/D;
      u = InvMat[0][0]*(Org[0]-TmpTri->P1[0])+
          InvMat[1][0]*(Org[1]-TmpTri->P1[1])+
          InvMat[2][0]*(Org[2]-TmpTri->P1[2]);
      v = InvMat[0][1]*(Org[0]-TmpTri->P1[0])+
          InvMat[1][1]*(Org[1]-TmpTri->P1[1])+
          InvMat[2][1]*(Org[2]-TmpTri->P1[2]);
      if ((u>=0.0) && (v>=0.0) && (u+v<=1.0))
        return 1;
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInMeshXY
******************************************************************************/
int ClickInMeshXY(ObjectStruct *Mesh, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsMeshIntersection(Mesh, Origin, Direction);
}

/****************************************************************************
*  ClickInMeshXZ
*****************************************************************************/
int ClickInMeshXZ(ObjectStruct *Mesh, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsMeshIntersection(Mesh, Origin, Direction);
}

/****************************************************************************
*  ClickInMeshZY
*****************************************************************************/
int ClickInMeshZY(ObjectStruct *Mesh, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsMeshIntersection(Mesh, Origin, Direction);
}

/*****************************************************************************
*  ToolNewMesh FIXME
******************************************************************************/
void ToolNewMesh(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Label;

  Dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New Mesh"));

  Label = gtk_label_new(_("Sorry, the New Mesh function"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);
  Label = gtk_label_new(_("hasn't been implemented yet"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);

  Button = gtk_button_new_with_label("Close");
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button,
                     TRUE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  gtk_widget_show(Dialog);
}

/*****************************************************************************
*  CreateMesh
******************************************************************************/
ObjectStruct *CreateMesh(void)
{
  MeshStruct *MMesh;
  ObjectStruct *Mesh;

  MMesh = g_malloc(sizeof(MeshStruct));
  Mesh = (ObjectStruct *)MMesh;
  InitObject(Mesh);
  Mesh->Type = MESH_OBJECT;
  MMesh->FirstTriangle = NULL;
  return Mesh;
}

/*****************************************************************************
*  AddTriangleToMesh
******************************************************************************/
void AddTriangleToMesh(ObjectStruct *Mesh, Vector P1, Vector P2, Vector P3)
{
  TriangleListStruct *TmpTri;
  MeshStruct *MMesh = (MeshStruct *)Mesh;
  Vector V1, V2;
    
  if (Mesh == NULL)
    return;
  TmpTri = g_malloc(sizeof(TriangleListStruct));
  TmpTri->Next = MMesh->FirstTriangle;
  MMesh->FirstTriangle = TmpTri;
  TmpTri->P1[0] = P1[0]; TmpTri->P1[1] = P1[1]; TmpTri->P1[2] = P1[2];
  TmpTri->P2[0] = P2[0]; TmpTri->P2[1] = P2[1]; TmpTri->P2[2] = P2[2];
  TmpTri->P3[0] = P3[0]; TmpTri->P3[1] = P3[1]; TmpTri->P3[2] = P3[2];

  V1[0] = TmpTri->P2[0] - TmpTri->P1[0];
  V1[1] = TmpTri->P2[1] - TmpTri->P1[1];
  V1[2] = TmpTri->P2[2] - TmpTri->P1[2];
  V2[0] = TmpTri->P3[0] - TmpTri->P1[0];
  V2[1] = TmpTri->P3[1] - TmpTri->P1[1];
  V2[2] = TmpTri->P3[2] - TmpTri->P1[2];
  VCross(TmpTri->N1, V1, V2);
  VCross(TmpTri->N2, V1, V2);
  VCross(TmpTri->N3, V1, V2);
  AddTriangleToObjectMesh(Mesh, TmpTri->P1, TmpTri->P2, TmpTri->P3, 
                                TmpTri->N1, TmpTri->N2, TmpTri->N3);
}

