/* parserc.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include "giram.h"
#include "utils.h"
#include "tools.h"

int OldRCChar, IsOldRCChar;

/*****************************************************************************
*  CreateDefaultRCFile
******************************************************************************/
void CreateDefaultRCFile(gchar *FileName)
{
  FILE *Handle;

  Handle = fopen(FileName, "w");
  fprintf(Handle, "# This is the giramrc file.\n");
  fprintf(Handle, "# Every thing after a '#' on a line is comment.\n");
  fprintf(Handle, "# Every blank line is ignored.\n\n");

  /* The Library Path */
  fprintf(Handle, "# The variable library_path contain a colon-seperated\n");
  fprintf(Handle, "# of paths where the Pov Loader search for included files\n");
  fprintf(Handle, "(library_path \"/usr/local/povray3/include:/root/povray3/include\")\n");
  AddLibraryPath("/usr/local/povray3/include");
  AddLibraryPath("/root/povray3/include");
  
  /* The ToolBox Style */
  fprintf(Handle, "\n# This is the setting for the main tool box. The ToolBox\n");
  fprintf(Handle, "# is either static or dynamic. The static ToolBox shows all\n");
  fprintf(Handle, "# the tools at once where the dynamic one shows only the \n");
  fprintf(Handle, "# group of tools, other tools can be accessed by popping a\n");
  fprintf(Handle, "# window up with the right mouse bouton click\n");
  fprintf(Handle, "# (toolbox static)\n");
  fprintf(Handle, "(toolbox dynamic)\n\n");
  DynamicToolBox = TRUE;

  fprintf(Handle, "# If the main tool box is dynamic, then you can choose if\n");
  fprintf(Handle, "# the popped window arrange its icon in a vertical, \n");
  fprintf(Handle, "# horizontal or square way\n");
  fprintf(Handle, "# (dynamic-toolbox-shape vertical)\n");
  fprintf(Handle, "# (dynamic-toolbox-shape horizontal)\n");
  fprintf(Handle, "(dynamic-toolbox-shape square)\n\n");
  DynamicToolBoxShape = SQUARE_TOOLBOX;
    
  fclose(Handle);
}

/*****************************************************************************
*  NextRCChar
******************************************************************************/
int NextRCChar(FILE *Handle)
{
  if (IsOldRCChar)
  {
    IsOldRCChar = 0;
    return OldRCChar;
  }
  return getc(Handle);
}

/*****************************************************************************
*  IgnoreRCSpace
******************************************************************************/
void IgnoreRCSpace(FILE *Handle)
{
  int c;

  do
  {
    c = NextRCChar(Handle);
  } while (c==' ' || c=='\t' || c=='' || c=='\r' || c=='\n');
  OldRCChar = c; IsOldRCChar = 1;
}

/*****************************************************************************
*  ParseRCRightParen
******************************************************************************/
int ParseRCRightParen(FILE *Handle)
{
  int c;

  IgnoreRCSpace(Handle);
  c = NextRCChar(Handle);
  if (c != ')') return 1;
  return 0;
}

/*****************************************************************************
*  ParseRCProperty
******************************************************************************/
char *ParseRCProperty(FILE *Handle)
{
  char *LocalString;
  int LocalStringMaxLength, LocalStringLength;
  int c;

  LocalStringMaxLength = 20;
  LocalString = g_malloc(21);
  LocalStringLength = 0;
  c = NextRCChar(Handle);
  while((c>='a' && c<='z') || c=='_' || c=='-')
  {
    LocalString[LocalStringLength++] = (char)c;
    if (LocalStringLength == LocalStringMaxLength)
    {
      LocalStringMaxLength *= 2;
      LocalString = g_realloc(LocalString, LocalStringMaxLength);
    }
    c = NextRCChar(Handle);
  }
  OldRCChar = c; IsOldRCChar = 1;
  LocalString[LocalStringLength] = 0;
  return LocalString;
}

/*****************************************************************************
*  GetNextRCProperty
******************************************************************************/
char *GetNextRCProperty(FILE *Handle)
{
  char *LocalString = NULL;
  int done = FALSE;
  int c;

  while (!done)
  {
    IgnoreRCSpace(Handle);
    c = NextRCChar(Handle);
    switch (c)
    {
      case EOF:
        done = TRUE;
        break;
      case '#': /* comments */
        while (c != '\r' && c != '\n')
        {
          c = NextRCChar(Handle);
        }
        if (c == '\r')
        {
          c = NextRCChar(Handle);
          if (c == '\n') c = NextRCChar(Handle);
        }
        if (c == '\n')
        {
          c = NextRCChar(Handle);
          if (c == '\r') c = NextRCChar(Handle);
        }
        OldRCChar = c; IsOldRCChar = 1;
        break;
      case '(':
        done = TRUE;
        LocalString = ParseRCProperty(Handle);
        break;
      default:
        printf("Bug Bug Bug !!!\n");
    }
  }
  return LocalString;
}

/*****************************************************************************
*  ParseRCString
******************************************************************************/
char *ParseRCString(FILE *Handle)
{
  char *String;
  int StringLength, StringMaxLength;
  int c;

  StringMaxLength = 20;
  String = g_malloc(21);
  StringLength = 0;
  IgnoreRCSpace(Handle);
  c = NextRCChar(Handle);
  if (c != '"') return NULL;
  c = NextRCChar(Handle);
  while (c != '"')
  {
    if (c == EOF) return NULL;
    String[StringLength++] = (char)c;
    if (StringLength == StringMaxLength)
    {
      StringMaxLength *= 2;
      String = g_realloc(String, StringMaxLength);
    }
    c = NextRCChar(Handle);
  }
  String[StringLength] = 0;
  return String;
}

/*****************************************************************************
*  ParseRCint
******************************************************************************/
int ParseRCint(FILE *Handle)
{
  char *LocalString;
  int LocalStringMaxLength, LocalStringLength;
  int c;
  int Val = 0;

  LocalStringMaxLength = 20;
  LocalString = g_malloc(21);
  LocalStringLength = 0;
  IgnoreRCSpace(Handle);
  c = NextRCChar(Handle);
  while((c>='a' && c<='z') || (c>='0' && c<='9') || c=='_' || c=='-')
  {
    LocalString[LocalStringLength++] = (char)c;
    if (LocalStringLength == LocalStringMaxLength)
    {
      LocalStringMaxLength *= 2;
      LocalString = g_realloc(LocalString, LocalStringMaxLength);
    }
    c = NextRCChar(Handle);
  }
  OldRCChar = c; IsOldRCChar = 1;
  LocalString[LocalStringLength] = 0;
  if (!strcmp(LocalString, "static") ||
      !strcmp(LocalString, "off") ||
      !strcmp(LocalString, "0") ||
      !strcmp(LocalString, "vertical") ||
      !strcmp(LocalString, "false") )
  {
    Val = 0;
  } else if (!strcmp(LocalString, "dynamic") ||
             !strcmp(LocalString, "on") ||
             !strcmp(LocalString, "1") ||
             !strcmp(LocalString, "horizontal") ||
             !strcmp(LocalString, "true") )
  {
    Val = 1;
  } else if (!strcmp(LocalString, "square"))
  {
    Val = 2;
  }
  g_free(LocalString);
  return Val;
}

/*****************************************************************************
*  ParseRC
******************************************************************************/
void ParseRC(FILE *Handle)
{
  char *Property = NULL;
  char *LocalString;
  char *TmpString1, *TmpString2;
  int i;
  
  int done = FALSE;
  
  IsOldRCChar = 0;
  do
  {
    Property = GetNextRCProperty(Handle);
    if (!Property)
    {
      done = TRUE;
    }
    else if (!strcmp(Property, "library_path"))
    {
      LocalString = ParseRCString(Handle);
      ParseRCRightParen(Handle);
      TmpString1 = LocalString;
      TmpString2 = g_malloc(strlen(LocalString)+1);
      while (*TmpString1)
      {
        for (i=0 ; TmpString1[i] != ':' && TmpString1[i] != 0 ; i++)
          TmpString2[i] = TmpString1[i];
        TmpString2[i] = 0;
        TmpString1 += i;
        if (*TmpString1 == ':') TmpString1++;
        AddLibraryPath(TmpString2);
      }
      g_free(Property);
    }
    else if (!strcmp(Property, "plugins_path"))
    {
      g_free(Property);
    } else if (!strcmp(Property, "toolbox"))
    {
      DynamicToolBox = ParseRCint(Handle);
      ParseRCRightParen(Handle);
      g_free(Property);
    } else if (!strcmp(Property, "dynamic-toolbox-shape"))
    {
      DynamicToolBoxShape = ParseRCint(Handle);
      ParseRCRightParen(Handle);
      g_free(Property);
    }
  } while (!done);
  fclose(Handle);
}

/*****************************************************************************
*  ParseGiramRCFile
******************************************************************************/
int ParseGiramRCFile()
{
  FILE *Handle;
  DIR *DirHandle;
  gchar *FileName;

  FileName = g_malloc(strlen(g_get_home_dir())+
                      strlen("/.giram-" VERSION "/giramrc")+1);
  strcpy(FileName, g_get_home_dir());
  strcat(FileName, "/.giram-" VERSION );
  DirHandle = opendir(FileName);
  if (DirHandle)
  {
    closedir(DirHandle);
  } else
  {
    if (mkdir(FileName, 0777))
    {
      printf("Giram encountered a problem during the creation of\n");
      printf("the ~/.giram-" VERSION " directory");
      return 1;
    }
  }
  strcat(FileName, "/giramrc");
  Handle = fopen(FileName, "rb");
  if (Handle)
  {
    ParseRC(Handle);
  } else
  {
    CreateDefaultRCFile(FileName);
  }
  return 0;
}

