/* plane.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "plane.h"

/*****************************************************************************
*  InsidePlane
******************************************************************************/
int InsidePlane(ObjectStruct *Plane, double x, double y, double z)
{ /* XXX: clipped_by */
  double a;
  Vector Vect;
  PlaneStruct *PPlane = (PlaneStruct *)Plane;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Plane->Trans)
  {
    MInverseEvaluatePoint(Vect, Plane->Trans, Vect);
  }
  a  = Vect[0]*PPlane->Normal[0];
  a += Vect[1]*PPlane->Normal[1];
  a += Vect[2]*PPlane->Normal[2];
  if (a > PPlane->Distance)
    return Plane->Inverse;
  else
    return !(Plane->Inverse);
}

/*****************************************************************************
*  FindIntersectionPlaneSegment FIXME
******************************************************************************/
int FindIntersectionPlaneSegment(Vector InterPoint, Vector InterNorm,
                                 Vector InPoint, Vector OutPoint,
                                 ObjectStruct *Plane)
{
  return 0;
}

/*****************************************************************************
*  ComputePlaneTriangleMesh
******************************************************************************/
void ComputePlaneTriangleMesh(ObjectStruct *Plane)
{ /* XXX: clipped_by */
  Vector CentralPoint;
  Vector U1, U2;
  double ULength;
  Vector P1, P2, P3, P4;
  Vector N;
  int i,j;
  PlaneStruct *PPlane = (PlaneStruct *)Plane;

  if (Plane->FirstTriangle)
    DestroyObjectTriangleMesh(Plane);
  N[0] = PPlane->Normal[0];
  N[1] = PPlane->Normal[1];
  N[2] = PPlane->Normal[2];
  if (Plane->Inverse)
  {
    N[0] = -N[0];
    N[1] = -N[1];
    N[2] = -N[2];
  }
  CentralPoint[0] = PPlane->Normal[0]*PPlane->Distance;
  CentralPoint[1] = PPlane->Normal[1]*PPlane->Distance;
  CentralPoint[2] = PPlane->Normal[2]*PPlane->Distance;

  if ( (fabs(N[0])>fabs(N[1])) && (fabs(N[0])>fabs(N[2])) )
  { /* The plane normal is upon the X axis */
    U1[0] = -N[1]/N[0];
    U1[1] = 1;
    U1[2] = 0;
  } else if ( (fabs(N[1])>fabs(N[0])) && (fabs(N[1])>fabs(N[2])) )
  { /* The plane normal is upon the Y axis */
    U1[0] = 1;
    U1[1] = -N[0]/N[1];
    U1[2] = 0;
  } else
  { /* The plane normal is upon the Z axis */
    U1[0] = 0;
    U1[1] = 1;
    U1[2] = -N[1]/N[2];
  }
  ULength = VLength(U1);
  U1[0]/=ULength; U1[1]/=ULength; U1[2]/=ULength;
  VCross(U2, N, U1);
  ULength = VLength(U2);
  U2[0]/=ULength; U2[1]/=ULength; U2[2]/=ULength;
  
  if (Plane->Trans)
  {
    MEvaluateVector(N, Plane->Trans, N);
  }
  
  for (i=-PPlane->NbSquare; i<PPlane->NbSquare ;i++)
    for (j=-PPlane->NbSquare; j<PPlane->NbSquare ;j++)
    {
      P1[0] = CentralPoint[0]+PPlane->SquareSize*(i*U1[0]+j*U2[0]);
      P1[1] = CentralPoint[1]+PPlane->SquareSize*(i*U1[1]+j*U2[1]);
      P1[2] = CentralPoint[2]+PPlane->SquareSize*(i*U1[2]+j*U2[2]);

      P2[0] = CentralPoint[0]+PPlane->SquareSize*((i+1)*U1[0]+j*U2[0]);
      P2[1] = CentralPoint[1]+PPlane->SquareSize*((i+1)*U1[1]+j*U2[1]);
      P2[2] = CentralPoint[2]+PPlane->SquareSize*((i+1)*U1[2]+j*U2[2]);

      P3[0] = CentralPoint[0]+PPlane->SquareSize*(i*U1[0]+(j+1)*U2[0]);
      P3[1] = CentralPoint[1]+PPlane->SquareSize*(i*U1[1]+(j+1)*U2[1]);
      P3[2] = CentralPoint[2]+PPlane->SquareSize*(i*U1[2]+(j+1)*U2[2]);

      P4[0] = CentralPoint[0]+PPlane->SquareSize*((i+1)*U1[0]+(j+1)*U2[0]);
      P4[1] = CentralPoint[1]+PPlane->SquareSize*((i+1)*U1[1]+(j+1)*U2[1]);
      P4[2] = CentralPoint[2]+PPlane->SquareSize*((i+1)*U1[2]+(j+1)*U2[2]);

      if (Plane->Trans)
      {
        MEvaluatePoint(P1, Plane->Trans, P1);
        MEvaluatePoint(P2, Plane->Trans, P2);
        MEvaluatePoint(P3, Plane->Trans, P3);
        MEvaluatePoint(P4, Plane->Trans, P4);
      }
      AddTriangleToObjectMesh(Plane, P1, P2, P3, N,N,N);
      AddTriangleToObjectMesh(Plane, P2, P3, P4, N,N,N);
    }
}

/*****************************************************************************
*  TranslatePlane
******************************************************************************/
void TranslatePlane(ObjectStruct *Plane, Vector Vect)
{
  TransformStruct Trans;
  PlaneStruct *PPlane = (PlaneStruct *)Plane;

  if (Plane->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Plane->Trans, &Trans);
  } else
  {
    PPlane->Distance -= VDot(PPlane->Normal, Vect); 
  }
  if (Plane->ClippedBy)
    TranslateObject(Plane->ClippedBy, Vect);
}

/*****************************************************************************
*  RotatePlane
******************************************************************************/
void RotatePlane(ObjectStruct *Plane, Vector Vect)
{
  TransformStruct Trans;
  PlaneStruct *PPlane = (PlaneStruct *)Plane;

  ComputeRotateTrans(&Trans, Vect);
  if (Plane->Trans)
  {
    ComposeTrans(Plane->Trans, &Trans);
  } else
  {
    MEvaluateVector(PPlane->Normal, &Trans, PPlane->Normal); 
  }
  if (Plane->ClippedBy)
    RotateObject(Plane->ClippedBy, Vect);
}

/*****************************************************************************
*  ScalePlane
******************************************************************************/
void ScalePlane(ObjectStruct *Plane, Vector Vect)
{
  TransformStruct Trans;
  double Length;
  PlaneStruct *PPlane = (PlaneStruct *)Plane;

  if (Plane->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Plane->Trans, &Trans);
  } else
  {
    PPlane->Normal[0] /= Vect[0];
    PPlane->Normal[1] /= Vect[1];
    PPlane->Normal[2] /= Vect[2];
    Length = VLength(PPlane->Normal);
    PPlane->Normal[0] /= Length;
    PPlane->Normal[1] /= Length;
    PPlane->Normal[2] /= Length;
    PPlane->Distance  /= Length;
  }
  if (Plane->ClippedBy)
    ScaleObject(Plane->ClippedBy, Vect);
}

/*****************************************************************************
*  IsPlaneIntersection FIXME
******************************************************************************/
int IsPlaneIntersection(ObjectStruct *Plane, Vector Origin, Vector Direction)
{
  Vector Org, Dir;
  int i;

  if (Plane->Trans)
  {
    MInverseEvaluatePoint(Org, Plane->Trans, Origin);
    MInverseEvaluateVector(Dir, Plane->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  
  /* FIXME */
  return 0;
}

/*****************************************************************************
*  ClickInPlaneXY
******************************************************************************/
int ClickInPlaneXY(ObjectStruct *Plane, gdouble x, gdouble y)
{ 
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsPlaneIntersection(Plane, Origin, Direction);
}

/*****************************************************************************
*  ClickInPlaneXZ
******************************************************************************/
int ClickInPlaneXZ(ObjectStruct *Plane, gdouble x, gdouble y)
{
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsPlaneIntersection(Plane, Origin, Direction);
}

/*****************************************************************************
*  ClickInPlaneZY
******************************************************************************/
int ClickInPlaneZY(ObjectStruct *Plane, gdouble x, gdouble y)
{
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsPlaneIntersection(Plane, Origin, Direction);
}
 
/*****************************************************************************
*  ToolNewPlane
******************************************************************************/
void ToolNewPlane(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ObjectStruct *Plane;
  PlaneStruct *PPlane;
  ViewUserDataStruct *UserData;
  FrameViewStruct *TmpView;
  
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  Plane = g_malloc(sizeof(PlaneStruct));
  InitObject(Plane);
  Plane->Type = PLANE_OBJECT;
  PPlane = (PlaneStruct *)Plane;
  PPlane->Distance = 0.0;
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      PPlane->Normal[0] = 0.0;
      PPlane->Normal[1] = 0.0;
      PPlane->Normal[2] = 1.0;
      break;
    case ORTHO_XZ_CAMERA:
      PPlane->Normal[0] = 0.0;
      PPlane->Normal[1] = 1.0;
      PPlane->Normal[2] = 0.0;
      break;
    case ORTHO_ZY_CAMERA:
      PPlane->Normal[0] = 1.0;
      PPlane->Normal[1] = 0.0;
      PPlane->Normal[2] = 0.0;
      break;
  }
  PPlane->NbSquare = 4;
  PPlane->SquareSize = 2.5;
  Plane->Next = UserData->FrameView->FirstObject;
  UserData->FrameView->FirstObject = Plane;
  ComputePlaneTriangleMesh(Plane);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
}

/*****************************************************************************
*  PDBCreatePlane
******************************************************************************/
int PDBCreatePlane(int *Id, Vector *Normal, double *Distance)
{
  ObjectStruct *Plane;
  PlaneStruct *PPlane;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  PPlane = g_malloc(sizeof(PlaneStruct));
  Plane = (ObjectStruct *)PPlane;
  InitObject(Plane);
  Plane->Type = PLANE_OBJECT;
  PPlane->Normal[0] = (*Normal)[0];
  PPlane->Normal[1] = (*Normal)[1];
  PPlane->Normal[2] = (*Normal)[2];
  PPlane->Distance = *Distance;
  PPlane->NbSquare = 4;
  PPlane->SquareSize = 2.5;

  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
                  ;
  Plane->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Plane;
  ComputePlaneTriangleMesh(Plane);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
  gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Plane->Id;
}

