/* povfront.c
 * povfront - A GPLed Front-end for POVray raytracer
 * Copyright (C) 1999 Philippe DAVID <clovis1@club-internet.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/****************************************************************
* Name : 		povfront.c
* Parameters :
* Author : Philippe DAVID          update : 26/4/1999
* Return :
* Comments : 
****************************************************************/

#include "povfront.h"

/****************************************************************
 * Name : 		InitProgramme
 * parameters :       
 * Author : Philippe DAVID          Update : 17/03/1999
 * Return : 
 * Comments : Initialise global variables
 *            This is call after gtk_init call and before creating widget
 ****************************************************************/
void
InitProgramme ()
{
  /*** set options if the default file cannot be read (first use)  */
  strcpy (gImageSize, "+W800 +H600");
  strcpy (gAAmethode, "1");
  strcpy (gPalette, "G");
  gQualityValue = 9;
  gQualityThreshold = 0.3;
  gQualityDepth = 3;
  strcpy (g_command_to_invoque, "povray");
  g_invoque_command_number = 1;
  strcpy (g_entry_save_path, "");
  strcpy (g_entry_pov_version, "3.1");
  g_local_invoque_command_number = g_invoque_command_number;

  g_tooltips_background_color.pixel = 65535;
  g_tooltips_background_color.red = 65500;
  g_tooltips_background_color.green = 65500;
  g_tooltips_background_color.blue = 0;

  g_tooltips_foreground_color.pixel = 65535;
  g_tooltips_foreground_color.red = 0;
  g_tooltips_foreground_color.green = 0;
  g_tooltips_foreground_color.blue = 0;

  g_jobs_list = NULL;

  textdomain ("povfront");

  /* Get the HOME environment variable  */
  strcpy (g_home_directory, getenv ("HOME"));
  sprintf (g_default_config_file, "%s/.povfront", g_home_directory);
  if (access (g_default_config_file, F_OK) == -1)
    {
      /* if (mkdir (g_default_config_file, (mode_t) ACCESSPERMS) == -1) */
      if (mkdir (g_default_config_file, 0777) == -1)
      {
        exit (-1);
      }
      else
      {
        sprintf (g_default_config_file, "%s/.povfront/default.txt", g_home_directory);
        strcpy (g_config_file_to_parse, "");
        return; /* I don't try to read the default file which will be created by the user  */
      }
    }
  else /* The directory ~/.povfront exist  */
    {
      sprintf (g_default_config_file, "%s/.povfront/default.txt", g_home_directory);
      if (access (g_default_config_file, F_OK) == -1)
	{
	  strcpy (g_config_file_to_parse, "");
	  return;
	}
	{
	  strcpy (g_config_file_to_parse, g_default_config_file);
	}
    }
}

/****************************************************************
 * Name : 		delete_event
 * parameters :       
 * Author : Philippe DAVID          Update : 6 /09/ 1998
 * Return : 
 * Comments :
 ****************************************************************/
gint
delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  return (FALSE);
}


/****************************************************************
 * Name : 		destroy
 * parameters : Aucun
 * Author : Philippe DAVID          Update : 6 /09/ 1998
 * Return :
 * Comments :
 ****************************************************************/
void
destroy (GtkWidget *widget, gpointer data)
{
  if (gSaveSettingsExit)
    {
      write_config_file (g_default_config_file);
    }
  gtk_main_quit ();
}

/****************************************************************
 * Name : 		invoque_command
 * parameters : 
 * Author : Philippe DAVID          Update : 13 /02/ 1999
 * Return :
 * Comments :
 ****************************************************************/
void
invoque_command (GtkWidget *widget, gchar *p_command_to_invoque)
{
  if (strcmp (p_command_to_invoque, "1") == 0)
    {
      g_local_invoque_command_number = 1;
    }
  else
    {
      g_local_invoque_command_number = 2;
    }
}

/****************************************************************
 * Name : 		ImageSize
 * parameters : 
 * Author : Philippe DAVID          Update : 17/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
ImageSize (GtkWidget *p_widget, gchar *p_data)
{
  gchar *l_size;

  l_size = gtk_object_get_data (GTK_OBJECT (p_widget), "Size");
  strcpy (gImageSize, p_data);
  gSizeType = atoi (l_size);
}

/****************************************************************
 * Name : 		AAmethode
 * parameters : 
 * Author : Philippe DAVID          Update : 21/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
AAmethode (GtkWidget *widget, gpointer pData)
{
  strcpy (gAAmethode, pData);
}

/****************************************************************
 * Name : 		apply_options
 * parameters : 
 * Author : Philippe DAVID          Update : 10/4/1999
 * Return :
 * Comments :
 ****************************************************************/
void
apply_options (GtkWidget *widget, gpointer pData)
{
  GList *l_tooltips_list;
  gint l_list_length;
  gint i;

  strcpy (g_command_to_invoque, gtk_entry_get_text (GTK_ENTRY (gEntryCommand)));
  strcpy (g_alt_command_to_invoque, gtk_entry_get_text (GTK_ENTRY (gEntryAltCommand)));
  g_invoque_command_number = g_local_invoque_command_number;
  gShowTooltips = gShowTooltipsLocal;
  l_tooltips_list = g_tooltips_list;         /* Use a local pointer for the list  */
  if (l_tooltips_list != NULL)
    {
      l_tooltips_list = g_list_first (l_tooltips_list);
      l_list_length = g_list_length (l_tooltips_list);
      for (i=1; i <= l_list_length; i++)
	{
	  if (gShowTooltips)
	    {
	      gtk_tooltips_enable (GTK_TOOLTIPS (l_tooltips_list->data));
	    }
	  else
	    {
	      gtk_tooltips_disable (GTK_TOOLTIPS (l_tooltips_list->data));
	    }
	  l_tooltips_list = g_list_next (l_tooltips_list);
	}
    }
  gTestOverwrite = gTestOverwriteLocal;
  gSaveSettingsExit = gSaveSettingsExitLocal;
}

/****************************************************************
 * Name : 		Options
 * parameters : 
 * Author : Philippe DAVID          Update : 10/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
Options (GtkWidget *widget, gpointer pData)
{
  GtkWidget *lWindow;
  GtkWidget *lCheckTooltips;
  GtkWidget *lCheckOverwrite;
  GtkWidget *lCheckSaveSettingsExit;
  GtkWidget *lRadioCommand;
  GtkWidget *lRadioAltCommand;
  GtkWidget *lHBox;
  GtkWidget *lVBoxRadio;
  GtkWidget *lVBoxEntry;
  GtkWidget *lButtonApply;
  GtkWidget *lButtonCancel;
  GSList *lGrpRadio;

  /* Dialog window creation  */
  lWindow = gtk_dialog_new ();
  gtk_container_border_width (GTK_CONTAINER (lWindow), 5);
  gtk_window_set_title (GTK_WINDOW (lWindow), gettext ("Povfront : Options setting"));

  /* Show tool tips option  */
  lCheckTooltips = gtk_check_button_new_with_label (gettext ("Show tool tips"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lCheckTooltips, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (lCheckTooltips), gShowTooltips);
  gShowTooltipsLocal = gShowTooltips;
  gtk_signal_connect (GTK_OBJECT (lCheckTooltips), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gShowTooltipsLocal);
  gtk_widget_show (lCheckTooltips);

  /* Overwrite option  */
  lCheckOverwrite = gtk_check_button_new_with_label (gettext ("Test overwrite of output image"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lCheckOverwrite, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (lCheckOverwrite), gTestOverwrite);
  gTestOverwriteLocal = gTestOverwrite;
  gtk_signal_connect (GTK_OBJECT (lCheckOverwrite), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gTestOverwriteLocal);
  gtk_widget_show (lCheckOverwrite);

  /* Save settings as default when exit option  */
  lCheckSaveSettingsExit = gtk_check_button_new_with_label (gettext ("Save settings as default when exit"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lCheckSaveSettingsExit, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (lCheckSaveSettingsExit), gSaveSettingsExit);
  gSaveSettingsExitLocal = gSaveSettingsExit;
  gtk_signal_connect (GTK_OBJECT (lCheckSaveSettingsExit), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gSaveSettingsExitLocal);
  gtk_widget_show (lCheckSaveSettingsExit);

  /* Radio buttons  */
  lHBox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lHBox, FALSE, FALSE, 0);

  lVBoxRadio = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBox), lVBoxRadio, TRUE, TRUE, 0);

  lRadioCommand = gtk_radio_button_new_with_label (NULL, gettext ("Command :"));
  gtk_box_pack_start (GTK_BOX (lVBoxRadio), lRadioCommand, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (lRadioCommand), "pressed", GTK_SIGNAL_FUNC (invoque_command), "1");
  gtk_widget_show (lRadioCommand);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (lRadioCommand));

  lRadioAltCommand = gtk_radio_button_new_with_label (lGrpRadio, gettext ("Alt. command :"));
  gtk_box_pack_start (GTK_BOX (lVBoxRadio), lRadioAltCommand, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (lRadioAltCommand), "pressed", GTK_SIGNAL_FUNC (invoque_command), "2");
  gtk_widget_show (lRadioAltCommand);
  gtk_widget_show (lVBoxRadio);

  if (g_invoque_command_number == 1)
    {
      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (lRadioCommand), TRUE);
    }
  else
    {
      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (lRadioAltCommand), TRUE);
    }

  /* Entry command  */
  lVBoxEntry = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBox), lVBoxEntry, TRUE, TRUE, 0);

  gEntryCommand = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxEntry), gEntryCommand, FALSE, FALSE, 0);
  gtk_entry_set_text (GTK_ENTRY (gEntryCommand), g_command_to_invoque);
  gtk_widget_show (gEntryCommand);

  gEntryAltCommand = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxEntry), gEntryAltCommand, FALSE, FALSE, 0);
  gtk_entry_set_text (GTK_ENTRY (gEntryAltCommand), g_alt_command_to_invoque);
  gtk_widget_show (gEntryAltCommand);

  gtk_widget_show (lVBoxEntry);
  gtk_widget_show (lHBox);

  /* Buttons creation  */
  lButtonApply = gtk_button_new_with_label (gettext ("Apply"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonApply, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (lButtonApply), "clicked", GTK_SIGNAL_FUNC (apply_options), NULL);
  gtk_signal_connect_object( GTK_OBJECT(lButtonApply), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  gtk_widget_show (lButtonApply);

  lButtonCancel = gtk_button_new_with_label (gettext ("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonCancel, FALSE, FALSE, 0);
  gtk_signal_connect_object( GTK_OBJECT(lButtonCancel), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  gtk_widget_show (lButtonCancel);

  gtk_widget_show (lWindow);
}

/****************************************************************
 * Name : 	      AdjustmentValueChanged
 * parameters : 
 * Author : Philippe DAVID          Update : 29/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
AdjustmentValueChanged (GtkAdjustment *pAdjustment, gfloat *pValue)
{
  *pValue = (gfloat) pAdjustment->value;
}

/****************************************************************
 * Name : 		LoadConfigOk
 * parameters : 
 * Author : Philippe DAVID          Update : 10/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
LoadConfigOk (GtkWidget *widget, GtkFileSelection *pFileSelection)
{
  gchar *l_selected_file;
  /* Get the selected file  */
  l_selected_file =  gtk_file_selection_get_filename (GTK_FILE_SELECTION (pFileSelection));
  if (load_config_file (l_selected_file))
    {
      strcpy (g_entry_load_path, l_selected_file);
    }
  gtk_widget_destroy (GTK_WIDGET (pFileSelection));
}

/****************************************************************
 * Name : 	      LoadConfig
 * parameters : callback
 * Author : Philippe DAVID          Update : 10/03/1999
 * Return :
 * Comments : callback for the menu option "load config."
 ****************************************************************/
void
LoadConfig (GtkWidget *widget, gpointer pData)
{
  GtkWidget *lFileSelection;

  lFileSelection = gtk_file_selection_new (gettext ("Load configuration"));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (lFileSelection), g_entry_load_path);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (lFileSelection)->ok_button),
		      "clicked", (GtkSignalFunc) LoadConfigOk, lFileSelection );
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (lFileSelection)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (lFileSelection));
  gtk_widget_show (lFileSelection);
}

/****************************************************************
 * Name : 	      read_line
 * parameters : P_FILE_HANDLE is the file descriptor to use for reading
 * Author : Philippe DAVID          Update : 14/03/1999
 * Return :   Pointer to the first char. of the string read
 * Comments : read the line until the end of the file
 ****************************************************************/
gint
read_line (FILE *p_file_handle, gchar *p_line)
{
  gchar l_char;
  gint i = 0;
  gchar l_line[256];

  l_char = fgetc (p_file_handle);
  while ((l_char != '\n') && (l_char != '\r') && (l_char != EOF))
    {
      l_line[i] = l_char;
      ++i;
      if (i == 255) /* If the line is too long, it is supsect  */
	{
	  povfront_message (gettext ("ERROR: this file is not a configuration file\n"));
	  return 0;
	}
      l_char = fgetc (p_file_handle);
    }
  l_line [i]=0;
  strcpy (p_line, l_line);
  if (l_char == EOF)
    {
      return 0;
    }
  else
    {
      return 1;
    }
}

/****************************************************************
 * Name : 	      load_config_file
 * parameters : P_FILE is the file to load
 * Author : Philippe DAVID          Update : 26/4/1999
 * Return : return 1 if succes and 0 if it fail
 * Comments :
 ****************************************************************/
gint
load_config_file (gchar *p_file)
{
  FILE *l_file_handle;
  gchar l_car;
  gchar l_option [256];
  gfloat l_value;
  gchar l_string [256];
  gchar l_one_line [256];
  gint l_line_number = 0;   /* Used to report error while parsing the file  */
  gchar l_message[256];

  if (p_file == NULL) /* NULL if no config. file in the command line  */
    {
      return (load_config_file (g_default_config_file));
    }
  else
    {
      l_file_handle = fopen (p_file, "r");
      if (l_file_handle == NULL)
	{
	  sprintf (l_message, gettext ("ERROR: load_config_file: cannot open the file %s\n"), p_file);
	  povfront_message (l_message);
	  return 0;
	}
      while (1)
	{
	  l_string [0] = 0;  /* To abort side effect  */
	  if (!read_line (l_file_handle, l_one_line))
	      {
		return 1;
	      }
	  l_car = sscanf (l_one_line, "%s\t%f\t%s\n", l_option, &l_value, l_string);
	  ++l_line_number;
	  if (strcmp ("command", l_option) == 0)
	    {
	      strcpy (g_command_to_invoque, l_string);
	    }
	  else if (strcmp ("altcommand", l_option) == 0)
	    {
	      strcpy (g_alt_command_to_invoque, l_string);
	    }
	  else if (strcmp ("path", l_option) == 0)
	    {
	      strcpy (g_entry_save_path, l_string);
	    }
	  else if (strcmp ("exit", l_option) == 0)
	    {
	      gSaveSettingsExit = l_value;
	    }
	  else if (strcmp ("showtooltips", l_option) == 0)
	    {
	      gShowTooltips = l_value;
	    }
	  else if (strcmp ("testoverwrite", l_option) == 0)
	    {
	      gTestOverwrite = l_value;
	    }
	  else if (strcmp ("povfile", l_option) == 0)
	    {
	      if (!g_plugin)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckPovFile), (gint) l_value);
		  gtk_entry_set_text (GTK_ENTRY (gEntryPovFile), l_string);
		}
	    }
	  else if (strcmp ("sizeX1", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryXSize1), l_string);
	    }
	  else if (strcmp ("sizeY1", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryYSize1), l_string);
	    }
	  else if (strcmp ("sizeX2", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryXSize2), l_string);
	    }
	  else if (strcmp ("sizeY2", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryYSize2), l_string);
	    }
	  else if (strcmp ("size", l_option) == 0)
	    {
	      gSizeType = l_value;
	      if (l_value == 1)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize1), TRUE);
		  strcpy (gImageSize, "+W1280 +H1024");
		}
	      else if (l_value == 2)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize2), TRUE);
		  strcpy (gImageSize, "+W1024 +H768");
		}
	      else if (l_value == 3)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize3), TRUE);
		  strcpy (gImageSize, "+W800 +H600");
		}
	      else if (l_value == 4)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize4), TRUE);
		  strcpy (gImageSize, "+W768 +H576");
		}
	      else if (l_value == 5)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize5), TRUE);
		  strcpy (gImageSize, "+W640 +H480");
		}
	      else if (l_value == 6)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize6), TRUE);
		  strcpy (gImageSize, "+W320 +H240");
		}
	      else if (l_value == 7)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize7), TRUE);
		  strcpy (gImageSize, "+W320 +H200");
		}
	      else if (l_value == 8)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize8), TRUE);
		  strcpy (gImageSize, "+W160 +H100");
		}
	      else if (l_value == 9)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize9), TRUE);
		  strcpy (gImageSize, "");
		}
	      else if (l_value == 10)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutSize10), TRUE);
		  strcpy (gImageSize, "");
		}
	    }
	  else if (strcmp ("substartX", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckSub), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntrySubStartX), l_string);
	    }
	  else if (strcmp ("subendX", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntrySubEndX), l_string);
	    }
	  else if (strcmp ("substartY", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntrySubStartY), l_string);
	    }
	  else if (strcmp ("subendY", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntrySubEndY), l_string);
	    }
	  else if (strcmp ("previewstart", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOutPreview), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryOutPreview1), l_string);
	    }
	  else if (strcmp ("previewend", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryOutPreview2), l_string);
	    }
	  else if (strcmp ("inioutput", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckIniFile), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryIniFile), l_string);
	    }
	  else if (strcmp ("imageoutput", l_option) == 0)
	    {
	      if (!g_image_plugin)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOutFile), (gint) l_value);
		  gtk_entry_set_text (GTK_ENTRY (gEntryOutFile), l_string);
		}
	    }
	  else if (strcmp ("format", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry), l_string);
	    }
	  else if (strcmp ("abort", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut1), (gint) l_value);
	    }
	  else if (strcmp ("continue", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut2), (gint) l_value);
	    }
	  else if (strcmp ("pause", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut3), (gint) l_value);
	    }
	  else if (strcmp ("verbose", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut4), (gint) l_value);
	    }
	  else if (strcmp ("drawvista", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut5), (gint) l_value);
	    }
	  else if (strcmp ("display", l_option) == 0)
	    {
	      strcpy (gPalette, l_string);
	      if (strcmp ("G", l_string) == 0)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutColor1), TRUE);
		}
	      else if (strcmp ("H", l_string) == 0)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutColor2), TRUE);
		}
	      else if (strcmp ("T", l_string) == 0)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioOutColor3), TRUE);
		}
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut6), (gint) l_value);
	    }
	  else if (strcmp ("alphaout", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut7), (gint) l_value);
	    }
	  else if (strcmp ("antialiasing", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAntialiasing), (gint) l_value);
	    }
	  else if (strcmp ("threshold", l_option) == 0)
	    {
	      gtk_adjustment_set_value (GTK_ADJUSTMENT (g_adjust_Threshold), l_value);
	    }
	  else if (strcmp ("jitter", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheck_jitter), (gint) l_value);
	    }
	  else if (strcmp ("jittervalue", l_option) == 0)
	    {
	      gtk_adjustment_set_value (GTK_ADJUSTMENT (g_adjust_Jitter), l_value);
	    }
	  else if (strcmp ("methode", l_option) == 0)
	    {
	      strcpy (gAAmethode, l_string);
	      if (strcmp ("1", l_string) == 0)
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioAA1), TRUE);
		}
	      else
		{
		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (g_radioAA2), TRUE);
		}
	    }
	  else if (strcmp ("aadepth", l_option) == 0)
	    {
	      gtk_adjustment_set_value (GTK_ADJUSTMENT (g_adjust_Depth), l_value);
	    }
	  else if (strcmp ("quality", l_option) == 0)
	    {
	      gtk_adjustment_set_value (GTK_ADJUSTMENT (g_adjust_Quality), l_value);
	    }
	  else if (strcmp ("radiosity", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality1), (gint) l_value);
	    }
	  else if (strcmp ("boundingbox", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality2), (gint) l_value);
	    }
	  else if (strcmp ("lightbuffer", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality3), (gint) l_value);
	    }
	  else if (strcmp ("vistabuffer", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality4), (gint) l_value);
	    }
	  else if (strcmp ("removebounding", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality5), (gint) l_value);
	    }
	  else if (strcmp ("splitunion", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality6), (gint) l_value);
	    }
	  else if (strcmp ("fieldrender", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAnimField), (gint) l_value);
	    }
	  else if (strcmp ("cyclic", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAnimCyclic), (gint) l_value);
	    }
	  else if (strcmp ("oddfield", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAnimOdd), (gint) l_value);
	    }
	  else if (strcmp ("prescenecommand", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryPreScene), l_string);
	    }
	  else if (strcmp ("preframecommand", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryPreFrame), l_string);
	    }
	  else if (strcmp ("postscenecommand", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryPostScene), l_string);
	    }
	  else if (strcmp ("postframecommand", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryPostFrame), l_string);
	    }
	  else if (strcmp ("userabortcommand", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryUserAbort), l_string);
	    }
	  else if (strcmp ("fatalerrorcommand", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryFatalError), l_string);
	    }
	  else if (strcmp ("library1", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary1), l_string);
	    }
	  else if (strcmp ("library2", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary2), l_string);
	    }
	  else if (strcmp ("library3", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary3), l_string);
	    }
	  else if (strcmp ("library4", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary4), l_string);
	    }
	  else if (strcmp ("library5", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary5), l_string);
	    }
	  else if (strcmp ("library6", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary6), l_string);
	    }
	  else if (strcmp ("library7", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary7), l_string);
	    }
	  else if (strcmp ("library8", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary8), l_string);
	    }
	  else if (strcmp ("library9", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary9), l_string);
	    }
	  else if (strcmp ("library10", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibrary10), l_string);
	    }
	  else if (strcmp ("version", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryLibraryVersion), l_string);
	    }
	  else if (strcmp ("debugstream", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckDebugStream), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryDebugStream), l_string);
	    }
	  else if (strcmp ("fatalstream", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckFatalStream), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryFatalStream), l_string);
	    }
	  else if (strcmp ("renderstream", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckRenderStream), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryRenderStream), l_string);
	    }
	  else if (strcmp ("statstream", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckStatStream), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryStatStream), l_string);
	    }
	  else if (strcmp ("warningstream", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckWarningStream), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryWarningStream), l_string);
	    }
	  else if (strcmp ("allstream", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAllStream), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryAllStream), l_string);
	    }
	  else if (strcmp ("extanimloop", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimClock), l_string);
	    }
	  else if (strcmp ("initframe", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimFrameIni), l_string);
	    }
	  else if (strcmp ("finalframe", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimFrameFin), l_string);
	    }
	  else if (strcmp ("initclock", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimClockIni), l_string);
	    }
	  else if (strcmp ("finalclock", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimClockFin), l_string);
	    }
	  else if (strcmp ("initialsub", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimSubsetIni), l_string);
	    }
	  else if (strcmp ("finalsub", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryAnimSubsetFin), l_string);
	    }
	  else if (strcmp ("outputfile", l_option) == 0)
	    {
	      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckHistoFile), (gint) l_value);
	      gtk_entry_set_text (GTK_ENTRY (gEntryHistoFile), l_string);
	    }
	  else if (strcmp ("histosizeX", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryHistoXSize), l_string);
	    }
	  else if (strcmp ("histosizeY", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (gEntryHistoYSize), l_string);
	    }
	  else if (strcmp ("histoformat", l_option) == 0)
	    {
	      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry), l_string);
	    }
	  else
	    {
	      sprintf (l_message, "ERROR : line %d : %s (unrecognized word)\n", l_line_number, l_string);
	      povfront_message (l_message);
	      return 0;
	    }
	}
      fclose (l_file_handle);
    }
  return 1;
}

/****************************************************************
 * Name : 		write_config_file
 * parameters : 
 * Author : Philippe DAVID          Update : 26/04/1999
 * Return :
 * Comments :
 ****************************************************************/
gint
write_config_file (gchar *p_selected_file)
{
  FILE *l_conf_file_handler;
  gchar l_message[256];

  l_conf_file_handler = fopen (p_selected_file, "w");
  if (l_conf_file_handler == NULL)
    {
      sprintf (l_message, gettext ("ERROR: write_config: file %s cannot be opened\n"), p_selected_file);
      povfront_message (l_message);
      return 0;
    }

  fprintf (l_conf_file_handler, "command\t0\t%s\n", g_command_to_invoque);
  fprintf (l_conf_file_handler, "altcommand\t0\t%s\n", g_alt_command_to_invoque);
  fprintf (l_conf_file_handler, "path\t0\t%s\n", g_entry_save_path);
  fprintf (l_conf_file_handler, "exit\t%d\n", gSaveSettingsExit);
  fprintf (l_conf_file_handler, "showtooltips\t%d\n", gShowTooltips);
  fprintf (l_conf_file_handler, "testoverwrite\t%d\n", gTestOverwrite);
  /* Input POV File  */
  if (g_plugin)
    {
      fprintf (l_conf_file_handler, "povfile\t1\t%s\n", g_pov_file);
    }
  else
    {
      fprintf (l_conf_file_handler, "povfile\t%d\t%s\n", gInputPovFile,
	       gtk_entry_get_text (GTK_ENTRY (gEntryPovFile)));
    }
  /* Image size  */
  fprintf (l_conf_file_handler, "sizeX1\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntryXSize1)));
  fprintf (l_conf_file_handler, "sizeY1\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntryYSize1)));
  fprintf (l_conf_file_handler, "sizeX2\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntryXSize2)));
  fprintf (l_conf_file_handler, "sizeY2\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntryYSize2)));
  fprintf (l_conf_file_handler, "size\t%d\t\n", gSizeType);
  /* Rendering Window flags  */
  fprintf (l_conf_file_handler, "substartX\t%d\t%s\n", gSubRender, gtk_entry_get_text (GTK_ENTRY(gEntrySubStartX)));
  fprintf (l_conf_file_handler, "subendX\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntrySubEndX)));
  fprintf (l_conf_file_handler, "substartY\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntrySubStartY)));
  fprintf (l_conf_file_handler, "subendY\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntrySubEndY)));
  /* Preview flags  */
  fprintf (l_conf_file_handler, "previewstart\t%d\t%s\n", gPreview, gtk_entry_get_text (GTK_ENTRY(gEntryOutPreview1)));
  fprintf (l_conf_file_handler, "previewend\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY(gEntryOutPreview2)));
  /* Output Ini file  */
  fprintf (l_conf_file_handler, "inioutput\t%d\t%s\n", gOutputIniFile, gtk_entry_get_text (GTK_ENTRY (gEntryIniFile)));
   /* Output image file  */
  if (!g_image_plugin)
    {
      fprintf (l_conf_file_handler, "imageoutput\t%d\t%s\n", gOutputImage, gtk_entry_get_text (GTK_ENTRY (gEntryOutFile)));
    }
  /* Image format  */
  fprintf (l_conf_file_handler, "format\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry)));
  /* Test Abort flag  */
  fprintf (l_conf_file_handler, "abort\t%d\n", gTestAbort);
  /* Continue flag  */
  fprintf (l_conf_file_handler, "continue\t%d\n", gContinue);
   /* Pause flag  */
  fprintf (l_conf_file_handler, "pause\t%d\n", gPause);
  /* Verbose flag  */
  fprintf (l_conf_file_handler, "verbose\t%d\n", gVerbose);
  /* Draw vista flag  */
  fprintf (l_conf_file_handler, "drawvista\t%d\n", gDrawVista);
  /* Display Image flag  */
  fprintf (l_conf_file_handler, "display\t%d\t%s\n", gDisplayImage, gPalette);
  /* Output Alpha flag  */
  fprintf (l_conf_file_handler, "alphaout\t%d\n", gAlphaOut);
  /* Antialiasing flag with threshold  */
  fprintf (l_conf_file_handler, "antialiasing\t%d\n", gAntialiasing);
  fprintf (l_conf_file_handler, "threshold\t%f\n", gQualityThreshold);
  /* Jitter  */
  fprintf (l_conf_file_handler, "jitter\t%d\n", g_jitter);
  fprintf (l_conf_file_handler, "jittervalue\t%f\n", g_jitter_value);
  /* Antialiasing methode and depth  */
  fprintf (l_conf_file_handler, "methode\t0\t%s\n", gAAmethode);
  fprintf (l_conf_file_handler, "aadepth\t%f\n", gQualityDepth);
  /* Quality level  */
  fprintf (l_conf_file_handler, "quality\t%f\n", gQualityValue);
  /* Radiosity flag  */
  fprintf (l_conf_file_handler, "radiosity\t%d\n", gRadiosity);
  /* Bounding flag  */
  fprintf (l_conf_file_handler, "boundingbox\t%d\n", gBoundingBox);
  /* Light Buffer flag  */
  fprintf (l_conf_file_handler, "lightbuffer\t%d\n", gLightBuffer);
  /* Vista Buffer flag  */
  fprintf (l_conf_file_handler, "vistabuffer\t%d\n", gVistaBuffer);
  /* Remove bounds flag  */
  fprintf (l_conf_file_handler, "removebounding\t%d\n", gRemoveBounding);
  /* Split Union flag  */
  fprintf (l_conf_file_handler, "splitunion\t%d\n", gSplit);
  /* Field render flag  */
  fprintf (l_conf_file_handler, "fieldrender\t%d\n", g_field_render);
  /* Cyclic animation flag  */
  fprintf (l_conf_file_handler, "cyclic\t%d\n", g_cyclic);
  /* Odd fieldflag  */
  fprintf (l_conf_file_handler, "oddfield\t%d\n", g_odd_field);
  /* Pre scene command  */
  fprintf (l_conf_file_handler, "prescenecommand\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryPreScene)));
  /* Pre frame command  */
  fprintf (l_conf_file_handler, "preframecommand\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryPreFrame)));
  /* Post scene command  */
  fprintf (l_conf_file_handler, "postscenecommand\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryPostScene)));
  /* Post frame command  */
  fprintf (l_conf_file_handler, "postframecommand\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryPostFrame)));
  /* User abort command  */
  fprintf (l_conf_file_handler, "userabortcommand\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryUserAbort)));
  /* Fatal Error command  */
  fprintf (l_conf_file_handler, "fatalerrorcommand\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryFatalError)));
  /* Library path option  */
  fprintf (l_conf_file_handler, "library1\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary1)));
  fprintf (l_conf_file_handler, "library2\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary2)));
  fprintf (l_conf_file_handler, "library3\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary3)));
  fprintf (l_conf_file_handler, "library4\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary4)));
  fprintf (l_conf_file_handler, "library5\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary5)));
  fprintf (l_conf_file_handler, "library6\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary6)));
  fprintf (l_conf_file_handler, "library7\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary7)));
  fprintf (l_conf_file_handler, "library8\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary8)));
  fprintf (l_conf_file_handler, "library9\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary9)));
  fprintf (l_conf_file_handler, "library10\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibrary10)));

  /* POV file version  */
  fprintf (l_conf_file_handler, "version\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryLibraryVersion)));

  /* Stream flag  */
  fprintf (l_conf_file_handler, "debugstream\t%d\t%s\n", gDebugStream
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryDebugStream)));
  fprintf (l_conf_file_handler, "fatalstream\t%d\t%s\n", gFatalStream
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryFatalStream)));
  fprintf (l_conf_file_handler, "renderstream\t%d\t%s\n", gRenderStream
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryRenderStream)));
  fprintf (l_conf_file_handler, "statstream\t%d\t%s\n", gStatStream
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryStatStream)));
  fprintf (l_conf_file_handler, "warningstream\t%d\t%s\n", gWarningStream
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryWarningStream)));
  fprintf (l_conf_file_handler, "allstream\t%d\t%s\n", gAllStream
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryAllStream)));

  /* External animation loop  */
  fprintf (l_conf_file_handler, "extanimloop\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimClock)));
  /* Initial frame for internal animation  */
  fprintf (l_conf_file_handler, "initframe\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimFrameIni)));
  /* Final frame for internal animation  */
  fprintf (l_conf_file_handler, "finalframe\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimFrameFin)));
  /* Initial clock value for internal animation  */
  fprintf (l_conf_file_handler, "initclock\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimClockIni)));
  /* final clock value for internal animation  */
  fprintf (l_conf_file_handler, "finalclock\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimClockFin)));
  /*  initial subset of frame for animation  */
  fprintf (l_conf_file_handler, "initialsub\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimSubsetIni)));
  /* Final subset of frame for animation  */
  fprintf (l_conf_file_handler, "finalsub\t0\t%s\n",  gtk_entry_get_text (GTK_ENTRY (gEntryAnimSubsetFin)));
  /* Histogram option  */
  fprintf (l_conf_file_handler, "outputfile\t%d\t%s\n", gOutputHistoFile
	   ,gtk_entry_get_text (GTK_ENTRY (gEntryHistoFile)));
  fprintf (l_conf_file_handler, "histoformat\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry)));
  fprintf (l_conf_file_handler, "histosizeX\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryHistoXSize)));
  fprintf (l_conf_file_handler, "histosizeY\t0\t%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryHistoYSize)));

  fclose (l_conf_file_handler);
  return 1;
}

/****************************************************************
 * Name : 		write_default_config
 * parameters : 
 * Author : Philippe DAVID          Update : 28/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
write_default_config (GtkWidget *widget, gpointer *p_data)
{
  write_config_file (g_default_config_file);
}

/****************************************************************
 * Name : 		SaveConfigOk
 * parameters : 
 * Author : Philippe DAVID          Update : 28/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
SaveConfigOk (GtkWidget *widget, GtkFileSelection *pFileSelection)
{
  gchar *l_selected_file;
  /* Get the selected file  */
  l_selected_file =  gtk_file_selection_get_filename (GTK_FILE_SELECTION (pFileSelection));
  if (write_config_file (l_selected_file))
    {
      strcpy (g_entry_save_path, l_selected_file);
    }
  gtk_widget_destroy (GTK_WIDGET (pFileSelection));
}

/****************************************************************
 * Name : 		Save config
 * parameters : 
 * Author : Philippe DAVID          Update : 28/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
SaveConfig (GtkWidget *pWidget, gpointer pData)
{
  GtkWidget *lFileSelection;

  lFileSelection = gtk_file_selection_new (gettext ("Save configuration"));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (lFileSelection), g_entry_save_path);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (lFileSelection)->ok_button),
		      "clicked", (GtkSignalFunc) SaveConfigOk, lFileSelection );
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (lFileSelection)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (lFileSelection));
  gtk_widget_show (lFileSelection);
}

/****************************************************************
 * Name : 		APropos
 * parameters : 
 * Author : Philippe DAVID          Update : 29/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
APropos (GtkWidget *widget, gpointer pData)
{
  GtkWidget *lWindow;
  GtkWidget *lButtonClose;
  GtkWidget *lLabel;
  GtkWidget *lSeparator;

  lWindow = gtk_dialog_new ();
  gtk_container_border_width (GTK_CONTAINER (lWindow), 5);
  gtk_window_set_title (GTK_WINDOW (lWindow), gettext ("POVFRONT: about ..."));

  lLabel = gtk_label_new ("POVFRONT version 0.9");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lLabel = gtk_label_new (gettext ("A front end for POVRAY"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lLabel = gtk_label_new (gettext ("This program is free software (GPL)"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lSeparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lSeparator, TRUE, TRUE, 0);
  gtk_widget_show (lSeparator);

  lLabel = gtk_label_new (gettext ("Author: Philippe P.E. DAVID"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lLabel = gtk_label_new ("Clovis1@club-internet.fr");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lLabel = gtk_label_new (gettext ("If you use it, tell it to me by E-Mail"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lLabel = gtk_label_new (gettext ("and make me some comments about povfront"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lButtonClose = gtk_button_new_with_label ("Close");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonClose, FALSE, FALSE, 10);
  gtk_signal_connect_object( GTK_OBJECT(lButtonClose), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  gtk_widget_show (lButtonClose);
  gtk_widget_show (lWindow);
}

/****************************************************************
 * Name : 		povfront_message
 * parameters : 
 * Author : Philippe DAVID          Update : 17/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
povfront_message (gchar *pData)
{
  GtkWidget *lWindow;
  GtkWidget *lButtonClose;
  GtkWidget *lLabel;

  lWindow = gtk_dialog_new ();
  gtk_container_border_width (GTK_CONTAINER (lWindow), 5);
  gtk_window_set_title (GTK_WINDOW (lWindow), "POVFRONT");

  lLabel = gtk_label_new (pData);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lButtonClose = gtk_button_new_with_label (gettext ("Close"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonClose, FALSE, FALSE, 10);
  gtk_signal_connect_object( GTK_OBJECT(lButtonClose), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  gtk_widget_show (lButtonClose);
  gtk_widget_show (lWindow);
}

/****************************************************************
 * Name : 		povfront_message_overwrite
 * parameters : 
 * Author : Philippe DAVID          Update : 11/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
povfront_message_overwrite (gchar *pData)
{
  GtkWidget *lWindow;
  GtkWidget *lButtonOverwrite;
  GtkWidget *lButtonCancel;
  GtkWidget *lLabel;

  lWindow = gtk_dialog_new ();
  gtk_container_border_width (GTK_CONTAINER (lWindow), 5);
  gtk_window_set_title (GTK_WINDOW (lWindow), "POVFRONT");

  lLabel = gtk_label_new (pData);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lLabel = gtk_label_new ("Overwrite this file?");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lButtonOverwrite = gtk_button_new_with_label (gettext ("Overwrite"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonOverwrite, FALSE, FALSE, 10);
  gtk_signal_connect (GTK_OBJECT(lButtonOverwrite), "clicked",
		      GTK_SIGNAL_FUNC (Render2), NULL);
  gtk_signal_connect_object (GTK_OBJECT(lButtonOverwrite), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  gtk_widget_show (lButtonOverwrite);

  lButtonCancel = gtk_button_new_with_label (gettext ("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonCancel, FALSE, FALSE, 10);
  gtk_signal_connect_object (GTK_OBJECT(lButtonCancel), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  gtk_widget_show (lButtonCancel);

  gtk_widget_show (lWindow);
}

/****************************************************************
 * Name : 		Render
 * parameters :
 * Author : Philippe DAVID          Update : 11/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
Render (GtkWidget *widget, gpointer data)
{
  gchar *l_image_file_name;
  FILE  *l_output_image_handle;

  /* Test Overwrite of output images if necessary  */
  if (gOutputImage)
    {
      if (!g_image_plugin)
	{
	  l_image_file_name = gtk_entry_get_text (GTK_ENTRY (gEntryOutFile));
	}
      else
	{
	  l_image_file_name = g_image_file_to_compute;
	}
      l_output_image_handle = fopen (l_image_file_name, "r");
      if (l_output_image_handle != NULL)
	{
	  fclose (l_output_image_handle); /* I don't want to read but to see if the file exist  */
	  if (gTestOverwrite)
	    {
	      /* Then the image already exist  */
	      if (!gContinue)
		{
		  povfront_message_overwrite (l_image_file_name);
		}
	      else
		{
		  Render2 (NULL, NULL);
		}
	    }
	  else
	    {
	      Render2 (NULL, NULL);
	    }
	}
      else
	{
	  if (gContinue)
	    {
	      povfront_message (gettext ("You specify the CONTINUE flag, but the file does not exist; rendering in any way"));
	    }
	  Render2 (NULL, NULL);
	}
    }
  else
    {
      Render2 (NULL, NULL);
    }
}

/****************************************************************
 * Name : 		Render2
 * parameters :
 * Author : Philippe DAVID          Update : 26/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
Render2 (GtkWidget *widget, gpointer data)
{
  gchar  lLigneDeCommande [1000];
  gchar *lCurseur;
  gchar *lTampon;
  gchar *lText;
  gchar *l_arg[ARG_MAX];
  gint   l_index_argument = 0;
  pid_t  l_pid_pov;
  job   *l_the_last_job;
  gchar *l_time_char;
  time_t l_job_time;
  gchar  l_ini_file_name[512];
  FILE  *l_ini_file_handle;

  /* Preparing the ini file contening the command to invoque (script page  */
  sprintf (l_ini_file_name, "%s/.povfront/inifile", g_home_directory);
  l_ini_file_handle = fopen (l_ini_file_name, "w");
  
  /* Error checking on fopen for inifile */
  if (l_ini_file_handle == NULL)
  {
    perror("Error opening inifile");
    exit(-1);
  }
  
  fprintf (l_ini_file_handle, "Pre_Scene_Command=%s\n \
Pre_Frame_Command=%s\n \
Post_Scene_Command=%s\n \
Post_Frame_Command=%s\n \
User_Abort_Command=%s\n \
Fatal_Error_Command=%s\n", gtk_entry_get_text (GTK_ENTRY (gEntryPreScene))
	  , gtk_entry_get_text (GTK_ENTRY (gEntryPreFrame))
	  , gtk_entry_get_text (GTK_ENTRY (gEntryPostScene))
	  , gtk_entry_get_text (GTK_ENTRY (gEntryPostFrame))
	  , gtk_entry_get_text (GTK_ENTRY (gEntryUserAbort))
	  , gtk_entry_get_text (GTK_ENTRY (gEntryFatalError)));
  fclose (l_ini_file_handle);

  /* Choice of the right command to cinvoque  */
  if (g_invoque_command_number == 1)
    {
      sprintf (lLigneDeCommande, "%s ", g_command_to_invoque);
      l_arg [l_index_argument++] = g_command_to_invoque;
    }
  else
    {
      sprintf (lLigneDeCommande, "%s ", g_alt_command_to_invoque);
      l_arg [l_index_argument++] = g_alt_command_to_invoque;
    }
  lCurseur = lLigneDeCommande + strlen (lLigneDeCommande);

  /* Add the ini file to parse on the command line  */
  sprintf (lCurseur, "%s ", l_ini_file_name);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;
  
  /* Input POV File  */
  if (gInputPovFile)
    {
      sprintf (lCurseur, "+I%s ", gtk_entry_get_text (GTK_ENTRY (gEntryPovFile)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  else
    {
      if (g_plugin)
	{
	  sprintf (lCurseur, "+I%s ", g_pov_file);
	  l_arg [l_index_argument++] = lCurseur;
 	  lCurseur += strlen (lCurseur) + 1;
	}
    }

  /* Image size  */
  if (gSizeType == 9)
    {
      sprintf (lCurseur, "+W%s", gtk_entry_get_text (GTK_ENTRY(gEntryXSize1)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
      sprintf (lCurseur, "+H%s" ,gtk_entry_get_text (GTK_ENTRY(gEntryYSize1)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  else
    {
      if (gSizeType == 10)
	{
	  sprintf (lCurseur, "+W%s", gtk_entry_get_text (GTK_ENTRY(gEntryXSize2)));
	  l_arg [l_index_argument++] = lCurseur;
	  lCurseur += strlen (lCurseur) + 1;
	  sprintf (lCurseur, "+H%s" ,gtk_entry_get_text (GTK_ENTRY(gEntryYSize2)));
	  l_arg [l_index_argument++] = lCurseur;
	  lCurseur += strlen (lCurseur) + 1;
	}
      else
	{
	  sprintf (lCurseur, "%s ", gImageSize);
	  l_arg [l_index_argument++] = lCurseur;
	  lCurseur += strlen (lCurseur) + 1;
	}
    }
  
  /* Rendering Window flags  */
  if (gSubRender)
    {
      sprintf (lCurseur, "+SC%s", gtk_entry_get_text (GTK_ENTRY(gEntrySubStartX)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
      sprintf (lCurseur, "+EC%s" , gtk_entry_get_text (GTK_ENTRY(gEntrySubEndX)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
      sprintf (lCurseur, "+SR%s" , gtk_entry_get_text (GTK_ENTRY(gEntrySubStartY)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
      sprintf (lCurseur, "+ER%s" , gtk_entry_get_text (GTK_ENTRY(gEntrySubEndY)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Preview flags  */
  if (gPreview)
    {
      sprintf (lCurseur, "+SP%s", gtk_entry_get_text (GTK_ENTRY(gEntryOutPreview1)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
      sprintf (lCurseur, "+EP%s", gtk_entry_get_text (GTK_ENTRY(gEntryOutPreview2)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Output Ini file  */
  if (gOutputIniFile)
    {
      sprintf (lCurseur, "+GI%s ",  gtk_entry_get_text (GTK_ENTRY (gEntryIniFile)));
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Output image file  */
  if (gOutputImage)
    {
      if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry)), "CSV") == 0)
	{
	  sprintf (lCurseur, "+FC");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry)), "PNG") == 0)
	{
	  sprintf (lCurseur, "+FN");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry)), "PPM") == 0)
	{
	  sprintf (lCurseur, "+FP");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry)), "BMP") == 0)
	{
	  sprintf (lCurseur, "+FS");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry)), "TGA") == 0)
	{
	  sprintf (lCurseur, "+FT");
	}
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
      if (!g_image_plugin)
	{
	  sprintf (lCurseur, "+O%s ",  gtk_entry_get_text (GTK_ENTRY (gEntryOutFile)));
	}
      else
	{
	  sprintf (lCurseur, "+O%s ",  g_image_file_to_compute);
	}
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Test Abort flag  */
  if (gTestAbort)
    {
      sprintf (lCurseur, "+X ");
    }
  else
    {
      sprintf (lCurseur, "-X ");
    }
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Continue flag  */
  if (gContinue) sprintf (lCurseur, "+C ");
  else sprintf (lCurseur, "-C ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Pause flag  */
  if (gPause) sprintf (lCurseur, "+P ");
  else sprintf (lCurseur, "-P ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Verbose flag  */
  if (gVerbose) sprintf (lCurseur, "+V ");
  else sprintf (lCurseur, "-V ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Draw vista flag  */
  if (gDrawVista) sprintf (lCurseur, "+UD ");
  else sprintf (lCurseur, "-UD ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Display Image flag  */
  if (gDisplayImage) sprintf (lCurseur, "+D0%s ", gPalette);
  else sprintf (lCurseur, "-D0%s ", gPalette);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Output Alpha flag  */
  if (gAlphaOut) sprintf (lCurseur, "+UA ");
  else sprintf (lCurseur, "-UA ");
  lCurseur += strlen (lCurseur) + 1;

  /* Antialiasing flag with threshold  */
  if (gAntialiasing) sprintf (lCurseur, "+A%f ", gQualityThreshold);
  else sprintf (lCurseur, "-A%f ", gQualityThreshold);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Jitter  */
  if (g_jitter) sprintf (lCurseur, "+J%f ", g_jitter_value);
  else sprintf (lCurseur, "-J%f ",  g_jitter_value);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Antialiasing methode  */
  sprintf (lCurseur, "+AM%s", gAAmethode);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;
  sprintf (lCurseur, "+R%d", (gint) gQualityDepth);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Quality level  */
  sprintf (lCurseur, "+Q%d ", (gint) gQualityValue);
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Radiosity flag  */
  if (gRadiosity) sprintf (lCurseur, "+QR ");
  else sprintf (lCurseur, "-QR ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Bounding flag  */
  if (gBoundingBox) sprintf (lCurseur, "+MB ");
  else sprintf (lCurseur, "-MB ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Light Buffer flag  */
  if (gLightBuffer) sprintf (lCurseur, "+UL ");
  else sprintf (lCurseur, "-UL ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* vista Buffer flag  */
  if (gVistaBuffer) sprintf (lCurseur, "+UV ");
  else sprintf (lCurseur, "-UV ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Remove bounds flag  */
  if (gRemoveBounding) sprintf (lCurseur, "+UR ");
  else sprintf (lCurseur, "-UR ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Split Union flag  */
  if (gRemoveBounding) sprintf (lCurseur, "+SU ");
  else sprintf (lCurseur, "-SU ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* field render flag  */
  if (g_field_render) sprintf (lCurseur, "+UF ");
  else sprintf (lCurseur, "-UF ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* odd field flag  */
  if (g_odd_field) sprintf (lCurseur, "+UO ");
  else sprintf (lCurseur, "-UO ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* Library path option  */
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary1));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary2));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary3));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary4));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary5));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary6));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary7));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary8));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary9));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
     lCurseur += strlen (lCurseur) + 1;
    }
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibrary10));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+L%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
     lCurseur += strlen (lCurseur) + 1;
    }

  /* POV file version  */
  lTampon = gtk_entry_get_text (GTK_ENTRY (gEntryLibraryVersion));
  if (strlen (lTampon) != 0)
    {
      sprintf (lCurseur, "+MV%s ", lTampon);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Stream flag  */
  if (gDebugStream) sprintf (lCurseur, "-GD%s ", gtk_entry_get_text (GTK_ENTRY (gEntryDebugStream)));
  else sprintf (lCurseur, "-GD ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  if (gFatalStream) sprintf (lCurseur, "-GF%s ", gtk_entry_get_text (GTK_ENTRY (gEntryFatalStream)));
  else sprintf (lCurseur, "-GF ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  if (gRenderStream) sprintf (lCurseur, "-GR%s ", gtk_entry_get_text (GTK_ENTRY (gEntryRenderStream)));
  else sprintf (lCurseur, "-GR ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  if (gStatStream) sprintf (lCurseur, "-GS%s ", gtk_entry_get_text (GTK_ENTRY (gEntryStatStream)));
  else sprintf (lCurseur, "-GS ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  if (gWarningStream) sprintf (lCurseur, "-GW%s ", gtk_entry_get_text (GTK_ENTRY (gEntryWarningStream)));
  else sprintf (lCurseur, "-GW ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  if (gAllStream) sprintf (lCurseur, "-GA%s ", gtk_entry_get_text (GTK_ENTRY (gEntryAllStream)));
  else sprintf (lCurseur, "-GA ");
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur) + 1;

  /* External animation loop  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimClock))) != NULL)
    {
      sprintf (lCurseur, "+K%s ",  lText);
      l_arg [l_index_argument++] = lCurseur;
     lCurseur += strlen (lCurseur) + 1;
    }

  /* Initial frame for internal animation  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimFrameIni))) != NULL)
    {
      sprintf (lCurseur, "+KFI%s ",  lText);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Final frame for internal animation  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimFrameFin))) != NULL)
    {
      sprintf (lCurseur, "+KFF%s ",  lText);
      l_arg [l_index_argument++] = lCurseur;
     lCurseur += strlen (lCurseur) + 1;
    }

  /* Initial clock value for internal animation  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimClockIni))) != NULL)
    {
      sprintf (lCurseur, "+KI%s ",  lText);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* final clock value for internal animation  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimClockFin))) != NULL)
    {
      sprintf (lCurseur, "+KF%s ",  lText);
      l_arg [l_index_argument++] = lCurseur;
     lCurseur += strlen (lCurseur) + 1;
    }

  /*  initial subset of frame for animation  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimSubsetIni))) != NULL)
    {
      sprintf (lCurseur, "+SF%s",  lText);
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur) + 1;
    }

  /* Final subset of frame for animation  */
  if ((lText = gtk_entry_get_text (GTK_ENTRY (gEntryAnimSubsetFin))) != NULL)
    {
      sprintf (lCurseur, "+EF%s",  lText);
      l_arg [l_index_argument++] = lCurseur;
     lCurseur += strlen (lCurseur) + 1;
    }

  /* Histogram option  */
  if (gOutputHistoFile)
    {
      if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry)), "CSV") == 0)
	{
	  sprintf (lCurseur, "+HTC");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry)), "PNG") == 0)
	{
	  sprintf (lCurseur, "+HTN");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry)), "PPM") == 0)
	{
	  sprintf (lCurseur, "+HTP");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry)), "BMP") == 0)
	{
	  sprintf (lCurseur, "+HTS");
	}
      else if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry)), "TGA") == 0)
	{
	  sprintf (lCurseur, "+HTT");
	}
      l_arg [l_index_argument++] = lCurseur;
      lCurseur += strlen (lCurseur + 1);
      sprintf (lCurseur, "+HS%s.%s +HN%s",
	       gtk_entry_get_text (GTK_ENTRY (gEntryHistoXSize)),
	       gtk_entry_get_text (GTK_ENTRY (gEntryHistoYSize)),
	       gtk_entry_get_text (GTK_ENTRY (gEntryHistoFile)));
    }
  else
    {
      sprintf (lCurseur, "+HTX");
    }
  l_arg [l_index_argument++] = lCurseur;
  lCurseur += strlen (lCurseur + 1);

  l_arg [l_index_argument] = NULL;

  l_pid_pov = fork ();
  if (l_pid_pov == (pid_t) 0)
    {
      if (execvp (l_arg[0], l_arg) == -1)
	{
	  povfront_message (gettext ("ERROR: Cannot launch rendering\n"));
	  return;
	}
      else
	{
	  exit (1);
	}
    }
  else
    {
      if (l_pid_pov == -1)
	{
	  povfront_message (gettext ("ERROR: Not enough memory to launch a new process\n"));
	  return;
	}
      else
	{
	  g_pid_last_process = l_pid_pov;
	  l_the_last_job = (job *) malloc (sizeof (job));
	  if (l_the_last_job == NULL)
	    {
	      povfront_message (gettext ("ERROR: Cannot allocate memory"));
	      return;
	    }
	  else
	    {
	      l_job_time = time (NULL);
	      l_time_char =  ctime (&l_job_time);
	      l_the_last_job->time = (gchar *) malloc (strlen (l_time_char));
	      strcpy (l_the_last_job->time, l_time_char);
	      l_the_last_job->pid = l_pid_pov;
	      l_the_last_job->state = POVF_RUNNING;
	      if (g_plugin)
		{
		  l_the_last_job->file = malloc (strlen (g_pov_file));
		  if (l_the_last_job->file == NULL)
		    {
		      povfront_message (gettext ("ERROR: Cannot allocate memory"));
		      return;
		    }
		  else
		    {
		      strcpy (l_the_last_job->file, g_pov_file);
		    }
		}
	      else
		{
		  l_the_last_job->file = malloc (strlen (gtk_entry_get_text (GTK_ENTRY (gEntryPovFile))));
		  if (l_the_last_job->file == NULL)
		    {
		      povfront_message (gettext ("ERROR: Cannot allocate memory"));
		      return;
		    }
		  else
		    {
		      strcpy (l_the_last_job->file, gtk_entry_get_text (GTK_ENTRY (gEntryPovFile)));
		    }
		}
	      if (g_image_plugin)
		{
		  l_the_last_job->image = malloc (strlen (g_image_file_to_compute));
		  if (l_the_last_job == NULL)
		    {
		      povfront_message (gettext ("ERROR: Cannot allocate memory"));
		      return;
		    }
		  else
		    {
		      strcpy (l_the_last_job->image, g_image_file_to_compute);
		    }
		}
	      else
		{
		  l_the_last_job->image = malloc (strlen (gtk_entry_get_text (GTK_ENTRY (gEntryOutFile))));
		  if (l_the_last_job == NULL)
		    {
		      povfront_message (gettext ("ERROR: Cannot allocate memory"));
		      return;
		    }
		  else
		    {
		      strcpy (l_the_last_job->image, gtk_entry_get_text (GTK_ENTRY (gEntryOutFile)));
		    }
		}
	      g_jobs_list = g_list_append (g_jobs_list, l_the_last_job);
	    }
	}
    }
}

/****************************************************************
 * Name : 		Abort
 * parameters :
 * Author : Philippe DAVID          Update : 20/03/1999
 * Return :
 * Comments : Abort the last job launched
 ****************************************************************/
void
Abort (GtkWidget *widget, gpointer data)
{
  if (kill (g_pid_last_process, SIGKILL) != 0)
    {
      povfront_message (gettext ("BUG: Cannot kill the processus"));
    }
}

/****************************************************************
 * Name : 		Help
 * parameters :
 * Author : Philippe DAVID          Update : 7 /02/ 1999
 * Return :
 * Comments :
 ****************************************************************/
void
Help (GtkWidget *widget, gpointer data)
{
  povfront_message ("Help : Not implemented yet\n");
}

/****************************************************************
 * Name : 		refresh
 * parameters :
 * Author : Philippe DAVID          Update : 27/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
refresh (GtkWidget *widget, GtkWidget *p_gtklist)
{
  refresh_list (p_gtklist);
}
/****************************************************************
 * Name : 		refresh_list
 * parameters :
 * Author : Philippe DAVID          Update : 3/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
refresh_list (GtkWidget *p_gtklist)
{
  job *l_a_job;
  gchar *l_row [5];
  gint i;
  gint l_status;
  gint l_list_length;
  GList *l_jobs_list;

  l_jobs_list = g_jobs_list;         /* Use a local pointer for the list  */
  gtk_clist_clear (GTK_CLIST (p_gtklist));
  if (l_jobs_list != NULL)
    {
      l_jobs_list = g_list_first (l_jobs_list);
      l_list_length = g_list_length (l_jobs_list);
      for (i=1; i <= l_list_length; i++)
	{
	  l_a_job = l_jobs_list->data;

	  l_row [0] = "";      /* Setting default state  */
	  l_row [1] = gettext ("Done");
	  l_row [2] = l_a_job->time;
	  l_row [3] = l_a_job->file;
	  l_row [4] = l_a_job->image;

	  if (l_a_job->state == POVF_HALTED)
	    {
	      l_row [1] = gettext ("Halted");
	      l_row [0] = malloc (15);
	      sprintf (l_row [0], "%d", (int) l_a_job->pid);
	    }
	  else if (l_a_job->state != POVF_DONE)
	    {
	      if (waitpid (l_a_job->pid, &l_status, WNOHANG|WUNTRACED) == -1)
		{
		  if (errno == ECHILD)
		    {
		      l_a_job->state = POVF_DONE;
		      l_row [1] = gettext ("Done");
		    }
		}
	      else
		{
		  if (WIFEXITED (l_status))
		    {
		      if (WEXITSTATUS (l_status) == 0)
			{
			  l_a_job->state = POVF_FINISHED;
			  l_row [1] = gettext ("Finished");
			}
		      else if (WIFSIGNALED (l_status))
			{
			  l_a_job->state = POVF_ABORTED;
			  l_row [1] = gettext ("Aborted");
			}
		      else
			{
			  l_a_job->state = POVF_FINISHED;
			  l_row [1] = gettext ("Error");
			}
		    }
		  else if (WIFSTOPPED (l_status))
		    {
		      l_a_job->state = POVF_HALTED;
		      l_row [1] = gettext ("Halted");
		      l_row [0] = malloc (15);
		      sprintf (l_row [0], "%d", (int) l_a_job->pid);
		    }
		  else
		    {
		      l_row [1] = gettext ("Running");
		      l_row [0] = malloc (15);
		      sprintf (l_row [0], "%d", (int) l_a_job->pid);
		    }
		}
	    }
	  gtk_clist_append (GTK_CLIST (p_gtklist), l_row);
	  free (l_row [0]);
	  l_jobs_list = g_list_next (l_jobs_list);
	}
    }
}

/****************************************************************
 * Name : 		stop_jobs
 * parameters :
 * Author : Philippe DAVID          Update : 27/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
stop_jobs (GtkObject *p_jobs_list)
{
  GList *l_list;
  guint l_data;
  gint i, l_length;
  GList *l_process;
  pid_t l_process_pid;
  job *l_a_job;

  l_list = GTK_CLIST (p_jobs_list)->selection;
  l_list = g_list_first (l_list);
  l_length = g_list_length (l_list);
  for (i=1; i<= l_length; i++)
    {
      l_data = (guint) l_list->data;
      l_process = (GList *) g_list_nth (g_jobs_list, l_data);
      l_a_job = (job *) l_process->data;
      l_process_pid = l_a_job->pid;
      if (kill (l_process_pid, SIGSTOP) == -1)
	{
	  povfront_message (gettext ("ERROR"));
	}
      l_list = g_list_next (l_list);
    }
}

/****************************************************************
 * Name : 		continue_jobs
 * parameters :
 * Author : Philippe DAVID          Update : 27/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
continue_jobs (GtkObject *p_jobs_list)
{
  GList *l_list;
  guint l_data;
  gint i, l_length;
  GList *l_process;
  pid_t l_process_pid;
  job *l_a_job;

  l_list = GTK_CLIST (p_jobs_list)->selection;
  l_list = g_list_first (l_list);
  l_length = g_list_length (l_list);
  for (i=1; i<= l_length; i++)
    {
      l_data = (guint) l_list->data;
      l_process = (GList *) g_list_nth (g_jobs_list, l_data);
      l_a_job = (job *) l_process->data;
      l_process_pid = l_a_job->pid;
      if (kill (l_process_pid, SIGCONT) == -1)
	{
	  povfront_message (gettext ("ERROR: Unable to send the signal to the proc."));
	}
      else
	{
	  l_a_job->state = POVF_RUNNING;
	}
      l_list = g_list_next (l_list);
    }
}

/****************************************************************
 * Name : 		kill_jobs
 * parameters :
 * Author : Philippe DAVID          Update : 27/03/1999
 * Return :
 * Comments :
 ****************************************************************/
void
kill_jobs (GtkObject *p_jobs_list)
{
  GList *l_list;
  guint l_data;
  gint i, l_length;
  GList *l_process;
  pid_t l_process_pid;
  job *l_a_job;

  l_list = GTK_CLIST (p_jobs_list)->selection;
  l_list = g_list_first (l_list);
  l_length = g_list_length (l_list);
  for (i=1; i<= l_length; i++)
    {
      l_data = (guint) l_list->data;
      l_process = (GList *) g_list_nth (g_jobs_list, l_data);
      l_a_job = (job *) l_process->data;
      l_process_pid = l_a_job->pid;
      if (kill (l_process_pid, SIGKILL) == -1)
	{
	  povfront_message (gettext ("ERROR: Unable to kill the proc."));
	}
      else
	{
	  l_a_job->state = POVF_FINISHED;
	}
      l_list = g_list_next (l_list);
    }
}

/****************************************************************
 * Name : 		greater_than
 * parameters :
 * Author : Philippe DAVID          Update : 3/04/1999
 * Return :
 * Comments :
 ****************************************************************/
gint
greater_than (gint *a, gint *b)
{
  if (*a > *b) return 1;
  else return 0;
}

/****************************************************************
 * Name : 		delete_jobs
 * parameters :
 * Author : Philippe DAVID          Update : 3/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
delete_jobs (GtkObject *p_jobs_list)
{
  GList *l_list;
  GList *l_process;
  GList *l_copy_of_list = NULL;  /* Usefull because gtk_clist_remove reset the selection list  */
  job *l_a_job;
  guint l_process_state;  /* State of the process to delete from the list  */
  guint *l_data;
  guint *l_range;
  guint i, l_length;

  l_list = GTK_CLIST (p_jobs_list)->selection;
  l_length = g_list_length (l_list);
  /* We begin to copy the selection list because the first call to gtk_clist_remove delete it  */
  for (i=1; i<=l_length; i++)
    {
      l_data = (guint *) malloc (sizeof (gint));
      *l_data = (guint) l_list->data;
      l_copy_of_list = g_list_insert_sorted (l_copy_of_list, l_data, (GCompareFunc) greater_than);
      l_list = g_list_next (l_list);
    }
  l_copy_of_list = g_list_last (l_copy_of_list);
  for (i=1; i<=l_length; i++)
    {
      l_range = (guint *) l_copy_of_list->data;
      l_process = (GList *) g_list_nth (g_jobs_list, *l_range);
      l_a_job = (job *) l_process->data;
      l_process_state = l_a_job->state;
      /* We remove just the done process  */
      if (l_process_state == POVF_DONE)
	{
	  free (l_a_job->time);
	  free (l_a_job->file);
	  free (l_a_job->image);
	  /* Remove from the job control list  */
	  gtk_clist_remove (GTK_CLIST (g_gtklist_jobs), *l_range);
	  /* Remove from the process list  */
	  g_jobs_list = g_list_remove (g_jobs_list, l_a_job);
	}
      l_copy_of_list = g_list_previous (l_copy_of_list);
    }
  g_list_free (l_copy_of_list);
}

/****************************************************************
 * Name : 		control
 * parameters :
 * Author : Philippe DAVID          Update : 10/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
control (GtkWidget *widget, gpointer data)
{
  GtkWidget *lWindow;
  GtkWidget *lButtonStop;
  GtkWidget *lButtonContinue;
  GtkWidget *lButtonKill;
  GtkWidget *lButtonDelete;
  GtkWidget *lButtonRefresh;
  GtkWidget *lButtonClose;
  GtkTooltips *l_button_stop_tip;
  GtkTooltips *l_button_continue_tip;
  GtkTooltips *l_button_kill_tip;
  GtkTooltips *l_button_delete_tip;
  GtkTooltips *l_button_refresh_tip;
  GtkTooltips *l_button_close_tip;
  GtkWidget *l_scrolled_window;
  gchar *l_title[5] = {gettext ("PID")
		       , gettext ("State")
		       , gettext ("Launch time")
		       , gettext ("File")
		       , gettext ("Image")};

  /* Dialog window creation  */
  lWindow = gtk_dialog_new ();
  gtk_container_border_width (GTK_CONTAINER (lWindow), 5);
  gtk_window_set_title (GTK_WINDOW (lWindow), gettext ("Povfront : Jobs control"));

  /* this is the scolled window to put the GtkList widget inside */
  l_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_usize (l_scrolled_window, 300, 150);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->vbox), l_scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (l_scrolled_window);

  /* Create the GtkList widget  */
  g_gtklist_jobs = gtk_clist_new_with_titles (5, l_title);
  gtk_container_add (GTK_CONTAINER (l_scrolled_window), g_gtklist_jobs);

  refresh_list (g_gtklist_jobs);

  gtk_clist_set_column_width (GTK_CLIST (g_gtklist_jobs), 0, 35);
  gtk_clist_set_column_width (GTK_CLIST (g_gtklist_jobs), 1, 45);
  gtk_clist_set_column_width (GTK_CLIST (g_gtklist_jobs), 2, 135);
  gtk_clist_set_column_width (GTK_CLIST (g_gtklist_jobs), 3, 250);
  gtk_clist_set_column_width (GTK_CLIST (g_gtklist_jobs), 4, 250);
  gtk_clist_set_selection_mode (GTK_CLIST (g_gtklist_jobs), GTK_SELECTION_MULTIPLE);
  gtk_widget_show (g_gtklist_jobs);

  /* Buttons creation  */
  lButtonStop = gtk_button_new_with_label (gettext ("Stop"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonStop, TRUE, FALSE, 10);
  gtk_signal_connect_object (GTK_OBJECT (lButtonStop), "clicked"
			     , GTK_SIGNAL_FUNC (stop_jobs), (GtkObject *) g_gtklist_jobs);
  l_button_stop_tip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, l_button_stop_tip);
  gtk_tooltips_set_colors (l_button_stop_tip, &g_tooltips_background_color, &g_tooltips_foreground_color);
  gtk_tooltips_set_tip (l_button_stop_tip, lButtonStop, gettext ("Stop the selected process"), NULL);
  gtk_widget_show (lButtonStop);

  lButtonContinue = gtk_button_new_with_label (gettext ("Continue"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonContinue, TRUE, FALSE, 10);
  gtk_signal_connect_object (GTK_OBJECT (lButtonContinue), "clicked"
			     , GTK_SIGNAL_FUNC (continue_jobs), (GtkObject *) g_gtklist_jobs);
  l_button_continue_tip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, l_button_continue_tip);
  gtk_tooltips_set_tip (l_button_continue_tip, lButtonContinue, gettext ("Make the selected process continue"), NULL);
  gtk_widget_show (lButtonContinue);

  lButtonKill = gtk_button_new_with_label (gettext ("Abort"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonKill, TRUE, FALSE, 10);
  gtk_signal_connect_object (GTK_OBJECT (lButtonKill), "clicked"
			     , GTK_SIGNAL_FUNC (kill_jobs), (GtkObject *) g_gtklist_jobs);
  l_button_kill_tip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, l_button_kill_tip);
  gtk_tooltips_set_tip (l_button_kill_tip, lButtonKill, gettext ("Kill the selected process"), NULL);
  gtk_widget_show (lButtonKill);

  lButtonDelete = gtk_button_new_with_label (gettext ("Delete"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonDelete, TRUE, FALSE, 10);
  gtk_signal_connect_object (GTK_OBJECT (lButtonDelete), "clicked"
			     , GTK_SIGNAL_FUNC (delete_jobs), (GtkObject *) g_gtklist_jobs);
  l_button_delete_tip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, l_button_delete_tip);
  gtk_tooltips_set_tip (l_button_delete_tip, lButtonDelete, gettext ("Delete the selected 'Done' process"), NULL);
  gtk_widget_show (lButtonDelete);

  lButtonRefresh = gtk_button_new_with_label (gettext ("Refresh"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonRefresh, TRUE, FALSE, 10);
  gtk_signal_connect (GTK_OBJECT (lButtonRefresh), "clicked", GTK_SIGNAL_FUNC (refresh), g_gtklist_jobs);
  l_button_refresh_tip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, l_button_refresh_tip);
  gtk_tooltips_set_tip (l_button_refresh_tip, lButtonRefresh, gettext ("Refresh the list"), NULL);
  gtk_widget_show (lButtonRefresh);

  lButtonClose = gtk_button_new_with_label (gettext ("Close"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (lWindow)->action_area), lButtonClose, TRUE, FALSE, 10);
  gtk_signal_connect_object( GTK_OBJECT(lButtonClose), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (lWindow));
  l_button_close_tip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, l_button_close_tip);
  gtk_tooltips_set_tip (l_button_close_tip, lButtonClose, gettext ("Close the job control"), NULL);
  gtk_widget_show (lButtonClose);

  gtk_widget_show (lWindow);
}
/*** END OF control  */

/****************************************************************
 * Name : 		SwitchOption
 * parameters :
 * Author : Philippe DAVID          Update : 13 /02/ 1999
 * Return :
 * Comments : Attention  l'initialisation
 ****************************************************************/
void
SwitchOption (GtkWidget *widget, gint *pSwitch)
{
  *pSwitch = (*pSwitch)? FALSE: TRUE;
}

/****************************************************************
 * Name : 		SwitchPalette
 * parameters :
 * Author : Philippe DAVID          Update : 27/02/1999
 * Return :
 * Comments : Switch the palette code
 ****************************************************************/
void
SwitchPalette (GtkWidget *widget, gchar *pNewCode)
{
  strcpy (gPalette, pNewCode);
}

/****************************************************************
 * Name : 		FileSelectionOk
 * parameters :        
 * Author : Philippe DAVID          Update : 21 Fevrier 1999
 * Return :
 * Comments :
 ****************************************************************/
void
FileSelectionOk (GtkWidget *widget, GtkFileSelection *pFileSelection)
{
  gchar *lSelectedFile;
  GtkWidget *lEntry;

  lSelectedFile =  gtk_file_selection_get_filename (GTK_FILE_SELECTION (pFileSelection));
  lEntry = gtk_object_get_data (GTK_OBJECT (pFileSelection), "Entry");
  gtk_entry_set_text (GTK_ENTRY (lEntry), (gchar*) lSelectedFile);
  gtk_widget_destroy (GTK_WIDGET (pFileSelection));
}

/****************************************************************
 * Name : 		FileSelection
 * parameters :       
 * Author : Philippe DAVID          Update : 21/02/1999
 * Return :
 * Comments :
 ****************************************************************/
void
FileSelection (GtkWidget *pWidget, gpointer pEntry)
{
  GtkWidget *lFileSelection;
  gchar *lCurrentPath;
  gchar *lWindowName;

  lWindowName = gtk_object_get_data (GTK_OBJECT (pWidget), "FileSelect");
  lFileSelection = gtk_file_selection_new (lWindowName);
  gtk_object_set_data (GTK_OBJECT (lFileSelection), "Entry", pEntry);
  lCurrentPath = gtk_entry_get_text (GTK_ENTRY (pEntry));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (lFileSelection), lCurrentPath);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (lFileSelection)->ok_button),
		      "clicked", (GtkSignalFunc) FileSelectionOk, lFileSelection );
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (lFileSelection)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (lFileSelection));
  gtk_widget_show (lFileSelection);
}

/****************************************************************
 * Name : 		FileView
 * parameters :       
 * Author : Philippe DAVID          Update : 26/04/1999
 * Return :
 * Comments :
 ****************************************************************/
void
FileView (GtkWidget *pWidget, gpointer *p_file_name)
{
  GtkWidget *l_text_window;
  gchar *l_window_name;
  FILE *l_file_handle;
  GtkWidget *l_hbox;
  GtkWidget *l_window;
  GtkWidget *l_button_close;
  GtkWidget *l_vscrollbar;
  gchar l_buffer[1024];
  gint l_length;

  l_file_handle = fopen (gtk_entry_get_text (GTK_ENTRY (p_file_name)), "r");
  if (l_file_handle == NULL)
    {
      povfront_message (gettext ("This file does not exist"));
      return;
    }
  else
    {
      l_window = gtk_dialog_new ();
      l_window_name = gtk_object_get_data (GTK_OBJECT (pWidget), "FileSelect");
      gtk_window_set_title (GTK_WINDOW (l_window), l_window_name);

      l_button_close = gtk_button_new_with_label (gettext ("Close"));
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (l_window)->action_area), l_button_close, TRUE, FALSE, 10);
      gtk_signal_connect_object( GTK_OBJECT(l_button_close), "clicked",
				 GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (l_window));
      gtk_widget_show (l_button_close);

      l_hbox = gtk_hbox_new (FALSE, FALSE);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (l_window)->vbox), l_hbox, TRUE, TRUE, 0);
      gtk_widget_show (l_hbox);

      l_text_window = gtk_text_new (NULL, NULL);
      gtk_text_set_editable (GTK_TEXT (l_text_window), FALSE);
      gtk_box_pack_start (GTK_BOX (l_hbox), l_text_window, TRUE, TRUE, 0);

      l_vscrollbar = gtk_vscrollbar_new (GTK_TEXT(l_text_window)->vadj);
      gtk_box_pack_start (GTK_BOX (l_hbox), l_vscrollbar, FALSE, FALSE, 0);
      gtk_widget_show (l_vscrollbar);

      do
	{
	  l_length = fread (l_buffer, 1, 1024, l_file_handle);
	  gtk_text_insert (GTK_TEXT (l_text_window), NULL, NULL, NULL, l_buffer, l_length);
	}
      while (l_length == 1024);
      fclose (l_file_handle);

      gtk_widget_show (l_text_window);
      gtk_widget_show (l_window);
    }
}

/****************************************************************
 * Name : 		parse_options
 * parameters : 
 * Author : Philippe DAVID          Update : 17/03/1999
 * Return : 0 if error >0 otherwhile
 * Comments :
 ****************************************************************/
int
povf_parse_options (int argc, char *argv[])
{
  int i;
  FILE *l_pov_file_handler;

  for (i=1; i<argc; i++)
    {
      if (strcmp ("--version", argv[i]) == 0)
	{
	  printf (gettext ("povfront version 0.9.0 is a GNU's GPL utility\nfrom Ph. P. E. DAVID\n"));
	  exit (0);
	}
      else if (strcmp ("--help", argv[i]) == 0)
	{
	  printf (gettext ("\nUsage: povfront [-c FILE] [-i IMAGE] [SOURCE]\n"));
	  printf (gettext ("povfront is a front end for POVRAY ray tracing engine\n\n"));
	  printf (gettext ("  -c FILE\twhere FILE specify a configuration file\n"));
	  printf (gettext ("  -i IMAGE\twhere IMAGE specify the image file to compute\n"));
	  printf (gettext ("\t  SOURCE is a pov source file\n"));
	  printf (gettext ("\t--version\tdisplay the name and the version of the software\n\n"));
	  printf (gettext ("\t--help\t\tdisplay this text\n\n"));
	  printf (gettext ("please report any bug to clovis1@club-internet.fr\n"));
	  printf ("\n");
	  exit (0);
	}
      else if (strcmp ("-c", argv[i]) == 0)
	{
	  ++i;
	  strcpy (g_config_file_to_parse, argv[i]);
	}
      else if (strcmp ("-i", argv[i]) == 0)
	{
	  ++i;
	  strcpy (g_image_file_to_compute, argv[i]);
	  g_image_plugin = 1;
	}
      else /* Option is not configuration file  */
	{
	  strcpy (g_pov_file, argv[i]);
	  g_plugin = 1;       /* indicate to the soft that the file name cannot be changed  */
	  l_pov_file_handler = fopen (g_pov_file, "r");
	  if (l_pov_file_handler == NULL)
	    {
	      printf (gettext ("ERROR: This file cannot be opened : %s\n"), g_pov_file);
	      exit (1);
	    }
	  else /* The file can be opened  */
	    {
	      fclose (l_pov_file_handler);
	    }
	}
    }
  return 0;
}

/****************************************************************
 * Name : 		main
 * parameters : Aucun
 * Author : Philippe DAVID          Update : 26/04/1999
 * Return : 
 * Comments :
 ****************************************************************/
int
main (int argc, char *argv[])
{
  GtkWidget *lseparator;
  GtkWidget *lwindow;
  GtkWidget *lMenuBar;

  GtkWidget *lFileMenu;
  GtkWidget *lFileItem;
  GtkWidget *lFileMenuItemFile;
  GtkWidget *lFileMenuItemSave;
  GtkWidget *lFileMenuSeparator;
  GtkWidget *lFileMenuItemQuit;

  GtkWidget *lOptionMenu;
  GtkWidget *lOptionItem;
  GtkWidget *lOptionMenuItemOption;
  GtkWidget *lOptionMenuItemDefault;

  GtkWidget *lHelpMenu;
  GtkWidget *lHelpItem;
  GtkWidget *lHelpMenuItemApropos;
  GtkWidget *lHelpMenuItemHelp;

  GtkWidget *lJobsMenu;
  GtkWidget *lJobsItem;
  GtkWidget *lJobsMenuItemControl;

  GtkWidget *lVBox;
  GtkWidget *lVBoxOutMain;
  GtkWidget *lHBoxIniFile;
  GtkWidget *lHBoxButtons;
  GtkWidget *lButtonRender;
  GtkWidget *lButtonAbort;
  GtkWidget *lLabel;
  GtkTooltips *lCheckPovFileTip;
  GtkWidget *lButtonIniFile;
  GtkWidget *lButtonOutFile;
  GtkWidget *lSeparator;
  GtkWidget *lNoteBook;
  GtkWidget *lHBoxOutput;
  GtkWidget *lVBoxAnimation;
  GtkWidget *lHBoxScript;
  GtkWidget *lVBoxScriptLabel;
  GtkWidget *lVBoxScriptEntry;

  GtkWidget *lVBoxLibrary;
  GtkWidget *lHBoxLibrary;
  GtkWidget *lVBoxLibraryEntry;
  GtkWidget *lVBoxLibraryBrowse;
  GtkWidget *lButtonLibraryBrowse1;
  GtkWidget *lButtonLibraryBrowse2;
  GtkWidget *lButtonLibraryBrowse3;
  GtkWidget *lButtonLibraryBrowse4;
  GtkWidget *lButtonLibraryBrowse5;
  GtkWidget *lButtonLibraryBrowse6;
  GtkWidget *lButtonLibraryBrowse7;
  GtkWidget *lButtonLibraryBrowse8;
  GtkWidget *lButtonLibraryBrowse9;
  GtkWidget *lButtonLibraryBrowse10;
  GtkWidget *lHBoxLibraryVersion;

  GSList    *lGrpRadio;
  GSList    *lGrpRadioColor;
  GtkWidget *lVBox1Output;
  GtkWidget* lVBox2Output;
  GtkWidget* lHBoxOut2;
  GtkWidget* lHBoxOut3;
  GtkWidget* lLabel2;
  GtkWidget* lLabel3;
  GtkWidget* lLabel4;
  GdkPixmap *lSubsetPixmap;
  GtkWidget *lSubsetPixmapWid;
  GdkBitmap *lMaskSubset;
  GtkWidget* lHBoxOutputPreview; 
  GtkWidget* lEntryOutDepth;
  GtkWidget* lTableOutSub;
  GtkWidget* lSeparator2;
  GtkWidget* lSeparatorAnim;
  GtkWidget* lHBoxOutputDepth;
  GtkObject *g_adjust_;
  GtkObject *g_adjust_2;
  GtkObject *g_adjust_3;
  GtkObject *g_adjust_4;
  GtkObject *g_adjust_5;
  GtkObject *g_adjust_6;
  GtkObject *g_adjust_7;
  GtkObject *g_adjust_8;
  GtkObject *g_adjust_9;
  GtkObject *g_adjust_10;
  GtkObject *g_adjust_11;

  /*** Variables for Head  */
  GtkWidget* lHBoxHead;
  GtkWidget* lVBoxHeadCheck;
  GtkWidget* lVBoxHeadEntry;
  GtkWidget* lVBoxHeadBrowse;
  GtkWidget *lHBoxHeadFileType;
  GtkWidget *lButtonPovFile;
  GtkWidget *l_label_file_name;

  /*** Variables for Quality page  */
  GtkWidget *lVBoxQuality;
  GtkWidget *lHScaleQuality;
  GtkTooltips *lHScaleQualitytip;
  GtkWidget *lLabelQuality;
  GtkWidget *lHSeparatorQuality;
  GtkWidget *lLabelAA;
  GtkWidget *lTableQuality;
  GtkWidget *lhbox_aa;
  GtkWidget *lvbox_methode_aa;
  GtkWidget *lvbox_labels_aa;
  GtkWidget *lvbox_scales_aa;
  GtkWidget *lHScaleJitter;
  GtkWidget *lLabelDepth;
  GtkWidget *lHScaleDepth;
  GtkWidget *lLabelThreshold;
  GtkWidget *lHScaleThreshold;
  GSList    *lGrpRadioAA;
  GtkWidget *lHSeparatorHistogram;
  GtkWidget *lLabelHistogram;
  GtkWidget *lHBoxHistoFile;
  GtkWidget *lHBoxHistoType;
  GtkWidget *lHBoxHistoSize;
  GtkTooltips *lCheckHistoFileTip;
  GtkWidget *lButtonHistoFile;
  GList     *lListFileType;
  GtkObject *lHistoAdjust;
  GtkObject *lHistoAdjust2;
  /*** Variables for Animation page  */
  GtkWidget *lLabelAnim;
  GtkWidget *lHBoxAnimClock;
  GtkWidget *l_hbox_internal_animation;
  GtkWidget *l_vbox_internal_animation1;
  GtkWidget *l_vbox_internal_animation2;
  /*** Variables for the messages page  */
  GtkWidget *lHBoxMessages;
  GtkWidget *lVBoxMessagesCheck;
  GtkWidget *lVBoxMessagesEntry;
  GtkWidget *lVBoxMessagesBrowse;
  GtkWidget *lVBoxMessagesView;
  GtkWidget *lButtonDebugStream;
  GtkWidget *lButtonFatalStream;
  GtkWidget *lButtonRenderStream;
  GtkWidget *lButtonStatStream;
  GtkWidget *lButtonWarningStream;
  GtkWidget *lButtonAllStream;
  GtkWidget *lButtonDebugView;
  GtkWidget *lButtonFatalView;
  GtkWidget *lButtonRenderView;
  GtkWidget *lButtonStatView;
  GtkWidget *lButtonWarningView;
  GtkWidget *lButtonAllView;

  GtkStyle *lStyle;


  gtk_init (&argc, &argv);

  gtk_rc_parse ("~/.povfront/.povfrontrc"); 
           
  InitProgramme ();                        /* Some variables to set  */

  povf_parse_options (argc, argv);

  /*** Main window creation  */
  lwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (lwindow, "main window");

  gtk_signal_connect (GTK_OBJECT (lwindow), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  gtk_signal_connect (GTK_OBJECT (lwindow), "destroy",
		      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_container_border_width (GTK_CONTAINER (lwindow), 3);
  gtk_window_set_title (GTK_WINDOW (lwindow), gettext ("povfront : a front end for povray"));

  /*** Menu creation  */
  /* Menu File  */
  lFileMenu = gtk_menu_new ();
  
  lFileMenuItemFile = gtk_menu_item_new_with_label (gettext ("Load Config."));
  gtk_object_set_data (GTK_OBJECT (lFileMenuItemFile), "FileSelect", gettext ("Load configuration file"));
  gtk_signal_connect_object( GTK_OBJECT(lFileMenuItemFile), "activate",
			     GTK_SIGNAL_FUNC (LoadConfig), NULL);
  gtk_menu_append (GTK_MENU (lFileMenu), lFileMenuItemFile);
  gtk_widget_show (lFileMenuItemFile);

  lFileMenuItemSave = gtk_menu_item_new_with_label (gettext ("Save Config."));
  gtk_object_set_data (GTK_OBJECT (lFileMenuItemFile), "FileSelect", gettext ("Save configuration file"));
  gtk_signal_connect_object( GTK_OBJECT(lFileMenuItemSave), "activate",
			     GTK_SIGNAL_FUNC (SaveConfig), NULL);
  gtk_menu_append (GTK_MENU (lFileMenu), lFileMenuItemSave);
  gtk_widget_show (lFileMenuItemSave);

  lFileMenuSeparator = gtk_menu_item_new();
  gtk_menu_append (GTK_MENU (lFileMenu), lFileMenuSeparator);
  gtk_widget_show (lFileMenuSeparator);

  lFileMenuItemQuit = gtk_menu_item_new_with_label (gettext ("Quit"));
  gtk_signal_connect_object( GTK_OBJECT(lFileMenuItemQuit), "activate",
			     GTK_SIGNAL_FUNC (destroy), (gpointer) lwindow);
  gtk_menu_append (GTK_MENU (lFileMenu), lFileMenuItemQuit);
  gtk_widget_show (lFileMenuItemQuit);

  /* Option Menu  */
  lOptionMenu = gtk_menu_new ();

  lOptionMenuItemOption = gtk_menu_item_new_with_label (gettext ("Configure"));
  gtk_signal_connect_object( GTK_OBJECT(lOptionMenuItemOption), "activate",
			     GTK_SIGNAL_FUNC (Options), NULL);
  gtk_menu_append (GTK_MENU (lOptionMenu), lOptionMenuItemOption);
  gtk_widget_show (lOptionMenuItemOption);

  lOptionMenuItemDefault = gtk_menu_item_new_with_label (gettext ("Save default"));
  gtk_signal_connect_object (GTK_OBJECT (lOptionMenuItemDefault), "activate",
			     GTK_SIGNAL_FUNC (write_default_config), NULL);
  gtk_menu_append (GTK_MENU (lOptionMenu), lOptionMenuItemDefault);
  gtk_widget_show (lOptionMenuItemDefault);

  /* Jobs Menu  */
  lJobsMenu = gtk_menu_new ();

  lJobsMenuItemControl = gtk_menu_item_new_with_label (gettext ("Control"));
  gtk_signal_connect_object( GTK_OBJECT(lJobsMenuItemControl), "activate",
			     GTK_SIGNAL_FUNC (control), NULL);
  gtk_menu_append (GTK_MENU (lJobsMenu), lJobsMenuItemControl);
  gtk_widget_show (lJobsMenuItemControl);

  /* Help Menu  */
  lHelpMenu = gtk_menu_new ();

  lHelpMenuItemApropos = gtk_menu_item_new_with_label (gettext ("About..."));
  gtk_signal_connect_object( GTK_OBJECT(lHelpMenuItemApropos), "activate",
			     GTK_SIGNAL_FUNC (APropos), NULL);
  gtk_menu_append (GTK_MENU (lHelpMenu), lHelpMenuItemApropos);
  gtk_widget_show (lHelpMenuItemApropos);

  lHelpMenuItemHelp = gtk_menu_item_new_with_label (gettext ("Help"));
  gtk_signal_connect_object( GTK_OBJECT(lHelpMenuItemHelp), "activate",
			     GTK_SIGNAL_FUNC (Help), NULL);
  gtk_menu_append (GTK_MENU (lHelpMenu), lHelpMenuItemHelp);
  gtk_widget_show (lHelpMenuItemHelp);

  /* Menu bar  */
  lMenuBar = gtk_menu_bar_new ();

  lFileItem = gtk_menu_item_new_with_label (gettext ("File"));
  gtk_menu_bar_append (GTK_MENU_BAR (lMenuBar), lFileItem);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (lFileItem), lFileMenu);
  gtk_widget_show (lFileItem);

  lOptionItem = gtk_menu_item_new_with_label (gettext ("Options"));
  gtk_menu_bar_append (GTK_MENU_BAR (lMenuBar), lOptionItem);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (lOptionItem), lOptionMenu);
  gtk_widget_show (lOptionItem);

  lJobsItem = gtk_menu_item_new_with_label (gettext ("Jobs"));
  gtk_menu_bar_append (GTK_MENU_BAR (lMenuBar), lJobsItem);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (lJobsItem), lJobsMenu);
  gtk_widget_show (lJobsItem);

  lHelpItem = gtk_menu_item_new_with_label (gettext ("Help"));
  gtk_menu_item_right_justify (GTK_MENU_ITEM (lHelpItem));
  gtk_menu_bar_append (GTK_MENU_BAR (lMenuBar), lHelpItem);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (lHelpItem), lHelpMenu);
  gtk_widget_show (lHelpItem);

  /*** declaration of the main verticale box of the window  */
  lVBox = gtk_vbox_new (FALSE, 0);

  gtk_box_pack_start (GTK_BOX (lVBox), lMenuBar, FALSE, FALSE, 0);
  gtk_widget_show (lMenuBar );

  /* POV file selection  */
  lHBoxHead = gtk_hbox_new (FALSE, 0);            /* Box of the Header  */
  lVBoxHeadCheck = gtk_vbox_new (FALSE, 0);
  lVBoxHeadEntry = gtk_vbox_new (FALSE, 0);
  lVBoxHeadBrowse = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBox), lHBoxHead, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxHead), lVBoxHeadCheck, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxHead), lVBoxHeadEntry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxHead), lVBoxHeadBrowse, FALSE, FALSE, 0);

  if (g_plugin)
    {
      l_label_file_name = gtk_label_new (gettext ("POV File :"));
      gtk_box_pack_start (GTK_BOX (lVBoxHeadCheck), l_label_file_name, TRUE, FALSE, 7);
      gtk_widget_show (l_label_file_name);

      l_label_file_name = gtk_label_new (g_pov_file);
      gtk_box_pack_start (GTK_BOX (lVBoxHeadEntry), l_label_file_name, TRUE, FALSE, 7);
      gtk_widget_show (l_label_file_name);

      l_label_file_name = gtk_label_new (" ");
      gtk_box_pack_start (GTK_BOX (lVBoxHeadBrowse), l_label_file_name, TRUE, FALSE, 7);
      gtk_widget_show (l_label_file_name);
    }
  else
    {
      gCheckPovFile = gtk_check_button_new_with_label (gettext ("Pov File"));
      gtk_box_pack_start (GTK_BOX (lVBoxHeadCheck), gCheckPovFile, TRUE, FALSE, 0);
      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckPovFile), gInputPovFile);
      gtk_signal_connect (GTK_OBJECT (gCheckPovFile), "clicked", (GtkSignalFunc) SwitchOption, &gInputPovFile );
      gtk_widget_show (gCheckPovFile);
      lCheckPovFileTip = gtk_tooltips_new ();
      g_tooltips_list = g_list_append (g_tooltips_list, lCheckPovFileTip);
      gtk_tooltips_set_tip (lCheckPovFileTip, gCheckPovFile, gettext ("Enable/Disable parsing a pov File as input"), NULL);

      gEntryPovFile = gtk_entry_new ();
      gtk_box_pack_start (GTK_BOX (lVBoxHeadEntry), gEntryPovFile, TRUE, TRUE, 0);
      gtk_widget_show (gEntryPovFile);

      lButtonPovFile = gtk_button_new_with_label (gettext ("Browse"));
      gtk_box_pack_start (GTK_BOX (lVBoxHeadBrowse), lButtonPovFile, TRUE, FALSE, 0);
      gtk_object_set_data (GTK_OBJECT (lButtonPovFile), "FileSelect", gettext ("POV File Selection"));
      gtk_signal_connect (GTK_OBJECT (lButtonPovFile), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryPovFile);
      gtk_widget_show (lButtonPovFile);
    }

  /* Selection of the generated image  */
  if (!g_image_plugin)
    {
      gCheckOutFile = gtk_check_button_new_with_label (gettext ("Output Image"));
      gtk_box_pack_start (GTK_BOX (lVBoxHeadCheck), gCheckOutFile, TRUE, FALSE, 0);
      gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOutFile), gOutputImage);
      gtk_signal_connect (GTK_OBJECT (gCheckOutFile), "clicked", (GtkSignalFunc) SwitchOption, &gOutputImage );
      gtk_widget_show (gCheckOutFile);

      gEntryOutFile = gtk_entry_new ();
     gtk_box_pack_start (GTK_BOX (lVBoxHeadEntry), gEntryOutFile, TRUE, TRUE, 0);
      gtk_widget_show (gEntryOutFile);

      lButtonOutFile = gtk_button_new_with_label (gettext ("Browse"));
      lButtonOutFile = gtk_button_new_with_label (gettext ("Browse"));
      gtk_object_set_data (GTK_OBJECT (lButtonOutFile), "FileSelect", gettext ("Image File Selection"));
      gtk_box_pack_start (GTK_BOX (lVBoxHeadBrowse), lButtonOutFile, TRUE, FALSE, 0);
      gtk_signal_connect (GTK_OBJECT (lButtonOutFile), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryOutFile);
      gtk_widget_show (lButtonOutFile);
    }
  else
    {
      gOutputImage = 1; /* Force outpout of image  */
    }

  gtk_widget_show (lVBoxHeadCheck);
  gtk_widget_show (lVBoxHeadEntry);
  gtk_widget_show (lVBoxHeadBrowse);
  gtk_widget_show (lHBoxHead);

  lHBoxHeadFileType = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBox), lHBoxHeadFileType, TRUE, TRUE, 0);
  gtk_widget_show (lHBoxHeadFileType);

  gCheckAntialiasing = gtk_check_button_new_with_label (gettext ("Antialiasing"));
  gtk_box_pack_start (GTK_BOX (lHBoxHeadFileType), gCheckAntialiasing, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAntialiasing), gAntialiasing);
  gtk_signal_connect (GTK_OBJECT (gCheckAntialiasing), "clicked", (GtkSignalFunc) SwitchOption, &gAntialiasing );
  gtk_widget_show (gCheckAntialiasing);

  g_combo_out_file = gtk_combo_new ();
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (g_combo_out_file)->entry), FALSE);
  lListFileType = NULL;
  lListFileType  = g_list_prepend (lListFileType, "CSV");
  lListFileType  = g_list_prepend (lListFileType, "PNG");
  lListFileType  = g_list_prepend (lListFileType, "PPM");
  lListFileType  = g_list_prepend (lListFileType, "BMP");
  lListFileType  = g_list_prepend (lListFileType, "TGA");
  gtk_combo_set_popdown_strings (GTK_COMBO (g_combo_out_file), lListFileType);
  gtk_box_pack_start (GTK_BOX (lHBoxHeadFileType), g_combo_out_file, TRUE, FALSE, 0);
  gtk_widget_show (g_combo_out_file);

  lSeparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBox), lSeparator, TRUE, TRUE, 5);
  gtk_widget_show (lSeparator);

  /* lHBoxButtons = gtk_handle_box_new ();  */
  lHBoxButtons = gtk_hbox_new (TRUE, 0);
  gtk_widget_set_name (lHBoxButtons, "main buttons");
  gtk_box_pack_start (GTK_BOX (lVBox), lHBoxButtons, TRUE, TRUE, 0);

  lButtonRender = gtk_button_new_with_label (gettext ("Render"));
  gtk_widget_set_name (lButtonRender, "Render");
  gtk_box_pack_start (GTK_BOX (lHBoxButtons), lButtonRender, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (lButtonRender), "clicked", (GtkSignalFunc) Render, NULL );
  gtk_widget_show (lButtonRender);

  lButtonAbort = gtk_button_new_with_label (gettext ("Abort the last job"));
  gtk_widget_set_name (lButtonAbort, "Abort");
  gtk_box_pack_start (GTK_BOX (lHBoxButtons), lButtonAbort, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (lButtonAbort), "clicked", (GtkSignalFunc) Abort, NULL );
  gtk_widget_show (lButtonAbort);

  lSeparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBox), lSeparator, TRUE, TRUE, 5);
  gtk_widget_show (lSeparator);

  lNoteBook = gtk_notebook_new ();
  gtk_box_pack_start (GTK_BOX (lVBox), lNoteBook, TRUE, TRUE, 0);

  /*** NoteBook definition  */

  /*** Outpout page definition */
  lLabel = gtk_label_new (gettext ("Output"));
  gtk_widget_show (lLabel);
  lVBoxOutMain = gtk_vbox_new (FALSE, 0);
  lHBoxOutput = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBoxOutMain), lHBoxOutput, TRUE, TRUE, 0);
  lVBox1Output = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOutput), lVBox1Output, TRUE, TRUE, 0);

  g_radioOutSize1 = gtk_radio_button_new_with_label (NULL, "1280 x 1024");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize1, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize1), "Size", "1");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize1), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W1280 +H1024");
  gtk_widget_show (g_radioOutSize1);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize1));

  g_radioOutSize2 = gtk_radio_button_new_with_label (lGrpRadio, "1024 x 768");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize2, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize2), "Size", "2");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize2), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W1024 +H768");
  gtk_widget_show (g_radioOutSize2);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize2));

  g_radioOutSize3 = gtk_radio_button_new_with_label (lGrpRadio, "800 x 600");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize3, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize3), "Size", "3");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize3), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W800 +H600");
  gtk_widget_show (g_radioOutSize3);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize3));

  g_radioOutSize4 = gtk_radio_button_new_with_label (lGrpRadio, "768 x 576");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize4, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize4), "Size", "4");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize4), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W768 +H576");
  gtk_widget_show (g_radioOutSize4);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize4));

  g_radioOutSize5 = gtk_radio_button_new_with_label (lGrpRadio, "640 x 480");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize5, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize5), "Size", "5");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize5), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W640 +H480");
  gtk_widget_show (g_radioOutSize5);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize5));

  g_radioOutSize6 = gtk_radio_button_new_with_label (lGrpRadio, "320 x 240");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize6, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize6), "Size", "6");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize6), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W320 +H240");
  gtk_widget_show (g_radioOutSize6);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize6));

  g_radioOutSize7 = gtk_radio_button_new_with_label (lGrpRadio, "320 x 200");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize7, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize7), "Size", "7");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize7), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W320 +H200");
  gtk_widget_show (g_radioOutSize7);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize7));

  g_radioOutSize8 = gtk_radio_button_new_with_label (lGrpRadio, "160 x 100");
  gtk_box_pack_start (GTK_BOX (lVBox1Output), g_radioOutSize8, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize8), "Size", "8");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize8), "pressed", GTK_SIGNAL_FUNC (ImageSize), "+W160 +H100");
  gtk_widget_show (g_radioOutSize8);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize8));

  /*** Customized size outpout 1  */
  lHBoxOut2 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBox1Output), lHBoxOut2, TRUE, TRUE, 0);

  g_radioOutSize9 = gtk_radio_button_new_with_label (lGrpRadio, ":");
  gtk_box_pack_start (GTK_BOX (lHBoxOut2), g_radioOutSize9, FALSE, FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize9), "Size", "9");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize9), "pressed", GTK_SIGNAL_FUNC (ImageSize), "");
  gtk_widget_show (g_radioOutSize9);
  lGrpRadio = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutSize9));

  g_adjust_ = gtk_adjustment_new (100, 0, 10000, 1, 10, 50);
  gEntryXSize1 = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_), 4, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOut2), gEntryXSize1, TRUE, TRUE, 0);
  gtk_widget_show (gEntryXSize1);

  g_adjust_2 = gtk_adjustment_new (100, 0, 10000, 1, 10, 50);
  gEntryYSize1 = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_2), 4, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOut2), gEntryYSize1, TRUE, TRUE, 0);
  gtk_widget_show (gEntryYSize1);

  gtk_widget_show (lHBoxOut2);

  /*** Customized size outpout 2  */
  lHBoxOut3 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBox1Output), lHBoxOut3, TRUE, TRUE, 0);

  g_radioOutSize10 = gtk_radio_button_new_with_label (lGrpRadio, ":");
  gtk_box_pack_start (GTK_BOX (lHBoxOut3), g_radioOutSize10, FALSE, FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (g_radioOutSize10), "Size", "10");
  gtk_signal_connect (GTK_OBJECT (g_radioOutSize10), "pressed", GTK_SIGNAL_FUNC (ImageSize), "");
  gtk_widget_show (g_radioOutSize10);
  
  g_adjust_3 = gtk_adjustment_new (50, 0, 10000, 1, 10, 50);
  gEntryXSize2 = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_3), 4, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOut3), gEntryXSize2, TRUE, TRUE, 0);
  gtk_widget_show (gEntryXSize2);

  g_adjust_4 = gtk_adjustment_new (50, 0, 10000, 1, 10, 50);
  gEntryYSize2 = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_4), 4, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOut3), gEntryYSize2, TRUE, TRUE, 0);
  gtk_widget_show (gEntryYSize2);
  
  gtk_widget_show (lHBoxOut3);
  
  lSeparator2 = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBox1Output), lSeparator2, TRUE, TRUE, 5);
  gtk_widget_show (lSeparator2);
  
  /*** Table for subset of rendering  */
  lTableOutSub = gtk_table_new (3, 3, TRUE);
  gtk_box_pack_start (GTK_BOX (lVBox1Output), lTableOutSub, TRUE, TRUE, 0);

  gCheckSub = gtk_check_button_new_with_label (gettext ("Sub"));
  gtk_table_attach_defaults (GTK_TABLE(lTableOutSub), gCheckSub, 0, 1, 0, 1);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckSub), gSubRender);
  gtk_signal_connect (GTK_OBJECT (gCheckSub), "clicked", (GtkSignalFunc) SwitchOption, &gSubRender );
  gtk_widget_show (gCheckSub);
  
  g_adjust_5 = gtk_adjustment_new (50, 0, 10000, 1, 10, 50);
  gEntrySubStartX = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_5), 4, 0);
  gtk_table_attach_defaults (GTK_TABLE(lTableOutSub), gEntrySubStartX, 1, 2, 0, 1);
  gtk_widget_show (gEntrySubStartX);

  g_adjust_6 = gtk_adjustment_new (100, 0, 10000, 1, 10, 50);
  gEntrySubEndX = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_6), 4, 0);
  gtk_table_attach_defaults (GTK_TABLE(lTableOutSub), gEntrySubEndX, 2, 3, 0, 1);
  gtk_widget_show (gEntrySubEndX);
  
  g_adjust_7 = gtk_adjustment_new (50, 0, 10000, 1, 10, 50);
  gEntrySubStartY = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_7), 4, 0);
  gtk_table_attach_defaults (GTK_TABLE(lTableOutSub), gEntrySubStartY, 0, 1, 1, 2);
  gtk_widget_show (gEntrySubStartY);
  
  g_adjust_8 = gtk_adjustment_new (100, 0, 10000, 1, 10, 50);
  gEntrySubEndY = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_8), 4, 0);
  gtk_table_attach_defaults (GTK_TABLE(lTableOutSub), gEntrySubEndY, 0, 1, 2, 3);
  gtk_widget_show (gEntrySubEndY);
  
  /* Pixmap for the subset of the image  */
  lStyle = gtk_widget_get_style (lwindow);
  gtk_widget_realize (lwindow);
  lSubsetPixmap = gdk_pixmap_create_from_xpm_d (lwindow->window, &lMaskSubset,
					      &lStyle->bg[GTK_STATE_NORMAL], (gchar **) subset_xpm);
  lSubsetPixmapWid = gtk_pixmap_new (lSubsetPixmap, lMaskSubset);
  gtk_widget_show (lSubsetPixmapWid);
  gtk_table_attach_defaults (GTK_TABLE (lTableOutSub), lSubsetPixmapWid, 1, 3, 1, 3);
  
  gtk_widget_show (lTableOutSub);
  
  lSeparator = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (lHBoxOutput), lSeparator, TRUE, TRUE, 5);
  gtk_widget_show (lSeparator);
  
  /*** Right vertical column of options */
  lVBox2Output = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOutput), lVBox2Output, TRUE, TRUE, 0);
  
  gCheckOut1 = gtk_check_button_new_with_label (gettext ("Test abort"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut1, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut1), gTestAbort);
  gtk_signal_connect (GTK_OBJECT (gCheckOut1), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gTestAbort);
  gtk_widget_show (gCheckOut1);
  
  gCheckOut2 = gtk_check_button_new_with_label (gettext ("Continue"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut2, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut2), gContinue);
  gtk_signal_connect (GTK_OBJECT (gCheckOut2), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gContinue);
  gtk_widget_show (gCheckOut2);
  
  gCheckOut3 = gtk_check_button_new_with_label (gettext ("Pause"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut3, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut3), gPause);
  gtk_signal_connect (GTK_OBJECT (gCheckOut3), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gPause);
  gtk_widget_show (gCheckOut3);
  
  gCheckOut4 = gtk_check_button_new_with_label (gettext ("Verbose"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut4, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut4), gVerbose);
  gtk_signal_connect (GTK_OBJECT (gCheckOut4), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gVerbose);
  gtk_widget_show (gCheckOut4);
  
  gCheckOut5 = gtk_check_button_new_with_label (gettext ("Draw vista"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut5, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut5), gDrawVista);
  gtk_signal_connect (GTK_OBJECT (gCheckOut5), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gDrawVista);
  gtk_widget_show (gCheckOut5);
  
  gCheckOut6 = gtk_check_button_new_with_label (gettext ("Display Image"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut6, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut6), gDisplayImage);
  gtk_signal_connect (GTK_OBJECT (gCheckOut6), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gDisplayImage);
  gtk_widget_show (gCheckOut6);
  
  gCheckOut7 = gtk_check_button_new_with_label (gettext ("Alpha out"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), gCheckOut7, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOut7), gAlphaOut);
  gtk_signal_connect (GTK_OBJECT (gCheckOut7), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gAlphaOut);
  gtk_widget_show (gCheckOut7);
  
  lSeparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBox2Output), lSeparator, TRUE, TRUE, 0);
  gtk_widget_show (lSeparator);
  
  lLabel2 = gtk_label_new (gettext ("Palette"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), lLabel2, TRUE, TRUE, 0);
  gtk_widget_show (lLabel2);
  
  g_radioOutColor1 = gtk_radio_button_new_with_label (NULL, gettext ("Gray scale"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), g_radioOutColor1, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_radioOutColor1), "clicked", GTK_SIGNAL_FUNC (SwitchPalette), "G");
  gtk_widget_show (g_radioOutColor1);
  lGrpRadioColor = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutColor1));
  
  g_radioOutColor2 = gtk_radio_button_new_with_label (lGrpRadioColor, gettext ("high color"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), g_radioOutColor2, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_radioOutColor2), "clicked", GTK_SIGNAL_FUNC (SwitchPalette), "H");
  gtk_widget_show (g_radioOutColor2);
  lGrpRadioColor = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioOutColor2));
  
  g_radioOutColor3 = gtk_radio_button_new_with_label (lGrpRadioColor, gettext ("True color"));
  gtk_box_pack_start (GTK_BOX (lVBox2Output), g_radioOutColor3, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_radioOutColor3), "clicked", GTK_SIGNAL_FUNC (SwitchPalette), "T");
  gtk_widget_show (g_radioOutColor3);
  
  lSeparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBox2Output), lSeparator, TRUE, TRUE, 5);
  gtk_widget_show (lSeparator);
  
  lHBoxOutputPreview = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBox2Output), lHBoxOutputPreview, TRUE, TRUE, 0);
  
  gCheckOutPreview = gtk_check_button_new_with_label (gettext ("Preview"));
  gtk_box_pack_start (GTK_BOX (lHBoxOutputPreview), gCheckOutPreview, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckOutPreview), gPreview);
  gtk_signal_connect (GTK_OBJECT (gCheckOutPreview), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gPreview);
  gtk_widget_show (gCheckOutPreview);
  
  g_adjust_9 = gtk_adjustment_new (8, 2, 64, 2, 8, 4);
  gEntryOutPreview1 = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_9), 2, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxOutputPreview), gEntryOutPreview1, TRUE, TRUE, 0);
  gtk_widget_show (gEntryOutPreview1);
  
  lLabel3 = gtk_label_new ("to");
  gtk_box_pack_start (GTK_BOX (lHBoxOutputPreview), lLabel3, TRUE, TRUE, 0);
  gtk_widget_show (lLabel3);
  
  g_adjust_10 = gtk_adjustment_new (8, 2, 64, 2, 8, 4);
  gEntryOutPreview2 = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_10), 2,0);
  gtk_box_pack_start (GTK_BOX (lHBoxOutputPreview), gEntryOutPreview2, TRUE, TRUE, 0);
  gtk_widget_show (gEntryOutPreview2);

  gtk_widget_show (lHBoxOutputPreview);

  lHBoxOutputDepth = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lVBox2Output), lHBoxOutputDepth, TRUE, TRUE, 0);
  lLabel4 = gtk_label_new (gettext ("Depth"));
  gtk_box_pack_start (GTK_BOX (lHBoxOutputDepth), lLabel4, TRUE, TRUE, 0);
  gtk_widget_show (lLabel4);
  
  g_adjust_11 = gtk_adjustment_new (16, 2, 32, 2, 8, 4);
  lEntryOutDepth = gtk_spin_button_new (GTK_ADJUSTMENT (g_adjust_11), 2,0);
  gtk_box_pack_start (GTK_BOX (lHBoxOutputDepth), lEntryOutDepth, FALSE, FALSE, 0);
  gtk_widget_show (lEntryOutDepth);

  gtk_widget_show (lHBoxOutputDepth);
  gtk_widget_show (lVBox1Output);
  gtk_widget_show (lVBox2Output);
  gtk_widget_show (lHBoxOutput);

  lSeparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxOutMain), lSeparator, TRUE, TRUE, 0);
  gtk_widget_show (lSeparator);
  
  /*** Boite de selection du fichier INI en sortie  */
  lHBoxIniFile = gtk_hbox_new (FALSE,0);
  gtk_box_pack_start (GTK_BOX (lVBoxOutMain), lHBoxIniFile, TRUE, FALSE, 0);

  gCheckIniFile = gtk_check_button_new_with_label (gettext ("Output .ini file"));
  gtk_box_pack_start (GTK_BOX (lHBoxIniFile), gCheckIniFile, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckIniFile), gOutputIniFile);
  gtk_signal_connect (GTK_OBJECT (gCheckIniFile), "clicked", (GtkSignalFunc) SwitchOption, &gOutputIniFile );
  gtk_widget_show (gCheckIniFile);
  
  gEntryIniFile = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lHBoxIniFile), gEntryIniFile, TRUE, TRUE, 0);
  gtk_widget_show (gEntryIniFile);

  lButtonIniFile = gtk_button_new_with_label (gettext ("Browse"));
  gtk_box_pack_start (GTK_BOX (lHBoxIniFile), lButtonIniFile, FALSE, FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (lButtonIniFile), "FileSelect", gettext ("INI File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonIniFile), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryIniFile);
  gtk_widget_show (lButtonIniFile);

  gtk_widget_show (lHBoxIniFile);
  gtk_widget_show (lVBoxOutMain);
  gtk_notebook_append_page (GTK_NOTEBOOK (lNoteBook), lVBoxOutMain, lLabel);
  /*** END of the outpout page contends  */
  
  /*** Quality page definition  */
  lLabel = gtk_label_new (gettext ("Quality"));
  gtk_widget_show (lLabel);

  lVBoxQuality = gtk_vbox_new (FALSE, 0);

  lTableQuality = gtk_table_new (4, 2, FALSE);
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lTableQuality, TRUE, TRUE, 0);
  /*** Contents of the quality page  */
  lLabelQuality = gtk_label_new (gettext ("Quality"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), lLabelQuality, 0, 1, 0, 1);
  gtk_widget_show (lLabelQuality);

  g_adjust_Quality = gtk_adjustment_new (gQualityValue, 0, 11, 1, 1, 0);
  lHScaleQuality = gtk_hscale_new (GTK_ADJUSTMENT (g_adjust_Quality));
  gtk_scale_set_digits (GTK_SCALE (lHScaleQuality), 0);
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), lHScaleQuality, 1, 2, 0, 1);
  gtk_signal_connect (GTK_OBJECT (g_adjust_Quality), "value_changed",
		      GTK_SIGNAL_FUNC (AdjustmentValueChanged), &gQualityValue);
  gtk_widget_show (lHScaleQuality);
  lHScaleQualitytip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, lHScaleQualitytip);
  gtk_tooltips_set_tip (lHScaleQualitytip, lHScaleQuality, gettext ("0 : Just show quick colors, full ambient lighting only\n1 : Quick colors at 5 or below\n2 : Specified diffuse and ambient light\n3 :\n4 : Shadows but no ext. lights\n5 : Shadows, including ext. lights\n6 : Texture patterns\n7 :\n8 : Reflected, refracted, and transmitted rays\n9 : Media\n10 : Radiosity, no media\n11 : Radiosity and media"), NULL);

  gCheckQuality1 = gtk_check_button_new_with_label (gettext ("Radiosity"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), gCheckQuality1, 0, 1, 2, 3);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality1), gRadiosity);
  gtk_signal_connect (GTK_OBJECT (gCheckQuality1), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gRadiosity);
  gtk_widget_show (gCheckQuality1);

  gCheckQuality2 = gtk_check_button_new_with_label (gettext ("Bounding Box"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), gCheckQuality2, 0, 1, 3, 4);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality2), gBoundingBox);
  gtk_signal_connect (GTK_OBJECT (gCheckQuality2), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gBoundingBox);
  gtk_widget_show (gCheckQuality2);

  gCheckQuality3 = gtk_check_button_new_with_label (gettext ("Light buffer"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), gCheckQuality3, 0, 1, 4, 5);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality3), gLightBuffer);
  gtk_signal_connect (GTK_OBJECT (gCheckQuality3), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gLightBuffer);
  gtk_widget_show (gCheckQuality3);

  gCheckQuality4 = gtk_check_button_new_with_label (gettext ("Vista buffer"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), gCheckQuality4, 1, 2, 2, 3);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality4), gVistaBuffer);
  gtk_signal_connect (GTK_OBJECT (gCheckQuality4), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gVistaBuffer);
  gtk_widget_show (gCheckQuality4);

  gCheckQuality5 = gtk_check_button_new_with_label (gettext ("Remove bounding"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), gCheckQuality5, 1, 2, 3, 4);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality5), gRemoveBounding);
  gtk_signal_connect (GTK_OBJECT (gCheckQuality5), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gRemoveBounding);
  gtk_widget_show (gCheckQuality5);

  gCheckQuality6 = gtk_check_button_new_with_label (gettext ("Split"));
  gtk_table_attach_defaults (GTK_TABLE(lTableQuality), gCheckQuality6, 1, 2, 4, 5);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckQuality6), gSplit);
  gtk_signal_connect (GTK_OBJECT (gCheckQuality6), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gSplit);
  gtk_widget_show (gCheckQuality6);

  lHSeparatorQuality = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lHSeparatorQuality, FALSE, FALSE, 0);
  gtk_widget_show (lHSeparatorQuality);

  lLabelAA = gtk_label_new (gettext ("Antialiasing Options"));
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lLabelAA, FALSE, FALSE, 0);
  gtk_widget_show (lLabelAA);

  lhbox_aa = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lhbox_aa, FALSE, FALSE, 0);

  lvbox_methode_aa = gtk_vbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lhbox_aa), lvbox_methode_aa, TRUE, TRUE, 0);
  gtk_widget_show (lvbox_methode_aa);

  g_radioAA1 = gtk_radio_button_new_with_label (NULL, gettext ("Methode 1"));
  gtk_box_pack_start (GTK_BOX (lvbox_methode_aa), g_radioAA1, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_radioAA1), "pressed", GTK_SIGNAL_FUNC (AAmethode), "1");
  gtk_widget_show (g_radioAA1);
  lGrpRadioAA = gtk_radio_button_group (GTK_RADIO_BUTTON (g_radioAA1));
  
  g_radioAA2 = gtk_radio_button_new_with_label (lGrpRadioAA, gettext ("Methode 2"));
  gtk_box_pack_start (GTK_BOX (lvbox_methode_aa), g_radioAA2, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_radioAA2), "pressed", GTK_SIGNAL_FUNC (AAmethode), "2");
  gtk_widget_show (g_radioAA2);

  gtk_widget_show (lvbox_methode_aa);

  lseparator = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (lhbox_aa), lseparator, TRUE, TRUE, 0);
  gtk_widget_show (lseparator);

  lvbox_labels_aa  = gtk_vbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lhbox_aa), lvbox_labels_aa, TRUE, TRUE, 0);

  gCheck_jitter = gtk_check_button_new_with_label (gettext ("Jitter"));
  gtk_box_pack_start (GTK_BOX (lvbox_labels_aa), gCheck_jitter, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheck_jitter), g_jitter);
  gtk_signal_connect (GTK_OBJECT (gCheck_jitter), "clicked", (GtkSignalFunc) SwitchOption, &g_jitter );
  gtk_widget_show (gCheck_jitter);

  lLabelDepth = gtk_label_new (gettext ("Depth"));
  gtk_box_pack_start (GTK_BOX (lvbox_labels_aa), lLabelDepth, TRUE, TRUE, 0);
  gtk_widget_show (lLabelDepth);
  
  lLabelThreshold = gtk_label_new (gettext ("Threshold"));
  gtk_box_pack_start (GTK_BOX (lvbox_labels_aa), lLabelThreshold, TRUE, TRUE, 0);
  gtk_widget_show (lLabelThreshold);

  gtk_widget_show (lvbox_labels_aa);

  lvbox_scales_aa = gtk_vbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (lhbox_aa), lvbox_scales_aa, TRUE, TRUE, 0);

  g_adjust_Jitter = gtk_adjustment_new (1.0, 0, 1, 0.1, 0.1, 0);
  lHScaleJitter = gtk_hscale_new (GTK_ADJUSTMENT (g_adjust_Jitter));
  gtk_box_pack_start (GTK_BOX (lvbox_scales_aa), lHScaleJitter, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_adjust_Jitter), "value_changed",
		      GTK_SIGNAL_FUNC (AdjustmentValueChanged), &g_jitter_value);
  gtk_widget_show (lHScaleJitter);
  
  g_adjust_Depth = gtk_adjustment_new (3, 1, 9, 1, 1, 0);
  lHScaleDepth = gtk_hscale_new (GTK_ADJUSTMENT (g_adjust_Depth));
  gtk_scale_set_digits (GTK_SCALE (lHScaleDepth), 0);
  gtk_box_pack_start (GTK_BOX (lvbox_scales_aa), lHScaleDepth, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_adjust_Depth), "value_changed",
		      GTK_SIGNAL_FUNC (AdjustmentValueChanged), &gQualityDepth);
  gtk_widget_show (lHScaleDepth);
  
  g_adjust_Threshold = gtk_adjustment_new (0.3, 0, 1, 0.1, 0.1, 0.1);
  lHScaleThreshold = gtk_hscale_new (GTK_ADJUSTMENT (g_adjust_Threshold));
  gtk_box_pack_start (GTK_BOX (lvbox_scales_aa), lHScaleThreshold, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (g_adjust_Threshold), "value_changed",
		      GTK_SIGNAL_FUNC (AdjustmentValueChanged), &gQualityThreshold);
  gtk_widget_show (lHScaleThreshold);

  gtk_widget_show (lvbox_scales_aa);

  gtk_widget_show (lhbox_aa);

  lHSeparatorHistogram = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lHSeparatorHistogram, TRUE, TRUE, 0);
  gtk_widget_show (lHSeparatorHistogram);
  
  /* Histogram options  */
  lLabelHistogram = gtk_label_new (gettext ("Histogram Options"));
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lLabelHistogram, TRUE, TRUE, 0);
  gtk_widget_show (lLabelHistogram);
  
  lHBoxHistoFile = gtk_hbox_new (FALSE,0);
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lHBoxHistoFile, TRUE, FALSE, 0);
  
  gCheckHistoFile = gtk_check_button_new_with_label (gettext ("Histo. File"));
  gtk_box_pack_start (GTK_BOX (lHBoxHistoFile), gCheckHistoFile, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckHistoFile), gOutputHistoFile);
  gtk_signal_connect (GTK_OBJECT (gCheckHistoFile), "clicked", (GtkSignalFunc) SwitchOption, &gOutputHistoFile );
  gtk_widget_show (gCheckHistoFile);
  lCheckHistoFileTip = gtk_tooltips_new ();
  g_tooltips_list = g_list_append (g_tooltips_list, lCheckHistoFileTip);
  gtk_tooltips_set_tip (lCheckHistoFileTip, gCheckHistoFile, gettext ("Enable/Disable outpout of histogram file"), NULL);
  
  gEntryHistoFile = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lHBoxHistoFile), gEntryHistoFile, TRUE, TRUE, 0);
  gtk_widget_show (gEntryHistoFile);
  
  lButtonHistoFile = gtk_button_new_with_label (gettext ("Browse"));
  gtk_box_pack_start (GTK_BOX (lHBoxHistoFile), lButtonHistoFile, FALSE, FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (lButtonHistoFile), "FileSelect", gettext ("Histogram File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonHistoFile), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryHistoFile);

  /* Histogram file type  */
  lHBoxHistoType = gtk_hbox_new (FALSE,0);
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lHBoxHistoType, TRUE, TRUE, 0);

  lLabelHistogram = gtk_label_new (gettext ("Histogram file type"));
  gtk_box_pack_start (GTK_BOX (lHBoxHistoType), lLabelHistogram, TRUE, TRUE, 0);
  gtk_widget_show (lLabelHistogram);

  g_combo_histo_type = gtk_combo_new ();
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (g_combo_histo_type)->entry), FALSE);
  gtk_combo_set_popdown_strings (GTK_COMBO (g_combo_histo_type), lListFileType);
  gtk_box_pack_start (GTK_BOX (lHBoxHistoType), g_combo_histo_type, FALSE, TRUE, 0);
  gtk_widget_show (g_combo_histo_type);
  
  /* Histogram image size  */
  lHBoxHistoSize = gtk_hbox_new (FALSE,0);
  gtk_box_pack_start (GTK_BOX (lVBoxQuality), lHBoxHistoSize, TRUE, TRUE, 0);
  
  lLabelHistogram = gtk_label_new (gettext ("Histogram image size"));
  gtk_box_pack_start (GTK_BOX (lHBoxHistoSize), lLabelHistogram, TRUE, TRUE, 0);
  gtk_widget_show (lLabelHistogram);
  
  lHistoAdjust = gtk_adjustment_new (100, 0, 10000, 1, 10, 50);
  gEntryHistoXSize = gtk_spin_button_new (GTK_ADJUSTMENT (lHistoAdjust), 4, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxHistoSize), gEntryHistoXSize, FALSE, FALSE, 0);
  gtk_widget_show (gEntryHistoXSize);
  
  lHistoAdjust2 = gtk_adjustment_new (100, 0, 10000, 1, 10, 50);
  gEntryHistoYSize = gtk_spin_button_new (GTK_ADJUSTMENT (lHistoAdjust2), 4, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxHistoSize), gEntryHistoYSize, FALSE, FALSE, 0);
  gtk_widget_show (gEntryHistoYSize);
  
  gtk_widget_show (lHBoxHistoSize);
  gtk_widget_show (lHBoxHistoType);
  gtk_widget_show (lButtonHistoFile);
  gtk_widget_show (lHBoxHistoFile);
  
  gtk_widget_show (lTableQuality);
  gtk_widget_show (lVBoxQuality);
  gtk_notebook_append_page (GTK_NOTEBOOK (lNoteBook), lVBoxQuality, lLabel);
  /*** END of the quality page contends  */
  
  /*** Definition de la page Animation  */
  lLabel = gtk_label_new (gettext ("Animation"));
  gtk_widget_show (lLabel);
  lVBoxAnimation = gtk_vbox_new (FALSE, 0);
  /*** Contents of the Animation page  */
  /* Clock Option of external Animation  */
  lLabelAnim = gtk_label_new (gettext ("External Animation Loop"));
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), lLabelAnim, TRUE, TRUE, 0);
  gtk_widget_show (lLabelAnim);
  
  lHBoxAnimClock = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), lHBoxAnimClock, TRUE, TRUE, 0);
  lLabelAnim = gtk_label_new (gettext ("Clock"));
  gtk_box_pack_start (GTK_BOX (lHBoxAnimClock), lLabelAnim, TRUE, TRUE, 0);
  gtk_widget_show (lLabelAnim);
  gEntryAnimClock = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lHBoxAnimClock), gEntryAnimClock, FALSE, FALSE, 0);
  gtk_widget_show (gEntryAnimClock);
  gtk_widget_show (lHBoxAnimClock);
  /* Initial and Final values for Clock, frames and subset  */
  lSeparatorAnim = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), lSeparatorAnim, TRUE, TRUE, 0);
  gtk_widget_show (lSeparatorAnim);
  lLabelAnim = gtk_label_new (gettext ("Internal Animation Loop"));
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), lLabelAnim, TRUE, TRUE, 0);
  gtk_widget_show (lLabelAnim);
  
  l_hbox_internal_animation = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), l_hbox_internal_animation, TRUE, TRUE, 0);

  l_vbox_internal_animation1 = gtk_vbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (l_hbox_internal_animation), l_vbox_internal_animation1, TRUE, TRUE, 0);

  lLabelAnim = gtk_label_new (gettext ("Initial Frame"));
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation1), lLabelAnim, TRUE, TRUE, 5);
  gtk_widget_show (lLabelAnim);

  lLabelAnim = gtk_label_new (gettext ("Final Frame"));
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation1), lLabelAnim, TRUE, TRUE, 5);
  gtk_widget_show (lLabelAnim);

  lLabelAnim = gtk_label_new (gettext ("Initial Clock"));
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation1), lLabelAnim, TRUE, TRUE, 5);
  gtk_widget_show (lLabelAnim);

  lLabelAnim = gtk_label_new (gettext ("Final Clock"));
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation1), lLabelAnim, TRUE, TRUE, 5);
  gtk_widget_show (lLabelAnim);

  lLabelAnim = gtk_label_new (gettext ("Initial Subset"));
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation1), lLabelAnim, TRUE, FALSE, 5);
  gtk_widget_show (lLabelAnim);

  lLabelAnim = gtk_label_new (gettext ("Final Subset"));
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation1), lLabelAnim, TRUE, FALSE, 5);
  gtk_widget_show (lLabelAnim);
  gtk_widget_show (l_vbox_internal_animation1);
  
  l_vbox_internal_animation2 = gtk_vbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (l_hbox_internal_animation), l_vbox_internal_animation2, TRUE, TRUE, 0);

  gEntryAnimFrameIni = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation2), gEntryAnimFrameIni, TRUE, TRUE, 5);
  gtk_widget_show (gEntryAnimFrameIni);
  
  gEntryAnimFrameFin = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation2), gEntryAnimFrameFin, TRUE, TRUE, 5);
  gtk_widget_show (gEntryAnimFrameFin);
  
  gEntryAnimClockIni = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation2), gEntryAnimClockIni, TRUE, TRUE, 5);
  gtk_widget_show (gEntryAnimClockIni);
  
  gEntryAnimClockFin = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation2), gEntryAnimClockFin, TRUE, TRUE, 5);
  gtk_widget_show (gEntryAnimClockFin);
  
  gEntryAnimSubsetIni = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation2), gEntryAnimSubsetIni, TRUE, FALSE, 5);
  gtk_widget_show (gEntryAnimSubsetIni);
  
  gEntryAnimSubsetFin = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (l_vbox_internal_animation2), gEntryAnimSubsetFin, TRUE, FALSE, 5);
  gtk_widget_show (gEntryAnimSubsetFin);
  
  gCheckAnimCyclic = gtk_check_button_new_with_label (gettext ("Cyclic animation"));
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), gCheckAnimCyclic, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (gCheckAnimCyclic), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &g_cyclic);
  gtk_widget_show (gCheckAnimCyclic);
  gtk_widget_show (l_vbox_internal_animation2);
  gtk_widget_show (l_hbox_internal_animation);
 
  gCheckAnimField = gtk_check_button_new_with_label (gettext ("Field rendering"));
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), gCheckAnimField, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAnimField), g_field_render);
  gtk_signal_connect (GTK_OBJECT (gCheckAnimField), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &g_field_render);
  gtk_widget_show (gCheckAnimField);
  
  gCheckAnimOdd = gtk_check_button_new_with_label (gettext ("Odd field"));
  gtk_box_pack_start (GTK_BOX (lVBoxAnimation), gCheckAnimOdd, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAnimOdd), g_odd_field);
  gtk_signal_connect (GTK_OBJECT (gCheckAnimOdd), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &g_odd_field);
  gtk_widget_show (gCheckAnimOdd);
  
  gtk_widget_show (lVBoxAnimation);
  gtk_notebook_append_page (GTK_NOTEBOOK (lNoteBook), lVBoxAnimation, lLabel);
  /*** END of the Animation page contends  */
  
  /*** Definition de la page Script  */
  
  lLabel = gtk_label_new (gettext ("Script"));
  gtk_widget_show (lLabel);
  lHBoxScript = gtk_hbox_new (FALSE, 0);
  gtk_notebook_append_page (GTK_NOTEBOOK (lNoteBook), lHBoxScript, lLabel);
  gtk_widget_show (lHBoxScript);

  /*** Contents of the Script page  */
  lVBoxScriptLabel = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxScript), lVBoxScriptLabel, TRUE, TRUE, 0);
  gtk_widget_show (lVBoxScriptLabel);
  lLabel = gtk_label_new (gettext ("Pre scene command"));
  gtk_box_pack_start (GTK_BOX (lVBoxScriptLabel), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);
  lLabel = gtk_label_new (gettext ("Pre frame command"));
  gtk_box_pack_start (GTK_BOX (lVBoxScriptLabel), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);
  lLabel = gtk_label_new (gettext ("Post scene command"));
  gtk_box_pack_start (GTK_BOX (lVBoxScriptLabel), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);
  lLabel = gtk_label_new (gettext ("Post frame command"));
  gtk_box_pack_start (GTK_BOX (lVBoxScriptLabel), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);
  lLabel = gtk_label_new (gettext ("User abort command"));
  gtk_box_pack_start (GTK_BOX (lVBoxScriptLabel), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);
  lLabel = gtk_label_new (gettext ("Fatal error command"));
  gtk_box_pack_start (GTK_BOX (lVBoxScriptLabel), lLabel, TRUE, TRUE, 0);
  gtk_widget_show (lLabel);

  lVBoxScriptEntry = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxScript), lVBoxScriptEntry, TRUE, TRUE, 0);
  gtk_widget_show (lVBoxScriptEntry);

  gEntryPreScene = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxScriptEntry), gEntryPreScene, TRUE, TRUE, 0);
  gtk_widget_show (gEntryPreScene);
  gEntryPreFrame = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxScriptEntry), gEntryPreFrame, TRUE, TRUE, 0);
  gtk_widget_show (gEntryPreFrame);
  gEntryPostScene = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxScriptEntry), gEntryPostScene, TRUE, TRUE, 0);
  gtk_widget_show (gEntryPostScene);
  gEntryPostFrame = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxScriptEntry), gEntryPostFrame, TRUE, TRUE, 0);
  gtk_widget_show (gEntryPostFrame);
  gEntryUserAbort = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxScriptEntry), gEntryUserAbort, TRUE, TRUE, 0);
  gtk_widget_show (gEntryUserAbort);
  gEntryFatalError = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxScriptEntry), gEntryFatalError, TRUE, TRUE, 0);
  gtk_widget_show (gEntryFatalError);
  /*** END of the outpout Script contends  */
  
  /*** Definition of the Messages page  */
  lLabel = gtk_label_new (gettext ("Messages"));
  gtk_widget_show (lLabel);
  lHBoxMessages = gtk_hbox_new (FALSE, 0);
  gtk_notebook_append_page (GTK_NOTEBOOK (lNoteBook), lHBoxMessages, lLabel);
  /*** Contents of the Messages page  */
  lVBoxMessagesCheck = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxMessages), lVBoxMessagesCheck, FALSE, FALSE, 0);
  lVBoxMessagesEntry = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxMessages), lVBoxMessagesEntry, TRUE, TRUE, 0);
  lVBoxMessagesBrowse = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxMessages), lVBoxMessagesBrowse, FALSE, FALSE, 0);
  lVBoxMessagesView = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lHBoxMessages), lVBoxMessagesView, FALSE, FALSE, 0);
  
  /* Check button for messages options  */
  gCheckDebugStream = gtk_check_button_new_with_label (gettext ("Debug"));
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesCheck), gCheckDebugStream, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckDebugStream), gDebugStream);
  gtk_signal_connect (GTK_OBJECT (gCheckDebugStream), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gDebugStream);
  gtk_widget_show (gCheckDebugStream);
  
  gCheckFatalStream = gtk_check_button_new_with_label (gettext ("Fatal"));
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesCheck), gCheckFatalStream, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckFatalStream), gFatalStream);
  gtk_signal_connect (GTK_OBJECT (gCheckFatalStream), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gFatalStream);
  gtk_widget_show (gCheckFatalStream);
  
  gCheckRenderStream = gtk_check_button_new_with_label (gettext ("Render"));
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesCheck), gCheckRenderStream, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckRenderStream), gRenderStream);
  gtk_signal_connect (GTK_OBJECT (gCheckRenderStream), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gRenderStream);
 gtk_widget_show (gCheckRenderStream);
  
  gCheckStatStream = gtk_check_button_new_with_label (gettext ("Stat"));
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesCheck), gCheckStatStream, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckStatStream), gStatStream);
  gtk_signal_connect (GTK_OBJECT (gCheckStatStream), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gStatStream);
  gtk_widget_show (gCheckStatStream);
  
  gCheckWarningStream = gtk_check_button_new_with_label (gettext ("Warning"));
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesCheck), gCheckWarningStream, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckWarningStream), gWarningStream);
  gtk_signal_connect (GTK_OBJECT (gCheckWarningStream), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gWarningStream);
  gtk_widget_show (gCheckWarningStream);
  
  gCheckAllStream = gtk_check_button_new_with_label (gettext ("All"));
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesCheck), gCheckAllStream, TRUE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (gCheckAllStream), gAllStream);
  gtk_signal_connect (GTK_OBJECT (gCheckAllStream), "clicked", GTK_SIGNAL_FUNC (SwitchOption), &gAllStream);
  gtk_widget_show (gCheckAllStream);
  
  /* Entry field for messages options  */
  gEntryDebugStream = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesEntry), gEntryDebugStream, TRUE, TRUE, 0);
  gtk_widget_show (gEntryDebugStream);
  
  gEntryFatalStream = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesEntry), gEntryFatalStream, TRUE, TRUE, 0);
  gtk_widget_show (gEntryFatalStream);
  
  gEntryRenderStream = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesEntry), gEntryRenderStream, TRUE, TRUE, 0);
  gtk_widget_show (gEntryRenderStream);
  
  gEntryStatStream = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesEntry), gEntryStatStream, TRUE, TRUE, 0);
  gtk_widget_show (gEntryStatStream);
  
  gEntryWarningStream = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesEntry), gEntryWarningStream, TRUE, TRUE, 0);
  gtk_widget_show (gEntryWarningStream);
  
  gEntryAllStream = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesEntry), gEntryAllStream, TRUE, TRUE, 0);
  gtk_widget_show (gEntryAllStream);
  
  /* Browse button for file selection  */
  lButtonDebugStream = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonDebugStream), "FileSelect", gettext ("Debug stream File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonDebugStream), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryDebugStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesBrowse), lButtonDebugStream, TRUE, FALSE, 0);
  gtk_widget_show (lButtonDebugStream);
  
  lButtonFatalStream = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonFatalStream), "FileSelect", gettext ("Fatal stream File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonFatalStream), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryFatalStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesBrowse), lButtonFatalStream, TRUE, FALSE, 0);
  gtk_widget_show (lButtonFatalStream);
  
  lButtonRenderStream = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonRenderStream), "FileSelect", gettext ("Render stream File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonRenderStream), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryRenderStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesBrowse), lButtonRenderStream, TRUE, FALSE, 0);
  gtk_widget_show (lButtonRenderStream);
  
  lButtonStatStream = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonStatStream), "FileSelect", gettext ("Stat stream File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonStatStream), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryStatStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesBrowse), lButtonStatStream, TRUE, FALSE, 0);
  gtk_widget_show (lButtonStatStream);
  
  lButtonWarningStream = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonWarningStream), "FileSelect", gettext ("Warning stream File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonWarningStream), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryWarningStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesBrowse), lButtonWarningStream, TRUE, FALSE, 0);
  gtk_widget_show (lButtonWarningStream);
  
  lButtonAllStream = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonAllStream), "FileSelect", gettext ("All stream File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonAllStream), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryAllStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesBrowse), lButtonAllStream, TRUE, FALSE, 0);
  gtk_widget_show (lButtonAllStream);

  /* View buttons for the messages files  */
  lButtonDebugView = gtk_button_new_with_label (gettext ("View"));
  gtk_object_set_data (GTK_OBJECT (lButtonDebugView), "FileSelect", gettext ("View Debug stream File"));
  gtk_signal_connect (GTK_OBJECT (lButtonDebugView), "clicked", GTK_SIGNAL_FUNC (FileView), gEntryDebugStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesView), lButtonDebugView, TRUE, FALSE, 0);
  gtk_widget_show (lButtonDebugView);
  
  lButtonFatalView = gtk_button_new_with_label (gettext ("View"));
  gtk_object_set_data (GTK_OBJECT (lButtonFatalView), "FileSelect", gettext ("View Fatal stream File"));
  gtk_signal_connect (GTK_OBJECT (lButtonFatalView), "clicked", GTK_SIGNAL_FUNC (FileView), gEntryFatalStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesView), lButtonFatalView, TRUE, FALSE, 0);
  gtk_widget_show (lButtonFatalView);
  
  lButtonRenderView = gtk_button_new_with_label (gettext ("View"));
  gtk_object_set_data (GTK_OBJECT (lButtonRenderView), "FileSelect", gettext ("View Render stream File"));
  gtk_signal_connect (GTK_OBJECT (lButtonRenderView), "clicked", GTK_SIGNAL_FUNC (FileView), gEntryRenderStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesView), lButtonRenderView, TRUE, FALSE, 0);
  gtk_widget_show (lButtonRenderView);
  
  lButtonStatView = gtk_button_new_with_label (gettext ("View"));
  gtk_object_set_data (GTK_OBJECT (lButtonStatView), "FileSelect", gettext ("View Stat stream File"));
  gtk_signal_connect (GTK_OBJECT (lButtonStatView), "clicked", GTK_SIGNAL_FUNC (FileView), gEntryStatStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesView), lButtonStatView, TRUE, FALSE, 0);
  gtk_widget_show (lButtonStatView);
  
  lButtonWarningView = gtk_button_new_with_label (gettext ("View"));
  gtk_object_set_data (GTK_OBJECT (lButtonWarningView), "FileSelect", gettext ("View Warning stream File"));
  gtk_signal_connect (GTK_OBJECT (lButtonWarningView), "clicked", GTK_SIGNAL_FUNC (FileView), gEntryWarningStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesView), lButtonWarningView, TRUE, FALSE, 0);
  gtk_widget_show (lButtonWarningView);
  
  lButtonAllView = gtk_button_new_with_label (gettext ("View"));
  gtk_object_set_data (GTK_OBJECT (lButtonAllView), "FileSelect", gettext ("View All stream File"));
  gtk_signal_connect (GTK_OBJECT (lButtonAllView), "clicked", GTK_SIGNAL_FUNC (FileView), gEntryAllStream);
  gtk_box_pack_start (GTK_BOX (lVBoxMessagesView), lButtonAllView, TRUE, FALSE, 0);
  gtk_widget_show (lButtonAllView);

  gtk_widget_show (lVBoxMessagesCheck);	
  gtk_widget_show (lVBoxMessagesEntry);	
  gtk_widget_show (lVBoxMessagesBrowse);	
  gtk_widget_show (lVBoxMessagesView);	
  gtk_widget_show (lHBoxMessages);	
  /*** END of the Messages page contends  */
  
  /*** Definition de la page library du note book  */
  lLabel = gtk_label_new ("Library");
  gtk_widget_show (lLabel);
  /*** Contents of the library page  */
  lVBoxLibrary = gtk_vbox_new (FALSE,0);
  gtk_notebook_append_page (GTK_NOTEBOOK (lNoteBook),lVBoxLibrary, lLabel);
  
  lHBoxLibrary = gtk_hbox_new (FALSE,0);
  gtk_box_pack_start (GTK_BOX (lVBoxLibrary), lHBoxLibrary, TRUE, TRUE, 0);
  lVBoxLibraryEntry = gtk_vbox_new (TRUE, 3);
  gtk_box_pack_start (GTK_BOX (lHBoxLibrary), lVBoxLibraryEntry, TRUE, TRUE, 0);
  gEntryLibrary1 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary1, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary1);
  gEntryLibrary2 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary2, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary2);
  gEntryLibrary3 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary3, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary3);
  gEntryLibrary4 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary4, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary4);
  gEntryLibrary5 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary5, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary5);
  gEntryLibrary6 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary6, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary6);
  gEntryLibrary7 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary7, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary7);
  gEntryLibrary8 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary8, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary8);
  gEntryLibrary9 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary9, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary9);
  gEntryLibrary10 = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryEntry), gEntryLibrary10, TRUE, TRUE, 0);
  gtk_widget_show (gEntryLibrary10);
  gtk_widget_show (lVBoxLibraryEntry);
  
  lVBoxLibraryBrowse = gtk_vbox_new (TRUE, 3);
  gtk_box_pack_start (GTK_BOX (lHBoxLibrary), lVBoxLibraryBrowse, FALSE, TRUE, 0);
  lButtonLibraryBrowse1 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse1), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse1), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary1);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse1, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse1);
  lButtonLibraryBrowse2 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse2), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse2), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary2);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse2, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse2);
  lButtonLibraryBrowse3 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse3), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse3), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary3);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse3, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse3);
  lButtonLibraryBrowse4 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse4), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse4), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary4);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse4, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse4);
  lButtonLibraryBrowse5 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse5), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse5), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary5);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse5, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse5);
  lButtonLibraryBrowse6 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse6), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse6), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary6);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse6, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse6);
  lButtonLibraryBrowse7 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse7), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse7), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary7);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse7, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse7);
  lButtonLibraryBrowse8 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse8), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse8), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary8);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse8, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse8);
  lButtonLibraryBrowse9 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse9), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse9), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary9);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse9, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse9);
  lButtonLibraryBrowse10 = gtk_button_new_with_label (gettext ("Browse"));
  gtk_object_set_data (GTK_OBJECT (lButtonLibraryBrowse10), "FileSelect", gettext ("Library File Selection"));
  gtk_signal_connect (GTK_OBJECT (lButtonLibraryBrowse10), "clicked", GTK_SIGNAL_FUNC (FileSelection), gEntryLibrary10);
  gtk_box_pack_start (GTK_BOX (lVBoxLibraryBrowse), lButtonLibraryBrowse10, TRUE, FALSE, 0);
  gtk_widget_show (lButtonLibraryBrowse10);
  
  gtk_widget_show (lVBoxLibraryBrowse);
  gtk_widget_show (lHBoxLibrary);
  
  lHBoxLibraryVersion = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (lVBoxLibrary),lHBoxLibraryVersion , TRUE, FALSE, 0);
  
  lLabel = gtk_label_new (gettext ("POV File Version"));
  gtk_box_pack_start (GTK_BOX (lHBoxLibraryVersion), lLabel, TRUE, FALSE, 0);
  gtk_widget_show (lLabel);
  
  gEntryLibraryVersion = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (gEntryLibraryVersion), g_entry_pov_version);
  gtk_box_pack_start (GTK_BOX (lHBoxLibraryVersion), gEntryLibraryVersion, TRUE, FALSE, 0);
  gtk_widget_show (gEntryLibraryVersion);
  
  gtk_widget_show (lHBoxLibraryVersion);
  gtk_widget_show (lVBoxLibrary);
  gtk_widget_show (lNoteBook);
  /***  END of notebook definition  */

  gtk_container_add (GTK_CONTAINER (lwindow), lVBox);

  gtk_widget_show (lHBoxIniFile);
  gtk_widget_show (lHBoxButtons);
  gtk_widget_show (lVBox);

  if (strlen (g_config_file_to_parse) == 0)
    {
      povfront_message (gettext ("You don't have a default configuration file, don't forget to save one\n"));
    }
  else
    {
      load_config_file (g_config_file_to_parse);
    }

  gtk_widget_show (lwindow);

  gtk_main ();

  return 0;
}
/*** END OF povfront.c  */
