/* povfront.c
 * povfront - A GPLed Front-end for POVray raytracer
 * Copyright (C) 1999 Philippe DAVID <clovis1@club-internet.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/****************************************************************
* Name : 		povfront.h
* Author : Philippe DAVID          update : 3/4/1999
****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <limits.h>     /* ARG_MAX  */
#include <unistd.h>     /* execvp  */
#include <sys/types.h>  /* waitpid  */
#include <sys/wait.h>
#include <signal.h>     /* For kill function  */
#include <sys/stat.h>      /* For mkdir function  */
#include <errno.h>
#include <time.h>

/*** i18n things  */
#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) gettext(String)
#ifdef gettext_noop
#define N_(String) gettext_noop(String)
#else
#define N_(String) (String)
#endif
#else /* NLS is disabled  */
#define _(String) (String)
#define N_(String) (String)
#define textdomain(String) 
#define gettext(String) (String)
#define dgettext(Domain,String) (String)
#define dcgettext(Domain,String,Type) (String)
#define bindtextdomain(Domain,Directory) (Domain)
#endif

#include "subset.xpm"

/*** Global Settings  */
GtkWidget *gEntryPovFile;          /* Input POV file  */
GtkWidget *gEntryIniFile;          /* Output Ini file  */
GtkWidget *gEntryOutFile;          /* Image file name  */
GtkWidget *gEntryLibrary1;         /* Library path  */
GtkWidget *gEntryLibrary2;         /* Library path  */
GtkWidget *gEntryLibrary3;         /* Library path  */
GtkWidget *gEntryLibrary4;         /* Library path  */
GtkWidget *gEntryLibrary5;         /* Library path  */
GtkWidget *gEntryLibrary6;         /* Library path  */
GtkWidget *gEntryLibrary7;         /* Library path  */
GtkWidget *gEntryLibrary8;         /* Library path  */
GtkWidget *gEntryLibrary9;         /* Library path  */
GtkWidget *gEntryLibrary10;        /* Library path  */
GtkWidget *gEntryLibraryVersion;   /* Version of the POV engine  */
GtkWidget *gEntryXSize1;             /* Custom Size X1 of the output image  */
GtkWidget *gEntryYSize1;             /* Custom Size Y1 of the output image  */
GtkWidget *gEntryXSize2;             /* Custom size X2 of the output image  */
GtkWidget *gEntryYSize2;             /* Custom size Y2 of the output image  */
GtkWidget *gEntrySubStartX;          /* sub size of the output image  */
GtkWidget *gEntrySubEndX;            /* sub size of the output image  */
GtkWidget *gEntrySubStartY;          /* sub size of the output image  */
GtkWidget *gEntrySubEndY;            /* sub size of the output image  */
GtkWidget *gEntryOutPreview1;        /* Preview flag of image  */
GtkWidget *gEntryOutPreview2;        /* Preview flag of image  */
GtkWidget *gEntryHistoFile;          /* Histogram file name  */
GtkWidget *gEntryHistoXSize;         /* Histogram grid X size  */
GtkWidget *gEntryHistoYSize;         /* Histogram grid Y size  */
GtkWidget *gEntryAnimClock;          /* value for clock in case of external animation loop  */
GtkWidget *gEntryAnimFrameIni;       /* Initial frame for internal animation  */
GtkWidget *gEntryAnimFrameFin;       /* Final frame for internal animation  */
GtkWidget *gEntryAnimClockIni;       /* Initial clock for internal animation  */
GtkWidget *gEntryAnimClockFin;       /* Final clock for internal animation  */
GtkWidget *gEntryAnimSubsetIni;      /* Initial frame for subset internal animation  */
GtkWidget *gEntryAnimSubsetFin;      /* Final frame for subset internal animation  */
GtkWidget *gEntryPreScene;
GtkWidget *gEntryPreFrame;
GtkWidget *gEntryPostScene;
GtkWidget *gEntryPostFrame;
GtkWidget *gEntryUserAbort;
GtkWidget *gEntryFatalError;
GtkWidget *gEntryDebugStream;        /* output messages stream to console or file  */
GtkWidget *gEntryFatalStream;        /* output messages stream to console or file  */
GtkWidget *gEntryRenderStream;       /* output messages stream to console or file  */
GtkWidget *gEntryStatStream;         /* output messages stream to console or file  */
GtkWidget *gEntryWarningStream;      /* output messages stream to console or file  */
GtkWidget *gEntryAllStream;          /* output messages stream to console or file  */
GtkWidget *gEntryCommand;            /* Command to invoque to run the rendering  */
GtkWidget *gEntryAltCommand;         /* Alternate command to invoque to run the rendering  */
GtkWidget *g_combo_histo_type;       /* Combo for the file type of histogram file  */
GtkWidget *g_combo_out_file;            /* Combo for the file type of the image  */
GtkWidget *gCheckAntialiasing;
GtkWidget *gCheckPovFile;
GtkWidget *gCheckSub;
GtkWidget *gCheckOutPreview;
GtkWidget *gCheckOut1;               /* Test abort  */
GtkWidget *gCheckOut2;               /* Continue flag  */
GtkWidget *gCheckOut3;               /* Pause flag  */
GtkWidget *gCheckOut4;               /* Verbose flag  */
GtkWidget *gCheckOut5;               /* Draw vista flag  */
GtkWidget *gCheckOut6;               /* Display image flag  */
GtkWidget *gCheckOut7;               /* Alpha out flag  */
GtkWidget *gCheckIniFile;
GtkWidget *gCheckOutFile;
GtkWidget *gCheckQuality1;
GtkWidget *gCheckQuality2;
GtkWidget *gCheckQuality3;
GtkWidget *gCheckQuality4;
GtkWidget *gCheckQuality5;
GtkWidget *gCheckQuality6;
GtkWidget *gCheck_jitter;
GtkWidget *gCheckHistoFile;
GtkWidget *gCheckAnimCyclic;
GtkWidget *gCheckAnimField;
GtkWidget *gCheckAnimOdd;
GtkWidget *gCheckDebugStream;
GtkWidget *gCheckFatalStream;
GtkWidget *gCheckRenderStream;
GtkWidget *gCheckStatStream;
GtkWidget *gCheckWarningStream;
GtkWidget *gCheckAllStream;
GtkObject *g_adjust_Threshold;
GtkObject *g_adjust_Depth;
GtkObject *g_adjust_Jitter;
GtkObject *g_adjust_Quality;
GtkWidget *g_radioAA1;
GtkWidget *g_radioAA2;
GtkWidget* g_radioOutSize1;
GtkWidget* g_radioOutSize2;
GtkWidget* g_radioOutSize3;
GtkWidget* g_radioOutSize4;
GtkWidget* g_radioOutSize5;
GtkWidget* g_radioOutSize6;
GtkWidget* g_radioOutSize7;
GtkWidget* g_radioOutSize8;
GtkWidget* g_radioOutSize9;
GtkWidget* g_radioOutSize10;
GtkWidget* g_radioOutColor1;
GtkWidget* g_radioOutColor2;
GtkWidget* g_radioOutColor3;
GtkWidget *g_gtklist_jobs;

gint gInputPovFile = 0;         /* Tell if the ommand take a povfile as input  */
gint gOutputIniFile = 0;        /* Tell if povray must provide a ini file  */
gint gOutputImage = 0;          /* Tell if povray write an image on the disk  */
gint gTestAbort = 1;            /* test abort while rendering  */
gint gContinue = 0;             /* retrieve last begining of rendering  */
gint gPause = 1;                /* test pause while rendering  */
gint gVerbose = 0;              /* verbose rendering  */
gint gDrawVista = 0;            /* draw vista  */
gint gDisplayImage = 1;         /* Display image while rendering  */
gint gAlphaOut = 0;             /* Output alpha channel  */
gint gSubRender = 0;            /* Subrendering of the image  */
gint gPreview = 0;              /* preview flag  */
gint gRadiosity = 0;            /* Radiosity flag  */
gint gBoundingBox = 0;          /* Bounding box flag  */
gint gLightBuffer = 0;          /* Light buffer flag  */
gint gVistaBuffer = 0;          /* Vista buffer flag  */
gint gRemoveBounding = 0;       /* Remove bounds flag  */
gint gSplit = 0;	        /* Split Union flag  */
gint gAntialiasing = 0;	        /* Antialiasing flag  */
gint gOutputHistoFile = 0;      /* Outpout Histogram file flag  */
gint gDebugStream = 0;          /* Output messages stream to console or file  */
gint gFatalStream = 0;          /* Output messages stream to console or file  */
gint gRenderStream = 0;         /* Output messages stream to console or file  */
gint gStatStream = 0;           /* Output messages stream to console or file  */
gint gWarningStream = 0;        /* Output messages stream to console or file  */
gint gAllStream = 0;            /* Output messages stream to console or file  */
gint g_jitter = 1;              /* Flag = 1 if jittering  */
gint g_field_render = 0;        /* Flag for field rendering in animation  */
gint g_odd_field = 0;           /* Flag for entrelace rendering in animation  */
gint g_cyclic = 0;              /* Flag for cyclic animation  */
gint gSaveSettingsExit = 0;     /* Save settings as default when exit  */
gint gSaveSettingsExitLocal = 0;     /* Save settings as default when exit  */
gint gShowTooltips = 1;         /* Show tooltips options  */
gint gShowTooltipsLocal = 1;         /* Show tooltips options  */
gint gTestOverwrite = 1;        /* Test Overwrite of iage output  */
gint gTestOverwriteLocal = 1;        /* Test Overwrite of iage output  */

gchar g_entry_save_path[256];   /* Current pah for saving configuration file  */
gchar g_entry_load_path[256];   /* Current pah for sloading configuration file  */
gchar g_entry_pov_version[10];  /* Version of povray engine  */
gchar gImageSize [20];          /* The povray parameters for the image size  */
gchar gAAmethode [2];           /* AntiAliasing methode: 1 or 2  */
gint gSizeType;                 /* The number of the size radio button activated  */
gchar gPalette [2];             /* Code for palette color;  */
gchar g_command_to_invoque[50]; /* Command to invoque povray  */
gchar g_alt_command_to_invoque[50];/* Alternate command to invoque povray  */
gint  g_invoque_command_number; /* Number of the command to invoque (radio button number)  */
gint  g_local_invoque_command_number; /* The same but before apply  */
gfloat gQualityValue;           /* Quality level for rendering  */
gfloat g_jitter_value = 1.0;    /* Value for the jittering  */
gfloat gQualityDepth;           /* Depth of the antialiasing  */
gfloat gQualityThreshold;       /* Threshold of the Antialising  */
char g_pov_file[256];           /* pov file in the command line  */
gint g_plugin = 0;              /* Set to 1 if the pov file name is on the command line  */
gint g_image_plugin = 0;        /* Set to 1 if the image file name is on the command line  */
gchar g_home_directory[256];    /* Home directory of the user  */
gchar g_default_config_file[256]; /* File of the default configguration with the path  */
gchar g_config_file_to_parse [256];
gchar g_image_file_to_compute [256]; /* Image File to produce when called as a plugin  */
pid_t g_pid_last_process;
GdkColor g_tooltips_background_color;
GdkColor g_tooltips_foreground_color;


/* Definition to handle the jobs  */
struct _job {
  pid_t pid;
  gint state;
  gchar *time;
  gchar *file;
  gchar *image;
};

typedef struct _job job;
GList *g_jobs_list;              /* List of jobs used by the job control  */
GList *g_tooltips_list;          /* List used to store the tooltips  */

typedef enum
{
  POVF_RUNNING,
  POVF_HALTED,
  POVF_ABORTED,
  POVF_FINISHED,
  POVF_DONE,
}proc_mode;

void Render2 (GtkWidget *widget, gpointer);
void Render (GtkWidget *widget, gpointer);
void SwitchOption (GtkWidget *widget, gint *pSwitch);
void write_default_config (GtkWidget *widget, gpointer *p_data);
void SwitchOption (GtkWidget *widget, gint *pSwitch);
void InitProgramme ();
gint write_config_file (gchar *p_selected_file);
gint load_config_file (gchar *p_file);
void povfront_message (gchar *pData);
void refresh_list (GtkWidget *p_gtklist);
