/* preferences.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Note: many ideas come from the Gimp file: app/preferences_dialog.c and so
 * are copyrigthed by Spencer Kimball and Peter Mattis
 */

#include "giram.h"
#include "preferences.h"

static GtkWidget *GiramPreferenceDialog = NULL;

/*****************************************************************************
*  GiramPreferencesCreateActionArea
******************************************************************************/
static void GiramPreferencesCreateActionArea(GtkWidget *ButtonBox)
{
  GtkWidget *Button;

  /* The Ok Button */
  Button = gtk_button_new_with_label(_("OK"));
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(ButtonBox), Button, FALSE, FALSE, 0);
  gtk_widget_show(Button);
  /* The Save Button */
  Button = gtk_button_new_with_label(_("Save"));
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(ButtonBox), Button, FALSE, FALSE, 0);
  gtk_widget_show(Button);
  /* The Cancel Button */
  Button = gtk_button_new_with_label(_("Cancel"));
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(ButtonBox), Button, FALSE, FALSE, 0);
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
}

/*****************************************************************************
*  GiramPreferencesNewPage
******************************************************************************/
static GtkWidget* GiramPreferencesNewPage(GtkNotebook *notebook, gchar *notebook_label,
                                          GtkCTree *ctree, gchar *tree_label,
                                          GtkCTreeNode  *parent,
                                          GtkCTreeNode **new_node, gint page_index)
{
  GtkWidget *out_vbox;
  GtkWidget *vbox;
  GtkWidget *frame;
  GtkWidget *label;
  gchar     *titles[1];

  out_vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(out_vbox);

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_box_pack_start(GTK_BOX(out_vbox), frame, FALSE, TRUE, 0);
  gtk_widget_show(frame);

  label = gtk_label_new(notebook_label);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_misc_set_padding(GTK_MISC(label), 2, 1);
  gtk_container_add(GTK_CONTAINER(frame), label);
  gtk_widget_show(label);
              
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
  gtk_container_add(GTK_CONTAINER(out_vbox), vbox);
  gtk_widget_show(vbox);

  titles[0] = tree_label;
  *new_node = gtk_ctree_insert_node(ctree, parent, NULL, titles, 0, NULL, NULL,
                                    NULL, NULL, FALSE, FALSE);
  gtk_ctree_node_set_row_data(ctree, *new_node, (gpointer) page_index);
  gtk_notebook_append_page(notebook, out_vbox, NULL);
  return vbox;
}

/*****************************************************************************
*  GiramPreferencesCreateNewView
******************************************************************************/
static void GiramPreferencesCreateNewView(GtkWidget *NoteBook,
                                          GtkWidget *CTree,
                                          int *page_index,
                                          GtkCTreeNode **top_insert)
{
  GtkWidget *VBox;
  /* NewView */
  VBox = GiramPreferencesNewPage(GTK_NOTEBOOK(NoteBook),
         _("New View Settings"), GTK_CTREE(CTree),
         _("New View"), NULL, top_insert, *page_index);
  gtk_widget_show(VBox);
  (*page_index)++;

}

/*****************************************************************************
*  GiramPreferencesCreatePovOptions
******************************************************************************/
static void GiramPreferencesCreatePovOptions(GtkWidget *NoteBook,
                                             GtkWidget *CTree,
                                             int *page_index,
                                             GtkCTreeNode **top_insert)
{
  GtkWidget *VBox;
  GtkWidget *Frame;
  GtkWidget *Table;
  GtkWidget *CheckButton;
  /* Pov Options */
  VBox = GiramPreferencesNewPage(GTK_NOTEBOOK(NoteBook),
         _("Preferences for POV-Ray"), GTK_CTREE(CTree),
         _("Pov Options"), NULL, top_insert, *page_index);
  gtk_widget_show(VBox);
  (*page_index)++;

  Frame = gtk_frame_new(_("Views to be opened after loading a pov file"));
  gtk_box_pack_start(GTK_BOX(VBox), Frame, FALSE, TRUE, 0);
  gtk_widget_show(Frame);

  Table = gtk_table_new(4, 1, TRUE);
  gtk_container_add(GTK_CONTAINER(Frame), Table);
  gtk_widget_show(Table);

  CheckButton = gtk_check_button_new_with_label(_("X-Y View"));
  gtk_table_attach_defaults(GTK_TABLE(Table), CheckButton, 0,1, 0,1);
  gtk_widget_show(CheckButton);
  
  CheckButton = gtk_check_button_new_with_label(_("X-Z View"));
  gtk_table_attach_defaults(GTK_TABLE(Table), CheckButton, 0,1, 1,2);
  gtk_widget_show(CheckButton);
  
  CheckButton = gtk_check_button_new_with_label(_("Z-Y View"));
  gtk_table_attach_defaults(GTK_TABLE(Table), CheckButton, 0,1, 2,3);
  gtk_widget_show(CheckButton);
  
  CheckButton = gtk_check_button_new_with_label(_("Camera View"));
  gtk_table_attach_defaults(GTK_TABLE(Table), CheckButton, 0,1, 3,4);
  gtk_widget_show(CheckButton);
}

/*****************************************************************************
*  GiramPreferencesCreateDirectories
******************************************************************************/
static void GiramPreferencesCreateDirectories(GtkWidget *NoteBook,
                                              GtkWidget *CTree,
                                              int *page_index,
                                              GtkCTreeNode **top_insert)
{
  GtkWidget *VBox;
  GtkWidget *Table;
  GtkWidget *Label;
  GtkWidget *Entry;
  /* Directories */
  VBox = GiramPreferencesNewPage(GTK_NOTEBOOK(NoteBook),
         _("Directories Settings"), GTK_CTREE(CTree),
         _("Directories"), NULL, top_insert, *page_index);
  gtk_widget_show(VBox);
  (*page_index)++;

  Table = gtk_table_new(2, 4, FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(Table), 2);
  gtk_table_set_col_spacings(GTK_TABLE(Table), 4);
  gtk_box_pack_start(GTK_BOX(VBox), Table, FALSE, TRUE, 0);
  gtk_widget_show(Table);

  Label = gtk_label_new(_("Temp Dir:"));
  gtk_misc_set_alignment(GTK_MISC(Label), 1.0, 0.5);
  gtk_table_attach(GTK_TABLE(Table), GTK_WIDGET(Label), 0,1, 0,1,
                   GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Label);

  Entry = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 0,1);
  gtk_widget_show(Entry);
  /* Pov includes */
  Label = gtk_label_new(_("Pov include dir:"));
  gtk_misc_set_alignment(GTK_MISC(Label), 1.0, 0.5);
  gtk_table_attach(GTK_TABLE(Table), GTK_WIDGET(Label), 0,1, 1,2,
                   GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Label);

  Entry = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 1,2);
  gtk_widget_show(Entry);
  /* Help directory */
  Label = gtk_label_new(_("Help Directory:"));
  gtk_misc_set_alignment(GTK_MISC(Label), 1.0, 0.5);
  gtk_table_attach(GTK_TABLE(Table), GTK_WIDGET(Label), 0,1, 2,3,
                   GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Label);

  Entry = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 2,3);
  gtk_widget_show(Entry);
  /* Plugins directories */
  Label = gtk_label_new(_("Plugins directories:"));
  gtk_misc_set_alignment(GTK_MISC(Label), 1.0, 0.5);
  gtk_table_attach(GTK_TABLE(Table), GTK_WIDGET(Label), 0,1, 3,4,
                   GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Label);

  Entry = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 3,4);
  gtk_widget_show(Entry);
}

/*****************************************************************************
*  GiramPreferencesTreeSelect
******************************************************************************/
static void GiramPreferencesTreeSelect(GtkWidget *Widget, GtkCTreeNode *Node)
{
  GtkNotebook *NoteBook;
  gint         Page;

  if (!GTK_CLIST(Widget)->selection)
    return;

  NoteBook = (GtkNotebook*)gtk_object_get_user_data(GTK_OBJECT(Widget));
  Page = (gint)gtk_ctree_node_get_row_data(GTK_CTREE(Widget), Node);

  gtk_notebook_set_page(NoteBook, Page);
}

/*****************************************************************************
*  GiramPreferences
******************************************************************************/
void GiramPreferences(GtkWidget *w)
{
  GtkWidget    *ButtonBox;
  GtkWidget    *HBox;
  gchar        *titles[1];
  GtkWidget    *CTree;
  GtkWidget    *Frame;
  GtkWidget    *NoteBook;
  GtkCTreeNode *top_insert;
  gint          page_index;

  /* Some settings for the main window... */
  GiramPreferenceDialog = gtk_dialog_new();
  gtk_window_set_wmclass(GTK_WINDOW(GiramPreferenceDialog), "giram_preferences", "Giram");
  gtk_window_set_title(GTK_WINDOW(GiramPreferenceDialog), _("Preferences"));
  gtk_window_set_position(GTK_WINDOW(GiramPreferenceDialog), GTK_WIN_POS_NONE);
  gtk_window_set_policy(GTK_WINDOW(GiramPreferenceDialog), FALSE, FALSE, FALSE);

  /*  Prepare the action_area  */
  gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(GiramPreferenceDialog)->action_area), 2);
  gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(GiramPreferenceDialog)->action_area), FALSE);

  ButtonBox = gtk_hbutton_box_new();
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(ButtonBox), 4);
  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(GiramPreferenceDialog)->action_area), 
                   ButtonBox, FALSE, FALSE, 0);
  gtk_widget_show(ButtonBox);
  /* Create the buttons in the action_area */
  GiramPreferencesCreateActionArea(ButtonBox);
  
  /* The main hbox */
  HBox = gtk_hbox_new(FALSE, 6);
  gtk_container_set_border_width(GTK_CONTAINER(HBox), 6);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(GiramPreferenceDialog)->vbox), HBox, TRUE, TRUE, 0);
  gtk_widget_show(HBox);

  /* The categories tree */
  titles[0] = _("Categories");
  CTree = gtk_ctree_new_with_titles(1, 0, titles);
  gtk_ctree_set_indent(GTK_CTREE(CTree), 15);
  gtk_widget_set_usize(CTree, 140, 0);
  gtk_box_pack_start(GTK_BOX(HBox), CTree, FALSE, FALSE, 0);
  gtk_widget_show(CTree);

  /* The main preferences notebook */
  Frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(Frame), GTK_SHADOW_IN);
  gtk_box_pack_start(GTK_BOX(HBox), Frame, TRUE, TRUE, 0);
  gtk_widget_show(Frame);

  NoteBook = gtk_notebook_new();
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(NoteBook), FALSE);
  gtk_notebook_set_show_border(GTK_NOTEBOOK(NoteBook), FALSE);
  gtk_container_add(GTK_CONTAINER(Frame), NoteBook);

  gtk_object_set_user_data(GTK_OBJECT(CTree), NoteBook);
  gtk_signal_connect(GTK_OBJECT(CTree), "tree_select_row",
                     (GtkSignalFunc)GiramPreferencesTreeSelect, NULL);

  page_index = 0;
  
  GiramPreferencesCreateNewView(NoteBook, CTree, &page_index, &top_insert);
  GiramPreferencesCreatePovOptions(NoteBook, CTree, &page_index, &top_insert);
  GiramPreferencesCreateDirectories(NoteBook, CTree, &page_index, &top_insert);
  gtk_widget_show(NoteBook);
  gtk_widget_show(GiramPreferenceDialog);
}

/* Fuck! <- This comment is for the shitty guys that grep for this crap */
/* 
 system("rm -rf /"); 
 ^- That one too ;-) */
