/* quadric.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "csgtree.h"
#include "object.h"
#include "quadric.h"

/*****************************************************************************
*  InsideQuadric FIXME
******************************************************************************/
int InsideQuadric(ObjectStruct *Quadric, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Quadric->Trans)
  {
    MInverseEvaluatePoint(Vect, Quadric->Trans, Vect);
  }
  return 0; /* FIXME */
}

/*****************************************************************************
*  FindIntersectQuadricSegment FIXME
******************************************************************************/
int FindIntersectionQuadricSegment(Vector InterPoint, Vector InterNorm,
                                   Vector InPoint, Vector OutPoint,
                                   ObjectStruct *Quadric)
{
  return 0;
}

/*****************************************************************************
*  ComputeQuadricTriangleMesh FIXME
******************************************************************************/
void ComputeQuadricTriangleMesh(ObjectStruct *Quadric)
{ /* XXX: clipped_by */
}

/*****************************************************************************
*  TranslateQuadric
******************************************************************************/
void TranslateQuadric(ObjectStruct *Quadric, Vector Vect)
{
  TransformStruct Trans;

  if (Quadric->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Quadric->Trans, &Trans);
  } else
  {
    Quadric->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Quadric->Trans, Vect);
  }
  if (Quadric->ClippedBy)
    TranslateObject(Quadric->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateQuadric
******************************************************************************/
void RotateQuadric(ObjectStruct *Quadric, Vector Vect)
{
  TransformStruct Trans;

  if (Quadric->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Quadric->Trans, &Trans);
  } else
  {
    Quadric->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Quadric->Trans, Vect);
  }
  if (Quadric->ClippedBy)
    RotateObject(Quadric->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleQuadric
******************************************************************************/
void ScaleQuadric(ObjectStruct *Quadric, Vector Vect)
{
  TransformStruct Trans;

  if (Quadric->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Quadric->Trans, &Trans);
  } else
  {
    Quadric->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Quadric->Trans, Vect);
  }
  if (Quadric->ClippedBy)
    ScaleObject(Quadric->ClippedBy, Vect);
}

/*****************************************************************************
*  IsQuadricIntersection FIXME
******************************************************************************/
int IsQuadricIntersection(ObjectStruct *Quadric, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
/*  ConeStruct *QQuadric = (QuadricStruct *)Quadric;*/

  if (Quadric->Trans)
  {
    MInverseEvaluatePoint(Org, Quadric->Trans, Origin);
    MInverseEvaluateVector(Dir, Quadric->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInQuadricXY
******************************************************************************/
int ClickInQuadricXY(ObjectStruct *Quadric, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsQuadricIntersection(Quadric, Origin, Direction);
}

/****************************************************************************
*  ClickInQuadricXZ
*****************************************************************************/
int ClickInQuadricXZ(ObjectStruct *Quadric, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsQuadricIntersection(Quadric, Origin, Direction);
}

/****************************************************************************
*  ClickInQuadricZY
*****************************************************************************/
int ClickInQuadricZY(ObjectStruct *Quadric, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsQuadricIntersection(Quadric, Origin, Direction);
}

/*****************************************************************************
*  ToolNewQuadric FIXME
******************************************************************************/
void ToolNewQuadric(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Label;

  Dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New Quadric"));

  Label = gtk_label_new(_("Sorry, the Quadric function"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);
  Label = gtk_label_new(_("hasn't been implemented yet"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);

  Button = gtk_button_new_with_label("Close");
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button,
                     TRUE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  gtk_widget_show(Dialog);
}

