/* sphere.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "snap.h"
#include "sphere.h"

/*****************************************************************************
*  InsideSphere
******************************************************************************/
int InsideSphere(ObjectStruct *Sphere, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Sphere->Trans)
  {
    MInverseEvaluatePoint(Vect, Sphere->Trans, Vect);
  }
  Vect[0] -= ((SphereStruct *)Sphere)->Center[0];
  Vect[1] -= ((SphereStruct *)Sphere)->Center[1];
  Vect[2] -= ((SphereStruct *)Sphere)->Center[2];
  if (Vect[0]*Vect[0]+Vect[1]*Vect[1]+Vect[2]*Vect[2] > 
           (((SphereStruct *)Sphere)->Radius * 
            ((SphereStruct *)Sphere)->Radius))
    return Sphere->Inverse;
  else
    return !(Sphere->Inverse);
}
  
/*****************************************************************************
*  SphereMeshNormal
******************************************************************************/
void SphereMeshNormal(Vector Result, ObjectStruct *Sphere, Vector Inter)
{
  double Norme;

  Result[0] = Inter[0] - ((SphereStruct *)Sphere)->Center[0];
  Result[1] = Inter[1] - ((SphereStruct *)Sphere)->Center[1];
  Result[2] = Inter[2] - ((SphereStruct *)Sphere)->Center[2];
  Norme = VLength(Result);
  Result[0] /= Norme;
  Result[1] /= Norme;
  Result[2] /= Norme;
  if (Sphere->Inverse)
  {
    Result[0] = -Result[0];
    Result[1] = -Result[1];
    Result[2] = -Result[2];
  }
}

/*****************************************************************************
*  ComputeSphereTriangleMesh
******************************************************************************/
void ComputeSphereTriangleMesh(ObjectStruct *Sphere)
{ /* XXX: clipped_by */
  Vector P1, P2, P3, P4;
  Vector N1, N2, N3, N4;
  int i,j;
  int DetailLevel;
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  DetailLevel = 16; /* FIXME: should be a private field */

  if (Sphere->FirstTriangle)
    DestroyObjectTriangleMesh(Sphere);
  /* First the North Pole */
  P1[0] = SSphere->Center[0];
  P1[1] = SSphere->Center[1]+SSphere->Radius;
  P1[2] = SSphere->Center[2];
  SphereMeshNormal(N1, Sphere, P1);
  if (Sphere->Trans)
  {
    MEvaluatePoint(P1, Sphere->Trans, P1);
    MEvaluateVector(N1, Sphere->Trans, N1);
  }
  for (i=0; i<DetailLevel; i++)
  { /* The "DetailLevel" triangles adjacents to the pole */
    P2[0] = SSphere->Center[0]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *cos(i*2.0*M_PI/DetailLevel);
    P2[1] = SSphere->Center[1]+SSphere->Radius*cos(M_PI/DetailLevel);
    P2[2] = SSphere->Center[2]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *sin(i*2.0*M_PI/DetailLevel);
    SphereMeshNormal(N2, Sphere, P2);
    if (Sphere->Trans)
    {
      MEvaluatePoint(P2, Sphere->Trans, P2);
      MEvaluateVector(N2, Sphere->Trans, N2);
    }
    P3[0] = SSphere->Center[0]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *cos((i+1)*2.0*M_PI/DetailLevel);
    P3[1] = SSphere->Center[1]+SSphere->Radius*cos(M_PI/DetailLevel);
    P3[2] = SSphere->Center[2]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *sin((i+1)*2.0*M_PI/DetailLevel);
    SphereMeshNormal(N3, Sphere, P3);
    if (Sphere->Trans)
    {
      MEvaluatePoint(P3, Sphere->Trans, P3);
      MEvaluateVector(N3, Sphere->Trans, N3);
    }
    AddTriangleToObjectMesh(Sphere, P1,P2,P3, N1,N2,N3);
  }
  /* Then the Mid Zone */
  for (i=1 ; i<DetailLevel-1 ; i++)
  {
    for (j=0 ; j<DetailLevel ; j++)
    {
      P1[0] = SSphere->Center[0]+SSphere->Radius*sin(i*M_PI/DetailLevel)
                                                *cos(j*2.0*M_PI/DetailLevel);
      P1[1] = SSphere->Center[1]+SSphere->Radius*cos(i*M_PI/DetailLevel);
      P1[2] = SSphere->Center[2]+SSphere->Radius*sin(i*M_PI/DetailLevel)
                                                *sin(j*2.0*M_PI/DetailLevel);
      SphereMeshNormal(N1, Sphere, P1);
      if (Sphere->Trans)
      {
        MEvaluatePoint(P1, Sphere->Trans, P1);
        MEvaluateVector(N1, Sphere->Trans, N1);
      }
      P2[0] = SSphere->Center[0]+SSphere->Radius*sin((i+1)*M_PI/DetailLevel)
                                                *cos(j*2.0*M_PI/DetailLevel);
      P2[1] = SSphere->Center[1]+SSphere->Radius*cos((i+1)*M_PI/DetailLevel);
      P2[2] = SSphere->Center[2]+SSphere->Radius*sin((i+1)*M_PI/DetailLevel)
                                                *sin(j*2.0*M_PI/DetailLevel);
      SphereMeshNormal(N2, Sphere, P2);
      if (Sphere->Trans)
      {
        MEvaluatePoint(P2, Sphere->Trans, P2);
        MEvaluateVector(N2, Sphere->Trans, N2);
      }
      P3[0] = SSphere->Center[0]+SSphere->Radius*sin(i*M_PI/DetailLevel)
                                              *cos((j+1)*2.0*M_PI/DetailLevel);
      P3[1] = SSphere->Center[1]+SSphere->Radius*cos(i*M_PI/DetailLevel);
      P3[2] = SSphere->Center[2]+SSphere->Radius*sin(i*M_PI/DetailLevel)
                                              *sin((j+1)*2.0*M_PI/DetailLevel);
      SphereMeshNormal(N3, Sphere, P3);
      if (Sphere->Trans)
      {
        MEvaluatePoint(P3, Sphere->Trans, P3);
        MEvaluateVector(N3, Sphere->Trans, N3);
      }
      P4[0] = SSphere->Center[0]+SSphere->Radius*sin((i+1)*M_PI/DetailLevel)
                                              *cos((j+1)*2.0*M_PI/DetailLevel);
      P4[1] = SSphere->Center[1]+SSphere->Radius*cos((i+1)*M_PI/DetailLevel);
      P4[2] = SSphere->Center[2]+SSphere->Radius*sin((i+1)*M_PI/DetailLevel)
                                              *sin((j+1)*2.0*M_PI/DetailLevel);
      SphereMeshNormal(N4, Sphere, P4);
      if (Sphere->Trans)
      {
        MEvaluatePoint(P4, Sphere->Trans, P4);
        MEvaluateVector(N4, Sphere->Trans, N4);
      }
      AddTriangleToObjectMesh(Sphere, P1,P2,P4, N1,N2,N4);
      AddTriangleToObjectMesh(Sphere, P1,P3,P4, N1,N3,N4);
    }
  }
  /* And the South Pole */
  P1[0]=SSphere->Center[0];
  P1[1]=SSphere->Center[1]-SSphere->Radius;
  P1[2]=SSphere->Center[2];
  SphereMeshNormal(N1, Sphere, P1);
  if (Sphere->Trans)
  {
    MEvaluatePoint(P1, Sphere->Trans, P1);
    MEvaluateVector(N1, Sphere->Trans, N1);
  }
  for (i=0 ; i<DetailLevel ; i++)
  { /* The "DetailLevel" triangles adjacents to the south pole */
    P2[0] = SSphere->Center[0]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *cos(i*2.0*M_PI/DetailLevel);
    P2[1] = SSphere->Center[1]-SSphere->Radius*cos(M_PI/DetailLevel);
    P2[2] = SSphere->Center[2]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *sin(i*2.0*M_PI/DetailLevel);
    SphereMeshNormal(N2, Sphere, P2);
    if (Sphere->Trans)
    {
      MEvaluatePoint(P2, Sphere->Trans, P2);
      MEvaluateVector(N2, Sphere->Trans, N2);
    }
    P3[0] = SSphere->Center[0]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *cos((i+1)*2.0*M_PI/DetailLevel);
    P3[1] = SSphere->Center[1]-SSphere->Radius*cos(M_PI/DetailLevel);
    P3[2] = SSphere->Center[2]+SSphere->Radius*sin(M_PI/DetailLevel)
                                              *sin((i+1)*2.0*M_PI/DetailLevel);
    SphereMeshNormal(N3, Sphere, P3);
    if (Sphere->Trans)
    {
      MEvaluatePoint(P3, Sphere->Trans, P3);
      MEvaluateVector(N3, Sphere->Trans, N3);
    }
    AddTriangleToObjectMesh(Sphere, P1,P2,P3, N1,N2,N3);
  }
}

/*****************************************************************************
*  FindIntersectionSphereSegment
******************************************************************************/
int FindIntersectionSphereSegment(Vector InterPoint, Vector InterNorm,
                                  Vector InPoint, Vector OutPoint,
                                  ObjectStruct *Sphere)
{
  Vector InP, OutP;
  Vector Vd;
  double Norme;
  double B, C, t;
  SphereStruct *SSphere = (SphereStruct *)Sphere;
  int i;

  if (Sphere->Trans)
  {
    MInverseEvaluatePoint(InP, Sphere->Trans, InPoint);
    MInverseEvaluatePoint(OutP, Sphere->Trans, OutPoint);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      InP[i] = InPoint[i];
      OutP[i] = OutPoint[i];
    }
  }
  Vd[0] = InP[0] - OutP[0]; Vd[1] = InP[1] - OutP[1]; Vd[2] = InP[2] - OutP[2];
  Norme = VLength(Vd);
  Vd[0] /= Norme; Vd[1] /= Norme; Vd[2] /= Norme;
  B = 2 * ( Vd[0]*(OutP[0]-SSphere->Center[0])+
            Vd[1]*(OutP[1]-SSphere->Center[1])+
            Vd[2]*(OutP[2]-SSphere->Center[2]));
  C = (OutP[0]-SSphere->Center[0])*(OutP[0]-SSphere->Center[0]) +
      (OutP[1]-SSphere->Center[1])*(OutP[1]-SSphere->Center[1]) +
      (OutP[2]-SSphere->Center[2])*(OutP[2]-SSphere->Center[2])
      - SSphere->Radius * SSphere->Radius;
  t = (- B - sqrt(B*B - 4.0*C)) / 2.0;
  if (t >= 0.0 && t < Norme)
  {
    InterPoint[0] = OutP[0]+t*Vd[0];
    InterPoint[1] = OutP[1]+t*Vd[1];
    InterPoint[2] = OutP[2]+t*Vd[2];
    SphereMeshNormal(InterNorm, Sphere, InterPoint);
    if (Sphere->Trans)
    {
      MEvaluatePoint(InterPoint, Sphere->Trans, InterPoint);
      MEvaluateVector(InterNorm, Sphere->Trans, InterNorm);
    }
    return 1;
  }
  t = (- B + sqrt(B*B - 4.0*C)) / 2.0;
  if (t >= 0.0 && t < Norme)
  {
    InterPoint[0] = OutP[0]+t*Vd[0];
    InterPoint[1] = OutP[1]+t*Vd[1];
    InterPoint[2] = OutP[2]+t*Vd[2];
    SphereMeshNormal(InterNorm, Sphere, InterPoint);
    if (Sphere->Trans)
    {
      MEvaluatePoint(InterPoint, Sphere->Trans, InterPoint);
      MEvaluateVector(InterNorm, Sphere->Trans, InterNorm);
    }
    return 1;
  }
  
  return 0;
}

/*****************************************************************************
*  TranslateSphere
******************************************************************************/
void TranslateSphere(ObjectStruct *Sphere, Vector Vect)
{
  TransformStruct Trans;
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  if (Sphere->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Sphere->Trans, &Trans);
  } else
  {
    SSphere->Center[0] += Vect[0];
    SSphere->Center[1] += Vect[1];
    SSphere->Center[2] += Vect[2];
  }
  if (Sphere->ClippedBy)
    TranslateObject(Sphere->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateSphere
******************************************************************************/
void RotateSphere(ObjectStruct *Sphere, Vector Vect)
{
  TransformStruct Trans;
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  ComputeRotateTrans(&Trans, Vect);
  if (Sphere->Trans)
  {
    ComposeTrans(Sphere->Trans, &Trans);
  } else
  { 
    MEvaluatePoint(SSphere->Center, &Trans, SSphere->Center);
  }
  if (Sphere->ClippedBy)
    RotateObject(Sphere->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleSphere
******************************************************************************/
void ScaleSphere(ObjectStruct *Sphere, Vector Vect)
{
  TransformStruct Trans;
  SphereStruct *SSphere = (SphereStruct *)Sphere;
  
  if (Sphere->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Sphere->Trans, &Trans);
  } else
  { 
    if (Vect[0] != Vect[1] || Vect[0] != Vect[2])
    {
      Sphere->Trans = g_malloc(sizeof(TransformStruct));
      ComputeScaleTrans(Sphere->Trans, Vect);
    } else
    {
      SSphere->Center[0] *= Vect[0];
      SSphere->Center[1] *= Vect[0];
      SSphere->Center[2] *= Vect[0];
      SSphere->Radius *= Vect[0];
    }
  }
  if (Sphere->ClippedBy)
    ScaleObject(Sphere->ClippedBy, Vect);
}

/*****************************************************************************
*  IsSphereIntersection
******************************************************************************/
int IsSphereIntersection(ObjectStruct *Sphere, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  Vector OriginToCenter;
  int i;
  double Length;
  double Tca;
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  if (Sphere->Trans)
  {
    MInverseEvaluatePoint(Org, Sphere->Trans, Origin);
    MInverseEvaluateVector(Dir, Sphere->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  OriginToCenter[0] = SSphere->Center[0] - Org[0];
  OriginToCenter[1] = SSphere->Center[1] - Org[1];
  OriginToCenter[2] = SSphere->Center[2] - Org[2];
  Length = VLength(Dir);
  Dir[0] /= Length; Dir[1] /= Length; Dir[2] /= Length;
  Tca = VDot(OriginToCenter, Dir);
  
  if (VDot(OriginToCenter, OriginToCenter) - Tca*Tca < 
      SSphere->Radius * SSphere->Radius)
    return 1;
  else
    return 0;
}

/*****************************************************************************
*  ClickInSphereXY
******************************************************************************/
int ClickInSphereXY(ObjectStruct *Sphere, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  if (Sphere->Trans)
  {
    Origin[0] = x;
    Origin[1] = y;
    return IsSphereIntersection(Sphere, Origin, Direction);
  } else if ( (x-SSphere->Center[0])*(x-SSphere->Center[0])+
              (y-SSphere->Center[1])*(y-SSphere->Center[1]) <
              SSphere->Radius * SSphere->Radius )
    return 1;
  else
    return 0;
}

/*****************************************************************************
*  ClickInSphereXZ
******************************************************************************/
int ClickInSphereXZ(ObjectStruct *Sphere, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  if (Sphere->Trans)
  {
    Origin[0] = x;
    Origin[2] = y;
    return IsSphereIntersection(Sphere, Origin, Direction);
  } else if ( (x-SSphere->Center[0])*(x-SSphere->Center[0])+
              (y-SSphere->Center[2])*(y-SSphere->Center[2]) <
              SSphere->Radius * SSphere->Radius )
    return 1;
  else
    return 0;
}

/*****************************************************************************
*  ClickInSphereZY
******************************************************************************/
int ClickInSphereZY(ObjectStruct *Sphere, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};
  SphereStruct *SSphere = (SphereStruct *)Sphere;

  if (Sphere->Trans)
  {
    Origin[2] = x;
    Origin[1] = y;
    return IsSphereIntersection(Sphere, Origin, Direction);
  } else if ( (x-SSphere->Center[2])*(x-SSphere->Center[2])+
              (y-SSphere->Center[1])*(y-SSphere->Center[1]) <
              SSphere->Radius * SSphere->Radius )
    return 1;
  else
    return 0;
}

double Xorg, Yorg, Radius;
GdkColormap *colormap;
GdkGC *gcPurple;
GdkColor Purple;
guint id;

#define NONE   1
#define MOVING 2
int SPHERE_STATE = NONE;

/*****************************************************************************
*  ToolInitNewSphere
******************************************************************************/
void ToolInitNewSphere(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ViewUserDataStruct *UserData;
  GdkColormap *colormap;
  GtkWidget *StatusBar;
          
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  id = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "new sphere");
  MouseToReal(bevent->x, bevent->y, DrawingArea, &Xorg, &Yorg);
    
  colormap = gdk_window_get_colormap(DrawingArea->window);
  gcPurple = gdk_gc_new(DrawingArea->window);
  Purple.red = 65535; Purple.green = 0; Purple.blue = 65535;
  /* XXX: the color should be settable... */
  gdk_color_alloc(colormap, &Purple);
  gdk_gc_set_foreground(gcPurple, &Purple);
  gdk_gc_set_function(gcPurple, GDK_XOR);

  Radius = 0.0;
 
  SPHERE_STATE = MOVING;
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, _("Sphere, center: "));
  gdk_pointer_grab(DrawingArea->window, FALSE,
                   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                   NULL, NULL, bevent->time);
}

/*****************************************************************************
*  ToolMotionNewSphere
******************************************************************************/
void ToolMotionNewSphere(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventMotion *Mev = (GdkEventMotion *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff, Xold, Yold;
  char *Message;
  Vector Center;

  if (SPHERE_STATE != MOVING)
    return;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;

  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);
  gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
               DrawingArea->allocation.width/2+(Xorg-Radius-Xoff)*Zoom,
               DrawingArea->allocation.height/2-(Yorg+Radius-Yoff)*Zoom,
               Radius*2.0*Zoom, Radius*2.0*Zoom,  0, 64*360);
  MouseToReal(Mev->x, Mev->y, DrawingArea, &Xold, &Yold);
  Radius = sqrt((Xold-Xorg)*(Xold-Xorg)+(Yold-Yorg)*(Yold-Yorg));
  gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
               DrawingArea->allocation.width/2+(Xorg-Radius-Xoff)*Zoom,
               DrawingArea->allocation.height/2-(Yorg+Radius-Yoff)*Zoom,
               Radius*2.0*Zoom, Radius*2.0*Zoom,  0, 64*360);
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      Center[0] = Xorg;
      Center[1] = Yorg;
      Center[2] = 0.0;
      break;
    case ORTHO_XZ_CAMERA:
      Center[0] = Xorg;
      Center[1] = 0.0;
      Center[2] = Yorg;
      break;
    case ORTHO_ZY_CAMERA:
      Center[0] = 0.0;
      Center[1] = Yorg;
      Center[2] = Xorg;
      break;
  }
  Message = g_strdup_printf(_("Sphere, center: <%g, %g, %g>, Radius: %g"),
                            Center[0], Center[1], Center[2], Radius);
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
  g_free(Message);
}    

/*****************************************************************************
*  ToolReleaseNewSphere
******************************************************************************/
void ToolReleaseNewSphere(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventButton *Bev = (GdkEventButton *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff, Xold, Yold;
  FrameViewStruct *TmpView;
  ObjectStruct *Sphere;
  SphereStruct *SSphere;

  if (SPHERE_STATE != MOVING)
    return;
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;

  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);
  MouseToReal(Bev->x, Bev->y, DrawingArea, &Xold, &Yold);
  Radius = sqrt((Xold-Xorg)*(Xold-Xorg)+(Yold-Yorg)*(Yold-Yorg));
  SSphere = g_malloc(sizeof(SphereStruct));
  Sphere = (ObjectStruct *)SSphere;
  InitObject(Sphere);
  Sphere->Type = SPHERE_OBJECT;
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      SSphere->Center[0] = Xorg;
      SSphere->Center[1] = Yorg;
      SSphere->Center[2] = 0.0;
      break;
    case ORTHO_XZ_CAMERA:
      SSphere->Center[0] = Xorg;
      SSphere->Center[1] = 0.0;
      SSphere->Center[2] = Yorg;
      break;
    case ORTHO_ZY_CAMERA:
      SSphere->Center[0] = 0.0;
      SSphere->Center[1] = Yorg;
      SSphere->Center[2] = Xorg;
      break;
  }
  SSphere->Radius = Radius;
  Sphere->Next = UserData->FrameView->FirstObject;
  UserData->FrameView->FirstObject = Sphere;
  ComputeSphereTriangleMesh(Sphere);
  SPHERE_STATE = NONE;

  gdk_gc_destroy(gcPurple);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
  gdk_pointer_ungrab(Bev->time);
}

/*****************************************************************************
*  CreateSphere
******************************************************************************/
ObjectStruct *CreateSphere(Vector Center, double Radius)
{
  SphereStruct *SSphere;
  ObjectStruct *Sphere;

  SSphere = g_malloc(sizeof(SphereStruct));
  Sphere = (ObjectStruct *)SSphere;
  InitObject(Sphere);
  Sphere->Type = SPHERE_OBJECT;

  SSphere->Center[0] = Center[0];
  SSphere->Center[1] = Center[1];
  SSphere->Center[2] = Center[2];
  SSphere->Radius = Radius;
  ComputeSphereTriangleMesh(Sphere);
  return Sphere;
}

/*****************************************************************************
*  PDBCreateSphere
******************************************************************************/
int PDBCreateSphere(int *Id, Vector *Center, double *Radius)
{
  ObjectStruct *Sphere;
  SphereStruct *SSphere;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  SSphere = g_malloc(sizeof(SphereStruct));
  Sphere = (ObjectStruct *)SSphere;
  InitObject(Sphere);
  Sphere->Type = SPHERE_OBJECT;
  SSphere->Center[0] = (*Center)[0];
  SSphere->Center[1] = (*Center)[1];
  SSphere->Center[2] = (*Center)[2];
  SSphere->Radius = *Radius;

  for (LocalFrame = FirstFrame ; 
       LocalFrame->Id != *Id ; 
       LocalFrame = LocalFrame->Next)
             ;
  Sphere->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Sphere;
  ComputeSphereTriangleMesh(Sphere);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Sphere->Id;
}

