/* superellipsoid.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "superellipsoid.h"

/*****************************************************************************
*  InsideSuperEllipsoid FIXME
******************************************************************************/
int InsideSuperEllipsoid(ObjectStruct *SuperEll, double x, double y, double z)
{ /* XXX: clipped_by */
  Vector Vect;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (SuperEll->Trans)
  {
    MInverseEvaluatePoint(Vect, SuperEll->Trans, Vect);
  }
  return 0; /* FIXME */
}

/*****************************************************************************
*  FindIntersectSuperEllipsoidSegment FIXME
******************************************************************************/
int FindIntersectionSuperEllipsoidSegment(Vector InterPoint, Vector InterNorm,
                                          Vector InPoint, Vector OutPoint,
                                          ObjectStruct *SuperEllipsoid)
{
  return 0;
}

/*****************************************************************************
*  SuperEllipsoidMeshNormal FIXME
******************************************************************************/
void SuperEllipsoidMeshNormal(Vector Result, ObjectStruct *SuperEllipsoid,
                              Vector Inter)
{ /* FIXME */
  Result[0] = 0.0;
  Result[1] = 0.0;
  Result[2] = 0.0;
}

/*****************************************************************************
*  ComputeSuperEllipsoidTriangleMesh
******************************************************************************/
void ComputeSuperEllipsoidTriangleMesh(ObjectStruct *SuperEllipsoid)
{ /* XXX: clipped_by */
  Vector P1, P2, P3, P4;
  Vector N1, N2, N3, N4;
  int i,j;
  int DetailLevel;
  SuperEllipsoidStruct *SSuperEllipsoid = (SuperEllipsoidStruct *)SuperEllipsoid;
  double t1, s1, c1, t2, s2, c2;

  DetailLevel = 16; /* FIXME: should be a private field */

  if (SuperEllipsoid->FirstTriangle)
    DestroyObjectTriangleMesh(SuperEllipsoid);
  /* First the North Pole */
  P1[0] = 0.0;
  P1[1] = 0.0;
  P1[2] = 1.0;
  SuperEllipsoidMeshNormal(N1, SuperEllipsoid, P1);
  if (SuperEllipsoid->Trans)
  {
    MEvaluatePoint(P1, SuperEllipsoid->Trans, P1);
    MEvaluateVector(N1, SuperEllipsoid->Trans, N1);
  }
  for (i=0; i<DetailLevel; i++)
  { /* The "DetailLevel" triangles adjacents to the pole */
    t1 = M_PI/DetailLevel;
    if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
    else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
    if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
    else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
    t2 = i*2.0*M_PI/DetailLevel;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    P2[0] = s1*c2;
    P2[1] = s1*s2;
    P2[2] = c1;
    SuperEllipsoidMeshNormal(N2, SuperEllipsoid, P2);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P2, SuperEllipsoid->Trans, P2);
      MEvaluateVector(N2, SuperEllipsoid->Trans, N2);
    }
    t2 = (i+1)*2.0*M_PI/DetailLevel;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    P3[0] = s1*c2;
    P3[1] = s1*s2;
    P3[2] = c1;
    SuperEllipsoidMeshNormal(N3, SuperEllipsoid, P3);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P3, SuperEllipsoid->Trans, P3);
      MEvaluateVector(N3, SuperEllipsoid->Trans, N3);
    }
    AddTriangleToObjectMesh(SuperEllipsoid, P1,P2,P3, N1,N2,N3);
  }
  /* Then the Mid Zone */
  for (i=1 ; i<DetailLevel-1 ; i++)
  {
    for (j=0 ; j<DetailLevel ; j++)
    {
      t1 = i*M_PI/DetailLevel;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = j*2.0*M_PI/DetailLevel;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      P1[0] = s1*c2;
      P1[1] = s1*s2;
      P1[2] = c1;
      SuperEllipsoidMeshNormal(N1, SuperEllipsoid, P1);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P1, SuperEllipsoid->Trans, P1);
        MEvaluateVector(N1, SuperEllipsoid->Trans, N1);
      }
      t1 = (i+1)*M_PI/DetailLevel;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = j*2.0*M_PI/DetailLevel;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      P2[0] = s1*c2;
      P2[1] = s1*s2;
      P2[2] = c1;
      SuperEllipsoidMeshNormal(N2, SuperEllipsoid, P2);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P2, SuperEllipsoid->Trans, P2);
        MEvaluateVector(N2, SuperEllipsoid->Trans, N2);
      }
      t1 = i*M_PI/DetailLevel;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = (j+1)*2.0*M_PI/DetailLevel;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      P3[0] = s1*c2;
      P3[1] = s1*s2;
      P3[2] = c1;
      SuperEllipsoidMeshNormal(N3, SuperEllipsoid, P3);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P3, SuperEllipsoid->Trans, P3);
        MEvaluateVector(N3, SuperEllipsoid->Trans, N3);
      }
      t1 = (i+1)*M_PI/DetailLevel;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = (j+1)*2.0*M_PI/DetailLevel;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      P4[0] = s1*c2;
      P4[1] = s1*s2;
      P4[2] = c1;
      SuperEllipsoidMeshNormal(N4, SuperEllipsoid, P4);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P4, SuperEllipsoid->Trans, P4);
        MEvaluateVector(N4, SuperEllipsoid->Trans, N4);
      }
      AddTriangleToObjectMesh(SuperEllipsoid, P1,P2,P4, N1,N2,N4);
      AddTriangleToObjectMesh(SuperEllipsoid, P1,P3,P4, N1,N3,N4);
    }
  }
  /* And the South Pole */
  P1[0] =  0.0;
  P1[1] =  0.0;
  P1[2] = -1.0;
  SuperEllipsoidMeshNormal(N1, SuperEllipsoid, P1);
  if (SuperEllipsoid->Trans)
  {
    MEvaluatePoint(P1, SuperEllipsoid->Trans, P1);
    MEvaluateVector(N1, SuperEllipsoid->Trans, N1);
  }
  for (i=0 ; i<DetailLevel ; i++)
  { /* The "DetailLevel" triangles adjacents to the south pole */
    t1 = M_PI-M_PI/DetailLevel;
    if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
    else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
    if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
    else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
    t2 = i*2.0*M_PI/DetailLevel;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    P2[0] = s1*c2;
    P2[1] = s1*s2;
    P2[2] = c1;
    SuperEllipsoidMeshNormal(N2, SuperEllipsoid, P2);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P2, SuperEllipsoid->Trans, P2);
      MEvaluateVector(N2, SuperEllipsoid->Trans, N2);
    }
    t2 = (i+1)*2.0*M_PI/DetailLevel;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    P3[0] = s1*c2;
    P3[1] = s1*s2;
    P3[2] = c1;
    SuperEllipsoidMeshNormal(N3, SuperEllipsoid, P3);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P3, SuperEllipsoid->Trans, P3);
      MEvaluateVector(N3, SuperEllipsoid->Trans, N3);
    }
    AddTriangleToObjectMesh(SuperEllipsoid, P1,P2,P3, N1,N2,N3);
  }
}

/*****************************************************************************
*  TranslateSuperEllipsoid
******************************************************************************/
void TranslateSuperEllipsoid(ObjectStruct *SuperEll, Vector Vect)
{
  TransformStruct Trans;

  if (SuperEll->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(SuperEll->Trans, &Trans);
  } else
  {
    SuperEll->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(SuperEll->Trans, Vect);
  }
  if (SuperEll->ClippedBy)
    TranslateObject(SuperEll->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateSuperEllipsoid
******************************************************************************/
void RotateSuperEllipsoid(ObjectStruct *SuperEll, Vector Vect)
{
  TransformStruct Trans;

  if (SuperEll->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(SuperEll->Trans, &Trans);
  } else
  {
    SuperEll->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(SuperEll->Trans, Vect);
  }
  if (SuperEll->ClippedBy)
    RotateObject(SuperEll->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleSuperEllipsoid
******************************************************************************/
void ScaleSuperEllipsoid(ObjectStruct *SuperEll, Vector Vect)
{
  TransformStruct Trans;

  if (SuperEll->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(SuperEll->Trans, &Trans);
  } else
  {
    SuperEll->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(SuperEll->Trans, Vect);
  }
  if (SuperEll->ClippedBy)
    ScaleObject(SuperEll->ClippedBy, Vect);
}

/*****************************************************************************
*  IsSuperEllipsoidIntersection FIXME
******************************************************************************/
int IsSuperEllipsoidIntersection(ObjectStruct *SuperEll, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;

  if (SuperEll->Trans)
  {
    MInverseEvaluatePoint(Org, SuperEll->Trans, Origin);
    MInverseEvaluateVector(Dir, SuperEll->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  return 0;
}

/*****************************************************************************
*  ClickInSuperEllipsoidXY
******************************************************************************/
int ClickInSuperEllipsoidXY(ObjectStruct *SuperEll, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsSuperEllipsoidIntersection(SuperEll, Origin, Direction);
}

/****************************************************************************
*  ClickInSuperEllipsoidXZ
*****************************************************************************/
int ClickInSuperEllipsoidXZ(ObjectStruct *SuperEll, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsSuperEllipsoidIntersection(SuperEll, Origin, Direction);
}

/****************************************************************************
*  ClickInSuperEllipsoidZY
*****************************************************************************/
int ClickInSuperEllipsoidZY(ObjectStruct *SuperEll, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsSuperEllipsoidIntersection(SuperEll, Origin, Direction);
}

/*****************************************************************************
*  NewSuperEllipsoidOK
******************************************************************************/
void NewSuperEllipsoidOK(GtkWidget *Dialog)
{
  GtkObject **Data;
  ViewUserDataStruct *UserData;
  ObjectStruct *SuperEllipsoid;
  SuperEllipsoidStruct *SSuperEllipsoid;
  FrameViewStruct *TmpView;
  
  Data = gtk_object_get_data(GTK_OBJECT(Dialog), "WidgetData");
  UserData = gtk_object_get_data(Data[2], "ViewData");
/*  printf("EW : %g\n", GTK_ADJUSTMENT(Data[0])->value);
  printf("NS : %g\n", GTK_ADJUSTMENT(Data[1])->value);*/
  SSuperEllipsoid = g_malloc(sizeof(SuperEllipsoidStruct));
  SuperEllipsoid = (ObjectStruct *)SSuperEllipsoid;
  InitObject(SuperEllipsoid);
  SuperEllipsoid->Type = SUPERELLIPSOID_OBJECT;
  SSuperEllipsoid->E = GTK_ADJUSTMENT(Data[0])->value;
  SSuperEllipsoid->N = GTK_ADJUSTMENT(Data[1])->value;

  SuperEllipsoid->Next = UserData->FrameView->FirstObject;
  UserData->FrameView->FirstObject = SuperEllipsoid;
  ComputeSuperEllipsoidTriangleMesh(SuperEllipsoid);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
  gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
}

/*****************************************************************************
*  ToolNewSuperEllipsoid
******************************************************************************/
void ToolNewSuperEllipsoid(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Table;
  GtkWidget *Label;
  GtkObject *Adjustment;
  GtkWidget *Scale;
  GtkObject **Data;

  Dialog = gtk_dialog_new();
  Data = g_malloc(3*sizeof(GtkObject*));
  Data[2] = GTK_OBJECT(DrawingArea);
  
  gtk_object_set_data(GTK_OBJECT(Dialog), "WidgetData", Data);
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New SuperEllipsoid"));

  /* The Ok Button */
  Button = gtk_button_new_with_label(_("OK"));
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)NewSuperEllipsoidOK,
                            GTK_OBJECT(Dialog));
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button, TRUE,
                     TRUE, 0);
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  /* The Close Button */
  Button = gtk_button_new_with_label(_("Close"));
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button, TRUE,
                     TRUE, 0);
  gtk_widget_show(Button);
  /* The East-West Parameter */
  Table = gtk_table_new(2, 3, FALSE);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Table, TRUE, TRUE, 0);
  
  Label = gtk_label_new(_("East-West Parameter: "));
  gtk_table_attach(GTK_TABLE(Table), Label, 0,1, 0,1,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Label);

  Adjustment = gtk_adjustment_new(1.0, 0.1, 5.0, 0.1, 0.1, 1.0);
  Scale = gtk_hscale_new(GTK_ADJUSTMENT(Adjustment));
  gtk_widget_set_usize(GTK_WIDGET(Scale), 150, 30);
  gtk_range_set_update_policy(GTK_RANGE(Scale), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_draw_value(GTK_SCALE(Scale), TRUE);
  gtk_table_attach(GTK_TABLE(Table), Scale, 1,2, 0,1,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Scale);
  Data[0] = Adjustment;            
  
  /* The North-South Parameter */
  Label = gtk_label_new(_("North-South Parameter: "));
  gtk_table_attach(GTK_TABLE(Table), Label, 0,1, 1,2,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Label);
    
  Adjustment = gtk_adjustment_new(1.0, 0.1, 5.0, 0.1, 0.1, 1.0);
  Scale = gtk_hscale_new(GTK_ADJUSTMENT(Adjustment));
  gtk_widget_set_usize(GTK_WIDGET(Scale), 150, 30);
  gtk_range_set_update_policy(GTK_RANGE(Scale), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_draw_value(GTK_SCALE(Scale), TRUE);
  gtk_table_attach(GTK_TABLE(Table), Scale, 1,2, 1,2,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(Scale);
  Data[1] = Adjustment;
  
  gtk_widget_show(Table);
  
  gtk_widget_show(Dialog);
  
}

/*****************************************************************************
*  PDBCreateSuperEllipsoid
******************************************************************************/
int PDBCreateSuperEllipsoid(int *Id, double *N, double *E)
{
  ObjectStruct *SuperEllipsoid;
  SuperEllipsoidStruct *SSuperEllipsoid;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  SSuperEllipsoid = g_malloc(sizeof(SuperEllipsoidStruct));
  SuperEllipsoid = (ObjectStruct *)SSuperEllipsoid;
  InitObject(SuperEllipsoid);
  SuperEllipsoid->Type = SUPERELLIPSOID_OBJECT;
  SSuperEllipsoid->E = *E;
  SSuperEllipsoid->N = *N;

  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
                  ;
  SuperEllipsoid->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = SuperEllipsoid;
  ComputeSuperEllipsoidTriangleMesh(SuperEllipsoid);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
  gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return SuperEllipsoid->Id;
}

