/* tool_select.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <math.h>
#include "giram.h"
#include "object.h"
#include "tool_select.h"

ObjectSelection *LocalSelection = NULL;

/*****************************************************************************
*  DestroySelection
******************************************************************************/
void DestroySelection(ObjectSelection **pSel)
{
  ObjectSelection *Tmp1, *Tmp2;

  if (*pSel)
  {
    Tmp1 = *pSel;
    while (Tmp1->Next != *pSel)
    {
      Tmp2 = Tmp1->Next;
      Tmp1->Object->Selected = 0;
      g_free(Tmp1);
      Tmp1 = Tmp2;
    }
    Tmp1->Object->Selected = 0;
    g_free(Tmp1);
    *pSel = NULL;
  }
}

/*****************************************************************************
*  AddObjectToSelection
******************************************************************************/
void AddObjectToSelection(ObjectSelection **pSel, ObjectStruct *Object)
{
  ObjectSelection *Prev;

  if (*pSel)
  {
    for (Prev = *pSel ; 
         Prev->Next != *pSel ; 
         Prev=Prev->Next)
               ;
    Prev->Next = g_malloc(sizeof(ObjectSelection));
    Prev->Next->Next = *pSel;
    Prev->Next->Object = Object;
  } else
  {
    *pSel = g_malloc(sizeof(ObjectSelection));
    (*pSel)->Next = *pSel;
    (*pSel)->Object = Object;
  }
  Object->Selected = 1;
}

/*****************************************************************************
*  ObjectInSelection
******************************************************************************/
int ObjectInSelection(ObjectSelection *Sel, ObjectStruct *Object)
{
  ObjectSelection *Prev;

  if (Sel)
  {
    for (Prev=Sel ; Prev->Next!=Sel ; Prev=Prev->Next)
    {
      if (Prev->Object == Object)
        return 1;
    }
    if (Prev->Object == Object)
      return 1;
  }
  return 0;
}

/*****************************************************************************
*  RemoveObjectFromSelection
******************************************************************************/
void RemoveObjectFromSelection(ObjectSelection **pSel, ObjectStruct *Object)
{
  ObjectSelection *Prev;
  ObjectSelection *Tmp;

  if (*pSel)
  {
    if (*pSel == (*pSel)->Next)
    {
      if ((*pSel)->Object == Object)
      {
        g_free(*pSel);
        *pSel = NULL;
      }
    } else
    {
      for (Prev=*pSel ; Prev->Next->Object != Object ; Prev = Prev->Next);
      Tmp = Prev->Next;
      if (*pSel == Tmp)
        *pSel = Prev;
      Prev->Next = Prev->Next->Next;
      g_free(Tmp);
    }
  }
  Object->Selected = 0;
}

/*****************************************************************************
*  DestroyLocalSelection
******************************************************************************/
void DestroyLocalSelection(void)
{
  ObjectSelection *Tmp1, *Tmp2;

  if (LocalSelection)
  {
    Tmp1 = LocalSelection;
    while (Tmp1->Next != LocalSelection)
    {
      Tmp2 = Tmp1->Next;
      g_free(Tmp1);
      Tmp1 = Tmp2;
    }
    g_free(Tmp1);
    LocalSelection = NULL;
  }
}

/*****************************************************************************
*  AddObjectToLocalSelection
******************************************************************************/
void AddObjectToLocalSelection(ObjectStruct *Object)
{
  ObjectSelection *Prev;
  if (LocalSelection)
  {
    for (Prev = LocalSelection ; Prev->Next != LocalSelection ; Prev=Prev->Next)
      ;
    Prev->Next = g_malloc(sizeof(ObjectSelection));
    Prev->Next->Next = LocalSelection;
    Prev->Next->Object = Object;
  } else
  {
    LocalSelection = g_malloc(sizeof(ObjectSelection));
    LocalSelection->Next = LocalSelection;
    LocalSelection->Object = Object;
  }
}

/*****************************************************************************
*  ToggleSelect
******************************************************************************/
void ToggleSelect(GtkWidget *DrawingArea, GdkEventButton *bev)
{
  ViewUserDataStruct *UserData;
  FrameStruct *LocalFrame;
  ObjectStruct *Object;
  gdouble x, y;
  FrameViewStruct *TmpView;
  ObjectSelection *Prev;
  ObjectSelection *TmpSelection; 
  static ObjectSelection *PreviousSelection;      /* FIXME: should be one */
  static gdouble old_x = -10.0, old_y = -10.0;     /* per scene/frame */    
  int RebuildLocal;
  
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  LocalFrame = UserData->FrameView;

  RebuildLocal = 1;
  if (bev->state & 8)
  { /* Alt is pressed */
    if (LocalSelection && (fabs(bev->x-old_x) < 4.0 /* XXX Should be settable */) 
                       && (fabs(bev->y-old_y) < 4.0))
    { /* Just rotate the selection */
      LocalSelection = LocalSelection->Next;
      RebuildLocal = 0;
    } else
    {
      DestroySelection(&PreviousSelection);
      if (LocalFrame->Selection)
      {
        for (Prev = LocalFrame->Selection ;
             Prev->Next != LocalFrame->Selection ;
             Prev = Prev->Next)
          AddObjectToSelection(&PreviousSelection, Prev->Object);
        AddObjectToSelection(&PreviousSelection, Prev->Object);
      }
    }
  }
  old_x = bev->x;
  old_y = bev->y;
  if (RebuildLocal)
  {
    DestroyLocalSelection();
    x = UserData->Xoffset+(bev->x-DrawingArea->allocation.width/2) / UserData->ZoomFactor;
    y = UserData->Yoffset-(bev->y-DrawingArea->allocation.height/2) / UserData->ZoomFactor;
    switch (UserData->CameraStyle)
    {
      case ORTHO_XY_CAMERA:
        for (Object = LocalFrame->FirstObject ; Object ; Object = Object->Next)
          if (ClickInObjectXY(Object, x, y))
            AddObjectToLocalSelection(Object);
        break;
      case ORTHO_XZ_CAMERA:
        for (Object = LocalFrame->FirstObject ; Object ; Object = Object->Next)
          if (ClickInObjectXZ(Object, x, y))
            AddObjectToLocalSelection(Object);
        break;
      case ORTHO_ZY_CAMERA:
        for (Object = LocalFrame->FirstObject ; Object ; Object = Object->Next)
          if (ClickInObjectZY(Object, x, y))
            AddObjectToLocalSelection(Object);
        break;
    }
  }
  /* Update the scene selection */
  if (bev->state & 8)
  { /* Alt */
    if (bev->state & 4)
    { /* Alt & Ctrl */
      if (bev->state & 1)
      { /* Alt & Ctrl & Shift */
        /* The Frame Selection is the first object of the Local Selection
         * but only if it was already present in the Previous Selection */
        DestroySelection(&(LocalFrame->Selection));
        if (LocalSelection)
        {
          if (ObjectInSelection(PreviousSelection, LocalSelection->Object))
            AddObjectToSelection(&(LocalFrame->Selection), 
                                 LocalSelection->Object);
        }
      } else
      { /* Alt & Ctrl & No Shift */
        /* The Frame Selection is the Previous Selection LESS the first object
         * of the Local Selection */
        DestroySelection(&(LocalFrame->Selection));
        if (PreviousSelection)
        {
          for (Prev = PreviousSelection ;
               Prev->Next != PreviousSelection ;
               Prev = Prev->Next)
            AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
          AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
        }
        if (LocalSelection)
        {
          if (ObjectInSelection(LocalFrame->Selection, LocalSelection->Object))
            RemoveObjectFromSelection(&(LocalFrame->Selection), 
                                      LocalSelection->Object);
        }
      }
    } else
    { /* Alt & No Ctrl */
      if (bev->state & 1)
      { /* Alt & No Ctrl & Shift */
        /* The Frame Selection is the Previous Selection PLUS the first object
         * of the Local Selection */
        DestroySelection(&(LocalFrame->Selection));
        if (PreviousSelection)
        {
          for (Prev = PreviousSelection ;
               Prev->Next != PreviousSelection ;
               Prev = Prev->Next)
            AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
          AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
        }
        if (LocalSelection)
        {
          if (!ObjectInSelection(LocalFrame->Selection, LocalSelection->Object))
            AddObjectToSelection(&(LocalFrame->Selection), 
                                 LocalSelection->Object);
        }
      } else
      { /* Alt & No Ctrl & No Shift */
        /* The Frame Selection is now the First object of the Local selection */
        DestroySelection(&(LocalFrame->Selection));
        if (LocalSelection)
          AddObjectToSelection(&(LocalFrame->Selection), 
                               LocalSelection->Object);
      }
    }
  } else
  { /* No Alt */
    if (bev->state & 4)
    { /* No Alt & Ctrl */
      if (bev->state & 1)
      { /* No Alt & Ctrl & Shift */
        /* The Frame selection is the object that were in the frame
         * selection AND are in the new selection */
        TmpSelection = NULL;
        if (LocalSelection)
        {
          for (Prev = LocalSelection ;
               Prev->Next != LocalSelection ;
               Prev = Prev->Next)
            if (ObjectInSelection(LocalFrame->Selection, Prev->Object))
              AddObjectToSelection(&TmpSelection, Prev->Object);
          if (ObjectInSelection(LocalFrame->Selection, Prev->Object))
            AddObjectToSelection(&TmpSelection, Prev->Object);
        }
        DestroySelection(&(LocalFrame->Selection));
        if (TmpSelection)
        {
          for (Prev = TmpSelection ;
               Prev->Next != TmpSelection ;
               Prev = Prev->Next)
            Prev->Object->Selected = 1;
          Prev->Object->Selected = 1;
        }
        LocalFrame->Selection = TmpSelection;
      } else
      { /* No Alt & Ctrl & No Shift */
        /* Take the object of the new Selection Out of the Frame Selection */
        if (LocalSelection)
        {
          for (Prev = LocalSelection ;
               Prev->Next != LocalSelection ;
               Prev = Prev->Next)
            if (ObjectInSelection(LocalFrame->Selection, Prev->Object))
              RemoveObjectFromSelection(&(LocalFrame->Selection), Prev->Object);
          if (ObjectInSelection(LocalFrame->Selection, Prev->Object))
            RemoveObjectFromSelection(&(LocalFrame->Selection), Prev->Object);
        }
      }
    } else
    { /* No Alt & No Ctrl */
      if (bev->state & 1)
      { /* No Alt & No Ctrl & Shift */
        /* Add the new selection to the Frame selection */
        if (LocalSelection)
        {
          for (Prev = LocalSelection ;
               Prev->Next != LocalSelection ;
               Prev=Prev->Next)
            if (!ObjectInSelection(LocalFrame->Selection, Prev->Object))
              AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
          if (!ObjectInSelection(LocalFrame->Selection, Prev->Object))
            AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
        }
      } else
      { /* No Alt & No Ctrl & No Shift */
        /* the Frame is selection is replaced by the new selection */
        DestroySelection(&(LocalFrame->Selection));
        if (LocalSelection)
        {
          for (Prev = LocalSelection ; 
               Prev->Next != LocalSelection ; 
               Prev=Prev->Next)
            AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
          AddObjectToSelection(&(LocalFrame->Selection), Prev->Object);
        }
      }
    }
  }
  
  /* FIXME: should update the CSG Tree selection */
  /* We redraw all the Views */
  for (TmpView = LocalFrame->FirstView ; TmpView ; TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);

}

