/* torus.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "snap.h"
#include "root_solver.h"
#include "torus.h"

/*****************************************************************************
*  InsideTorus
******************************************************************************/
int InsideTorus(ObjectStruct *Torus, double x, double y, double z)
{ /* XXX: clipped_by */
  TorusStruct *TTorus = (TorusStruct *)Torus;
  Vector Vect;
  double p, pp;

  Vect[0] = x;
  Vect[1] = y;
  Vect[2] = z;
  if (Torus->Trans)
  {
    MInverseEvaluatePoint(Vect, Torus->Trans, Vect);
  }
  p = sqrt(Vect[0]*Vect[0] + Vect[2]*Vect[2]);
  pp = Vect[1]*Vect[1] + (p-TTorus->Major)*(p-TTorus->Major);
  if (pp <= TTorus->Minor*TTorus->Minor)
    return !(Torus->Inverse);
  else
    return Torus->Inverse;
}

/*****************************************************************************
*  FindIntersectTorusSegment FIXME
******************************************************************************/
int FindIntersectionTorusSegment(Vector InterPoint, Vector InterNorm,
                                 Vector InPoint, Vector OutPoint,
                                 ObjectStruct *Torus)
{
  return 0;
}

/*****************************************************************************
*  ComputeTorusTriangleMesh
******************************************************************************/
void ComputeTorusTriangleMesh(ObjectStruct *Torus)
{ /* XXX: clipped_by */
  int i,j;
  int DetailLevel;
  Vector P1, P2, P3, P4;
  Vector N1, N2, N3, N4;
  TorusStruct *TTorus = (TorusStruct *)Torus;

  DetailLevel = 16; /* XXX should be an internal parameter */

  if (Torus->FirstTriangle)
    DestroyObjectTriangleMesh(Torus);

  for (i=0 ; i<DetailLevel ; i++)
    for (j=0 ; j<DetailLevel ; j++)
    {
      P1[0] = (TTorus->Major+TTorus->Minor*cos(j*2.0*M_PI/DetailLevel))*cos(i*2.0*M_PI/DetailLevel);
      P1[1] = TTorus->Minor*sin(j*2.0*M_PI/DetailLevel);
      P1[2] = (TTorus->Major+TTorus->Minor*cos(j*2.0*M_PI/DetailLevel))*sin(i*2.0*M_PI/DetailLevel);
      P2[0] = (TTorus->Major+TTorus->Minor*cos((j+1)*2.0*M_PI/DetailLevel))*cos(i*2.0*M_PI/DetailLevel);
      P2[1] = TTorus->Minor*sin((j+1)*2.0*M_PI/DetailLevel);
      P2[2] = (TTorus->Major+TTorus->Minor*cos((j+1)*2.0*M_PI/DetailLevel))*sin(i*2.0*M_PI/DetailLevel);
      P3[0] = (TTorus->Major+TTorus->Minor*cos(j*2.0*M_PI/DetailLevel))*cos((i+1)*2.0*M_PI/DetailLevel);
      P3[1] = TTorus->Minor*sin(j*2.0*M_PI/DetailLevel);
      P3[2] = (TTorus->Major+TTorus->Minor*cos(j*2.0*M_PI/DetailLevel))*sin((i+1)*2.0*M_PI/DetailLevel);
      P4[0] = (TTorus->Major+TTorus->Minor*cos((j+1)*2.0*M_PI/DetailLevel))*cos((i+1)*2.0*M_PI/DetailLevel);
      P4[1] = TTorus->Minor*sin((j+1)*2.0*M_PI/DetailLevel);
      P4[2] = (TTorus->Major+TTorus->Minor*cos((j+1)*2.0*M_PI/DetailLevel))*sin((i+1)*2.0*M_PI/DetailLevel);
     
      N1[0] = cos(i*2.0*M_PI/DetailLevel)*cos(j*2.0*M_PI/DetailLevel);
      N1[1] = sin(j*2.0*M_PI/DetailLevel);
      N1[2] = sin(i*2.0*M_PI/DetailLevel)*cos(j*2.0*M_PI/DetailLevel);
      N2[0] = cos(i*2.0*M_PI/DetailLevel)*cos((j+1)*2.0*M_PI/DetailLevel);
      N2[1] = sin((j+1)*2.0*M_PI/DetailLevel);
      N2[2] = sin(i*2.0*M_PI/DetailLevel)*cos((j+1)*2.0*M_PI/DetailLevel);
      N3[0] = cos((i+1)*2.0*M_PI/DetailLevel)*cos(j*2.0*M_PI/DetailLevel);
      N3[1] = sin(j*2.0*M_PI/DetailLevel);
      N3[2] = sin((i+1)*2.0*M_PI/DetailLevel)*cos(j*2.0*M_PI/DetailLevel);
      N4[0] = cos((i+1)*2.0*M_PI/DetailLevel)*cos((j+1)*2.0*M_PI/DetailLevel);
      N4[1] = sin((j+1)*2.0*M_PI/DetailLevel);
      N4[2] = sin((i+1)*2.0*M_PI/DetailLevel)*cos((j+1)*2.0*M_PI/DetailLevel);
      if (Torus->Trans)
      {
        MEvaluatePoint(P1, Torus->Trans, P1);
        MEvaluatePoint(P2, Torus->Trans, P2);
        MEvaluatePoint(P3, Torus->Trans, P3);
        MEvaluatePoint(P4, Torus->Trans, P4);
        MEvaluateVector(N1, Torus->Trans, N1);
        MEvaluateVector(N2, Torus->Trans, N2);
        MEvaluateVector(N3, Torus->Trans, N3);
        MEvaluateVector(N4, Torus->Trans, N4);
      }
      AddTriangleToObjectMesh(Torus, P1,P2,P3, N1,N2,N3);
      AddTriangleToObjectMesh(Torus, P4,P2,P3, N4,N2,N3);
    }
}

/*****************************************************************************
*  TranslateTorus
******************************************************************************/
void TranslateTorus(ObjectStruct *Torus, Vector Vect)
{
  TransformStruct Trans;

  if (Torus->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Torus->Trans, &Trans);
  } else
  {
    Torus->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Torus->Trans, Vect);
  }
  if (Torus->ClippedBy)
    TranslateObject(Torus->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateTorus
******************************************************************************/
void RotateTorus(ObjectStruct *Torus, Vector Vect)
{
  TransformStruct Trans;

  if (Torus->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Torus->Trans, &Trans);
  } else
  {
    Torus->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Torus->Trans, Vect);
  }
  if (Torus->ClippedBy)
    RotateObject(Torus->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleTorus
******************************************************************************/
void ScaleTorus(ObjectStruct *Torus, Vector Vect)
{
  TransformStruct Trans;

  if (Torus->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Torus->Trans, &Trans);
  } else
  {
    Torus->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Torus->Trans, Vect);
  }
  if (Torus->ClippedBy)
    ScaleObject(Torus->ClippedBy, Vect);
}

/*****************************************************************************
*  IsTorusIntersection FIXME
******************************************************************************/
int IsTorusIntersection(ObjectStruct *Torus, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
  double R2, r2;
  double Py2, Dy2, PDy2, k1, k2;
  double c[5], s[4];
  TorusStruct *TTorus = (TorusStruct *)Torus;

  if (Torus->Trans)
  {
    MInverseEvaluatePoint(Org, Torus->Trans, Origin);
    MInverseEvaluateVector(Dir, Torus->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }

  R2   = TTorus->Major * TTorus->Major;
  r2   = TTorus->Minor * TTorus->Minor;

  Py2  = Org[1] * Org[1];
  Dy2  = Dir[1] * Dir[1];
  PDy2 = Org[1] * Dir[1];

  k1   = Org[0] * Org[0] + Org[2] * Org[2] + Py2 - R2 - r2;
  k2   = Org[0] * Dir[0] + Org[2] * Dir[2] + PDy2;

  c[0] = 1.0;
  c[1] = 4.0 * k2;
  c[2] = 2.0 * (k1 + 2.0 * (k2 * k2 + R2 * Dy2));
  c[3] = 4.0 * (k2 * k1 + 2.0 * R2 * PDy2);
  c[4] = k1 * k1 + 4.0 * R2 * (Py2 - r2);

  return SolveQuartic(c, s);
}

/*****************************************************************************
*  ClickInTorusXY
******************************************************************************/
int ClickInTorusXY(ObjectStruct *Torus, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsTorusIntersection(Torus, Origin, Direction);
}

/****************************************************************************
*  ClickInTorusXZ
*****************************************************************************/
int ClickInTorusXZ(ObjectStruct *Torus, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsTorusIntersection(Torus, Origin, Direction);
}

/****************************************************************************
*  ClickInTorusZY
*****************************************************************************/
int ClickInTorusZY(ObjectStruct *Torus, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsTorusIntersection(Torus, Origin, Direction);
}

/****************************************************************************/
double Xorg, Yorg, Radius, MinorRadius;
GdkColormap *colormap;
GdkGC *gcPurple;
GdkColor Purple;
guint id;
Vector Center;

#define NONE           1
#define MOVING         2
#define BETWEEN_RADIUS 3
#define MINOR_RADIUS   4

int TORUS_STATE = NONE;

/*****************************************************************************
*  ToolInitNewTorus
******************************************************************************/
void ToolInitNewTorus(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  ViewUserDataStruct *UserData;
  GdkColormap *colormap;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;

  if (TORUS_STATE == BETWEEN_RADIUS)
  {
    TORUS_STATE = MINOR_RADIUS;
    return;
  }
  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  id = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "new torus");
  MouseToReal(bevent->x, bevent->y, DrawingArea, &Xorg, &Yorg);

  colormap = gdk_window_get_colormap(DrawingArea->window);
  gcPurple = gdk_gc_new(DrawingArea->window);
  Purple.red = 65535; Purple.green = 0; Purple.blue = 65535;
  /* XXX: the color should be settable... */
  gdk_color_alloc(colormap, &Purple);
  gdk_gc_set_foreground(gcPurple, &Purple);
  gdk_gc_set_function(gcPurple, GDK_XOR);

  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      Center[0] = Xorg;
      Center[1] = Yorg;
      Center[2] = 0.0;
      break;
    case ORTHO_XZ_CAMERA:
      Center[0] = Xorg;
      Center[1] = 0.0;
      Center[2] = Yorg;
      break;
    case ORTHO_ZY_CAMERA:
      Center[0] = 0.0;
      Center[1] = Yorg;
      Center[2] = Xorg;
      break;
  }
  Radius = 0.0;

  TORUS_STATE = MOVING;
  gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, _("Torus: "));
  gdk_pointer_grab(DrawingArea->window, FALSE,
                   GDK_BUTTON_PRESS_MASK | 
                   GDK_POINTER_MOTION_MASK | 
                   GDK_BUTTON_RELEASE_MASK,
                   NULL, NULL, bevent->time);
}

/*****************************************************************************
*  ToolMotionNewTorus
******************************************************************************/
void ToolMotionNewTorus(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventMotion *Mev = (GdkEventMotion *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff, Xold, Yold;
  gchar *Message;
  double NewRadius;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;
  if (TORUS_STATE == MOVING)
  {
    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-Radius-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+Radius-Yoff)*Zoom,
                 Radius*2.0*Zoom, Radius*2.0*Zoom,  0, 64*360);
    MouseToReal(Mev->x, Mev->y, DrawingArea, &Xold, &Yold);
    Radius = sqrt((Xold-Xorg)*(Xold-Xorg)+(Yold-Yorg)*(Yold-Yorg));
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-Radius-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+Radius-Yoff)*Zoom,
                 Radius*2.0*Zoom, Radius*2.0*Zoom,  0, 64*360);
    Message = g_strdup_printf(_("Torus: Center <%g, %g, %g>, Major Radius: %g"),
                              Center[0], Center[1], Center[2], Radius);
    gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
    g_free(Message);
  } else if ((TORUS_STATE == MINOR_RADIUS) || (TORUS_STATE == BETWEEN_RADIUS))
  {
    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-(Radius-MinorRadius)-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+(Radius-MinorRadius)-Yoff)*Zoom,
                 (Radius-MinorRadius)*2.0*Zoom, 
                 (Radius-MinorRadius)*2.0*Zoom, 0,64*360);
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-(Radius+MinorRadius)-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+(Radius+MinorRadius)-Yoff)*Zoom,
                 (Radius+MinorRadius)*2.0*Zoom, 
                 (Radius+MinorRadius)*2.0*Zoom, 0,64*360);
    MouseToReal(Mev->x, Mev->y, DrawingArea, &Xold, &Yold);
    NewRadius = sqrt((Xold-Xorg)*(Xold-Xorg)+(Yold-Yorg)*(Yold-Yorg));
    MinorRadius = fabs(NewRadius-Radius);
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-(Radius-MinorRadius)-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+(Radius-MinorRadius)-Yoff)*Zoom,
                 (Radius-MinorRadius)*2.0*Zoom, 
                 (Radius-MinorRadius)*2.0*Zoom, 0,64*360);
    gdk_draw_arc(DrawingArea->window, gcPurple, FALSE,
                 DrawingArea->allocation.width/2+(Xorg-(Radius+MinorRadius)-Xoff)*Zoom,
                 DrawingArea->allocation.height/2-(Yorg+(Radius+MinorRadius)-Yoff)*Zoom,
                 (Radius+MinorRadius)*2.0*Zoom, 
                 (Radius+MinorRadius)*2.0*Zoom, 0,64*360);
    Message = g_strdup_printf(_("Torus: Center <%g, %g, %g>, Major Radius: %g, Minor Radius: %g"),
            Center[0], Center[1], Center[2], Radius, MinorRadius);
    gtk_statusbar_push(GTK_STATUSBAR(StatusBar), id, Message);
    g_free(Message);
  }
}

/*****************************************************************************
*  ToolReleaseNewTorus
******************************************************************************/
void ToolReleaseNewTorus(GtkWidget *DrawingArea, GdkEvent *event)
{
  GdkEventButton *Bev = (GdkEventButton *)event;
  ViewUserDataStruct *UserData;
  GtkWidget *StatusBar;
  double Zoom, Xoff, Yoff;
  FrameViewStruct *TmpView;
  ObjectStruct *Torus;
  TorusStruct *TTorus;
  Vector Vect;

  if (TORUS_STATE == MOVING)
  {
    TORUS_STATE = BETWEEN_RADIUS;
    MinorRadius = 0.0;
    return;
  }

  if (TORUS_STATE != MINOR_RADIUS)
    return;

  UserData = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  StatusBar = UserData->StatusBar;
  Zoom = UserData->ZoomFactor;
  Xoff = UserData->Xoffset;
  Yoff = UserData->Yoffset;

  gtk_statusbar_pop(GTK_STATUSBAR(StatusBar), id);

  TTorus = g_malloc(sizeof(TorusStruct));
  Torus = (ObjectStruct *)TTorus;
  InitObject(Torus);
  Torus->Type = TORUS_OBJECT;
  switch (UserData->CameraStyle)
  {
    case ORTHO_XY_CAMERA:
      Vect[0] = 90.0;
      Vect[1] = 0.0;
      Vect[2] = 0.0;
      RotateTorus(Torus, Vect);
      break;
    case ORTHO_ZY_CAMERA:
      Vect[0] = 0.0;
      Vect[1] = 0.0;
      Vect[2] = 90.0;
      RotateTorus(Torus, Vect);
      break;
  }
  TranslateTorus(Torus, Center);
  TTorus->Major = Radius; TTorus->Minor = MinorRadius;
  Torus->Next = UserData->FrameView->FirstObject;
  UserData->FrameView->FirstObject = Torus;
  ComputeTorusTriangleMesh(Torus);
  TORUS_STATE = NONE;
  gdk_gc_destroy(gcPurple);
  for (TmpView = UserData->FrameView->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, UserData->FrameView);
  gdk_pointer_ungrab(Bev->time);
}

/*****************************************************************************
*  PDBCreateTorus
******************************************************************************/
int PDBCreateTorus(int *Id, double *MajorRadius, double *MinorRadius)
{
  ObjectStruct *Torus;
  TorusStruct *TTorus;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  TTorus = g_malloc(sizeof(TorusStruct));
  Torus = (ObjectStruct *)TTorus;
  InitObject(Torus);
  Torus->Type = TORUS_OBJECT;
  TTorus->Major = *MajorRadius;
  TTorus->Minor = *MinorRadius;

  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
                  ;
  Torus->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Torus;
  ComputeTorusTriangleMesh(Torus);
  for (TmpView = LocalFrame->FirstView ;
       TmpView ;
       TmpView = TmpView->Next)
  gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Torus->Id;
}

