/* triangle.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "triangle.h"

/*****************************************************************************
*  InsideTriangle
******************************************************************************/
int InsideTriangle(ObjectStruct *Triangle, double x, double y, double z)
{
  return 0; /* We cannot be "inside" a triangle... */
}

/*****************************************************************************
*  FindIntersectTriangleSegment
******************************************************************************/
int FindIntersectionTriangleSegment(Vector InterPoint, Vector InterNorm,
                                    Vector InPoint, Vector OutPoint,
                                    ObjectStruct *Triangle)
{
  return 0; /* Triangles shouldn't be used in CSG... */
}

/*****************************************************************************
*  ComputeTriangleTriangleMesh
******************************************************************************/
void ComputeTriangleTriangleMesh(ObjectStruct *Triangle)
{ /* XXX: clipped_by */
  Vector P1, P2, P3, N1, N2, N3;
  TriangleStruct *TTriangle = (TriangleStruct *)Triangle;

  if (Triangle->FirstTriangle)
    DestroyObjectTriangleMesh(Triangle);
  P1[0] = TTriangle->P1[0];
  P1[1] = TTriangle->P1[1];
  P1[2] = TTriangle->P1[2];
  P2[0] = TTriangle->P2[0];
  P2[1] = TTriangle->P2[1];
  P2[2] = TTriangle->P2[2];
  P3[0] = TTriangle->P3[0];
  P3[1] = TTriangle->P3[1];
  P3[2] = TTriangle->P3[2];
  N1[0] = TTriangle->N1[0];
  N1[1] = TTriangle->N1[1];
  N1[2] = TTriangle->N1[2];
  N2[0] = TTriangle->N2[0];
  N2[1] = TTriangle->N2[1];
  N2[2] = TTriangle->N2[2];
  N3[0] = TTriangle->N3[0];
  N3[1] = TTriangle->N3[1];
  N3[2] = TTriangle->N3[2];
  if (Triangle->Trans)
  {
    MEvaluatePoint(P1, Triangle->Trans, P1);
    MEvaluatePoint(P2, Triangle->Trans, P2);
    MEvaluatePoint(P3, Triangle->Trans, P3);
    MEvaluateVector(N1, Triangle->Trans, N1);
    MEvaluateVector(N2, Triangle->Trans, N2);
    MEvaluateVector(N3, Triangle->Trans, N3);
  }
  AddTriangleToObjectMesh(Triangle, P1,P2,P3, N1,N2,N3);
}

/*****************************************************************************
*  TranslateTriangle
******************************************************************************/
void TranslateTriangle(ObjectStruct *Triangle, Vector Vect)
{
  TransformStruct Trans;

  if (Triangle->Trans)
  {
    ComputeTranslateTrans(&Trans, Vect);
    ComposeTrans(Triangle->Trans, &Trans);
  } else
  { /* FIXME: can easily be optimized */
    Triangle->Trans = g_malloc(sizeof(TransformStruct));
    ComputeTranslateTrans(Triangle->Trans, Vect);
  }
  if (Triangle->ClippedBy)
    TranslateObject(Triangle->ClippedBy, Vect);
}

/*****************************************************************************
*  RotateTriangle
******************************************************************************/
void RotateTriangle(ObjectStruct *Triangle, Vector Vect)
{
  TransformStruct Trans;

  if (Triangle->Trans)
  {
    ComputeRotateTrans(&Trans, Vect);
    ComposeTrans(Triangle->Trans, &Trans);
  } else
  { /* FIXME: can easily be optimized */
    Triangle->Trans = g_malloc(sizeof(TransformStruct));
    ComputeRotateTrans(Triangle->Trans, Vect);
  }
  if (Triangle->ClippedBy)
    RotateObject(Triangle->ClippedBy, Vect);
}

/*****************************************************************************
*  ScaleTriangle
******************************************************************************/
void ScaleTriangle(ObjectStruct *Triangle, Vector Vect)
{
  TransformStruct Trans;

  if (Triangle->Trans)
  {
    ComputeScaleTrans(&Trans, Vect);
    ComposeTrans(Triangle->Trans, &Trans);
  } else
  { /* FIXME: can easily be optimized */
    Triangle->Trans = g_malloc(sizeof(TransformStruct));
    ComputeScaleTrans(Triangle->Trans, Vect);
  }
  if (Triangle->ClippedBy)
    ScaleObject(Triangle->ClippedBy, Vect);
}

/*****************************************************************************
*  IsTriangleIntersection
******************************************************************************/
int IsTriangleIntersection(ObjectStruct *Triangle, Vector Origin, Vector Direction)
{ /* XXX: clipped_by */
  Vector Org, Dir;
  int i;
  Matrix Mat, InvMat;
  double D;
  double u, v;
  TriangleStruct *TTriangle = (TriangleStruct *)Triangle;

  if (Triangle->Trans)
  {
    MInverseEvaluatePoint(Org, Triangle->Trans, Origin);
    MInverseEvaluateVector(Dir, Triangle->Trans, Direction);
  } else
  {
    for (i=0 ; i<3 ; i++)
    {
      Org[i] = Origin[i];
      Dir[i] = Direction[i];
    }
  }
  Mat[0][0] = TTriangle->P2[0] - TTriangle->P1[0];
  Mat[0][1] = TTriangle->P2[1] - TTriangle->P1[1];
  Mat[0][2] = TTriangle->P2[2] - TTriangle->P1[2];
  Mat[1][0] = TTriangle->P3[0] - TTriangle->P1[0];
  Mat[1][1] = TTriangle->P3[1] - TTriangle->P1[1];
  Mat[1][2] = TTriangle->P3[2] - TTriangle->P1[2];
  Mat[2][0] = -Dir[0];
  Mat[2][1] = -Dir[1];
  Mat[2][2] = -Dir[2];
  D = Mat[0][0]*Mat[1][1]*Mat[2][2]+Mat[0][1]*Mat[1][2]*Mat[2][0]+
      Mat[0][2]*Mat[1][0]*Mat[2][1]-Mat[2][0]*Mat[1][1]*Mat[0][2]-
      Mat[2][1]*Mat[1][2]*Mat[0][0]-Mat[2][2]*Mat[1][0]*Mat[0][1];
  if (D == 0.0)
    return 0;
  InvMat[0][0] =  (Mat[1][1]*Mat[2][2]-Mat[2][1]*Mat[1][2])/D;
  InvMat[0][1] = -(Mat[0][1]*Mat[2][2]-Mat[2][1]*Mat[0][2])/D;
  InvMat[0][2] =  (Mat[0][1]*Mat[1][2]-Mat[1][1]*Mat[0][2])/D;
  InvMat[1][0] = -(Mat[1][0]*Mat[2][2]-Mat[2][0]*Mat[1][2])/D;
  InvMat[1][1] =  (Mat[0][0]*Mat[2][2]-Mat[2][0]*Mat[0][2])/D;
  InvMat[1][2] = -(Mat[0][0]*Mat[1][2]-Mat[1][0]*Mat[0][2])/D;
  InvMat[2][0] =  (Mat[1][0]*Mat[2][1]-Mat[2][0]*Mat[1][1])/D;
  InvMat[2][1] = -(Mat[0][0]*Mat[2][1]-Mat[2][0]*Mat[0][1])/D;
  InvMat[2][2] =  (Mat[0][0]*Mat[1][1]-Mat[1][0]*Mat[0][1])/D;
  u = InvMat[0][0]*(Org[0]-TTriangle->P1[0])+
      InvMat[1][0]*(Org[1]-TTriangle->P1[1])+
      InvMat[2][0]*(Org[2]-TTriangle->P1[2]);
  v = InvMat[0][1]*(Org[0]-TTriangle->P1[0])+
      InvMat[1][1]*(Org[1]-TTriangle->P1[1])+
      InvMat[2][1]*(Org[2]-TTriangle->P1[2]);
  if ((u<0.0) || (u>1.0) || (v<0.0) || (v>1.0))
    return 0;
  if (u+v>1.0)
    return 0;
  return 1;
}

/*****************************************************************************
*  ClickInTriangleXY
******************************************************************************/
int ClickInTriangleXY(ObjectStruct *Triangle, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 0.0, 1.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[1] = y;
  return IsTriangleIntersection(Triangle, Origin, Direction);
}

/****************************************************************************
*  ClickInTriangleXZ
*****************************************************************************/
int ClickInTriangleXZ(ObjectStruct *Triangle, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {0.0, 1.0, 0.0, 0.0, 0.0};

  Origin[0] = x;
  Origin[2] = y;
  return IsTriangleIntersection(Triangle, Origin, Direction);
}

/****************************************************************************
*  ClickInTriangleZY
*****************************************************************************/
int ClickInTriangleZY(ObjectStruct *Triangle, gdouble x, gdouble y)
{ /* XXX: clipped_by */
  Vector Origin = {0.0, 0.0, 0.0, 0.0, 0.0};
  Vector Direction = {1.0, 0.0, 0.0, 0.0, 0.0};

  Origin[2] = x;
  Origin[1] = y;
  return IsTriangleIntersection(Triangle, Origin, Direction);
}

/*****************************************************************************
*  ToolNewTriangle FIXME
******************************************************************************/
void ToolNewTriangle(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget *Dialog;
  GtkWidget *Button;
  GtkWidget *Label;

  Dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(Dialog), _("New Triangle"));

  Label = gtk_label_new(_("Sorry, the New Triangle function"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);
  Label = gtk_label_new(_("hasn't been implemented yet"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Label, TRUE, TRUE, 0);
  gtk_widget_show(Label);

  Button = gtk_button_new_with_label("Close");
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialog)->action_area), Button,
                     TRUE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(Dialog));
  gtk_widget_grab_default(Button);
  gtk_widget_show(Button);
  gtk_widget_show(Dialog);
}

/*****************************************************************************
*  CreateTriangle
******************************************************************************/
ObjectStruct *CreateTriangle(Vector P1, Vector P2, Vector P3)
{
  TriangleStruct *TTriangle;
  ObjectStruct *Triangle;
  Vector V1, V2;

  TTriangle = g_malloc(sizeof(TriangleStruct));
  Triangle = (ObjectStruct *)TTriangle;
  InitObject(Triangle);
  Triangle->Type = TRIANGLE_OBJECT;

  TTriangle->P1[0] = P1[0];
  TTriangle->P1[1] = P1[1];
  TTriangle->P1[2] = P1[2];
  TTriangle->P2[0] = P2[0];
  TTriangle->P2[1] = P2[1];
  TTriangle->P2[2] = P2[2];
  TTriangle->P3[0] = P3[0];
  TTriangle->P3[1] = P3[1];
  TTriangle->P3[2] = P3[2];

  V1[0] = TTriangle->P2[0] - TTriangle->P1[0];
  V1[1] = TTriangle->P2[1] - TTriangle->P1[1];
  V1[2] = TTriangle->P2[2] - TTriangle->P1[2];
  V2[0] = TTriangle->P3[0] - TTriangle->P1[0];
  V2[1] = TTriangle->P3[1] - TTriangle->P1[1];
  V2[2] = TTriangle->P3[2] - TTriangle->P1[2];
  VCross(TTriangle->N1, V1, V2);
  VCross(TTriangle->N2, V1, V2);
  VCross(TTriangle->N3, V1, V2);

  ComputeTriangleTriangleMesh(Triangle);
  return Triangle;
}

/*****************************************************************************
*  PDBCreateTriangle
******************************************************************************/
int PDBCreateTriangle(int *Id, Vector *V1, Vector *V2, Vector *V3)
{
  ObjectStruct *Triangle;
  FrameStruct *LocalFrame;
  FrameViewStruct *TmpView;

  Triangle = CreateTriangle(*V1, *V2, *V3);
  for (LocalFrame = FirstFrame ;
       LocalFrame->Id != *Id ;
       LocalFrame = LocalFrame->Next)
    ;
  Triangle->Next = LocalFrame->FirstObject;
  LocalFrame->FirstObject = Triangle;
  ComputeTriangleTriangleMesh(Triangle);
  for (TmpView=LocalFrame->FirstView ; TmpView ; TmpView=TmpView->Next)
    gtk_widget_queue_draw(TmpView->DrawingArea);
  RebuildCSGTree(NULL, LocalFrame);
  return Triangle->Id;
}

