#!/usr/local/bin/python
#
# adzapper
# Copyright 1999 Adam Feuer
# Adam Feuer <adamf@pobox.com>
#
# This file is part of adzapper.
#
# adzapper is free software; you can redistribute it and/or modify
# it under the terms of the Zope Public License 1.0 as published by
# Digital Creations.
#
# adzapper is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Zope Public License for more details.
#
# You should have received a copy of the Zope Public License 1.0
# along with adzapper; see the file COPYING.
#
#  "This product includes software developed by Digital Creations
#  for use in the Z Object Publishing Environment
#  (http://www.zope.org/)."
#
#
# adzapper is an http proxy that filters out ads
#		
#
RCS_ID =  '$Id: adzapper.py,v 1.1.1.1 1999/08/14 22:42:11 adamf Exp $'

# global variables
MAJOR_VERSION = 0
MINOR_VERSION = 1
SUB_VERSION   = 26     # ;-)  

# zaplet config stuff
ZAPLET_DIR = 'zaplets'
ADZAPPER_CONFIG_FILE = 'adzapper.conf'

# python standard library modules
import socket
import string
import time
import re
import sys
import os
import getopt

# ZServer modules
import asyncore
import asynchat
import http_server
import resolver

# adzapper and zaplet modules
import debug
import platform_dependent
import zaplet
import blank_gif

# other constants
ACCESSCONTROL_ERROR = """<html>
<head>
<title>Error response</title
</head>
<body>
<h1>Access Control Error</h1>
<p>
You are not authorized to access this server.
</body>
"""

numeric_host = re.compile ('[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+')

class http_proxy_handler:

	url_re = re.compile (
		# groups:  1=host 2=port 3=uri
		'http://([^/:]+)(:[0-9]+)?([^ ]*).*',
		re.I
		)

	def __init__(self, resolver=None, zaplet_engine=None):
		self.resolver = resolver
		self.zaplet_engine = zaplet_engine
				
	def match (self, request):
		uri = request.uri
		self.match_obj = self.url_re.search(uri)
		if self.match_obj.group(1) != None:
			return 1
		else:
			return 0		


	def handle_request (self, request):
		# install the proxy version of the writable predicate
		request.channel.writable = request.channel.writable_for_proxy
		# our task: feed the request object a producer that will either
		# answer it correctly, or produce an error message.
		http_proxy_producer (
			self.match_obj.group(1),
			self.match_obj.group(2),
			self.match_obj.group(3),
			request,
			self.resolver,
			self.zaplet_engine
			)
		# for now, be sure and close the connection
		#request['Connection'] = 'close'
		#request.channel.producer_fifo.push (None)

# this producer implements two interfaces: the async_chat interface,
# and the 'stallable producer' interface.  when this object is at the
# front of an http channel's producer fifo, it will affect the channel's
# response to the 'writable' predicate, depending on whether or not
# any proxy data has accumulated.

class http_proxy_producer (asynchat.async_chat):

	def __init__ (self, host, port, uri, request, resolver,zaplet_engine):

		if zaplet_engine == None:
			debug.debug(2,"zaplet_engine == None, no access control")
		else:
			# access control - check requester's IP against our list of valid IPs
			if zaplet_engine.accesscontroller(request.requesterIP):
				self.send_accesscontrol_error()
				return

		# ok, passed access control-- now let's start getting the URL
		self.zaplet_engine = zaplet_engine

		self.host = host
		self.uri = uri
		self.request = request

		mn = numeric_host.search(host)
		if mn == None:
			resolved = 0
			self.ip = None
		else:
			resolved = 1
			self.ip = host

		if port == None:
			self.port = 80
			portstr = ''
		else:
			portstr = str(port)
			self.port = string.atoi (portstr[1:])


		self.url = 'http://' + host + portstr + uri
		debug.debug(2, "adzapper: about to block url: %s" % self.url)

		if self.zaplet_engine != None:
			# urlblocker function set-- so use it to check URL
			if self.zaplet_engine.urlblocker(self.url):
				self.send_replacement_gif(self.zaplet_engine.replacement_gif)
				return
		else:
			# no zaplet_engine function-- block nothing.
			debug.debug(2,"zaplet engine == None, no urlblocker")


		# using a proxy?
		if self.zaplet_engine.http_proxy == None:
			# no proxy
			if not resolved:
				resolver.resolve (host, self.resolver_callback)
			else:
				self.resolver_callback (host, 10, self.ip)
		else:
			# we are using a proxy
			self.ip = self.zaplet_engine.http_proxy_ip
			self.port = self.zaplet_engine.http_proxy_port
			self.resolver_callback (host, 10, self.ip) # host and 10 are just placeholders

		self.outgoing_buffer = ''


	def resolver_callback (self, host, ttl, ip):
			asynchat.async_chat.__init__ (self)
			self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
			# handle errors!
			self.ip = ip
			self.set_terminator (None)
			self.connect ((self.ip, self.port))

	def collect_incoming_data (self, data):
		s = 'http_proxy_producer: pushing %d bytes' % len(data)
		debug.debug(1,s)
		self.request.channel.push (data)

	def found_terminator (self):
		debug.debug(1,'found_terminator should not have been called')

	def handle_connect (self):
		debug.debug(5,'handle_connect: connected')
		
		if self.zaplet_engine != None:
			if self.zaplet_engine.http_proxy == None:
				# we're not using a proxy
				request_string = self.uri
			else:
				# we are using a proxy
				request_string = self.url
				
		# for now, just send all the headers, and use HTTP/1.0
		s = string.join (
				['%s %s HTTP/1.0' % (
					string.upper (self.request.command),
					request_string,
					)] + self.request.header,
				'\r\n'
				) + self.request.postdata + '\r\n\r\n'
		
		self.push (s)
		debug.debug(5,"")
		debug.debug(5,"..................")
		debug.debug(5, "Headers + postdata : ")
		debug.debug(5, s)
		debug.debug(5, "..................")
		debug.debug(5,"")
		debug.debug(5, "handle_connect: %s " % self.request)
		debug.debug(5, "handle_connect.postdata: [%s] " % self.request.postdata)
		
	def handle_close (self):
		self.done = 1
		self.close()
		self.request.channel.close_when_done()

	# this method is called by http_channel.writable_for_proxy(),
	# to let it know whether or not any data is actually available.
	def stalled (self):
		return len(self.outgoing_buffer) == 0 and not self.done

	def more (self):
		# just return the whole buffer.
		r = self.outgoing_buffer
		self.outgoing_buffer = ''
		return r

	def send_replacement_gif (self,gif):
		if gif == None:
			gif = blank_gif.default_gif
		self.request['Content-Type'] = 'image/gif'
		self.request['Content-Length'] = gif.length
		self.request.push (gif.data)
		self.request.done()

	def send_accesscontrol_error (self):
		self.request['Content-Type'] = 'text/html'
		self.request['Content-Length'] = len(ACCESSCONTROL_ERROR)
		self.request.push (ACCESSCONTROL_ERROR)
		self.request.done()


def print_help (program_name):
	print 'usage: %s [Options] [<port>]' % program_name
	print 'Options:'
	print '  -d <debug level>               Higher numbers produce greater detail'
	print '  -p                             Turn profiling on'
	print '  -v                             Display version number'
	print '  -n <nameserver ip address>     Use nameserver at this ip address'
	print '  -z <zaplet directory>          Path to zaplet directory'
	print '  -f <zaplet config file>        Name of zaplet configuration file in zaplet directory'
	print '  -h                             Print this help message'
	print '  <port>                         Port to run on. Default is port 51966.'

	
class adzapper:
	def __init__(self):
		# debugging defaults to off
		debug.debuglevel = 0 

		# profiling defaults to off
		self.profiling_on = 0
		
		# we don't have the nameserver IP address yet
		self.nameserver = None

		# set the defaults for zaplet directory and config file
		self.zaplet_dir = ZAPLET_DIR
		self.adzapper_config_file = ADZAPPER_CONFIG_FILE

		self.program_name = sys.argv[0]
		args = sys.argv[1:]
		debug.debug (1,("args: ",  args))

		try:
			optlist, arglist = getopt.getopt(args, 'hpvd:n:z:f:')
		except getopt.error , error_message:
			print error_message
			print
			print_help(self.program_name)
			raise SystemExit
	
		debug.debug(1,('args: ' , args))
		debug.debug(1,('optlist' , optlist))
		debug.debug(1,('arglist: ' , arglist))

		if arglist == []:
			self.port = 51966
		else:
			debug.debug(1, 'port: %s ' % arglist[0])
			self.port = string.atoi(arglist[0])

		if '-h' in args:
			print_help(self.program_name)
			raise SystemExit

		if '-v' in args:
			VERSION_STRING = '%d.%d.%d' % (MAJOR_VERSION,MINOR_VERSION,SUB_VERSION)
			print '%s version: %s' % (self.program_name,VERSION_STRING)
			raise SystemExit
	
		if '-p' in args:
			self.profiling_on = 1

		if '-d' in args:
			for list_item in optlist:
				if list_item[0] == '-d':
					debug.debuglevel = string.atoi(list_item[1])

		if '-n' in args:
			for list_item in optlist:
				if list_item[0] == '-n':
					self.nameserver = list_item[1]

		if '-z' in args:
			# note: if more then one '-z' is specified,
			# the last one on the command line is used.
			for list_item in optlist:
				if list_item[0] == '-z':
					self.zaplet_dir = list_item[1]
		if '-f' in args:
			for list_item in optlist:
				if list_item[0] == '-f':
					self.zaplet_config_file = list_item[1]

		debug.debug(1,'Debugging output is turned on at level %d' % debug.debuglevel)

		if (self.nameserver == None):
			# get the nameserver IP address for resolver
			self.nameserver = platform_dependent.getNameServer()
			if (self.nameserver == None):
				# nameserver defaults to localhost
				self.nameserver = '127.0.0.1'
		debug.debug(1,"nameserver: %s" % self.nameserver)

		# make the persistent resolver
		debug.debug(2,'Using nameserver %s' % self.nameserver)
		self.resolver = resolver.caching_resolver (self.nameserver)

		# initialize zaplet engine 
		self.zaplet_engine = zaplet.zaplet_engine(self.zaplet_dir,self.adzapper_config_file)

		# install signal handler to catch the signal SIGHUP 
	        # (only do this for Unix platforms)
		platform_dependent.setSighupHandler(self.zaplet_engine.__init__,
						    (self.zaplet_dir,self.adzapper_config_file))


	def run(self):
		# make the handler- need to give it the resolver and zaplet_engine
		ph = http_proxy_handler(self.resolver,self.zaplet_engine)
		hs = http_server.http_server ('', self.port)
		hs.install_handler (ph)
		if self.profiling_on:
			def profile_loop ():
				try:
					asyncore.loop()
				except KeyboardInterrupt:
					pass
				import profile
				profile.run ('profile_loop()', 'profile.out')
		else:
			asyncore.loop()
				
	
if __name__ == '__main__':

	adz = adzapper()
	adz.run()
