#
# This file is part of Zaplet
# Copyright 1999 Adam Feuer <adamf@pobox.com>
#
# Zaplet is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Zaplet is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zaplet; see the file COPYING-Zaplet. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#


# platform dependent code
# works on Win32 (WinNT, Win9x) and on Unix

RCS_ID =  '$Id: platform_dependent.py,v 1.3 1999/08/15 06:32:05 adamf Exp $'

import os
import sys
import re
import string


import debug

# Win32 Registry constants
WinNT_KEYNAME = "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters"
Win9x_KEYNAME = "SYSTEM\\CurrentControlSet\\Services\\VxD\\MSTCP"
VALUENAME = "NameServers"


# this is for unix only
class signalHandler:
	def __init__(self,handler,arglist,message=None):
		self.handler = handler
		self.arglist = arglist
		self.message = message

	def handle_it(self,signal, stackframe):
		if self.message:
			debug.debug(1,self.message)
		if self.handler:
			self.handler(self.arglist)
		return


# get the name server IP address
def getNameServer():

	platformName = sys.platform
	debug.debug(2,"platform: %s" % platformName)

	win32_re = re.compile("win32")
	winnt_re = re.compile("Windows_NT")


	# what platform are we on? (Win32 or Unix)
	if win32_re.search(platformName):
		# we are on win32
		debug.debug(2, "getnameserver: platform is Win32")
		osType = "Win32"

		# need to import these now
		import win32api, win32con
		
		KEYROOT = win32con.HKEY_LOCAL_MACHINE

		osName = os.environ['OS']

		# we need to know if it is WinNT or Win9x since the nameserver
		#  IP numbers are in different places
		if winnt_re.search(osName):
			debug.debug(2, "getnameserver: OS is Windows NT")
			KEYNAME = WinNT_KEYNAME
		else:
			debug.debug(2, "getnameserver: OS is Windows 9x")
			KEYNAME = Win9x_KEYNAME
	else:
		# we are on Unix (hopefully)
		debug.debug(2, "getnameserver: platform is Unix")
		osType = "Unix"


	if (osType == "Win32"):
		# get the nameserver out of the Windows registry
		
		keyRoot = KEYROOT
		keyName = WinNT_KEYNAME
		valueName = VALUENAME
		
		hkey = win32api.RegOpenKey(keyRoot,keyName)
		try:
			value, type = win32api.RegQueryValueEx(hkey,"NameServer")
		finally:
			win32api.RegCloseKey(hkey)
			nameservers = string.split(value, ' ')
			return nameservers[0]

	elif (osType == "Unix"):
		# see if there is a nameserver in /etc/resolv.conf
		# if so, use it-- it seems to be more reliable than localhost
		debug.debug(4,"getnameserver: osType == Unix, looking for nameserver.")
		nsip_re = re.compile('^(nameserver)\s((\d)+\.(\d)+\.(\d)+\.(\d)+)(\s*)$')
		try:
			f = open('/etc/resolv.conf','r')
			lines = f.readlines()
			f.close
		except:
				debug.debug(4,"getnameserver: file exception.")
				pass

		for l in lines:
			debug.debug(4,"getnameserver: resolv.conf line: %s" % l)
			m = nsip_re.match(l)
			if m != None:
				debug.debug(4,"getnameserver: Nameserver match: %s" % m.group(2))
				return m.group(2)
			
		debug.debug(4,"getnameserver: No nameserver match.")
		return None
	else:
		pass
	
	# unknown osType
	return None


def setSighupHandler(handler,arglist):
	if (sys.platform != 'win32'):
		# we are on Unix, hopefully!
		import signal

		# make a new handler object
		h = signalHandler(handler, arglist,"caught SIGHUP")
		signal.signal(signal.SIGHUP,h.handle_it)


if __name__=='__main__':

	nameserver = getNameServer()
	print "nameserver: %s" % nameserver



	




