#
# This file is part of Zaplet
# Copyright 1999 Adam Feuer <adamf@pobox.com>
#
# Zaplet is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Zaplet is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zaplet; see the file COPYING-Zaplet. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

# zaplet classes
# for use with adzapper filtering proxy

RCS_ID =  '$Id: zaplet.py,v 1.3 1999/08/14 23:50:18 adamf Exp $'

# constants
ALLOWDENY = 0
DENYALLOW = 1

import os
import re
import string
import os
import sys
import urlparse
import socket

# adzapper modules
import debug
import blank_gif

numeric_host = re.compile ('[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+')
numeric_network = re.compile ('[0-9]+\.([0-9]+\.)?([0-9]+\.)?([[0-9]+\*])?')

# returns true if string is enclosed in quotes
def quoted_string(arg):
	l = len(arg)
	if l > 1:
		if arg[0] == "\"" and arg[l-1] == "\"":
			return 1
		elif arg[0] == "\'" and arg[l-1] == "\'":
			return 1
	return 0

# returns true of string is enclosed in '/'
def regex_string(arg):
	l = len(arg)
	if l > 1:
		if arg[0] == '/' and arg[l-1] == '/':
			return 1
	return 0

# cut off the first and last characters of the string
def trim(arg):
	l = len(arg)
	if l > 2:
		return arg[1:l-1]
	else:
		return ""

class zaplet:
	# data structure object to hold zaplet
	def __init__(self):
		# zaplet version defaults to 0.1
		self.majorversion = 0
		self.minorversion = 1
		self.host = ''
		self.allow = ''
		self.block = ''
		self.lines = []

class zaplet_engine:

	# initialize the zaplet_engine object
	# build the {site : block_regex} dictionary

	def __init__(self, arg1, arg2=None):
		# init can be called 2 ways:
		# way 1: arg1 = 2 element list (ZAPLET_DIR,ADZAPPER_CONFIG_FILE)
		# way 2: arg1 = ZAPLET_DIR, arg2 = ADZAPPER_CONFIG_FILE

		if arg2 == None and type(arg1) == type(()):
			# way 1
			debug.debug(3, "way 1")
			if len(arg1) != 2:
				sys.stderr.write("zaplet_engine: argument list must have 2 elements.\n")
				sys.exit(0)
			else:
				self.zPath = arg1[0]
				self.zEngineConfigFile = arg1[1]
		elif arg2 != None and type(arg1) == type (arg2) == type(""):
			# way 2
			debug.debug(3, "way 2")
			self.zPath = arg1
			self.zEngineConfigFile = arg2
		else:
			sys.stderr.write("zaplet_engine: incorrect argument types.\n")
			sys.exit(0)
		
		# dictionary of (site, zaplet object instance) pairs
		self.zaplet_list = { }
		
		# access control list - lists of IPs we should allow or deny
		# defaults to off
		self.acOn = 0
		self.aclOrder = DENYALLOW  # defaults to deny all, then allow specific hosts and networks
		self.aclAllow = []
		self.aclDeny = []
		self.aclAllow_re = None
		self.aclDeny_re = None

		# replacement gif
		self.replacement_gif = blank_gif.default_gif

		# HTTP proxy - default is None
		self.http_proxy = None

		# adzapper.conf version defaults to 0.1
		self.zconf_majorversion = 0  
		self.zconf_minorversion = 1

		# make sure zPath has a '/' on the end-- read_config_file needs this.
                if self.zPath[len(self.zPath)-1] != '/':
                        self.zPath = self.zPath + '/'

		self.read_config_file(self.zPath,self.zEngineConfigFile)


	# read the adzapper.conf file
	# this file contains global settings for adzapper such as access control, etc.
	# note: zPath must have a trailing '/' 
	def read_config_file(self,zPath,zEngineConfigFile):
		debug.debug(1,"Initializing zaplet engine...")
		cpairs = []  # list of (site, conf file) pairs

		zengcfg = zPath + zEngineConfigFile

		# read the zaplet.conf file
		# this will hold configuration options for adzapper as a whole;
		# it is not currently used.
		try:
			debug.debug(1," opening zaplet.conf  file: " + zengcfg)
			cf = open(zengcfg, "r")
		except:
			sys.stderr.write("Cannot open zaplet.conf file "+ zengcfg +"\n")
			sys.exit(0)
			
	       	lines = cf.readlines()
		cf.close()
		for line in lines:
			line = string.strip(line)
			debug.debug(1," line: " + line)
			if (len(line) == 0) or re.search("^\s*#", line):
				continue
			if string.find(line, '#'):
				ltmp = string.split(line, '#')
				line = ltmp[0]
			tokens = re.split('\s+', line)
			command = string.lower(tokens[0])
			if (command == 'version'):
				vstr = tokens[1]
				vlist = string.split(vstr,'.')
				self.zconf_majorversion = string.atoi(vlist[0])
				self.zconf_minorversion = string.atoi(vlist[1])
				debug.debug(1,'zaplet.conf version: %i.%i' %
					    (self.zconf_majorversion, self.zconf_minorversion))

			elif (command == 'order'):
				arg = tokens[1]
				self.setaclOrder(arg)
			elif (command == 'allow'):
				args = tokens[1:]
				self.addaclAllow(args)
			elif (command == 'deny'):
				args = tokens[1:]
				self.addaclDeny(args)
			elif (command == 'replacement_gif'):
				arg = tokens[1]
				self.read_replacement_gif(arg)
			elif (command == 'http_proxy'):
				arg = tokens[1]
				self.set_http_proxy(arg)
			elif (command == 'pid_file'):
				arg = tokens[1]
				self.set_pid_file(arg)
			else:
				sys.stderr.write("Bad configuration line: "+line+"\n")

		# end for


		# initialize the access control
		if len(self.aclAllow) > 0:
			self.acOn = 1
			reStr = ''
			debug.debug(4,"init acl: %s" % self.aclAllow)
			for arg in self.aclAllow:
				reStr = reStr + '|' + arg
			# chop off initial '|'
			reStr = reStr[1:]
			debug.debug(4, "zaplet acl allow: %s" % reStr)
			self.aclAllow_re = re.compile(reStr)

		if len(self.aclDeny) > 0:
			self.acOn = 1
			reStr = ''
			debug.debug(4,"init acl: %s" % self.aclDeny)
			for arg in self.aclDeny:
				reStr = reStr + '|' + arg
			# chop off initial '|'
			reStr = reStr[1:]
			debug.debug(4, "zaplet acl deny: %s" % reStr)
			self.aclDeny_re = re.compile(reStr)			

		# make the list of zaplets
		zdir = os.listdir(zPath)
		dotfile =  re.compile('^\.')
		conffile = re.compile('.*\.conf$')
		zlist = []
		for file in zdir:
			if os.path.isfile(zPath + file):
				if not dotfile.search(file) and not conffile.search(file):
					zlist.append(file)
		debug.debug(1,"processed source directory: ")
		debug.debug(1,zlist)

		# now go thru list of zaplets, read them in and compile them.
		for site in zlist:
			debug.debug(1,"site: " + site + "        zaplet file: " + site)
			tz = self.read_zaplet(zPath,site)

			# if there is a 'host' statement, use it as the host;
			# otherwise, use the filename
			if tz.host == '':
				tz.host = site
				debug.debug(1,"tz.host: %s " % tz.host)


			self.zaplet_list[tz.host] = tz

		debug.debug(1,self.zaplet_list.keys())


	# given the name of a zapletfile, read it in and return a zaplet object,
	#  complete with compiled regexes
	def read_zaplet(self,zPath,zapletfile):
		# make a new zaplet
		z = zaplet()
		
		block_re_str = '' # the block regex that will be built for this zaplet
		allow_re_str = '' # the allow regex that will be built for this zaplet

		zcfile = zPath + zapletfile
		
		try:
			debug.debug(1," opening zaplet : " + zcfile)
			cf = open(zcfile, "r")
		except:
			sys.stderr.write("Cannot open zaplet " + zcfile + "\n")
			sys.exit(0)

		z.lines = cf.readlines()
		cf.close()

		in_zaplet = 0
		for line in z.lines:
			if in_zaplet:
				line = string.strip(line)
				if (len(line) == 0) or re.search("\s*#", line):
					continue
				if string.find(line, '#'):
					ltmp = string.split(line, '#')
					line = ltmp[0]
				tokens = re.split('\s+', line)
				command = string.lower(tokens[0])
				if (command == 'version'):
					vstr = tokens[1]
					vlist = string.split(vstr,'.')
					z.majorversion = string.atoi(vlist[0])
					z.minorversion = string.atoi(vlist[1])
					debug.debug(1,'zaplet version: %i.%i' % (z.majorversion, z.minorversion))
				elif (command == 'host'):
					z.host = tokens[1]
					debug.debug(1,'zaplet host: %s' % z.host)
				elif (command == 'block'):
					if len(tokens) > 1:
						arg = tokens[1]
					else:
						arg = ""
					if regex_string(arg):  
						# regular expression - don't escape
						# trim slashes and add to regex
						debug.debug (2,("allow regular expression: ",arg))
						re_add = trim(arg)
						debug.debug(2,("block trimmed regex: ", re_add))
						block_re_str =  block_re_str + '|' + re_add 
					elif quoted_string(arg):
						# literal string - trim quotes & escape
						debug.debug (2,("block quoted string: ",arg))
						re_add = trim(arg)
						debug.debug(2, ("block trimmed string: ", re_add))
						re_add = re.escape(re_add)
						block_re_str = block_re_str + '|' + re_add
					else:   # literal string - need to escape it.
						re_add = re.escape(arg)
						block_re_str = block_re_str + '|' + re_add
						debug.debug(2, ("block literal string: ", re_add))
				elif (command == 'allow'):
					if len(tokens) > 1:
						debug.debug(2, "tokens[1]: %s" % tokens[1])
						arg = tokens[1]
					else:
						arg = ""
					if regex_string(arg):  
						# regular expression - don't escape
						# trim slashes and add to regex
						debug.debug (2,"allow regular expression: %s" % arg)
						re_add = trim(arg)
						debug.debug(2,("allow trimmed regex: ", re_add))
						allow_re_str =  allow_re_str + '|' + re_add 
					elif quoted_string(arg):
						# literal string - trim quotes & escape
						debug.debug (2,("allow quoted string: ",arg))
						re_add = trim(arg)
						debug.debug(2, ("allow trimmed string: ", re_add))
						re_add = re.escape(re_add)
						allow_re_str = allow_re_str + '|' + re_add
					else:   # literal string - need to escape it.
						re_add = re.escape(arg)
						allow_re_str = allow_re_str + '|' + re_add
						debug.debug(2, ("allow literal string: ", re_add))

				elif (command == '</zaplet>'):
					in_zaplet = 0
					continue
				else:
					debug.debug(1,"unknown command: " + command)
					pass
			else: # not in_zaplet
				if re.search('^<zaplet>',line):
					in_zaplet = 1
					continue

						
		# chop off initial |
		allow_re_str = allow_re_str[1:]
		block_re_str = block_re_str[1:]

		debug.debug(1,"allow_re_str: %s" % allow_re_str)
		debug.debug(1,"block_re_str: %s" % block_re_str)

		# compile regexes
		if allow_re_str == "":
			allow_re_str = "^$"
		z.allow = allow_re_str
		z.allow_re = re.compile(z.allow)
		
		z.block = block_re_str
		z.block_re = re.compile(z.block)

		return z


	
	# check to see whether a named host is blocked -
	# uses zaplet database, searchs from most specific to least specific
	# (whole dns name www.foo.bar.com, then foo.bar.com, then bar.com, then .com)
	# returns 1 if url should be blocked, 0 if not.

	def checkNamedHost(self,host,url):
		srv_fields = string.split(host,'.')
		#debug.debug(1,srv_fields)
		while (srv_fields != []):
			tmps = string.join(srv_fields,'.');
			if (self.zaplet_list.has_key(tmps)):
				# we are on the site-- check for allows first
				if (self.zaplet_list[tmps].allow_re.search(url)):
					# it's specifically allowed, so let it thru
					debug.debug(2,"zap: allow- matched pattern- site: " + tmps + "    url: " + url)
					return 0
				# ok, it's not specifically allowed, now check for blocks
				elif (self.zaplet_list[tmps].block_re.search(url)):
					debug.debug(2,"zap: block matched pattern- site: " + tmps + "    url: " + url)
					return 1
				# it's not specifically blocked, so let it thru.
				else:
					# return-- we don't want to continue searching
					return 0
			del srv_fields[0]
		return 0



	# check to see whether a numeric host is blocked -
	# uses zaplet database, searchs from most specific to least specific
	# (whole ip number a.b.c.d, then network a.b.c, then net a.b, and then net a)
	# returns 1 if url should be blocked, 0 if not.

	def checkNumericHost(self,host,url):
		srv_fields = string.split(host,'.')
		debug.debug(1,"numeric host ip: %s" % srv_fields)
		lastfield = len(srv_fields)
		debug.debug(1,"numeric host ip fields: %d" % lastfield)
		for i in range(0,lastfield):
			tmps = string.join(srv_fields[0:lastfield-i],'.');
			debug.debug(1,"numeric host- iteration ip: %s" % tmps)
			if (self.zaplet_list.has_key(tmps)):
				# we are on the site-- check for allows first
				if (self.zaplet_list[tmps].allow_re.search(url)):
					# it's specifically allowed, so let it thru
					debug.debug(2,"zap: allow- matched pattern- site: " + tmps + "    url: " + url)
					return 0
				# ok, it's not specifically allowed, now check for blocks
				elif (self.zaplet_list[tmps].block_re.search(url)):
					debug.debug(2,"zap: block matched pattern- site: " + tmps + "    url: " + url)
					return 1
				# it's not specifically blocked, so let it thru.
				else:
					# return-- we don't want to continue searching
					return 0
				# default zaplet

		tmps = 'default-numeric'
		debug.debug(1,"checking default-numeric...")
		if (self.zaplet_list.has_key(tmps)):
			if (self.zaplet_list[tmps].allow_re.search(url)):
				# it's specifically allowed, so let it thru
				debug.debug(2,"zap: allow- matched pattern- site: " + tmps + "    url: " + url)
				return 0
			# ok, it's not specifically allowed, now check for blocks
			elif (self.zaplet_list[tmps].block_re.search(url)):
				debug.debug(2,"zap: block matched pattern- site: " + tmps + "    url: " + url)
				return 1
      		return 0


		

	# read in the user-supplied replacement gif
	def read_replacement_gif(self,file):
		file = self.zPath + file
		debug.debug(4,"replacement gif filename: %s " % file)
		tmplist = os.stat(file)
		self.replacement_gif.length = tmplist[6]
		file = open(file, 'r')
		self.replacement_gif.data = file.read(self.replacement_gif.length)
		file.close()



	# set the HTTP proxy url
	# (this is for setting adzapper to use a proxy server)
	def set_http_proxy(self,http_proxy_url):
		self.http_proxy = 1
		self.http_proxy_url = http_proxy_url
		
		url_tuple = urlparse.urlparse(http_proxy_url)
		netloc = url_tuple[1]
		if ':' in netloc:
			(host,port) = string.split(netloc,':')
		else:
			host = netloc
			port = '8080'

		self.http_proxy_ip = socket.gethostbyname(host)

		if len(port) == 0:
			port = '8080'
			
		self.http_proxy_port = string.atoi(port)

		debug.debug(3,"http proxy server: %s" % self.http_proxy_url)
		debug.debug(3,"http proxy ip, port: %s" % self.http_proxy_ip)
		debug.debug(3,"http proxy ip, port: %d" % self.http_proxy_port)


	# set the pid file; write the pid to this file
	def set_pid_file(self,file):
		self.pid_file = file
		debug.debug(4,"pid filename: %s " % self.pid_file)
		self.pid = "%d" % os.getpid()
		debug.debug(4,"pid: %s " % self.pid)
		file = open(file, 'w')
		file.write(self.pid)
		file.close()


	# set the access control order by interpreting the arguments
	def setaclOrder(self, arg=None):
		if arg == None:
			debug.debug(1,"syntax error: need either allow,deny or deny,allow keywords.")
		
		if (arg == 'allow,deny'):
			self.aclOrder = ALLOWDENY
			return
		if (arg == 'deny,allow'):
			self.aclOrder = DENYALLOW
			return
		debug.debug(1,"syntax error: order must be followed by \"allow,deny\" or \"deny,allow\"")


	# add an access control regex to the Allow acl
	# interpret allow strings and build regexes for them
	def addaclAllow(self,args):
		for arg in args:
			debug.debug(1,"addacl- arg: %s" % arg)
			if numeric_host.match(arg):
				debug.debug(1,"addacl- numeric host: %s" % arg)
				# it's a literal string, so we need to escape it.
				arg = re.escape(arg)
				self.aclAllow.append(arg)
			elif numeric_network.match(arg):
				debug.debug(1,"addacl- numeric network: %s" % arg)
				# if there is a '*' at the end, get rid of it
				if arg[len(arg)-1] == '*':
					arg = arg[0:len(arg)-1]
					debug.debug(1,"addacl- numeric network (chopped): %s" % arg)
				# it's a literal string, so we need to escape it.
				arg = re.escape(arg)
				self.aclAllow.append(arg)
			else:
				debug.debug(1,"addacl- no match: %s" % arg)
				if arg == 'all':
					self.aclAllow.append(arg)
				elif arg == 'from':
					pass # allow apache-like syntax
				debug.debug(1,"addacl- literal: %s" % arg)
		if 'all' in self.aclAllow:
			debug.debug(4,"addacl: found all")
			self.aclAllow = ['.*'] # match any IP address
				

	# add an access control regex to the Deny acl
	# interpret allow strings and build regexes for them
	def addaclDeny(self,args):
		for arg in args:
			debug.debug(1, "addacl- arg: %s" % arg)
			if numeric_host.match(arg):
				debug.debug(1,"addacl- numeric host: %s" % arg)
				self.aclDeny.append(arg)
			elif numeric_network.match(arg):
				debug.debug(1,"addacl- numeric network: %s" % arg)
				# if there is a '*' at the end, get rid of it
				if arg[len(arg)-1] == '*':
					arg = arg[0:len(arg)-1]
				debug.debug(1,"addacl- numeric network (chopped): %s" % arg)
				# it's a literal string, so we need to escape it.
				arg = re.escape(arg)
				self.aclDeny.append(arg)
			else:
				if arg == 'all':
					self.aclDeny.append(arg)
				elif arg == 'from':
					pass # allow apache-like syntax
				debug.debug(1,"addacl- literal: %s" % arg)
		if 'all' in self.aclDeny:
			debug.debug(4, "addacl: found all")
			self.aclDeny = ['.*'] # match any IP address


	# return 1 if requesterIP matches aclAllow_re
	# no aclAllow means nothing matches == nothing allowed
	def checkAllow(self,requesterIP):
		if self.aclAllow_re != None:
			if self.aclAllow_re.match(requesterIP):
				return 1
		return 0

	# return 1 if requesterIP matches aclDeny_re
	# no aclDeny means everything matches == everything denied
	def checkDeny(self,requesterIP):
		if self.aclDeny_re != None:
			if self.aclDeny_re.match(requesterIP):
				return 1
			else:
				return 0
		return 1


	# determine whether a particular IP address should be allowed to use the adzapper
	# returns 1 if an IP address should be blocked, 0 otherwise
	def accesscontroller(self,requesterIP):
		debug.debug(4, "requesterIP: [%s]" % requesterIP)
		if self.acOn == 0:
			return 0

		if self.aclOrder == ALLOWDENY:
			# check allow first, then deny
			if self.checkAllow(requesterIP):
				if self.checkDeny(requesterIP):
					return 1 # denied
				else:
					return 0 # not denied == allowed
			else:
				return 1 # not allowed == denied
			
		else: # self.aclOrder == DENYALLOW
			# check deny first, then do denies.
			if self.checkDeny(requesterIP):
				if self.checkAllow(requesterIP):
					return 0 # allowed
				else:
					return 1 # not allowed == denied
			else:
				return 0 # not denied == allowed



	# determine whether a URL should be zapped or not
	# returns 1 if it should be blocked, 0 otherwise
	def urlblocker(self, url):
		debug.debug(1,"zap: in zap.")
		url_tuple = urlparse.urlparse(url)
		host_port = url_tuple[1]
		if (string.find(host_port, ':') != -1):
			rem_host, rem_port = string.split(host_port, ':')
			rem_port = string.atoi(rem_port)
		else:
			rem_host, rem_port = host_port, 80

		# does the remote host match a zaplet in the database?
		if numeric_host.match(rem_host) == None:
			# has DNS name
			debug.debug(1, "named host.")
			if self.checkNamedHost(rem_host,url) == 1:
				# yes
				return 1
		else:
			# is a numeric host
			debug.debug(1, "numeric host.")
			if self.checkNumericHost(rem_host,url) == 1:
				return 1

		# default zaplet
		debug.debug(1,"checking default...")
		tmps = 'default'
		if (self.zaplet_list.has_key(tmps)):
			debug.debug(2,"zap: has default zaplet...")
			if (self.zaplet_list[tmps].allow_re.search(url)):
				# it's specifically allowed, so let it thru
				debug.debug(2,"zap: allow- matched pattern- site: " + tmps + "    url: " + url)
				return 0
			# ok, it's not specifically allowed, now check for blocks
			elif (self.zaplet_list[tmps].block_re.search(url)):
				debug.debug(2,"zap: block matched pattern- site: " + tmps + "    url: " + url)
				return 1
		debug.debug(2,"zap: no allows and no blocks. (%s)" % url)
      		return 0


