// //////////////////////////////////////////////////////////////////////////
// IceWM: src/NetStatus.cc
// by Mark Lawrence <nomad@null.net>
//
// !!! share code with cpu status
// //////////////////////////////////////////////////////////////////////////
#include "config.h"
#include "apppstatus.h"
#ifdef HAVE_NET_STATUS
#include "sysdep.h"
#include "prefs.h"

NetStatus::NetStatus(YWindow *aParent): YWindow(aParent) {
    // clear out the data
    for (int i = 0; i < NET_SAMPLES + 1; i++) {
        ppp_in[i] = ppp_out[i] = ppp_tot[i] = 0;
    }

    color[0] = new YColor(clrNetSend);
    color[1] = new YColor(clrNetReceive);
    color[2] = new YColor(clrNetIdle);

    setSize(NET_SAMPLES, 20);

    char status[64];

    sprintf(status, "%s - Status", netDevice);

    setToolTip(status);

    fUpdateTimer = new YTimer();
    if (fUpdateTimer) {
        fUpdateTimer->setInterval(NET_UPDATE_INTERVAL);
        fUpdateTimer->setTimerListener(this);
        fUpdateTimer->startTimer();
    }
    prev_ibytes = prev_obytes = 0;
    // set prev values for first updateStatus
    maxBytes = 0; // initially
    getCurrent(0, 0, 0);
    wasUp = false;
    updateStatus();
    maxBytes = 0; // initially
}

NetStatus::~NetStatus() {
    delete [] color;
    delete fUpdateTimer;
}

bool NetStatus::handleTimer(YTimer *t) {
    if (t != fUpdateTimer)
        return false;

    bool up = isUp();

    if (up) {
        if (!wasUp) {

            // clear out the data
            for (int i = 0; i < NET_SAMPLES + 1; i++) {
                ppp_in[i] = ppp_out[i] = ppp_tot[i] = 0;
            }
            this->show();

        }
        updateStatus();
    }
    else // link is down
        if (wasUp) this->hide();

    wasUp = up;
    return true;
}

void NetStatus::paint(Graphics &g, int /*x*/, int /*y*/,
                      unsigned int /*width*/, unsigned int /*height*/ )
{
    int h = height();

    //!!! this should really be unified with acpustatus.cc
    for (int i = 0; i < NET_SAMPLES; i++) {
        if (ppp_tot[i] > 0) {
            int in = (h * ppp_in[i] + maxBytes - 1) / maxBytes;
            int out = (h * ppp_out[i] + maxBytes - 1) / maxBytes;
            int t = h - in - 1;
            int l = out;

            if (t < h - 1) {
                g.setColor(color[0]);
                g.drawLine(i, t, i, h - 1);
                t--;
            }

            if (l > 0) {
                g.setColor(color[1]);
                //g.drawLine (i, h - tot -1, i, h - in);
                g.drawLine(i, 0, i, l);
                l++;
            }

            if (l < t) {
                g.setColor(color[2]);
                //g.drawLine (i, 0, i, h - tot - 2);
                g.drawLine(i, l, i, t - l);
            }
        }
        else {
            g.setColor(color[2]);
            g.drawLine(i, 0, i, h - 1);
        }
    }
}

bool NetStatus::isUp() {
#if 1
    return true;
#else
    struct ifreq ifr; // <net/if.h>

    if (s == -1)
        return false;
    
    strcpy(ifr.ifr_name, PPP_DEVICE);

    if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0) {
        //perror("ifr ioctl");
        return false;
    }

    if (ifr.ifr_flags & IFF_UP)
        return true;

    return false;
#endif
}

void NetStatus::updateStatus() {
    for (int i = 1; i < NET_SAMPLES; i++) {
        ppp_in[i-1] = ppp_in[i];
        ppp_out[i-1] = ppp_out[i];
        ppp_tot[i-1] = ppp_tot[i];
    }

    int last = NET_SAMPLES - 1;
    getCurrent(&ppp_in[last], &ppp_out[last], &ppp_tot[last]);
    repaint();
}

void NetStatus::getCurrent(int *in, int *out, int *tot) {
#if 0
    struct ifpppstatsreq req; // from <net/if_ppp.h> in the linux world

    memset(&req, 0, sizeof(req));

#ifdef linux
#undef ifr_name
#define ifr_name ifr__name

    req.stats_ptr = (caddr_t) &req.stats;

#endif // linux

    sprintf(req.ifr_name, PPP_DEVICE);

    if (ioctl(s, SIOCGPPPSTATS, &req) != 0) {
        if (errno == ENOTTY) {
            //perror ("ioctl");
        }
        else { // just not connected?
            //perror("??? ioctl?");
            return;
        }
    }

#endif

    unsigned long cur_ibytes = 0;
    unsigned long cur_obytes = 0;


    FILE *fp = fopen("/proc/net/dev", "r");
    if (!fp)
        return ;

    char buf[512];

    while (fgets(buf, sizeof(buf), fp) != NULL) {
        char *p = buf;
        while (*p == ' ')
            p++;
        if (strncmp(p, netDevice, strlen(netDevice)) == 0 &&
            p[strlen(netDevice)] == ':')
        {
            int ipackets, opackets;
            int ierrs, oerrs;
            int idrop, odrop;
            int ififo, ofifo;
            int iframe;
            int ocolls;
            int ocarrier;
            int icomp, ocomp;
            int imcast;

            p = strchr(p, ':') + 1;

            sscanf(p, "%lu %d %d %d %d %d %d %d" " %lu %d %d %d %d %d %d %d",
                   &cur_ibytes, &ipackets, &ierrs, &idrop, &ififo, &iframe, &icomp, &imcast,
                   &cur_obytes, &opackets, &oerrs, &odrop, &ofifo, &ocolls, &ocarrier, &ocomp);

            //printf("cur:%d %d %d\n", cur_ibytes, cur_obytes, maxBytes);

            break;
        }
    }
    fclose(fp);

    struct timeval curr_time;
    gettimeofday(&curr_time, NULL);

    double delta_t = (double) ((curr_time.tv_sec  - prev_time.tv_sec) * 1000000L
                             + (curr_time.tv_usec - prev_time.tv_usec)) / 1000000.0;

    int ni = (int)((cur_ibytes - prev_ibytes) / delta_t);
    int no = (int)((cur_obytes - prev_obytes) / delta_t);

    //printf("%d %d\n", ni, no);

    if (in) {
        *in  = ni;
        *out = no;
        *tot = *in + *out;
    }

    prev_time.tv_sec = curr_time.tv_sec;
    prev_time.tv_usec = curr_time.tv_usec;

    if (maxBytes == 0) // skip first read
        maxBytes = 1;
    else if (ni + no > maxBytes)
        maxBytes = ni + no;

    //printf("dif:%d %d %d\n", ni, no, maxBytes);

    prev_ibytes = cur_ibytes;
    prev_obytes = cur_obytes;
}

#endif
