#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "config.h"
#include "ylib.h"
#include <X11/Xatom.h>
#include "ylistbox.h"
#include "yscrollview.h"
#include "ymenu.h"
#include "yapp.h"
#include "sysdep.h"
#include "yaction.h"

#include <unistd.h>

#define LINE(c) ((c) == '\r' || (c) == '\n')
#define SPACE(c) ((c) == ' ' || (c) == '\t' || LINE(c))

class text_node {
public:
    text_node(char *t, int l, int _x, int _y) { text = t; len = l; next = 0; x = _x; y = _y; }

    int x, y;
    char *text;
    int len;
    text_node *next;
};

class node {
public:
    enum node_type {
        html, head, title,
        body,
        text, pre, line, paragraph, hrule, link,
        h1, h2, h3, h4, h5, h6,
        table, tr, td,
        div,
        ul, li,
        bold, font, italic,
    };

    node(node_type t) { next = 0; container = 0; type = t; wrap = 0; txt = 0; }

    node_type type;
    node *next;
    node *container;
    char *txt;
    text_node *wrap;
    int xr, yr;
    //int width, height;
};

node *root = NULL;

#define PRE 1
#define BOLD 2
#define LINK 4

node *add(node **first, node *last, node *n) {
    if (last)
        last->next = n;
    else
        *first = n;
    return n;
}

void skip_line(FILE *fp) {
    int c = getc(fp);
    //if (c == '\r')
    //    c = getc(fp);
    //if (c != '\n' && c != EOF)
    while (SPACE(c))
        c = getc(fp);
    if (c != EOF)
        ungetc(c, fp);
}
node *fix_list(node *list) {
    node *h, *l, *i;

    h = list;

    while (h && h->type != node::li) {
        l = h;
        h = h->next;
        delete l;
    }

    l = h;
    while (l && l->next) {
        i = l;
        while (i->next && i->next->type != node::li)
            i = i->next;

        l->container = l->next;
        l->next = i->next;
        i->next = 0;
        l = l->next;
    }
    return h;
}

node *parse(FILE *fp, int flags) {
    int c;
    node *f = NULL;
    node *l = NULL;

    c = getc(fp);
    while (c != EOF) {
        switch (c) {
        case '<':
            c = getc(fp);
            if (c == '!') {
                c = getc(fp);
                if (c == '-') {
                    c = getc(fp);
                    if (c == '-') {
                        // comment
                        c = getc(fp);
                        if (c != EOF) do {
                            if (c == '-') {
                                c = getc(fp);
                                if (c == '-') {
                                    do { c = getc(fp); } while (c == '-');
                                    if (c == '>' || c == EOF)
                                        break;
                                }
                            }
                            if (c == EOF)
                                break;
                            c = getc(fp);
                        } while (c != EOF);
                    }
                }
                while (c != EOF && c != '>') { c = getc(fp); }
            } else if (c == '/') {
                // ignore </BR> </P> </LI> ...
                int len = 0;
                char *buf = 0;

                c = getc(fp);
                while (SPACE(c)) c = getc(fp);
                do {
                    buf = (char *)realloc(buf, ++len);
                    buf[len-1] = TOUPPER(c);
                    c = getc(fp);
                } while (c != EOF && !SPACE(c) && c != '>');

                buf = (char *)realloc(buf, ++len);
                buf[len-1] = 0;

                if (strcmp(buf, "P") != 0 &&
                    strcmp(buf, "LI") != 0 &&
                    strcmp(buf, "BR") != 0)
                    return f;
            } else {
                int len = 0;
                char *buf = 0;

                while (SPACE(c)) c = getc(fp);
                do {
                    buf = (char *)realloc(buf, ++len);
                    buf[len-1] = TOUPPER(c);
                    c = getc(fp);
                } while (c != EOF && !SPACE(c) && c != '>');

                if (c != '>' && c != EOF)
                    do { c = getc(fp); } while (c != EOF && c != '>');
                if (c == '>')
                    skip_line(fp);

                buf = (char *)realloc(buf, ++len);
                buf[len-1] = 0;

                node::node_type type;
                node *n = 0;
                node *container = 0;

                if (strcmp(buf, "HR") == 0)
                    type = node::hrule;
                else if (strcmp(buf, "P") == 0)
                    type = node::paragraph;
                else if (strcmp(buf, "BR") == 0)
                    type = node::line;
                else if (strcmp(buf, "LI") == 0)
                    type = node::li;
                else {
                    if (strcmp(buf, "HTML") == 0)
                        type = node::html;
                    else if (strcmp(buf, "HEAD") == 0)
                        type = node::head;
                    else if (strcmp(buf, "TITLE") == 0)
                        type = node::title;
                    else if (strcmp(buf, "BODY") == 0)
                        type = node::body;
                    else if (strcmp(buf, "H1") == 0)
                        type = node::h1;
                    else if (strcmp(buf, "H2") == 0)
                        type = node::h2;
                    else if (strcmp(buf, "H3") == 0)
                        type = node::h3;
                    else if (strcmp(buf, "H4") == 0)
                        type = node::h4;
                    else if (strcmp(buf, "H5") == 0)
                        type = node::h5;
                    else if (strcmp(buf, "H6") == 0)
                        type = node::h6;
                    else if (strcmp(buf, "PRE") == 0)
                        type = node::pre;
                    else if (strcmp(buf, "FONT") == 0)
                        type = node::font;
                    else if (strcmp(buf, "TABLE") == 0)
                        type = node::table;
                    else if (strcmp(buf, "TR") == 0)
                        type = node::tr;
                    else if (strcmp(buf, "TD") == 0)
                        type = node::td;
                    else if (strcmp(buf, "DIV") == 0)
                        type = node::div;
                    else if (strcmp(buf, "UL") == 0)
                        type = node::ul;
                    else if (strcmp(buf, "A") == 0)
                        type = node::link;
                    else if (strcmp(buf, "B") == 0)
                        type = node::bold;
                    else if (strcmp(buf, "I") == 0)
                        type = node::italic;
                    else
                        break;

                    container = parse(fp, flags | ((type == node::pre) ? PRE : 0));

                    if (type == node::ul)
                        container = fix_list(container);

                }
                n = new node(type);
                if (container)
                    n->container = container;

                if (n)
                    l = add(&f, l, n);
                else
                    break;
            }
            break;
        default:
            {
                int len = 0;
                char *buf = 0;

                do {
                    if (c == '&')
                        c = '@';
                    if (c == '\r') {
                        c = getc(fp);
                        if (c != '\n')
                            ungetc(c, fp);
                        c = '\n';
                    }
                    if (!(flags & PRE) && SPACE(c))
                        c = ' ';
                    if ((flags & PRE) || c != ' ' || len == 0 || buf[len - 1] != ' ') {
                        buf = (char *)realloc(buf, ++len);
                        buf[len-1] = c;
                    }
                    c = getc(fp);
                } while (c != EOF && c != '<');

                if (c == '<') {
                    c = getc(fp);
                    if (c == '/') {

                        if (len && SPACE(buf[len - 1]))
                            len--;
                    }
                    ungetc(c, fp);
                    c = '<';
                }

                if (len) {
                    buf = (char *)realloc(buf, ++len);
                    buf[len-1] = 0;

                    node *n = new node(node::text);
                    n->txt = buf;
                    l = add(&f, l, n);
                }
                continue;
            }
        }
        c = getc(fp);
    }
    return f;
}

YIcon *file = 0;

extern Atom _XA_WIN_ICONS;

class HTextView: public YWindow,
    public YScrollBarListener, public YScrollable, public YActionListener
{
public:
    HTextView(YScrollView *v, YWindow *parent): YWindow(parent) {
        view = v;
        fVerticalScroll = view->getVerticalScrollBar();;
        fVerticalScroll->setScrollBarListener(this);
        fHorizontalScroll = view->getHorizontalScrollBar();
        fHorizontalScroll->setScrollBarListener(this);
        //setBitGravity(NorthWestGravity);
        tx = ty = 0;
        conWidth = conHeight = 0;


        font = YFont::getFont("fixed");
        fg = new YColor("rgb:00/00/00");
        bg = new YColor("rgb:CC/CC/CC");

        actionClose = new YAction();

        menu = new YMenu();
        menu->setActionListener(this);
        menu->addItem("Close", 0, "Ctrl+Q", actionClose);
    }

    ~HTextView() {
    }

    void setData(node *root) {
        fRoot = root;
        layout();
    }

    void layout() {
        int x = 0, y = 0;
        conWidth = conHeight = 0;
        layout(fRoot, 0, width(), x, y, conWidth, conHeight, 0);
        resetScroll();
    }

    void layout(node *n1, int left, int right, int &x, int &y, int &w, int &h, int pre) {
        node *n = n1;

        ///puts("{");
        while (n) {
            n->xr = x;
            n->yr = y;
            switch (n->type) {
            case node::title:
                break;
            case node::text:
                {
                    char *c;

                    while (n->wrap) {
                        text_node *t = n->wrap;
                        n->wrap = n->wrap->next;
                        delete t;
                    }

                    text_node **pwrap = &n->wrap;
                    for (char *b = n->txt; *b; b = c) {
                        int wc = 0;

                        if (pre) {
                            c = b;
                            while (*c && *c != '\n')
                                c++;
                        } else {
                            if (left == x)
                                while (SPACE(*b))
                                    b++;

                            c = b;

                            do {
                                char *d = c;

                                if (SPACE(*d))
                                    while (SPACE(*d))
                                        d++;
                                else
                                    while (*d && !SPACE(*d))
                                        d++;

                                int w1 = font->textWidth(b, d - b);

                                if (x + w1 < right) {
                                    wc = w1;
                                    c = d;
                                } else if (x == left) {
                                    if (wc == 0) {
                                        wc = w1;
                                        c = d;
                                    }
                                    break;
                                } else {
                                    if (wc == 0) {
                                        x = left;
                                        y = h;
                                        h += font->height();
                                        wc = w1;
                                        c = d;
                                    } else
                                        break;

                                }
                                ///printf("width=%d %d / %d / %d\n", wc, x, left, right);
                            } while (*c);
                        }

                        if (pre || c - b > 0) {
                            *pwrap = new text_node(b, c - b, x, y);
                            pwrap = &((*pwrap)->next);
                            if (y + font->height() > h)
                                h = y + font->height();

                            if (pre) {
                                wc = font->textWidth(b, c - b);
                                if (left + wc > w)
                                    w = left + wc;
                                x += wc;
                                if (*c == '\n') {
                                    c++;
                                    x = left;
                                    y = h;
                                }
                            } else {
                                x += wc;
                                if (x > w)
                                    w = x;
                            }
                        }
                        //printf("len=%d x=%d left=%d %s\n", c - b, x, left, b);
                    }
                }
                break;
            case node::line:
                //puts("<BR>");
                x = 0;
                y = h;
                break;
            case node::paragraph:
                //puts("<P>");
                h += font->height();
                x = n->xr = left;
                y = n->yr = h;
                break;
            case node::hrule:
                //puts("<HR>");
                x = n->xr = left;
                n->yr = h;
                h += 10;
                y = h;
                break;
            case node::ul:
                //puts("<UL>");
                x = n->xr = left;
                y = n->yr = h;
                if (n->container) {
                    layout(n->container, n->xr, right, x, y, w, h, pre);//n->width, n->height);
                }
                break;
            case node::li:
                //puts("<LI>");
                x = n->xr = left + 20;
                y = n->yr = h;
                if (n->container) {
                    layout(n->container, n->xr, right, x, y, w, h, pre);
                }
                break;
            case node::pre:
                x = left;
                y = h;
                if (n->container) {
                    layout(n->container, n->xr, right, x, y, w, h, 1);
                }
                break;
            default:
                if (n->container)
                    layout(n->container, left, right, x, y, w, h, pre);
                break;
            }
            n = n->next;
        }
        ///puts("}");
    }

    void draw(Graphics &g, node *n1) {
        node *n = n1;

        while (n) {
            switch (n->type) {
            case node::title:
                break;
            case node::text:
                if (n->wrap) {
                    text_node *t = n->wrap;

                    while (t) {
                        g.drawChars(t->text, 0, t->len, t->x - tx, t->y + font->ascent() - ty);
                        t = t->next;
                    }
                }
                break;

            case node::hrule:
                g.drawLine(2 - tx, n->yr + 4 - ty, width() - 2 - tx, n->yr + 4 - ty);
                g.drawLine(2 - tx, n->yr + 5 - ty, width() - 2 - tx, n->yr + 5 - ty);
                break;

            case node::li:

            default:
                if (n->container)
                    draw(g, n->container);
                break;
            }
            n = n->next;
        }
    }

    virtual void paint(Graphics &g, int wx, int wy, unsigned int wwidth, unsigned int wheight) {
        g.setColor(bg);
        g.fillRect(wx, wy, wwidth, wheight);
        g.setColor(fg);
        g.setFont(font);

        draw(g, fRoot);
    }

    void resetScroll() {
        fVerticalScroll->setValues(ty, height(), 0, contentHeight());
        fVerticalScroll->setBlockIncrement(height());
        fVerticalScroll->setUnitIncrement(font->height());
        fHorizontalScroll->setValues(tx, width(), 0, contentWidth());
        fHorizontalScroll->setBlockIncrement(width());
        fHorizontalScroll->setUnitIncrement(20);
        if (view)
            view->layout();
    }

    void setPos(int x, int y) {
        if (x != tx || y != ty) {
            int dx = x - tx;
            int dy = y - ty;

            tx = x;
            ty = y;

            scrollWindow(dx, dy);
        }
    }

    virtual void scroll(YScrollBar *sb, int delta) {
        if (sb == fHorizontalScroll)
            setPos(tx + delta, ty);
        else if (sb == fVerticalScroll)
            setPos(tx, ty + delta);
    }
    virtual void move(YScrollBar *sb, int pos) {
        if (sb == fHorizontalScroll)
            setPos(pos, ty);
        else if (sb == fVerticalScroll)
            setPos(tx, pos);
    }

    int contentWidth() {
        return conWidth;
    }
    int contentHeight() {
        return conHeight;
    }
    YWindow *getWindow() { return this; }

    virtual void handleClick(const XButtonEvent &up, int count) {
        if (up.button == 3) {
            menu->popup(0, 0, up.x_root, up.y_root, -1, -1,
                        YPopupWindow::pfCanFlipVertical |
                        YPopupWindow::pfCanFlipHorizontal |
                        YPopupWindow::pfPopupMenu);
            return ;
        }
    }

    virtual void actionPerformed(YAction *action, unsigned int modifiers) {
        if (action == actionClose)
            exit(0);
    }

    virtual void configure(int x, int y, unsigned int width, unsigned int height) {
        YWindow::configure(x, y, width, height);
        layout();
   }
private:
    node *fRoot;

    int tx, ty;
    int conWidth;
    int conHeight;

    YFont *font;
    YColor *bg, *fg;

    YScrollView *view;
    YScrollBar *fVerticalScroll;
    YScrollBar *fHorizontalScroll;

    YMenu *menu;
    YAction *actionClose;
};

class FileView: public YWindow {
public:
    FileView(char *path) {
        setDND(true);
        fPath = newstr(path);

        scroll = new YScrollView(this);
        view = new HTextView(scroll, this);
        scroll->setView(view);

        view->show();
        scroll->show();

        XStoreName(app->display(), handle(), fPath);
        file = getIcon("file");
        Pixmap icons[4];
        icons[0] = file->small()->pixmap();
        icons[1] = file->small()->mask();
        icons[2] = file->large()->pixmap();
        icons[3] = file->large()->mask();
        XChangeProperty(app->display(), handle(),
                        _XA_WIN_ICONS, XA_PIXMAP,
                        32, PropModeReplace,
                        (unsigned char *)icons, 4);

        setSize(640, 640);

        loadFile();
    }
    void loadFile() {
        FILE *fp = fopen(fPath, "r");
        if (fp == 0)
            fp = stdin;
        if (fp) {
            root = parse(fp, 0);

            view->setData(root);
        }
        fclose(fp);
    }

    virtual void configure(int x, int y, unsigned int width, unsigned int height) {
        YWindow::configure(x, y, width, height);
        scroll->setGeometry(0, 0, width, height);
    }

    virtual void handleClose() {
        delete this;
        app->exit(0);
    }

private:
    char *fPath;

    HTextView *view;
    YScrollView *scroll;
};

int main(int argc, char **argv) {
    YApplication app(&argc, &argv);

    if (argc > 1) {
        FileView *view = new FileView(argv[1]);
        view->show();

        return app.mainLoop();
    }
}

#if 0

#define S_LINK "\x1B[04m"
#define E_LINK "\x1B[0m"
#define S_HEAD "\x1B[01;31m"
#define E_HEAD "\x1B[0m"
#define S_BOLD "\x1B[01m"
#define E_BOLD "\x1B[0m"
#define S_ITALIC "\x1B[33m"
#define E_ITALIC "\x1B[0m"
#define S_DEBUG "\x1B[07;31m"
#define E_DEBUG "\x1B[0m"

int pos = 0;
int width = 80;

void clear_line() {
    if (pos != 0)
        printf("\n");
    pos = 0;
}

void new_line(int count = 1) {
    while (count-- > 0)
        putchar('\n');
    pos = 0;
}

void blank_line(int count = 1) {
    clear_line();
    new_line(count);
}

void out(char *text) {
    printf("%s", text);
    pos += strlen(text);
}

void tab(int len) {
    pos += len;
    while (len-- > 0) putchar(' ');
}

void pre(int left, char *text) {
    char *p = text;

    clear_line();
    while (*p) {
        tab(left);
        while (*p && *p != '\n') {
            putchar(*p);
            pos++;
            p++;
        }
        if (*p == '\n')
            p++;
        new_line();
    }
}

void wrap(int left, char *text) {
    //putchar('[');
    while (*text) {
        int chr = 0;
        int sp = 0;
        int col = pos;
        char *p = text;

        //printf("[%d:%s]", pos, text);
        while (*p && col <= width) {
            if (*p == ' ')
                sp = chr;
            chr++;
            col++;
            p++;
        }
        if (col <= width)
            sp = chr;
        if (sp == 0)
            sp = chr;
        //printf("(%d)", col);

        while (sp > 0) {
            putchar(*text);
            pos++;
            text++;
            sp--;
        }
        if (*text == ' ')
            text++;
        if (*text) {
            //if (pos != width)
                printf("\n");
            pos = 0;
            tab(left);
        }
    }
    //putchar(']');
}

node *blank(node *n) {
    if (n->next && n->next->type == node::text) {
        if (n->next->content.text[0] == ' ' &&
            n->next->content.text[1] == 0) {
            if (n->next->next &&
                (n->next->next->type == node::h1 ||
                 n->next->next->type == node::h2))
                return n->next;
            blank_line();
            return n->next;
        }
    }
    blank_line();
    return n;
}

void dump(int flags, int left, node *n, node *up) {
    while (n) {
        switch (n->type) {
        case node::html:
            dump(flags, left, n->content.container, n);
            break;
        case node::head:
            break;
        case node::body:
        case node::table:
        case node::font:
        case node::tr:
        case node::td:
        case node::div:
            dump(flags, left, n->content.container, n);
            break;
        case node::ul:
            dump(flags, left, n->content.container, n);
            if (up && up->type != node::li) {
                n = blank(n);
            }
            break;
        case node::li:
            clear_line();
            tab(left);
            out("  * ");
            dump(flags, left + 4, n->content.container, n);
            break;
        case node::bold:
            printf(S_BOLD);
            dump(flags | BOLD, left, n->content.container, n);
            printf(E_BOLD);
            break;
        case node::italic:
            printf(S_ITALIC);
            dump(flags | BOLD, left, n->content.container, n);
            printf(E_ITALIC);
            break;
        case node::h1:
        case node::h2:
        case node::h3:
        case node::h4:
        case node::h5:
        case node::h6:
            blank_line();
            printf(S_HEAD);
            dump(flags, left, n->content.container, n);
            printf(E_HEAD);
            n = blank(n);
            break;
        case node::pre:
            dump(flags | PRE, left + 4, n->content.container, n);
            break;
        case node::link:
            printf(S_LINK);
            dump(flags | LINK, left, n->content.container, n);
            printf(E_LINK);
            break;
        case node::hrule:
            clear_line();
            for (int i = 0; i < 72; i++)
                pos += printf("-");
            clear_line();
            break;
        case node::paragraph:
            clear_line();
            new_line(1);;
            break;
        case node::line:
            new_line(1);
            break;
        case node::text:
            if (flags & PRE)
                pre(left, n->content.text);
            else {
                wrap(left, n->content.text);
            }
            break;
        default:
            printf(S_DEBUG "?" E_DEBUG);
        }
        n = n->next;
    }
}

int main(int argc, char **argv) {
    FILE *fp = fopen(argv[1], "r");
    if (fp == 0)
        fp = stdin;
    char *e = getenv("COLUMNS");
    if (e) width = atoi(e);
    if (fp) {
        root = parse(fp, 0);
        dump(0, 0, root, 0);
        clear_line();
    }
}
#endif
