#include "config.h"
#include "ylib.h"
#include <X11/Xatom.h>
#include "ylistbox.h"
#include "yscrollview.h"
#include "ymenu.h"
#include "yapp.h"
#include "yaction.h"
#include "wmmgr.h"
#include "sysdep.h"
#include <dirent.h>

class ObjectList;
class ObjectListBox;

YIcon *folder = 0;
YIcon *file = 0;

class ObjectListItem: public YListItem {
public:
    ObjectListItem(char *container, char *name) {
        fContainer = container;
        fName = newstr(name);
        fFolder = false;

        struct stat sb;
        char *path = getLocation();
        if (lstat(path, &sb) == 0 && S_ISDIR(sb.st_mode))
            fFolder = true;
        delete path;
    }
    virtual ~ObjectListItem() { delete fName; fName = 0; }

    virtual const char *getText() { return fName; }
    bool isFolder() { return fFolder; }
    virtual YIcon *getIcon() { return isFolder() ? folder : file; }


    char *getLocation();
private:
    char *fContainer;
    char *fName;
    bool fFolder;
};

char *ObjectListItem::getLocation() {
    char *dir = fContainer;
    char *name = (char *)getText();
    int dlen;
    int nlen = (dlen = strlen(dir)) + 1 + strlen(name) + 1;
    char *npath;

    npath = new char[nlen];
    strcpy(npath, dir);
    if (dlen == 0 || dir[dlen - 1] != '/') {
        strcpy(npath + dlen, "/");
        dlen++;
    }
    strcpy(npath + dlen, name);
    return npath;
}

class ObjectListBox: public YListBox, public YActionListener {
public:
    ObjectListBox(ObjectList *list, YScrollView *view, YWindow *aParent): YListBox(view, aParent) {
        fObjList = list;

        actionOpenList = new YAction();
        actionOpenIcon = new YAction();
        actionOpen = new YAction();
        actionClose = new YAction();
        
        YMenu *openMenu = new YMenu();
        openMenu->addItem("List View", 0, 0, actionOpenList);
        openMenu->addItem("Icon View", 0, 0, actionOpenIcon);
        
        folderMenu = new YMenu();
        folderMenu->setActionListener(this);
        folderMenu->addItem("Open", 0, actionOpenList, openMenu);
    }

    virtual ~ObjectListBox() { }

    virtual bool handleKey(const XKeyEvent &key) {
        return YListBox::handleKey(key);
    }

    virtual void handleClick(const XButtonEvent &up, int count) {
        if (up.button == 3 && count == 1) {
            YMenu *menu = folderMenu;
            menu->popup(0, 0, up.x_root, up.y_root, -1, -1,
                        YPopupWindow::pfCanFlipVertical |
                        YPopupWindow::pfCanFlipHorizontal |
                        YPopupWindow::pfPopupMenu);
            return ;
        } else
            return YListBox::handleClick(up, count);
    }

    virtual void activateItem(YListItem *item);

    virtual void actionPerformed(YAction *action, unsigned int modifiers) {
        if (action == actionOpenList) {
        }
    }
private:
    ObjectList *fObjList;
    YMenu *folderMenu;
    YAction *actionClose;
    YAction *actionOpen;
    YAction *actionOpenList;
    YAction *actionOpenIcon;
};

class ObjectList: public YWindow {
public:
    static int winCount;

    ObjectList(char *path) {
        setDND(true);
        fPath = newstr(path);
        scroll = new YScrollView(this);
        list = new ObjectListBox(this,
                                 scroll,
                                 scroll);
        scroll->setView(list);
        updateList();
        list->show();
        scroll->show();
        
        XStoreName(app->display(), handle(), fPath);

        int w = desktop->width();
        int h = desktop->height();

        setGeometry(w / 3, h / 3, w / 3, h / 3);

        Pixmap icons[4];
        icons[0] = folder->small()->pixmap();
        icons[1] = folder->small()->mask();
        icons[2] = folder->large()->pixmap();
        icons[3] = folder->large()->mask();
        XChangeProperty(app->display(), handle(),
                        _XA_WIN_ICONS, XA_PIXMAP,
                        32, PropModeReplace,
                        (unsigned char *)icons, 4);
        winCount++;
    }

    ~ObjectList() { winCount--; }

    virtual void handleClose() {
        if (winCount == 1)
            app->exit(0);
        delete this;
    }

    void updateList();

    virtual void configure(int x, int y, unsigned int width, unsigned int height) {
        YWindow::configure(x, y, width, height);
        scroll->setGeometry(0, 0, width, height);
    }

    char *getPath() { return fPath; }

private:
    ObjectListBox *list;
    YScrollView *scroll;

    char *fPath;
};
int ObjectList::winCount = 0;

void ObjectList::updateList() {
    DIR *dir;

    if ((dir = opendir(fPath)) != NULL) {
        struct dirent *de;

        while ((de = readdir(dir)) != NULL) {
            char *n = de->d_name;

            if (n[0] == '.' && (n[1] == 0 || (n[1] == '.' && n[2] == 0)))
                ;
            else {
                ObjectListItem *o = new ObjectListItem(fPath, n);

                if (o)
                    list->addItem(o);
            }
        }
        closedir(dir);
    }
}

void ObjectListBox::activateItem(YListItem *item) {
    ObjectListItem *obj = (ObjectListItem *)item;
    char *path = obj->getLocation();

    if (obj->isFolder()) {
        //if (fork() == 0)
        //    execl("./icelist", "icelist", path, 0);
        ObjectList *list = new ObjectList(path);
        list->show();
    } else {
        if (fork() == 0)
            execl("./iceview", "iceview", path, 0);
    }
    delete path;
}

int main(int argc, char **argv) {
    YApplication app(&argc, &argv);

    folder = getIcon("folder");
    file = getIcon("file");

    ObjectList *list = new ObjectList(argv[1] ? argv[1] : (char *)"/");
    list->show();

    return app.mainLoop();
}
