/*
 * IceWM
 *
 * Copyright (C) 1997,1998 Marko Macek
 */
#include "config.h"

#ifndef NO_CONFIGURE_MENUS
#include "themes.h"

#include "ymenu.h"
#include "wmmgr.h"
#include "wmprog.h"
#include "ymenuitem.h"
#include "sysdep.h"
#include "base.h"
#include "prefs.h"
#include <dirent.h>

ThemesMenu::ThemesMenu(YWindow *parent): ObjectMenu(parent) {
    const char *homeDir = getenv("HOME");
    char *path;

    path = strJoin(libDir, "/themes/", NULL);
    findThemes(path, this);
    delete path;

    path = strJoin(configDir, "/themes/", NULL);
    findThemes(path, this);
    delete path;

    path = strJoin(homeDir, "/.icewm/themes/", NULL);
    findThemes(path, this);
    delete path;
}

ThemesMenu::~ThemesMenu() {
}

void ThemesMenu::findThemes(const char *path, YMenu *container) {
    DIR *dir;
    int nlen, plen = strlen(path);
    const char *tname = "/default.theme";
    char *npath;

    if ((dir = opendir(path)) != NULL) {
        struct dirent *de;

        while ((de = readdir(dir)) != NULL) {
            { // eliminate duplicates
                int i, found = 0;
                for (i = 0; i < container->itemCount(); i++) {
                    const char *n = container->item(i)->name();

                    if (n && strcmp(de->d_name, n) == 0) {
                        found = 1;
                        break;
                    }
                }
                if (found)
                    continue;
            }
            nlen = plen + 1 + strlen(de->d_name) + 1 + strlen(tname);
            npath = new char[nlen];

            if (npath) {
                strcpy(npath, path);
                if (plen == 0 || npath[plen - 1] != '/') {
                    strcpy(npath + plen, "/");
                    strcpy(npath + plen + 1, de->d_name);
                    plen++;
                } else {
                    strcpy(npath + plen, de->d_name);
                }
                strcat(npath, tname);

                if (access(npath, R_OK) == 0) {
                    char **args = (char **)MALLOC(4 * sizeof(char *));
                    if (args) {
                        args[0] = newstr("icewm"EXEEXT);
                        args[1] = newstr("-t");
                        args[2] = newstr(npath + plen);
                        args[3] = 0;
                        if (args[0] && args[1] && args[2]) {
                            DProgram *re_theme = DProgram::newProgram(de->d_name, 0, true, "icewm"EXEEXT, args);
                            if (re_theme) {
                                YMenuItem *item = container->add(new DObjectMenuItem(re_theme));
                                if (themeName && strcmp(themeName, npath + plen) == 0)
                                    item->setChecked(true);
                            }
                        }
                    }
                }
                delete npath;
            }
        }
        closedir(dir);
    }
}
#endif
