#!/usr/bin/perl
# logcolorize.pl - A PERL script to colorize log viewing
# Version 1.0.3
# License: GPL version 2 or later
# Copyright: (C) 1999, Michael T. Babcock
#                      <mikebabcock@pobox.com>
#                      http://www.linuxsupportline.com/~pgp/linux/
# Special Thanks 
# colorlogs.pl -- I ripped the colors and this header for this script.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
######################################################################
# Create the colors Assoc. Array
%colors = (
    'black'     => "\033[30m",
    'red'       => "\033[31m",
    'green'     => "\033[32m",
    'yellow'    => "\033[33m",
    'blue'      => "\033[34m",
    'magenta'   => "\033[35m",
    'purple'    => "\033[35m",
    'cyan'      => "\033[36m",
    'white'     => "\033[37m",
    'darkgray'  => "\033[30m");

# Define good notables:
@word_good=("starting\n", "starting.", "accepted.\n", "authenticated.\n", "Ready", "active", "reloading", "saved;", "restarting");
@word_warn=("dangling", "closed.\n", "Assuming", "root", "root\n", "exiting\n", "missing", "Ignored", "adminalert:", "deleting");
@word_note=("LOGIN", "DHCP_OFFER", "disconnected", "connect", "Successful");
@line_good=("up", "DHCP_ACK", "Cleaned", "alive", "found");
@line_warn=("invalid", "bad", "attackalert:", "wrong", "Lame", "FAILED", "failing", "unknown", "obsolete");
@line_bad=("DENY", "lost", "shutting", "dead", "DHCP_NAK", "failure;", "Unable", "inactive", "terminating", "refused", "rejected", "down");

# Now define what we want to use when:
$col_good = "green";
$col_warn = "yellow";
$col_bad = "red";
$col_note = "purple";

$col_norm =		"\033[00m";
$col_background =	"\033[07m";
$col_brighten = 	"\033[01m";
$col_underline = 	"\033[04m";
$col_blink = 		"\033[05m";

# Resets to normal colours and moves to the left one column ... 
$col_default = "$col_norm$colors{white}";

print "\n$col_norm$colors{cyan}logcolorize.pl 1.0.3 by $col_brighten$colors{cyan}Michael T. Babcock$col_norm$colors{cyan} <$col_brighten$colors{green}mikebabcock\@pobox.com$col_norm$colors{cyan}>\n";

$datespace=0;

mainloop: while (<>)
{
	$thisline = $_;
	@rec = split (/ /, $thisline);

	$output ="$col_brighten$colors{'cyan'}";
	$output.="$rec[0] ";
	if ($rec[1] eq "")
	{
		$datespace=1;
	}
	$output.="$rec[1+$datespace] $rec[2+$datespace] ";
	$output.="$col_brighten$colors{'blue'}$rec[3+$datespace]";
	if ($rec[(4+$datespace)] eq "last")
	{
		$output.="$col_norm$colors{'green'} last message repeated ";
		$output.="$col_brighten$rec[(7+$datespace)]$col_norm$colors{'green'} times\n";
		print "$output$col_default";
		next mainloop;
	}
	@service=split(/\[/,$rec[(4+$datespace)]);
	if ($#service eq 1)
	{
		@service2=split(/]/,$service[1]);
		$output.="$col_norm$colors{'green'} $service[0]$col_brighten$colors{'green'}\[";
		$output.="$col_brighten$colors{'white'}$service2[0]$col_brighten$colors{'green'}\]: ";
	} else {
		$output.="$col_norm$colors{'green'} $service[0] ";
	}
	$restcolor="$col_norm$colors{'cyan'}";
	$restoftheline="";
	for ($therest=(5+$datespace); $therest<=$#rec; $therest++)
	{
		$highlight=0;

		# Here we check for keywords to highlight ...
		for ($i=1; $i<$#word_good; $i++)
		{
			if ($word_good[$i] eq $rec[$therest])
			{
				$restoftheline.="$col_brighten$colors{$col_good}"; $highlight=1;
			}
		}
		for ($i=1; $i<$#word_warn; $i++)
		{
			if ($word_warn[$i] eq $rec[$therest])
			{
				$restoftheline.="$col_brighten$colors{$col_warn}"; $highlight=1;
			}
		}
		for ($i=1; $i<$#word_note; $i++)
		{
			if ($word_note[$i] eq $rec[$therest])
			{
				$restoftheline.="$col_brighten$colors{$col_note}"; $highlight=1;
			}
		}

		# Here we watch for words that indicate entire lines should be highlighted
		for ($i=1; $i<$#line_good; $i++)
		{
			if ($line_good[$i] eq $rec[$therest])
			{
				$restcolor="$col_norm$colors{$col_good}";
				$restoftheline.="$col_brighten$colors{$col_good}"; $highlight=1;
			}
		}
		for ($i=1; $i<$#line_warn; $i++)
		{
			if ($line_warn[$i] eq $rec[$therest])
			{
				$restcolor="$col_norm$colors{$col_warn}";
				$restoftheline.="$col_brighten$colors{$col_warn}"; $highlight=1;
			}
		}
		for ($i=1; $i<$#line_warn; $i++)
		{
			if ($line_warn[$i] eq $rec[$therest])
			{
				$restcolor="$col_norm$colors{$col_bad}";
				$restoftheline.="$col_brighten$colors{$col_bad}"; $highlight=1;
			}
		}

		$restoftheline.="$rec[$therest] ";
		if ($highlight == 1)
		{
			$restoftheline.=$restcolor;
		}
	}
	$output.="$restcolor$restoftheline";
	print "$output$col_default\33[1G";
}
