// client.cc
//
// Frank Hale 
// frankhale@yahoo.com
#include "sapphire.hh"

Client::Client() 
{
	shaded=False;
	
	name="";
	x=0;
	y=0;
	width=0;
	height=0;
	
	button1=False;
	button2=False;
	button3=False;
		
	close.window = maximize.window 
		     = minimize.window 
		     = shadeButton.window
		     = window 
		     = parent 
		     = trans 
		     = title 
		     = None;
}

Client::~Client() 
{
	XFreePixmap(wm->dpy, frame_pix);
	XFreePixmap(wm->dpy, button_pix);
}

Window Client::createTopLevelWindow(int x, int y, 
	unsigned int width,unsigned int height,unsigned int borderwidth)
{
	XSetWindowAttributes attrib;

	unsigned long create_mask = CWOverrideRedirect | 
		CWBackPixmap | CWBackPixel | CWBorderPixel | CWCursor | 
		CWEventMask;

	attrib.background_pixmap = None;
	attrib.background_pixel = wm->bg.pixel;
	attrib.border_pixel = wm->bd.pixel;
	attrib.override_redirect = True;
	attrib.cursor = XCreateFontCursor(wm->dpy, XC_left_ptr);
	attrib.event_mask =  StructureNotifyMask | SubstructureNotifyMask |
		SubstructureRedirectMask | ExposureMask | ButtonMotionMask | 
		ButtonPressMask | ButtonReleaseMask;
	
	return (XCreateWindow(wm->dpy, wm->root, x,y,width,height,borderwidth,
		DefaultDepth(wm->dpy, wm->screen), InputOutput, DefaultVisual(wm->dpy, wm->screen),
		create_mask, &attrib));
}

Window Client::createChildWindow(Window parent, int x, int y, 
	unsigned int width,unsigned int height,unsigned int borderwidth)
{
	XSetWindowAttributes attrib;

	unsigned long create_mask = 
		CWBackPixmap | CWBackPixel | CWBorderPixel | CWCursor | 
		CWEventMask;

	attrib.background_pixmap = None;
	attrib.background_pixel = wm->bg2.pixel;
	attrib.border_pixel = wm->bd.pixel;
	attrib.cursor = XCreateFontCursor(wm->dpy, XC_left_ptr);
	attrib.event_mask = StructureNotifyMask | SubstructureNotifyMask |
		SubstructureRedirectMask | ExposureMask | EnterWindowMask |
		LeaveWindowMask | ButtonMotionMask | ButtonPressMask | 
		ButtonReleaseMask;
	
	return (XCreateWindow(wm->dpy, parent, x,y,width,height,borderwidth,
		DefaultDepth(wm->dpy, wm->screen), InputOutput, DefaultVisual(wm->dpy, wm->screen),
		create_mask, &attrib));
}


void Client::make_new_client(Window w)
{
	XGrabServer(wm->dpy);

	Client *c = new Client();

	if (inititialize(c, w) == IsViewable) c->ignore_unmap++;
	else init_position(c);

	gravitate(c);
	reparent(c);

	if (wm_state(c) == IconicState) {
		XUnmapSubwindows(wm->dpy, c->title);
		XUnmapWindow(wm->dpy, c->title);
		XUnmapWindow(wm->dpy, c->parent);
		XUnmapWindow(wm->dpy, c->window);
	} else {
		XMapSubwindows(wm->dpy, c->parent);
		XMapSubwindows(wm->dpy, c->title);
		XMapWindow(wm->dpy, c->window);
		XMapRaised(wm->dpy, c->parent);
		set_wm_state(c, NormalState);		
	}

	wm->ClientList->insert(c);	

#ifdef DEBUG
	err("%s (%d, %d): %#x (%#x) @ %d,%d", c->name, wm_state(c),
		c->ignore_unmap, c->window, c->parent, c->x, c->y);
#endif

	XSync(wm->dpy, False);
	XUngrabServer(wm->dpy);
}

Client *Client::find_client(Window w)
{
	if (w && w != wm->root) {
		if(wm->ClientList->count()) {
			LinkedListIterator<Client> it(wm->ClientList);
			for(; it.current(); it++)
			{
				if (w == it.current()->parent ||
				    w == it.current()->title ||
				    w == it.current()->window ||
				    w == it.current()->close.window ||
				    w == it.current()->maximize.window ||
				    w == it.current()->minimize.window ||
				    w == it.current()->shadeButton.window)
				{
					return it.current();
				}				
			}
		}
	}

	return NULL;
}

int Client::inititialize(Client *c, Window w)
{
	XWindowAttributes attr;
	long dummy;
	XWMHints *hints = XGetWMHints(wm->dpy, w);

	c->window = w;
	c->ignore_unmap = 0;
	
	XGetTransientForHint(wm->dpy, c->window, &c->trans);

	c->trans = c->parent; // This is a fucking hack, gotta read my X manual!!!!

	XFetchName(wm->dpy, c->window, &c->name);

	c->size = XAllocSizeHints();
	XGetWMNormalHints(wm->dpy, c->window, c->size, &dummy);

	if (hints && (hints->flags & StateHint)) {
		set_wm_state(c, hints->initial_state);
		XFree(hints);
	}

	XGetWindowAttributes(wm->dpy, c->window, &attr);
		
	c->x = attr.x;
	c->y = attr.y;
	c->width = attr.width;
	c->height = attr.height;
	c->cmap = attr.colormap;

	return attr.map_state;
}

void Client::init_position(Client *c)
{
	int x, y;
	int xmax = DisplayWidth(wm->dpy, wm->screen);
	int ymax = DisplayHeight(wm->dpy, wm->screen);

	if (c->size->flags & (PSize | USSize) &&
		(c->size->width > MINSIZE) &&
		(c->size->height > MINSIZE)) {
			c->width = c->size->width;
			c->height = c->size->height;
	} else {
		if (c->width < MINSIZE) c->width = MINSIZE;
		if (c->height < MINSIZE) c->height = MINSIZE;
	}

	if (c->size->flags & (PPosition | USPosition) &&
		(c->size->x > MARGIN) &&
		(c->size->x + c->width < xmax - MARGIN) &&
		(c->size->y > MARGIN) &&
		(c->size->y + c->height + theight(c) < ymax - MARGIN)) {
			c->x = c->size->x;
			c->y = c->size->y;
	} else if (c->width > xmax || c->height > ymax) {
		c->x = MARGIN;
		c->y = MARGIN;
	} else {
		get_mouse_position(&x, &y);
		c->x = MARGIN + (x/ xmax) * (xmax - c->width - 2*MARGIN);
		c->y = MARGIN + (y/ ymax) * (ymax - theight(c) - c->height - 2*MARGIN);
	}

}

void Client::reparent(Client *c)
{
	XSelectInput(wm->dpy, c->window, ColormapChangeMask | EnterWindowMask | PropertyChangeMask);

	c->parent = createTopLevelWindow(c->x,c->y - theight(c),
		c->width,c->height + theight(c),BORDER);
		
	c->title = createChildWindow(c->parent, 0, 0
		, c->width, theight(c),
		0);	
	
	c->close.x     = 4;
	c->close.y     = 2;
	c->close.width = 15;
	c->close.height= theight(c)-6;

	c->maximize.x     = c->width - theight(c); 
	c->maximize.y     = 2;
	c->maximize.width = 15;
	c->maximize.height= theight(c)-6;

	c->minimize.x = c->width - theight(c) - 18;
	c->minimize.y = 2;
	c->minimize.width = 15;
	c->minimize.height = theight(c)-6;		     

	c->shadeButton.x = c->width - theight(c) - 36;
	c->shadeButton.y = 2;
	c->shadeButton.width = 15;
	c->shadeButton.height = theight(c)-6;	     

	c->close.window = 
		createChildWindow(c->title, c->close.x, c->close.y
		, c->close.width, c->close.height,
		1);
		
	c->maximize.window = 
		createChildWindow(c->title, c->maximize.x, c->maximize.y
		, c->maximize.width, c->maximize.height,
		1);

	c->minimize.window = 
		createChildWindow(c->title, c->minimize.x, c->minimize.y
		, c->minimize.width, c->minimize.height,
		1);

	c->shadeButton.window = 
		createChildWindow(c->title, c->shadeButton.x, c->shadeButton.y
		, c->shadeButton.width, c->shadeButton.height,
		1);

#ifdef SHAPE
	if (wm->shape) {
		XShapeSelectInput(wm->dpy, c->window, ShapeNotifyMask);
		set_shape(c);
	}
#endif

	XAddToSaveSet(wm->dpy, c->window);
	XSetWindowBorderWidth(wm->dpy, c->window, 0);
	XReparentWindow(wm->dpy, c->window, c->parent, 0, theight(c));
	XResizeWindow(wm->dpy, c->window, c->width, c->height);
		
	send_config(c);
}

void Client::do_gradient(Client *c)
{      		

	XClearWindow(wm->dpy, c->title);
	XClearWindow(wm->dpy, c->close.window);
	XClearWindow(wm->dpy, c->minimize.window);
	XClearWindow(wm->dpy, c->maximize.window);
	XClearWindow(wm->dpy, c->shadeButton.window);
	
	XSetWindowBackgroundPixmap(wm->dpy, c->title, c->frame_pix);
	XSetWindowBackgroundPixmap(wm->dpy, c->close.window, c->button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, c->maximize.window, c->button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, c->minimize.window, c->button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, c->shadeButton.window, c->button_pix);
}

void Client::reconfigure(Client *c)
{
	BImage Title(c->width, theight(c)+1, DefaultDepth(wm->dpy, wm->screen),
	wm->theme->getClientColorTo());	
		
	Title.renderHGradient(wm->theme->getClientColorFrom(), wm->theme->getClientColorTo());
	Title.renderBevel();
	
	c->frame_pix = Title.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
        c->title);
	
	BImage Button(15, theight(c), DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getTitleBarButtonColorTo());	
		
	Button.renderVGradient(wm->theme->getTitleBarButtonColorFrom(),wm->theme->getTitleBarButtonColorTo());
	Button.renderBevel();
	
        c->button_pix = Button.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
        c->shadeButton.window);	

	do_gradient(c);
	redraw(c);
}

void Client::set_wm_state(Client *c, int state)
{
	long data[2];

	data[0] = (long) state;
	data[1] = None; /* icon window */

	XChangeProperty(wm->dpy, c->window, wm->xa_wm_state, wm->xa_wm_state,
		32, PropModeReplace, (unsigned char *)data, 2);
}

int Client::wm_state(Client *c)
{
	Atom real_type;
	int real_format, state;
	unsigned long n, extra;
	unsigned char *data;

	if ((XGetWindowProperty(wm->dpy, c->window, wm->xa_wm_state, 0L, 2L, False,
				AnyPropertyType, &real_type, &real_format, &n,
				&extra, &data) == Success) && n) {
		state = *(int *)data;
		XFree(data);
		return state;
	} else return WithdrawnState;
}

void Client::send_config(Client *c)
{
	XConfigureEvent ce;

	ce.type = ConfigureNotify;
	ce.event = c->window;
	ce.window = c->window;
	ce.x = c->x;
	ce.y = c->y;
	ce.width = c->width;
	ce.height = c->height;
	ce.border_width = 0;
	ce.above = None;
	ce.override_redirect = 0;

	reconfigure(c);	

	XSendEvent(wm->dpy, c->window, False, StructureNotifyMask, (XEvent *)&ce);
}

void Client::remove_client(Client *c, int from_cleanup)
{
	XGrabServer(wm->dpy);

	XEvent foo;

	if(! XCheckTypedWindowEvent(wm->dpy, c->window, UnmapNotify, &foo))
	{

		if (from_cleanup) {
			if (wm_state(c) == IconicState) {
				XMapWindow(wm->dpy, c->window);
				printf("Mapping c->window\n");
			}
	} else set_wm_state(c, WithdrawnState);

		ungravitate(c);
		XRemoveFromSaveSet(wm->dpy, c->window);
		XReparentWindow(wm->dpy, c->window, wm->root, c->x, c->y);
		XSetWindowBorderWidth(wm->dpy, c->window, 1);
		XDestroyWindow(wm->dpy, c->parent);	

		if (c->name) XFree(c->name);
		if (c->size) XFree(c->size);

		XFreePixmap(wm->dpy, frame_pix);
		wm->ClientList->remove(c);
	}
	
	XSync(wm->dpy, False);
	XUngrabServer(wm->dpy);
}

void Client::move(Client *c)
{
	if (c) {
		drag(c);
		XMoveWindow(wm->dpy, c->parent, c->x, c->y - theight(c));		
		send_config(c);
	}
}

void Client::resize(Client *c)
{
	if (c) {
		sweep(c);
		XMoveResizeWindow(wm->dpy, c->parent,
			c->x, c->y - theight(c), c->width, c->height + theight(c));
		
		XMoveResizeWindow(wm->dpy, c->title,
			0, 0, c->width, theight(c)); 
			
		XMoveWindow(wm->dpy, c->maximize.window, c->width - theight(c), c->maximize.y);
		XMoveWindow(wm->dpy, c->minimize.window, c->width - theight(c) - 18, c->minimize.y);
		XMoveWindow(wm->dpy, c->shadeButton.window, c->width - theight(c) - 36, c->shadeButton.y);
		
		XMoveResizeWindow(wm->dpy, c->window,
			0, theight(c), c->width, c->height);
			
		send_config(c);
	
		reconfigure(c);	
	}
}

void Client::hide(Client *c)
{
	if (c) {
		c->ignore_unmap++;
		XUnmapWindow(wm->dpy, c->parent);
		XUnmapWindow(wm->dpy, c->window);
		set_wm_state(c, IconicState);
		
	} 
}

void Client::unhide(Client *c)
{
	if (c) {
		XMapWindow(wm->dpy, c->window);
		XMapRaised(wm->dpy, c->parent);
		set_wm_state(c, NormalState);
	}
}

// This repaints the titlebar and buttons on it, It also paints the window title.
void Client::redraw(Client *c)
{
	XEvent foo;
	
	// Make sure the window still exists.
	if(! XCheckTypedWindowEvent(wm->dpy, c->window, DestroyNotify, &foo))
	{				
		
		do_gradient(c);
		
		// Close Button
		//XDrawLine(wm->dpy, c->close.window, wm->border_gc, 2,2,c->close.width - 3,c->close.height-3);
		//XDrawLine(wm->dpy, c->close.window, wm->border_gc, 2,c->close.height - 3,c->close.width-3,2);
	
		// This creates a + on the button.
		//XDrawRectangle(wm->dpy, c->maximize.window,
		//	 wm->border_gc, 7, c->maximize.height-14, c->maximize.width-16, 11);		
		//XDrawRectangle(wm->dpy, c->maximize.window,
		//	 wm->border_gc, 2, c->maximize.height - 9, c->maximize.width - 5, 0); 

		// This puts a - on the button.
		//XDrawRectangle(wm->dpy, c->minimize.window,
		//	 wm->border_gc, 2, c->minimize.height - 9, c->minimize.width - 5, 0); 

		// Close Button
		XDrawLine(wm->dpy, c->close.window, wm->border_gc, 2,2,c->close.width - 3,c->close.height-3);
		XDrawLine(wm->dpy, c->close.window, wm->border_gc, 2,c->close.height - 3,c->close.width-3,2);
		//--------------------------------------------------------------

		// Maximize Button
		XDrawRectangle(wm->dpy, c->maximize.window,
			wm->border_gc, 2, 2, c->maximize.width - 5, c->maximize.height - 5);
	  	XDrawLine(wm->dpy,  c->maximize.window,
	   		wm->border_gc, 2, 3, c->maximize.width - 3, 3);
		//--------------------------------------------------------------
	
		// Minimize Button
		XDrawRectangle(wm->dpy, c->minimize.window,
			wm->border_gc, 2, c->minimize.height - 5, c->minimize.width - 5, 2);
		//--------------------------------------------------------------

		draw_shade_button(c, c->shaded);
		//--------------------------------------------------------------
		
		if (!c->trans) {
			if (c->name) XDrawString(wm->dpy, c->title, wm->string_gc, SPACE+ 25,
				SPACE + wm->font->ascent, c->name, strlen(c->name));		
		}

	}
}

void Client::draw_shade_button(Client *c, bool pressed)
{
	XClearWindow(wm->dpy, shadeButton.window);	

	if (pressed)
	{		
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				3, 6, 12, 6);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				4, 7, 11, 7);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				5, 8, 10, 8);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				6, 9, 9, 9);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				7, 10, 8, 10);	
	  	
				
	} else {

		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				7, 6, 8, 6);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				6, 7, 9, 7);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				5, 8, 10, 8);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				4, 9, 11, 9);
		XDrawLine(wm->dpy,  c->shadeButton.window,wm->border_gc, 
				3, 10, 12, 10);

	}
}

void Client::change_gravity(Client *c, int multiplier)
{
	int dy = 0;
	int gravity = (c->size->flags & PWinGravity) ?
		c->size->win_gravity : NorthWestGravity;

	switch (gravity) {
		case NorthWestGravity:
		case NorthEastGravity:
		case NorthGravity: dy = theight(c); break;
		case CenterGravity: dy = theight(c)/2; break;		
	}

	c->y += multiplier * dy;
}

#ifdef SHAPE
void Client::set_shape(Client *c)
{
	int n, order;
	XRectangle *rect;

	rect = XShapeGetRectangles(wm->dpy, c->window, ShapeBounding, &n, &order);
	if (n > 1) XShapeCombineShape(wm->dpy,
		c->parent, ShapeBounding, 0, theight(c),
		c->window, ShapeBounding, ShapeSet);
	XFree(rect);
}
#endif

void Client::draw_outline(Client *c)
{
	if(c->shaded)
	{
		XDrawRectangle(wm->dpy, wm->root, wm->invert_gc,
			c->x, c->y - theight(c) + BORDER/2,
			c->width, theight(c));
	
	} else {
	
		XDrawRectangle(wm->dpy, wm->root, wm->invert_gc,
			c->x + BORDER/2, c->y - theight(c) + BORDER/2,
			c->width + BORDER, c->height + theight(c) + BORDER);
	}
}

void Client::get_mouse_position(int *x, int *y)
{
	Window dw1, dw2;
	int t1, t2;
	unsigned int t3;

	XQueryPointer(wm->dpy, wm->root, &dw1, &dw2, x, y, &t1, &t2, &t3);
}

void Client::shade(Client *c)
{
	if (c->shaded)
	{
	   XResizeWindow(wm->dpy, c->parent, c->width, c->height + theight(c));
	   c->shaded=False;
	} else {
	   XResizeWindow(wm->dpy, c->parent, c->width, theight(c) - 1);
	   c->shaded=True;
	}
}

void Client::recalculate_sweep(Client *c, int x1, int y1, int x2, int y2)
{
	int basex, basey;

	c->width = abs(x1 - x2) - BORDER;
	c->height = abs(y1 - y2) - BORDER;

	if (c->size->flags & PResizeInc) {
		basex = (c->size->flags & PBaseSize) ? c->size->base_width :
			(c->size->flags & PMinSize) ? c->size->min_width : 0;
		basey = (c->size->flags & PBaseSize) ? c->size->base_height :
			(c->size->flags & PMinSize) ? c->size->min_height : 0;
		c->width -= (c->width - basex) % c->size->width_inc;
		c->height -= (c->height - basey) % c->size->height_inc;
	}
	if (c->size->flags & PMinSize) {
		if (c->width < c->size->min_width) c->width = c->size->min_width;
		if (c->height < c->size->min_height) c->height = c->size->min_height;
	}
	if (c->size->flags & PMaxSize) {
		if (c->width > c->size->max_width) c->width = c->size->max_width;
		if (c->height > c->size->max_height) c->height = c->size->max_height;
	}

	c->x = (x1 <= x2) ? x1 : x1 - c->width;
	c->y = (y1 <= y2) ? y1 : y1 - c->height;
}

void Client::sweep(Client *c)
{
	XEvent ev;
	int old_cx = c->x;
	int old_cy = c->y;

	if (!grab(wm->root, MouseMask, wm->resize_curs)) return;
	XGrabServer(wm->dpy);

	draw_outline(c);
	setmouse(c->window, c->width, c->height);
	for (;;) {
		XMaskEvent(wm->dpy, MouseMask, &ev);
		switch (ev.type) {
			case MotionNotify:
				draw_outline(c); 
				recalculate_sweep(c, old_cx, old_cy, ev.xmotion.x, ev.xmotion.y);				
				draw_outline(c);
				break;
			case ButtonRelease:
				draw_outline(c); 
				XUngrabServer(wm->dpy);
				ungrab();
				return;
		}
	}
}

void Client::drag(Client *c)
{
	XEvent ev;
	int x1, y1;
	int old_cx = c->x;
	int old_cy = c->y;

	if (!grab(wm->root, MouseMask, wm->move_curs)) return;
	XGrabServer(wm->dpy);
	get_mouse_position(&x1, &y1);

	draw_outline(c);
	for (;;) {
		XMaskEvent(wm->dpy, MouseMask, &ev);
		switch (ev.type) {
			case MotionNotify:
				draw_outline(c); /* clear */
				c->x = old_cx + (ev.xmotion.x - x1);
				c->y = old_cy + (ev.xmotion.y - y1);
				draw_outline(c);
				break;
			case ButtonRelease:
				draw_outline(c); /* clear */
				ungrab();
				return;
		}
	}
}

void Client::handle_leave_event(XCrossingEvent *e)
{
 	//Client *c = find_client(e->window);	
}

void Client::handle_enter_event(XCrossingEvent *e)
{
 	Client *c = find_client(e->window);

	if(c)
	{
		instcmap(c->cmap);
		XSetInputFocus(wm->dpy, c->window, RevertToParent, CurrentTime);
	}
}

void Client::handle_button_press_event(XButtonEvent *e)
{
	Client *c = find_client(e->window);

	if (c) {
		switch (e->button) {
			case Button1:
			{
				if(c->close.window == e->window)
				{
					XSetWindowBorderWidth(wm->dpy, c->close.window, 2);
				} else if (c->maximize.window == e->window)
				{
					XSetWindowBorderWidth(wm->dpy, c->maximize.window, 2);
				} else if (c->minimize.window == e->window)
				{
					XSetWindowBorderWidth(wm->dpy, c->minimize.window, 2);
					XLowerWindow(wm->dpy, c->parent); 
					
				} else if (c->shadeButton.window == e->window) {
					XSetWindowBorderWidth(wm->dpy, c->shadeButton.window, 2);
				} else if (c->title == e->window){
					XRaiseWindow(wm->dpy, c->parent);
					move(c);
				} 
	
			break;
			}

			case Button2:
			{	
			break;
			}
			case Button3:
			{
			break;
			}
		}
	}
}

void Client::handle_button_release_event(XButtonEvent *e)
{
	Client *c = find_client(e->window);

	if (c) {
		switch (e->button) {
			case Button1:
			{
				if(c->close.window == e->window)
				{
					XSetWindowBorderWidth(wm->dpy, c->close.window, 1);
					send_wm_delete(c);
				} else if (c->maximize.window == e->window)
				{
					XSetWindowBorderWidth(wm->dpy, c->maximize.window, 1);
					if(c->shaded)
					{
						shade(c);
						XRaiseWindow(wm->dpy, c->parent);
						resize(c);
					} else {
						XRaiseWindow(wm->dpy, c->parent);
						resize(c);
					}
					
				} else if (c->minimize.window == e->window)
				{
					XSetWindowBorderWidth(wm->dpy, c->minimize.window, 1);
				} else if (c->shadeButton.window == e->window) {
					XSetWindowBorderWidth(wm->dpy, c->shadeButton.window, 1);
					XRaiseWindow(wm->dpy, c->parent);
					shade(c);
				} 
	
			break;
			}

			case Button2:
			{	
				//if (box) c->resize(c);
				//else { 
				//	XRaiseWindow(wm->dpy, c->parent);  
				//	move(c);
				//}
				
			break;
			}
			case Button3:
			{
				//if(box) send_wm_delete(c);
				//else XLowerWindow(wm->dpy, c->parent); 
				
			break;
			}
		}
	}
}

void Client::handle_button_event(XButtonEvent *e)
{
}

void Client::handle_configure_request(XConfigureRequestEvent *e)
{
	Client *c = find_client(e->window);
	XWindowChanges wc;

	if (c) {
		ungravitate(c);
		if (e->value_mask & CWX) c->x = e->x;
		if (e->value_mask & CWY) c->y = e->y;
		if (e->value_mask & CWWidth) c->width = e->width;
		if (e->value_mask & CWHeight) c->height = e->height;
		gravitate(c);

		if (c->width != wc.width && c->height != wc.height)
		{
			XMoveResizeWindow(wm->dpy, c->parent,
				c->x, c->y - theight(c), c->width, c->height + theight(c));
		
			XMoveResizeWindow(wm->dpy, c->title,
				-1, -1, c->width, theight(c));
		
			XMoveWindow(wm->dpy, c->maximize.window, c->width - theight(c), c->maximize.y);
			XMoveWindow(wm->dpy, c->minimize.window, c->width - theight(c) - 18, c->minimize.y);
			XMoveWindow(wm->dpy, c->shadeButton.window, c->width - theight(c) - 36, c->shadeButton.y);
			
			XMoveResizeWindow(wm->dpy, c->window,
				0, theight(c), c->width, c->height);		
		}
		
		wc.x = c->x;
		wc.y = c->y - theight(c);
		wc.width = c->width;
		wc.height = c->height + theight(c);
		wc.border_width = BORDER;
		wc.sibling = e->above;
		wc.stack_mode = e->detail;
		
		XConfigureWindow(wm->dpy, c->parent, e->value_mask, &wc);

		send_config(c);
	}

	wc.x = c ? 0 : e->x;
	wc.y = c ? theight(c) : e->y;
	wc.width = e->width;
	wc.height = e->height;
	wc.sibling = e->above;
	wc.stack_mode = e->detail;

	XConfigureWindow(wm->dpy, e->window, e->value_mask, &wc);
}

void Client::handle_map_request(XMapRequestEvent *e)
{
	Client *c = find_client(e->window);

	if (c) {
		XMapWindow(wm->dpy, c->window);
		XMapRaised(wm->dpy, c->parent);
		set_wm_state(c, NormalState);
	} else
		make_new_client(e->window);
}

void Client::handle_unmap_event(XUnmapEvent *e)
{
	Client *c = find_client(e->window);

	if (c) {
		if (c->ignore_unmap) c->ignore_unmap--;
		else remove_client(c, 0);
	}
}

void Client::handle_client_message(XClientMessageEvent *e)
{
	Client *c = find_client(e->window);

	if (c && e->message_type == wm->xa_wm_change_state &&
		e->format == 32 && e->data.l[0] == IconicState) hide(c);	
}

void Client::handle_colormap_change(XColormapEvent *e)
{
	Client *c =  find_client(e->window);

	if (c && e->c_new) {
		c->cmap = e->colormap;
		instcmap(c->cmap);
	}
}

void Client::handle_property_change(XPropertyEvent *e)
{
	Client *c = find_client(e->window);
	long dummy;

	if (c) {
		switch (e->atom) {
			case XA_WM_NAME:
			{
				if (c->name) XFree(c->name);
				XFetchName(wm->dpy, c->window, &c->name);
				XClearWindow(wm->dpy, c->parent);
				redraw(c);
			}
			break;
			
			case XA_WM_NORMAL_HINTS:
			{
				XGetWMNormalHints(wm->dpy, c->window, c->size, &dummy);				
			}
			break;
		}
	}
}


void Client::handle_expose_event(XExposeEvent *e)
{
	Client *c = find_client(e->window);

	if (c && e->count == 0) { c->redraw(c); }
}

void Client::handle_reparent_event(XReparentEvent *e)
{
	Client *c = find_client(e->window);
	Client *dest = find_client(e->parent);

	if (c && !dest) remove_client(c, 0);
	
}

#ifdef SHAPE
void Client::handle_shape_change(XShapeEvent *e)
{
	Client *c = find_client(e->window);

	if (c) set_shape(c);
}
#endif

#ifdef DEBUG
void Client::dump_clients()
{
	if(wm->ClientList->count()) {
		LinkedListIterator<Client> it(wm->ClientList);
		for(; it.current(); it++)
		{
			err("%s -> %s (%d, %d): %#x (%#x) @ %d,%d", windowmanager_name, it.current()->name, it.current()->wm_state(it.current()),
			it.current()->ignore_unmap, it.current()->window, it.current()->parent, it.current()->x, it.current()->y);
		}
	}
}
#endif

void Client::send_wm_delete(Client *c)
{
	int i, n, found = 0;
	Atom *protocols;

	if (c) {
		if (XGetWMProtocols(wm->dpy, c->window, &protocols, &n))
			for (i=0; i<n; i++) if (protocols[i] == wm->xa_wm_delete) found++;
		if (found) send_xmessage(c->window, wm->xa_wm_protos, wm->xa_wm_delete);
		else XKillClient(wm->dpy, c->window);
	}
}

int Client::send_xmessage(Window w, Atom a, long x)
{
	XEvent ev;

	ev.type = ClientMessage;
	ev.xclient.window = w;
	ev.xclient.message_type = a;
	ev.xclient.format = 32;
	ev.xclient.data.l[0] = x;
	ev.xclient.data.l[1] = CurrentTime;

	return XSendEvent(wm->dpy, w, False, NoEventMask, &ev);
}

