#ifndef _CLIENT_HH_

class Menu;

class Client {

private:
	Window  title;
	Window	window;
	Window	parent;
	Window	trans;

	Pixmap frame_pix;
	Pixmap button_pix;
	
	struct button {
		Window window;
		int x,y;
		unsigned int width,height;
		bool pressed;
	};

	button shadeButton;
	button maximize;
	button minimize;
	button close;
	
	char	*name;

	Colormap	cmap;
	XSizeHints	*size;
	int		ignore_unmap;

	int		x;
	int		y;
	int		width;
	int		height;

	bool button1;
	bool button2;
	bool button3;
	
	bool shaded;
	
public: /* member functions */

	Client(); 
	~Client();

protected:
	
	Window createChildWindow(Window parent, int x, int y, unsigned int width,
		unsigned int height,unsigned int borderwidth);
	
	Window createTopLevelWindow(int x, int y, unsigned int width,
		unsigned int height,unsigned int borderwidth);

	void do_gradient(Client *c);	
	Client *find_client(Window w);
	Client *findClientByName(char *name);
	int inititialize(Client *c, Window w);
	void init_position(Client *c);
	void reparent(Client *c);
	void send_config(Client *c);
	void remove_client(Client *c, int from_cleanup);
	void resize(Client *c);
	void hide(Client *c);
	void move(Client *c);
	void unhide(Client *c);
	void change_gravity(Client *c, int multiplier);
	void shade(Client *c);
	
	#ifdef SHAPE
	void set_shape(Client *c);
	#endif

	void drag(Client *c);
	void sweep(Client *c);
	void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2);
	void draw_outline(Client *c);
	void draw_shade_button(Client *c, bool pressed);
	
	int wm_state(Client *c);
	void set_wm_state(Client *c, int state);

public:
	
	void redraw(Client *c);
	void get_mouse_position(int *x, int *y);
	void make_new_client(Window w);
	int send_xmessage(Window w, Atom a, long x);
	void send_wm_delete(Client *c);
	void handle_unmap_event(XUnmapEvent *e);
	void handle_button_event(XButtonEvent *e);
	void handle_button_press_event(XButtonEvent *e);
	void handle_button_release_event(XButtonEvent *e);
	void handle_client_message(XClientMessageEvent *e);
	void handle_colormap_change(XColormapEvent *e);
	void handle_configure_request(XConfigureRequestEvent *e);
	void handle_enter_event(XCrossingEvent *e);
	void handle_leave_event(XCrossingEvent *e);
	void handle_expose_event(XExposeEvent *e);
	void handle_map_request(XMapRequestEvent *e);
	void handle_property_change(XPropertyEvent *e);
	void handle_reparent_event(XReparentEvent *e);
	
	void reconfigure(Client *c);
	
	#ifdef SHAPE
	void handle_shape_change(XShapeEvent *e);
	#endif

	#ifdef DEBUG
	void dump_clients();
	#endif
};

#endif 
