#include "sapphire.hh"

//----------------------------------------------------------------------------//
// Sets up variables so we can get ready to use them
//----------------------------------------------------------------------------//
MenuLexer::MenuLexer(char *fileName, RootMenu *rm) : Scanner(fileName) 
{
	rootmenu = rm;
	
	temp = new char[MaxBufferSize];
	current_token = new char[MaxBufferSize];
	var = new char[MaxBufferSize];

	strcpy(temp, "");
	strcpy(current_token, "");
	strcpy(var, "");

	submenu_counter=0;
	function=99;
	
	createMenu=False;
	error=False;
	
	variables = new LinkedList<Data>;
}

MenuLexer::~MenuLexer() { 
		delete [] temp; 
		delete [] current_token;
		delete [] var;
				
		LinkedListIterator<Data> it(variables);
		for(; it.current(); it++)
		{
			variables->remove(it.current());
		}
		
		delete variables;
}

//--------------------------------------------------------------------------------------------//
// Preforms MenuLexerical Analysis 
// This represents the top level of the configuration file
//--------------------------------------------------------------------------------------------//
void MenuLexer::analyze()
{
	do {
		
		getNextToken(False);
		
	} while(statement() || menu(createMenu));
}

bool MenuLexer::menu(bool createMenu)
{
	if(match("menu"))
	{
		getNextToken(False);
		
		if(match("("))
		{
			getNextToken(False);		
			
			if(match(")"))
			{
			} else {
				
				quoted_text();
				
				char *name = new char[strlen(temp)];
				strcpy(name, temp);
				
				if(createMenu) rootmenu->setMenuTitle(name); 
				
				getNextToken(False);
				
				if(! match(")"))
				{
					expected(")");
				}
			}
		}
			
		getNextToken(False);
		
		if(match("{"))
		{
			
			do
			{
				getNextToken(False);
					
			} while(statement() || isVariable() || item(createMenu) || submenu(createMenu) || exit(createMenu) || theme(createMenu));
			
			if(match("}"))
			{
				if(createMenu) {
					
					rootmenu->doMenu();
					rootmenu->setSubMenuList(rootmenu->submenus);
				}
										
				// we only want 1 menu to be defined if there is more they are ignored.
				return False;
				
			} else {
				expected("}");				
			}

		} else {
			expected("{");
			
		}
	} else {
		expected("menu");
	}
		
	return False;
}

bool MenuLexer::theme(bool createMenu)
{
	// This parses the item syntax. 
	//
	// It looks like this:
	//
	// 	theme ( "MyTheme" ) { "~/themes/mytheme" }
	//
	char* name=NULL;
	
	if(match("theme"))
	{	
		getNextToken(False);
		
		if(match("("))
		{
			getNextToken(False);			
			
			if(quoted_text() || isVariable())
			{
				
				name=new char[strlen(temp)];	
				strcpy(name,temp);
				
			}
			
			getNextToken(False);
		
			if(match(")"))
			{
				getNextToken(False);
		
				if(match("{"))
				{
					do
					{
						getNextToken(False);
						
					} while(quoted_text() || isVariable());

					char* data=new char[strlen(temp)];
					strcpy(data,temp);

					if(match("}"))
					{
						if(createMenu) rootmenu->sub->insert(name,data, 99,99,2);

						return True;
					} else {
						expected("}");
					} 
				} else {
					expected("{");
				}
				
			
			} else {
				expected(")");
			}

		} else {
			expected("(");
		}	

	} else {
		return False;
	}

	return False;
}

bool MenuLexer::submenutheme(bool createMenu)
{
	// This parses the item syntax. 
	//
	// It looks like this:
	//
	// 	theme ( "MyTheme" ) { "~/themes/mytheme" }
	//


	char* name=NULL;	
	
	if(match("theme"))
	{	
		getNextToken(False);
		
		if(match("("))
		{
			getNextToken(False);			
			
			if(quoted_text() || isVariable())
			{
				name=new char[strlen(temp)];
				strcpy(name,temp);
			}
			
			getNextToken(False);
		
			if(match(")"))
			{
				getNextToken(False);
		
				if(match("{"))
				{
					do
					{
						getNextToken(False);
						
					} while(quoted_text() || isVariable());
	
					char* data=new char[strlen(temp)];
					strcpy(data,temp);

					if(match("}"))
					{
						if(createMenu) rootmenu->sub->insert(name,data, 99,99,2);
								
						return True;
					} else {
						expected("}");
					} 
				} else {
					expected("{");
				}
				
			
			} else {
				expected(")");
			}

		} else {
			expected("(");
		}	

	} else {
		return False;
	}

	return False;
}

bool MenuLexer::submenu(bool createMenu)
{
	char* submenu_name=NULL;
	
	if(match("submenu"))
	{
		getNextToken(False);
		
		if(match("("))
		{
			getNextToken(False);		
			
			if(match(")"))
			{
			} else {
				
				quoted_text();
				
				if(createMenu) {
				submenu_name = new char[strlen(temp)];
				strcpy(submenu_name, temp);
				
				rootmenu->sub = new Menu();
				rootmenu->sub->setMenuTitle(submenu_name);
				}
				
				getNextToken(False);
				
				if(! match(")"))
				{
					expected(")");
				}
				
			}
		}
			
		getNextToken(False);
		
		if(match("{"))
		{
			
			do
			{
				getNextToken(False);
					
			} while(statement() || isVariable() || submenuitem(createMenu) || exit(createMenu) || submenutheme(createMenu));
			
			if(match("}"))
			{
				if(createMenu) {
				
					// create the menu item that points to this submenu				
					rootmenu->insert(submenu_name, "", submenu_counter, submenu_counter, 99);
					rootmenu->submenus->insert(rootmenu->sub);
					rootmenu->sub->doMenu();				
					rootmenu->sub->setSubMenuList(rootmenu->submenus);
					submenu_counter++;
				}
				
				return True;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return False;
	}
		
	return False;
}

// Parses comments
bool MenuLexer::comments()
{
	// Comments start with a # and end with a # they can be multiple lines.
	if(match("#"))
	{
		while(1)
		{
			getNextToken(false);
		
			if(match("#"))
			{
				return true;
			} 
		}
	}

	// Experimental below - it doesn't work like I want it too.
		
	// /* comments */
	//
	// Comments resemble the standard C comments. They can be single line or multi-line. 
	// they can also be nested.
	
	/*if(match("/"))
	{	
		getNextToken(False);
		
		if(match("*"))
		{

			if(match("*"))
			{
				while(1)
				{
					getNextToken(False);
					
					comments(); // accept nested comments
					
					if(match("*"))
					{	
						
						getNextToken(False);

						if(match("/"))
						{
							return True;
							break;
						} 
			
						if(feof(inputfile)){
							expected("/");	
						}
					} 

					if(feof(inputfile)){
						expected("*");	
					}
				}
			} 
		} else {
			expected("*");
		}
	}
	*/

	return False;
}

//--------------------------------------------------------------------------------------------//
// Parses quoted text 
//--------------------------------------------------------------------------------------------//
bool MenuLexer::quoted_text()
{
	// "text..."
	//	

	// This function will work with the following escape sequence for putting quotes in you text
	//
	// escape sequence \" is used to add a quote to the text
	int holdLine=0;

	char *string = new char[MaxBufferSize];
	strcpy(string, "");
	
	if(match("\""))
	{
		holdLine=currentLine;
	
		do
		{
			getNextToken(True);
		
			if(feof(inputfile))
			{
				currentLine=holdLine+1;
				expected("\"");
			}
		
			// Check if there is an escape sequence of /" for quote.
			if(match("/"))
			{
				getNextToken(True);
				if(match("\""))
				{
					strcat(string, "\"");
					getNextToken(True);
				} else {
					strcat(string, "/");
				}
			} else {
				if(match("\""))
				{
					return True;
				}
			}
					
	
		// This concats the string and places it in variable called temp.
		// if you intended to use the string the contents of quoted_text would
		// be found in the variable temp which is global to the MenuLexer class.
		strcat(string, current_token); 
		strcpy(temp, string);
		
		} while(!match("\""));

	delete [] string;
				
	return True;
	
	} else {
		return False;
	}

	return False;
}

//--------------------------------------------------------------------------------------------//
bool MenuLexer::item(bool createMenu)
{
	// This parses the item syntax. 
	//
	// It looks like this:
	//
	// 	item ( "Xterm" ) { "xterm" }
	//
	char* name=NULL;
	
	if(match("item"))
	{	
		getNextToken(False);
		
		if(match("("))
		{
			getNextToken(False);			
			
			if(quoted_text() || isVariable())
			{
				
				name=new char[strlen(temp)];	
				strcpy(name,temp);
				
			}
			
			getNextToken(False);
		
			if(match(")"))
			{
				getNextToken(False);
		
				if(match("{"))
				{
					do
					{
						getNextToken(False);
						
					} while(quoted_text() || isVariable());

					char* data=new char[strlen(temp)];
					strcpy(data,temp);

					if(match("}"))
					{
						if(createMenu) rootmenu->insert(name,data, 0,99,function);

						return True;
					} else {
						expected("}");
					} 
				} else {
					expected("{");
				}
				
			
			} else {
				expected(")");
			}

		} else {
			expected("(");
		}	

	} else {
		return False;
	}

	return False;
}

//--------------------------------------------------------------------------------------------//
// need to pass this function a Menu* so it will insert the menu item in the proper sub menu.
bool MenuLexer::submenuitem(bool createMenu)
{
	// This parses the item syntax. 
	//
	// It looks like this:
	//
	// 	item ( "Xterm" ) { "xterm" }
	//

	char* name=NULL;	
	
	if(match("item"))
	{	
		getNextToken(False);
		
		if(match("("))
		{
			getNextToken(False);			
			
			if(quoted_text() || isVariable())
			{
				name=new char[strlen(temp)];
				strcpy(name,temp);
			}
			
			getNextToken(False);
		
			if(match(")"))
			{
				getNextToken(False);
		
				if(match("{"))
				{
					do
					{
						getNextToken(False);
						
					} while(quoted_text() || isVariable());
	
					char* data=new char[strlen(temp)];
					strcpy(data,temp);

					if(match("}"))
					{
						if(createMenu) rootmenu->sub->insert(name,data, 99,99,function);
								
						return True;
					} else {
						expected("}");
					} 
				} else {
					expected("{");
				}
				
			
			} else {
				expected(")");
			}

		} else {
			expected("(");
		}	

	} else {
		return False;
	}

	return False;
}

bool MenuLexer::exit(bool createMenu)
{
	// This will not be the final solution to having system functions executing from the root menu.
	// This is here right now just to make an exit entry in the menu. I will change this soon.
	// I already know that this function can cause unexpected results. It will only insert itself
	// on the root menu and not in submenus. Like I said above this will be changed.
	
	if(match("sapphire_exit"))
	{
		if(createMenu) rootmenu->insert("exit","", submenu_counter,99,1);
		
		return True;
	}

	return False;
}

//--------------------------------------------------------------------------------------------//
// Parses variables : var name = quoted_text 
//
// Used to store information that can be used in many places.
//--------------------------------------------------------------------------------------------//
bool MenuLexer::variable()
{
	char* name;

	if(match("var"))
	{
		getNextToken(False);
	
		name = current_token;

		getNextToken(False);
	
		if(match("="))
		{
			getNextToken(False);
			
			quoted_text();
			
			Data *d = new Data();
			
			strcpy (d->name, name);
			strcpy (d->data, temp);
			
			variables->insert(d);

			return True;
		} else {
			expected("=");
		}

	} 
	
	return False;
}

//--------------------------------------------------------------------------------------------//
// Parses legal statements
//--------------------------------------------------------------------------------------------//
bool MenuLexer::statement()
{
	// If there were more language statements they would go here.....
	
	if(comments())              return True;
	if(variable())              return True;
	
	// Its not a statement 
	return False;
}

//--------------------------------------------------------------------------------------------//
// Test the current token to see if it is a variable if it is then it writes the proper code
// to the HTML file.
//--------------------------------------------------------------------------------------------//
bool MenuLexer::isVariable()
{
	LinkedListIterator<Data> it(variables);
	
	for(; it.current(); it++)
	{
		if(match(it.current()->name))
		{
			return True;				
		} 
	}
	
	return False;
}

//-------------------------------------------------------------------------------------------//
// Matches a token by getting the next token and comparing it to the 
// match. If it matches it returns True otherwise it returns False
//--------------------------------------------------------------------------------------------//
bool MenuLexer::match(char *tok)
{
	if(strcmp(current_token, tok)==0)
	{
		return True; // found a match
	} else {
		return False; // no match
	}
}

//--------------------------------------------------------------------------------------------//
// If you expected a token but didn't get it then this function will
// print a error message in the form of 
// 
//--------------------------------------------------------------------------------------------//
void MenuLexer::expected(char *err)
{
	// Not meant to be perfect!!!!
	if(! error) {
		printf("\nexpected: %s\n", err);
		printf("menu file has an error on line: %d\n", currentLine);
	}
	
	//::exit(0);	
	error=True; 	
}
