// sapphire.hh
//
// Frank Hale 
// frankhale@yahoo.com
#ifndef _SAPPHIRE_HH_
#define _SAPPHIRE_HH_

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>
#include <X11/Xresource.h>
#include <X11/xpm.h>
#ifdef SHAPE
#include <X11/extensions/shape.h>
#endif

#define windowmanager_name      "Sapphire"
#define windowmanager_version   "Sapphire 0.12.3"

#include "linkedlist.hh"
#include "graphics.hh"

static char* HOME = getenv("HOME");

// Buffer size limit
#define MaxBufferSize 256

// Maximum number of Identifiers for configuration files
#define MaxMenuIdents 1

// used when the lexer finds an error. This will hold the current line that
// the scanner is on so it can attempt to report what line the error occurred 
// on.
extern int currentLine;

//----------------------------------------------------------------------------//
// Types of characters codes for the parser. These are used for the different 
// types of tokens.
// When the scanner see's one of these character types it returns it as a token.
//----------------------------------------------------------------------------//
enum charCodes  { 

// The codes below are collected with there type and when the parser sees
// a different type it returns the current token of the types it just collected.

	  letters,    	/*  a-z and A-Z            */
	  numbers,    	/*  0-9                    */
	  special,    	/*  example !@#$%          */
	  whitespace, 	/*  \n, \t, ' ', etc...    */

          // The codes below are returned 1 at a time. 
	  // This is useful for processing
          // tokens with special meaning.

	  openBrace,  	/*  {			         */
	  closeBrace, 	/*  }			         */
	  equals,     	/*  =			         */
	  plus,       	/*  +			         */
	  minus,      	/*  -			         */
	  forwardSlash, /*  /			         */
	  asterix,    	/*  *			         */
	  openBracket,  /*  [			         */
	  closeBracket,	/*  ]			         */
	  period,	/* .          	                 */
	  openParen,	/* (		        	 */
	  closedParen,	/* )			         */
	  greaterThan,  /* >			         */
	  lessThan,     /* <			         */ 
	  quote		/* "			         */
	 		  
};

extern charCodes charCodeMap[128];

/* default settings */
#define DEF_FONT	"-adobe-helvetica-medium-r-*-*-*-120-*-*-*-*-*-*"
#define MENU_FONT	"-*-lucida-medium-r-normal-*-15-*-*-*-*-*-*-*"
#define DEF_FG		"black"
#define DEF_BG2		"lightgray"
#define DEF_BG		"slategray"
#define DEF_BD		"black"
#define BORDER		1
#define SPACE		3
#define MARGIN		5
#define MINSIZE		10

/* some coding shorthand - I don't like shorthand code I think I will 
   change this and put this explicity in the code and scrub the macro's */

#define ChildMask	(SubstructureRedirectMask|SubstructureNotifyMask)
#define ButtonMask	(ButtonPressMask|ButtonReleaseMask)
#define MouseMask	(ButtonMask|PointerMotionMask)

#define streq(s1, s2) \
	(strcmp(s1, s2) == 0)
#define theight(c) \
	((c->trans ? 2*SPACE : wm->iheight) + BORDER)
#define instcmap(cmap) \
	XInstallColormap(wm->dpy, cmap)
#define grab(w, mask, curs) \
	(XGrabPointer(wm->dpy, w, False, mask, GrabModeAsync, GrabModeAsync, \
	None, curs, CurrentTime) == GrabSuccess)
#define ungrab() \
	XUngrabPointer(wm->dpy, CurrentTime)
#define setmouse(w, x, y) \
	XWarpPointer(wm->dpy, None, w, 0, 0, 0, 0, x, y)
#define gravitate(c) \
	c->change_gravity(c, 1)
#define ungravitate(c) \
	c->change_gravity(c, -1)

extern int handle_xerror(Display *dpy, XErrorEvent *e);
extern void err(char *fmt, ...);
extern void show_event(XEvent e);
extern void signalhandler(int i);

class MenuItem;
class Menu;
class RootMenu;
class Toolbar;
class InputBuffer;
class Scanner;
class MenuLexer;
class ThemeLexer;
class OutputBuffer;
class WMLexer;
class WMConfig;

#include "outputbuffer.hh"
#include "inputbuffer.hh"
#include "scanner.hh"
#include "data.hh"
#include "menulexer.hh"
#include "themelexer.hh"
#include "theme.hh"
#include "wmlexer.hh"
#include "wmconfig.hh"
#include "client.hh"
#include "wm.hh"
#include "toolbar.hh"
#include "menuitem.hh"
#include "menu.hh"
#include "rootmenu.hh"

#endif
