#include "sapphire.hh"

charCodes charCodeMap[128];

//--------------------------------------------------------------------------------------------//
// This sets up the charCodeMap with the various types of tokens.
//--------------------------------------------------------------------------------------------//
Scanner::Scanner(char *filename) : InputBuffer(filename)
{
	int i;

	fileName = filename;

	// Letter tokens
	for(i='a';i<='z';++i) charCodeMap[i]=letters;
	for(i='A';i<='Z';++i) charCodeMap[i]=letters;
	
	// Number tokens
	for(i='0';i<='9';++i) charCodeMap[i]=numbers;
	
	// Tokens
	charCodeMap['+'] = plus;
	charCodeMap['-'] = minus;
	charCodeMap['*'] = asterix;
	charCodeMap['/'] = forwardSlash;
	charCodeMap['('] = openParen;
	charCodeMap[')'] = closedParen;
	charCodeMap['{'] = openBrace;
	charCodeMap['}'] = closeBrace;
	charCodeMap['['] = openBracket;
	charCodeMap[']'] = closeBracket;
	charCodeMap[':'] = special;
	charCodeMap[';'] = special;
	charCodeMap['\''] = special;
	charCodeMap['\"'] = quote;
	charCodeMap['.'] = period;
	charCodeMap[','] = special;
	charCodeMap['<'] = lessThan;
	charCodeMap['>'] = greaterThan;
	charCodeMap['?'] = special;
	charCodeMap['&'] = special;
	charCodeMap['!'] = special;
	charCodeMap['@'] = special;
	charCodeMap['#'] = special;
	charCodeMap['$'] = special;
	charCodeMap['%'] = special;
	charCodeMap['^'] = special;
	charCodeMap['='] = equals;
	charCodeMap['|'] = special;
	charCodeMap['~'] = special;
	charCodeMap[' ']  = whitespace;	
	charCodeMap['\r'] = whitespace;
	charCodeMap['\n'] = whitespace;
	charCodeMap['\t'] = whitespace;
	charCodeMap['\0'] = whitespace;	
	charCodeMap[EOF] = whitespace;
}

//--------------------------------------------------------------------------------------------//
// Skips whitespace in the file 
//--------------------------------------------------------------------------------------------//
void Scanner::skipWhiteSpace()
{
	while(charCodeMap[currentChar]==whitespace)
	{
		if(feof(inputfile))
		{	
			return;
		}
		
		if(currentChar=='\n')
		{
			currentLine++;
		}
			
		getChar();
	}
}

//--------------------------------------------------------------------------------------------//
// getToken(bool spaces)
//
// This function will return the next token in the file. If spaces is 
// true then it will include spaces in the token, otherwise it will 
// separate tokens by spaces.
//--------------------------------------------------------------------------------------------//
char* Scanner::getToken(bool spaces)
{
	char *t = new char[MaxBufferSize];

	strcpy(t, "\n");

	int i=0;
	
	if(!feof(inputfile)) 
	{
		if(!spaces)
		{ 
			skipWhiteSpace(); 
		}
	
		switch(charCodeMap[currentChar])
		{
	
		case letters:
		{
			while(charCodeMap[currentChar] == letters) 
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
				
				getChar();
			}
			return t;
		}
		break;

		case openBrace: // 
		{
			if(charCodeMap[currentChar] == openBrace)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case closeBrace: // } 
		{
			if(charCodeMap[currentChar] == closeBrace)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case equals: // =
		{
			if(charCodeMap[currentChar] == equals)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case plus: // +
		{
			if(charCodeMap[currentChar] == plus)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case minus: // -
		{
			if(charCodeMap[currentChar] == minus)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case asterix: // *
		{
			if(charCodeMap[currentChar] == asterix)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}

		
			return t;
		}
		break;

		case forwardSlash: // /
		{
			if(charCodeMap[currentChar] == forwardSlash)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;
	
		case openBracket:
		{
			if(charCodeMap[currentChar] == openBracket)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;
		
		case closeBracket:
		{
			if(charCodeMap[currentChar] == closeBracket)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case period:
		{
			if(charCodeMap[currentChar] == period)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
		
			return t;
		}
		break;

		case openParen:
		{
			if(charCodeMap[currentChar] == openParen)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;

				getChar();
			}
	
			return t;
		}
		break;

		case closedParen:
		{
			if(charCodeMap[currentChar] == closedParen)
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
				
				getChar();
			}
		
			return t;
		}
		break;

		case greaterThan:
		{
			while(charCodeMap[currentChar] == greaterThan) 
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
		
				getChar();
			}
			return t;
		}
		break;

		case lessThan:
		{
			while(charCodeMap[currentChar] == lessThan) 
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
		
				getChar();
			}
			return t;
		}
		break;

		case quote:
		{
			while(charCodeMap[currentChar] == quote) 
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
		
				getChar();
			}
			return t;
		}
		break;

		case numbers:
		{
			while(charCodeMap[currentChar] == numbers) 
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
		
				getChar();
			}
			return t;
		}
		break;

		case special:
		{
			while(charCodeMap[currentChar] == special) 
			{
				t[i]=currentChar;		
				t[i+1]='\0';
				i++;
		
				getChar();
			}
			return t;
		}
		break;

		case whitespace:
		{
			if(spaces)
			{
				while(charCodeMap[currentChar] == whitespace) 
				{
					t[i]=currentChar;		
					t[i+1]='\0';
					i++;
		
					if(currentChar=='\n')
					{
						currentLine++;
					}
		
					getChar();
				}
				return t;

			} else {
				skipWhiteSpace();
				return t;
			}
		}
		break;

		}

	}	

	return t;
}
