#include "sapphire.hh"

//----------------------------------------------------------------------------//
// Sets up variables so we can get ready to use them
//----------------------------------------------------------------------------//
ThemeLexer::ThemeLexer(char *fileName) : Scanner(fileName) 
{		
	temp = new char[MaxBufferSize];
	current_token = new char[MaxBufferSize];
	var = new char[MaxBufferSize];

	strcpy(temp, "");
	strcpy(current_token, "");
	strcpy(var, "");

	variables = new LinkedList<Data>;
	wm->theme->setToolbarLocationTop(false);
}

ThemeLexer::~ThemeLexer() { 
		delete [] temp; 
		delete [] current_token;
		delete [] var;
		
		XFreePixmap(wm->dpy, root_pix);
				
		LinkedListIterator<Data> it(variables);
		for(; it.current(); it++)
		{
			variables->remove(it.current());
		}
		
		delete variables;
}

//--------------------------------------------------------------------------------------------//
// Preforms ThemeLexerical Analysis 
// This represents the top level of the configuration file
//--------------------------------------------------------------------------------------------//
void ThemeLexer::analyze()
{
	do {
		getNextToken(false);
	} while(statement() || theme());
}

bool ThemeLexer::theme()
{
	if(match("theme"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			do
			{
				getNextToken(false);
				
			} while(statement() || isVariable() || titlebar_color_from() ||
			        titlebar_color_to() || menu_color_from() || menu_color_to() || 
				menu_select_color_from() || menu_select_color_to() || 
				toolbar_color_from() || toolbar_color_to() || root_color() || 
				titlebar_button_color_from() || titlebar_button_color_to() || 
				show_root_color() || root_color_to() || root_color_from() || 
				set_root_hgradient() || set_root_vgradient() || set_toolbar_location_top() 
				|| execute());
			
			if(match("}"))
			{
				// we only want 1 theme to be defined if there is more they are ignored.
				return false;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

// Parses comments
bool ThemeLexer::comments()
{	
	// Comments start with a # and end with a # they can be multiple lines.
	if(match("#"))
	{
		while(1)
		{
			getNextToken(false);
		
			if(match("#"))
			{
				return true;
			} else {
			
				if(feof(inputfile))
				{
					expected("#");
				}
			}
		}
	}
	
	// Experimental below - it doesn't work like I want it too.
	
	// /* comments */
	//
	// Comments resemble the standard C comments. They can be single line or multi-line. 
	// they can also be nested.

	
	/*if(match("/"))
	{	
		getNextToken(false);
		
		if(match("*"))
		{

			if(match("*"))
			{
				while(1)
				{
					getNextToken(false);
					
					comments(); // accept nested comments
					
					if(match("*"))
					{	
						
						getNextToken(false);

						if(match("/"))
						{
							return true;
							break;
						} 
			
						if(feof(inputfile)){
							expected("/");	
						}
					} 

					if(feof(inputfile)){
						expected("*");	
					}
				}
			} 
		} else {
			expected("*");
		}
	}*/

	return false;
}

//--------------------------------------------------------------------------------------------//
// Parses quoted text 
//--------------------------------------------------------------------------------------------//
bool ThemeLexer::quoted_text()
{
	// "text..."
	//	

	// This function will work with the following escape sequence for putting quotes in you text
	//
	// escape sequence \" is used to add a quote to the text
	int holdLine=0;

	char *string = new char[MaxBufferSize];
	strcpy(string, "");
	
	if(match("\""))
	{
		holdLine=currentLine;
	
		do
		{
			getNextToken(true);
		
			if(feof(inputfile))
			{
				currentLine=holdLine+1;
				expected("\"");
			}
		
			// Check if there is an escape sequence of /" for quote.
			if(match("/"))
			{
				getNextToken(true);
				if(match("\""))
				{
					strcat(string, "\"");
					getNextToken(true);
				} else {
					strcat(string, "/");
				}
			} else {
				if(match("\""))
				{
					return true;
				}
			}
					
	
		// This concats the string and places it in variable called temp.
		// if you intended to use the string the contents of quoted_text would
		// be found in the variable temp which is global to the ThemeLexer class.
		strcat(string, current_token); 
		strcpy(temp, string);
		
		} while(!match("\""));

	delete [] string;
				
	return true;
	
	} else {
		
		return false;
	}

	return false;
}

bool ThemeLexer::r() 
{
	if(match("r"))
	{
		getNextToken(false);
	
		if(match("="))
		{
			getNextToken(false);
			
			quoted_text();
			
			// do something with value
			color_temp.r = atoi(temp);

			return true;
		} else {
			expected("=");
		}

	} 
	
	return false;
}

bool ThemeLexer::g() 
{
	if(match("g"))
	{
		getNextToken(false);
	
		if(match("="))
		{
			getNextToken(false);
			
			quoted_text();
			
			// do something with value
			color_temp.g = atoi(temp);
			
			return true;
		} else {
			expected("=");
		}

	} 
	
	return false;

}

bool ThemeLexer::b() 
{
	if(match("b"))
	{
		getNextToken(false);
	
		if(match("="))
		{
			getNextToken(false);
			
			quoted_text();
			
			// do something with value
			color_temp.b = atoi(temp);
			
			return true;
		} else {
			expected("=");
		}

	} 
	
	return false;

}

bool ThemeLexer::a() 
{
	if(match("a"))
	{
		getNextToken(false);
	
		if(match("="))
		{
			getNextToken(false);
			
			quoted_text();
			
			// do something with value
			color_temp.a = atoi(temp);

			return true;
		} else {
			expected("=");
		}

	} 
	
	return false;

}

//--------------------------------------------------------------------------------------------//
// Parses variables : var name = quoted_text 
//
// Used to store information that can be used in many places.
//--------------------------------------------------------------------------------------------//
bool ThemeLexer::variable()
{
	char* name;

	if(match("var"))
	{
		getNextToken(false);
	
		name = current_token;

		getNextToken(false);
	
		if(match("="))
		{
			getNextToken(false);
			
			quoted_text();
			
			Data *d = new Data();
			
			strcpy (d->name, name);
			strcpy (d->data, temp);
			
			variables->insert(d);

			return true;
		} else {
			expected("=");
		}

	} 
	
	return false;
}

bool ThemeLexer::titlebar_color_from()
{
	if(match("titlebarColorFrom"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
				
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{							
				wm->theme->setClientColorFrom(color_temp);
				
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}	
		
	return false;
}

bool ThemeLexer::titlebar_color_to()
{
	if(match("titlebarColorTo"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setClientColorTo(color_temp);
												
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::menu_color_from()
{
	if(match("menuColorFrom"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setMenuColorFrom(color_temp);
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::menu_color_to()
{
	if(match("menuColorTo"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setMenuColorTo(color_temp);				
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::menu_select_color_from()
{
	if(match("menuSelectColorFrom"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setMenuSelectColorFrom(color_temp);				
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::menu_select_color_to()
{
	if(match("menuSelectColorTo"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setMenuSelectColorTo(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::toolbar_color_from()
{
	if(match("toolbarColorFrom"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setToolbarColorFrom(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::toolbar_color_to()
{
	if(match("toolbarColorTo"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setToolbarColorTo(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::titlebar_button_color_from()
{
	if(match("titlebarButtonColorFrom"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setTitlebarButtonColorFrom(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::titlebar_button_color_to()
{
	if(match("titlebarButtonColorTo"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setTitlebarButtonColorTo(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::root_color_from()
{
	if(match("rootColorFrom"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setRootColorFrom(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::root_color_to()
{
	if(match("rootColorTo"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setRootColorTo(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}


bool ThemeLexer::root_color()
{
	if(match("rootColor"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(statement() || isVariable() || r() || g() || b() || a());
			
			if(match("}"))
			{
				wm->theme->setRootColor(color_temp);								
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

bool ThemeLexer::show_root_color()
{
	if(match("showRootBackgroundColor"))
	{	
		BImage rootImage(wm->xres, wm->yres, DefaultDepth(wm->dpy, wm->screen),
			wm->theme->getRootColor());

		rootImage.setBackgroundColor(wm->theme->getRootColor());
		
		root_pix = rootImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
			wm->root);			
		XSetWindowBackgroundPixmap(wm->dpy, wm->root, root_pix);
		XClearWindow(wm->dpy, wm->root);
		
		return true;
	} 
		
	return false;
}

bool ThemeLexer::set_root_vgradient()
{
	if(match("setRootHorizontalGradient"))
	{	
		BImage rootImage(wm->xres, wm->yres, DefaultDepth(wm->dpy, wm->screen),
			wm->theme->getRootColorFrom());
		rootImage.renderHGradient(wm->theme->getRootColorFrom(), wm->theme->getRootColorTo());
		
		root_pix = rootImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
			wm->root);			
		XSetWindowBackgroundPixmap(wm->dpy, wm->root, root_pix);
		XClearWindow(wm->dpy, wm->root);
		
		return true;
	} 
		
	return false;

}

bool ThemeLexer::set_root_hgradient()
{
	if(match("setRootVerticalGradient"))
	{	
		BImage rootImage(wm->xres, wm->yres, DefaultDepth(wm->dpy, wm->screen),
			wm->theme->getRootColorFrom());
		rootImage.renderVGradient(wm->theme->getRootColorFrom(), wm->theme->getRootColorTo());
		
		root_pix = rootImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
			wm->root);			
		XSetWindowBackgroundPixmap(wm->dpy, wm->root, root_pix);
		XClearWindow(wm->dpy, wm->root);

		
		return true;
	} 
			
	return false;
}

bool ThemeLexer::set_toolbar_location_top()
{
	if(match("setToolbarLocationTop"))
	{	
		wm->theme->setToolbarLocationTop(true);
 		return true;
 	}
}

bool ThemeLexer::execute()
{
	if(match("execute"))
	{
		getNextToken(false);
				
		if(match("{"))
		{
			
			do
			{
				getNextToken(false);
					
			} while(quoted_text() || isVariable());
			
			if(match("}"))
			{
				char command[strlen(temp)+8];
				sprintf(command, "exec %s &", temp);
				system(command);				
				
				return true;
				
			} else {
				expected("}");
			}

		} else {
			expected("{");
			
		}
	} else {
		return false;
	}
		
	return false;
}

//--------------------------------------------------------------------------------------------//
// Parses legal statements
//--------------------------------------------------------------------------------------------//
bool ThemeLexer::statement()
{
	// If there were more language statements they would go here.....
	
	if(comments())              return true;
	if(variable())              return true;
	
	// Its not a statement 
	return false;
}

//--------------------------------------------------------------------------------------------//
// Test the current token to see if it is a variable if it is then it writes the proper code
// to the HTML file.
//--------------------------------------------------------------------------------------------//
bool ThemeLexer::isVariable()
{
	LinkedListIterator<Data> it(variables);
	
	for(; it.current(); it++)
	{
		if(match(it.current()->name))
		{
			return true;				
		} 
	}
	
	return false;
}

//-------------------------------------------------------------------------------------------//
// Matches a token by getting the next token and comparing it to the 
// match. If it matches it returns true otherwise it returns false
//--------------------------------------------------------------------------------------------//
bool ThemeLexer::match(char *tok)
{
	if(strcmp(current_token, tok)==0)
	{
		//printf("found a match for: %s\n", tok);
		return true; // found a match
	} else {
		return false; // no match
	}
}

//--------------------------------------------------------------------------------------------//
// If you expected a token but didn't get it then this function will
// print a error message in the form of 
// 
//--------------------------------------------------------------------------------------------//
void ThemeLexer::expected(char *err)
{
	// Not meant to be perfect!!!!
	
	printf("\nexpected: %s\n", err);
	printf("theme file has an error on line: %d\n", currentLine);
	
}

