// toolbar.cc
//
// Frank Hale 
// frankhale@yahoo.com
#include "sapphire.hh"

Toolbar::Toolbar()
{
	month_abbreviation[0] = "Jan";
	month_abbreviation[1] = "Feb";
	month_abbreviation[2] =	"Mar";
	month_abbreviation[3] =	"Apr";
	month_abbreviation[4] =	 "May";
	month_abbreviation[5] =	 "Jun";
	month_abbreviation[6] =	 "Jul";
	month_abbreviation[7] =	 "Aug";
	month_abbreviation[8] =	 "Sep";
	month_abbreviation[9] =	 "Oct";
	month_abbreviation[10] = "Nov";
	month_abbreviation[11] = "Dec";

  	XSetWindowAttributes attrib;
  	unsigned long create_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
    		CWOverrideRedirect | CWEventMask;
  	attrib.background_pixmap = None;
  	attrib.background_pixel = wm->bg.pixel;
  	attrib.border_pixel = wm->bd.pixel;
  	attrib.override_redirect = True;
  	attrib.event_mask = StructureNotifyMask | ButtonPressMask | ButtonReleaseMask | ExposureMask;

	// check to see if toolbar is to be at top or bottom of the screen
  	// then place toolbar in correct location
	if (wm->theme->getToolbarLocationTop())
	{
  		frame.width = wm->xres - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = 0;
  	} else {
  		frame.width = wm->xres - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = wm->yres - frame.height;
	}
  
  	frame.window =
    		XCreateWindow(wm->dpy, wm->root, frame.x, frame.y,
		  frame.width, frame.height, 1, DefaultDepth(wm->dpy, wm->screen),
		  InputOutput, DefaultVisual(wm->dpy, wm->screen), create_mask, &attrib);

	BImage image(frame.width, frame.height, DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getToolbarColorTo());

	image.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());		
	image.renderBevel();
	frame_pix = image.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
		frame.window);			
	XSetWindowBackgroundPixmap(wm->dpy, frame.window, frame_pix);

	// Clock window --------------------------------------------------------------------
  	clock.x = wm->xres - 145;
  	clock.y =  4;
  	clock.width = 125;
  	clock.height = 15;

  	clock.window =  XCreateWindow(wm->dpy, frame.window, clock.x, clock.y,
		  clock.width, clock.height, 1, DefaultDepth(wm->dpy, wm->screen),
		  InputOutput, DefaultVisual(wm->dpy, wm->screen), create_mask, &attrib);
 
 	BImage clockImage(clock.width, clock.height, DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getToolbarColorTo());
	clockImage.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());
	clock_pix = clockImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
		clock.window);	
	
        XSetWindowBackgroundPixmap(wm->dpy, clock.window, clock_pix);
	// End Clock window -----------------------------------------------------------------

  	
	// Close Button ---------------------------------------------------------------------
  	close.x = 10;
	close.y = 4; 
	close.width = 15;
  	close.height = 15;

  	close.window =
    		 XCreateWindow(wm->dpy, frame.window, close.x, close.y,
		  close.width, close.height, 1, DefaultDepth(wm->dpy, wm->screen),
		  InputOutput, DefaultVisual(wm->dpy, wm->screen), create_mask, &attrib);

  	BImage buttonImage(close.width, close.height, DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getTitleBarButtonColorTo());	
		
  	buttonImage.renderHGradient(wm->theme->getTitleBarButtonColorFrom(),wm->theme->getTitleBarButtonColorTo());
  	buttonImage.renderBevel();
	
  	button_pix = buttonImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
  		Button.window);

  	XSetWindowBackgroundPixmap(wm->dpy, close.window, button_pix);	
	// End Close Button -----------------------------------------------------------------

	// cycleUp Button ---------------------------------------------------------------------
  	cycleUp.x = 36;
	cycleUp.y = 4;
	cycleUp.width = 15;
  	cycleUp.height = 15;

  	cycleUp.window =
    		XCreateWindow(wm->dpy, frame.window, cycleUp.x, cycleUp.y,
		  cycleUp.width, cycleUp.height, 1, DefaultDepth(wm->dpy, wm->screen),
		  InputOutput, DefaultVisual(wm->dpy, wm->screen), create_mask, &attrib);

	XSetWindowBackgroundPixmap(wm->dpy, cycleUp.window, button_pix);	
	// End cycleUp Button -----------------------------------------------------------------

	// cycleDown Button ---------------------------------------------------------------------
  	cycleDown.x = 54;
	cycleDown.y = 4;
	cycleDown.width = 15;
  	cycleDown.height = 15;

  	cycleDown.window =
    		XCreateWindow(wm->dpy, frame.window, cycleDown.x, cycleDown.y,
		  cycleDown.width, cycleDown.height, 1, DefaultDepth(wm->dpy, wm->screen),
		  InputOutput, DefaultVisual(wm->dpy, wm->screen), create_mask, &attrib);

	XSetWindowBackgroundPixmap(wm->dpy, cycleDown.window, button_pix);	
	// End cycleDown Button -----------------------------------------------------------------	
   	
	XMapSubwindows(wm->dpy, frame.window);
	XMapWindow(wm->dpy, frame.window);
}

Toolbar::~Toolbar()
{
	XFreePixmap(wm->dpy, clock_pix);
	XFreePixmap(wm->dpy, frame_pix);
	XFreePixmap(wm->dpy, button_pix);
}

void Toolbar::reconfigure()
{
	BImage image(frame.width, frame.height, DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getToolbarColorTo());

	image.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());		
	frame_pix = image.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
		frame.window);			
	
	XSetWindowBackgroundPixmap(wm->dpy, frame.window, frame_pix);
	
	BImage buttonImage(close.width, close.height, DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getTitleBarButtonColorTo());	
		
	buttonImage.renderHGradient(wm->theme->getTitleBarButtonColorFrom(),wm->theme->getTitleBarButtonColorTo());
	buttonImage.renderBevel();
	
        button_pix = buttonImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
        close.window);
	XSetWindowBackgroundPixmap(wm->dpy, close.window, button_pix);
	
	image.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());

 	BImage clockImage(clock.width, clock.height, DefaultDepth(wm->dpy, wm->screen),
		wm->theme->getToolbarColorTo());
	clockImage.renderVGradient(wm->theme->getToolbarColorFrom(), wm->theme->getToolbarColorTo());
	clock_pix = clockImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
		clock.window);	
        XSetWindowBackgroundPixmap(wm->dpy, clock.window, clock_pix);

	XSetWindowBackgroundPixmap(wm->dpy, cycleUp.window, button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, cycleDown.window, button_pix);
	
	XClearWindow(wm->dpy, frame.window);

	// check to see if toolbar is to be at top or bottom of the screen
  	// then place toolbar in correct location
	if (wm->theme->getToolbarLocationTop())
	{
  		frame.width = wm->xres - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = 0;
		
		XMoveResizeWindow(wm->dpy, frame.window, frame.x, frame.y , frame.width, frame.height);
  	} else {
  		frame.width = wm->xres - 1;
  		frame.height = 25;
  		frame.x = 0;
  		frame.y = wm->yres - frame.height;
		
		XMoveResizeWindow(wm->dpy, frame.window, frame.x, frame.y , frame.width, frame.height);
	}
	
	checkClock(True);
}

void Toolbar::do_button_press_event(XButtonEvent *e)
{	
	wm->Grab();

	switch (e->button) {
			
		case Button1:
		if(close.window == e->window)
		{
			XSetWindowBorderWidth(wm->dpy, close.window, 2);
		} else if(cycleUp.window == e->window)
		{
			XSetWindowBorderWidth(wm->dpy, cycleUp.window, 2);
		} else if(cycleDown.window == e->window)
		{
			XSetWindowBorderWidth(wm->dpy, cycleDown.window, 2);
		}
		break;
	
		case Button2:
		{ 
		}
			
		break;
				
		case Button3:
		{
		}
		break;
	}
	
	wm->Ungrab();
}

void Toolbar::do_button_release_event(XButtonEvent *e)
{
	switch (e->button) {
			
		case Button1:
		if(close.window == e->window)
		{
			XSetWindowBorderWidth(wm->dpy, close.window, 1);
			wm->quit_nicely();
		} else if(cycleUp.window == e->window)
		{
			XSetWindowBorderWidth(wm->dpy, cycleUp.window, 1);
			XCirculateSubwindowsUp(wm->dpy, RootWindow(wm->dpy, wm->screen));
		} else if(cycleDown.window == e->window)
		{
			XSetWindowBorderWidth(wm->dpy, cycleDown.window, 1);
			XCirculateSubwindowsDown(wm->dpy, RootWindow(wm->dpy, wm->screen));
		}
		break;
	
		case Button2:
		{ 
		}
			
		break;
				
		case Button3:
		{
		}
		break;
	}
}

void Toolbar::redraw(bool state)
{
	BImage buttonImage(close.width, close.height, DefaultDepth(wm->dpy, wm->screen),
	wm->theme->getTitleBarButtonColorTo());	
		
	buttonImage.renderHGradient(wm->theme->getTitleBarButtonColorFrom(),wm->theme->getTitleBarButtonColorTo());
	buttonImage.renderBevel();
	
        button_pix = buttonImage.convertToPixmap(wm->dpy, DefaultVisual(wm->dpy, wm->screen),
        close.window);

	
	XClearWindow(wm->dpy, close.window);
	
	XDrawLine(wm->dpy, close.window, wm->border_gc, 2,2,close.width - 3,close.height-3);
	XDrawLine(wm->dpy, close.window, wm->border_gc, 2,close.height - 3,close.width-3,2);

	int hh = cycleUp.height / 2, hw = cycleUp.width / 2;

	XClearWindow(wm->dpy, cycleUp.window);
	XClearWindow(wm->dpy, cycleDown.window);

	XPoint cycleDownpts[3];
	cycleDownpts[0].x = hw + 3; 
	cycleDownpts[0].y = hh -2;
	cycleDownpts[1].x = -3; 
	cycleDownpts[1].y = 6;
	cycleDownpts[2].x = -3; 
	cycleDownpts[2].y = -6;
		
		
	XFillPolygon(wm->dpy, cycleDown.window, wm->border_gc, cycleDownpts, 3, Convex,
	       CoordModePrevious);	
	

	XPoint cycleUppts[3];
	cycleUppts[0].x = hw - 4; 
	cycleUppts[0].y = hh + 4;
	cycleUppts[1].x = 7; 
	cycleUppts[1].y = 0;
	cycleUppts[2].x = -3; 
	cycleUppts[2].y = -7;
    
	XFillPolygon(wm->dpy, cycleUp.window, wm->border_gc, cycleUppts, 3, Convex,
	       CoordModePrevious);		
	
	XSetWindowBackgroundPixmap(wm->dpy, cycleUp.window, button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, cycleDown.window, button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, close.window, button_pix);
	XSetWindowBackgroundPixmap(wm->dpy, frame.window, frame_pix);
	
	checkClock(True);
}

void Toolbar::handle_expose_event(XExposeEvent *e)
{
	if(e->count == 0) redraw(False);
}

void Toolbar::checkClock(bool redraw)
{
  static int hour, minute, day, month, year;
  time_t tmp;
  struct tm *tt;
  
  if ((tmp = time(NULL)) != -1) {
    tt = localtime(&tmp);
    if (tt->tm_min != minute || tt->tm_hour != hour || day != tt->tm_mday ||
    	month != tt->tm_mon || year != tt->tm_year) {

      hour = tt->tm_hour;
      minute = tt->tm_min;
      day = tt->tm_mday;
      month = tt->tm_mon;
      year = tt->tm_year;
      XClearWindow(wm->dpy, clock.window);
      redraw = True;
    }
  }
  
  if (redraw) {
    char t[18];
    sprintf(t, "%02d:%02d %cm %02d %s %02d",
            ((hour > 12) ? hour - 12 : ((hour == 0) ? 12 : hour)), minute,
	    ((hour >= 12) ? 'p' : 'a'), day, month_abbreviation[month], year);
	  
    int len = strlen(t);
	    
    XDrawString(wm->dpy, clock.window, wm->string_gc,
		5, wm->font->ascent, t, len);  
  }
}
