#include "sapphire.hh"

//----------------------------------------------------------------------------//
// Sets up variables so we can get ready to use them
//----------------------------------------------------------------------------//
WMLexer::WMLexer(char *fileName) : Scanner(fileName) 
{
	temp = new char[MaxBufferSize];
	current_token = new char[MaxBufferSize];

	error=False;

	strcpy(temp, "");
	strcpy(current_token, "");
}

WMLexer::~WMLexer() { 
		delete [] temp; 
		delete [] current_token;
}

//--------------------------------------------------------------------------------------------//
// Preforms WMLexerical Analysis 
// This represents the top level of the configuration file
//--------------------------------------------------------------------------------------------//
void WMLexer::analyze()
{
	do {

		getNextToken(False);		
		
	} while(comments() || startUpTheme());
}

bool WMLexer::startUpTheme()
{
	if(match("startUpTheme"))
	{
		getNextToken(False);
				
		if(match("{"))
		{
			getNextToken(False);
			
			quoted_text();
			
			getNextToken(False);
			
			if(match("}"))
			{
				// Set the theme here
				wm->setTheme(temp);
										
				// we only want 1 menu to be defined if there is more they are ignored.
				return False;
				
			} else {
				expected("}");				
			}

		} else {
			expected("{");
			
		}
	} else {
		expected("startUpTheme");
	}
		
	return False;
}

// Parses comments
bool WMLexer::comments()
{
	// Comments start with a # and end with a # they can be multiple lines.
	if(match("#"))
	{
		while(1)
		{
			getNextToken(false);
		
			if(match("#"))
			{
				return true;
			} 
		}
	}
	
	return False;
}

//--------------------------------------------------------------------------------------------//
// Parses quoted text 
//--------------------------------------------------------------------------------------------//
bool WMLexer::quoted_text()
{
	// "text..."
	//	

	// This function will work with the following escape sequence for putting quotes in you text
	//
	// escape sequence \" is used to add a quote to the text
	int holdLine=0;

	char *string = new char[MaxBufferSize];
	strcpy(string, "");
	
	if(match("\""))
	{
	
		holdLine=currentLine;
	
		do
		{
			getNextToken(True);
		
			if(feof(inputfile))
			{
				currentLine=holdLine+1;
				expected("\"");
			}
		
			// Check if there is an escape sequence of /" for quote.
			if(match("/"))
			{
				getNextToken(True);
				if(match("\""))
				{
					strcat(string, "\"");
					getNextToken(True);
				} else {
					strcat(string, "/");
				}
			} else {
				if(match("\""))
				{
					return True;
				}
			}
					
	
		// This concats the string and places it in variable called temp.
		// if you intended to use the string the contents of quoted_text would
		// be found in the variable temp which is global to the WMLexer class.
		strcat(string, current_token); 
		strcpy(temp, string);
		
		} while(!match("\""));

	delete [] string;
				
	return True;
	
	} else {
		return False;
	}

	return False;
}

//-------------------------------------------------------------------------------------------//
// Matches a token by getting the next token and comparing it to the 
// match. If it matches it returns True otherwise it returns False
//--------------------------------------------------------------------------------------------//
bool WMLexer::match(char *tok)
{
	if(strcmp(current_token, tok)==0)
	{
		return True; // found a match
	} else {
		return False; // no match
	}
}

//--------------------------------------------------------------------------------------------//
// If you expected a token but didn't get it then this function will
// print a error message in the form of 
// 
//--------------------------------------------------------------------------------------------//
void WMLexer::expected(char *err)
{
	// Not meant to be perfect!!!!
	if(! error) {
		printf("\nexpected: %s\n", err);
		printf("wmconfig file has an error on line: %d\n", currentLine);
	}
	
	error=True; 	
}
