/* copy.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999 David Odin <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include "giram.h"
#include "object.h"
#include "copy.h"

/*****************************************************************************
*  CopyObject
******************************************************************************/
ObjectStruct *CopyObject(ObjectStruct *OrgObject)
{
  ObjectStruct *LocalObject;

  if (OrgObject)
  {
    switch (OrgObject->Type)
    {
      case BICUBIC_PATCH_OBJECT:
        LocalObject = g_malloc(sizeof(BicubicPatchStruct));
        {
/*          BicubicPatchStruct *BBicubicPatch = (BicubicPatchStruct *)LocalObject;
FIXME          BicubicPatchStruct *OrgBicubicPatch = (BicubicPatchStruct *)OrgObject;
*/
        }
        break;
      case BLOB_OBJECT:
        LocalObject = g_malloc(sizeof(BlobStruct));
        {
/*          BlobStruct *BBlob = (BlobStruct *)LocalObject;
FIXME          BlobStruct *OrgBlob = (BlobStruct *)OrgObject;
*/
        }
        break;
      case BOX_OBJECT:
        LocalObject = g_malloc(sizeof(BoxStruct));
        {
          BoxStruct *BBox = (BoxStruct *)LocalObject;
          BoxStruct *OrgBox = (BoxStruct *)OrgObject;

          BBox->MinCorner[0] = OrgBox->MinCorner[0];
          BBox->MinCorner[1] = OrgBox->MinCorner[1];
          BBox->MinCorner[2] = OrgBox->MinCorner[2];
          BBox->MaxCorner[0] = OrgBox->MaxCorner[0];
          BBox->MaxCorner[1] = OrgBox->MaxCorner[1];
          BBox->MaxCorner[2] = OrgBox->MaxCorner[2];
        }
        break;
      case CONE_OBJECT:
        LocalObject = g_malloc(sizeof(ConeStruct));
        {
          ConeStruct *CCone = (ConeStruct *)LocalObject;
          ConeStruct *OrgCone = (ConeStruct *)OrgObject;

          CCone->Open = OrgCone->Open;
          CCone->Radius1 = OrgCone->Radius1;
          CCone->Radius2 = OrgCone->Radius2;
        }
        break;
      case CSG_OBJECT:
        LocalObject = g_malloc(sizeof(CSGStruct));
        {
          CSGStruct *CCSG = (CSGStruct *)LocalObject;
          CSGStruct *OrgCSG = (CSGStruct *)OrgObject;
          ObjectStruct *Object1, *Object2;

          CCSG->Type = OrgCSG->Type;
          CCSG->FirstObject = CopyObject(OrgCSG->FirstObject);
          Object1 = CCSG->FirstObject;
          Object2 = OrgCSG->FirstObject;
          while (Object2)
          {
            Object1->Next = CopyObject(Object2->Next);
            Object1 = Object1->Next;
            Object2 = Object2->Next;
          }
        }
        break;
      case CUBIC_OBJECT:
        LocalObject = g_malloc(sizeof(CubicStruct));
        {
          CubicStruct *CCubic = (CubicStruct *)LocalObject;
          CubicStruct *OrgCubic = (CubicStruct *)OrgObject;
          int i;

          for (i=0 ; i<20 ; i++)
            CCubic->T[i] = OrgCubic->T[i];
        }
        break;
      case CYLINDER_OBJECT:
        LocalObject = g_malloc(sizeof(CylinderStruct));
        {
          CylinderStruct *CCylinder = (CylinderStruct *)LocalObject;
          CylinderStruct *OrgCylinder = (CylinderStruct *)OrgObject;

          CCylinder->Open = OrgCylinder->Open;
        }
        break;
      case DISC_OBJECT:
        LocalObject = g_malloc(sizeof(DiscStruct));
        {
          DiscStruct *DDisc = (DiscStruct *)LocalObject;
          DiscStruct *OrgDisc = (DiscStruct *)OrgObject;

          DDisc->Radius = OrgDisc->Radius;
          DDisc->HoleRadius = DDisc->HoleRadius;
        }
        break;
      case HEIGHT_FIELD_OBJECT:
        LocalObject = g_malloc(sizeof(HeightFieldStruct));
        {
          HeightFieldStruct *HHeightField = (HeightFieldStruct *)LocalObject;
          HeightFieldStruct *OrgHeightField = (HeightFieldStruct *)OrgObject;

          HHeightField->FileType = OrgHeightField->FileType;
          HHeightField->Hierarchy = OrgHeightField->Hierarchy;
          HHeightField->Smooth = OrgHeightField->Smooth;
          HHeightField->WaterLevel = OrgHeightField->WaterLevel;
        }
        break;
      case JULIA_FRACTAL_OBJECT:
        LocalObject = g_malloc(sizeof(JuliaFractalStruct));
        {
/*          JuliaFractalStruct *JJuliaFractal = (JuliaFractalStruct *)LocalObject;
FIXME          JuliaFractalStruct *OrgJuliaFractal = (JuliaFractalStruct *)OrgObject;
*/
        }
        break;
      case LATHE_OBJECT:
        LocalObject = g_malloc(sizeof(LatheStruct));
        {
/*          LatheStruct *LLathe = (LatheStruct *)LocalObject;
FIXME          LatheStruct *OrgLathe = (LatheStruct *)OrgObject;
*/
        }
        break;
      case MESH_OBJECT:
        LocalObject = g_malloc(sizeof(MeshStruct));
        {
          MeshStruct *MMesh = (MeshStruct *)LocalObject;
          MeshStruct *OrgMesh = (MeshStruct *)OrgObject;
          TriangleListStruct *TmpTri, *NewTri;

          MMesh->FirstTriangle = NULL;
          for (TmpTri = OrgMesh->FirstTriangle ; TmpTri ; TmpTri=TmpTri->Next)
          {
            NewTri = g_malloc(sizeof(TriangleListStruct));
            NewTri->Next = MMesh->FirstTriangle;
            MMesh->FirstTriangle = NewTri;
            NewTri->P1[0] = TmpTri->P1[0]; NewTri->N1[0] = TmpTri->N1[0];
            NewTri->P1[1] = TmpTri->P1[1]; NewTri->N1[1] = TmpTri->N1[1];
            NewTri->P1[2] = TmpTri->P1[2]; NewTri->N1[2] = TmpTri->N1[2];
            NewTri->P2[0] = TmpTri->P2[0]; NewTri->N2[0] = TmpTri->N2[0];
            NewTri->P2[1] = TmpTri->P2[1]; NewTri->N2[1] = TmpTri->N2[1];
            NewTri->P2[2] = TmpTri->P2[2]; NewTri->N2[2] = TmpTri->N2[2];
            NewTri->P3[0] = TmpTri->P3[0]; NewTri->N3[0] = TmpTri->N3[0];
            NewTri->P3[1] = TmpTri->P3[1]; NewTri->N3[1] = TmpTri->N3[1];
            NewTri->P3[2] = TmpTri->P3[2]; NewTri->N3[2] = TmpTri->N3[2];
          }
        }
        break;
      case PLANE_OBJECT:
        LocalObject = g_malloc(sizeof(PlaneStruct));
        {
          PlaneStruct *PPlane = (PlaneStruct *)LocalObject;
          PlaneStruct *OrgPlane = (PlaneStruct *)OrgObject;

          PPlane->Normal[0] = OrgPlane->Normal[0];
          PPlane->Normal[1] = OrgPlane->Normal[1];
          PPlane->Normal[2] = OrgPlane->Normal[2];
          PPlane->Distance = OrgPlane->Distance;
          PPlane->NbSquare = OrgPlane->NbSquare;
          PPlane->SquareSize = OrgPlane->SquareSize;
        }
        break;
      case POLY_OBJECT:
        LocalObject = g_malloc(sizeof(PolyStruct));
        {
/*          PolyStruct *PPoly = (PolyStruct *)LocalObject;
FIXME          PolyStruct *OrgPoly = (PolyStruct *)OrgObject;
*/
        }
        break;
      case POLYGON_OBJECT:
        LocalObject = g_malloc(sizeof(PolygonStruct));
        {
/*          PolygonStruct *PPolygon = (PolygonStruct *)LocalObject;
FIXME          PolygonStruct *OrgPolygon = (PolygonStruct *)OrgObject;
*/
        }
        break;
      case PRISM_OBJECT:
        LocalObject = g_malloc(sizeof(PrismStruct));
        {
/*          PrismStruct *PPrism = (PrismStruct *)LocalObject;
FIXME          PrismStruct *OrgPrism = (PrismStruct *)OrgObject;
*/
        }
        break;
      case QUADRIC_OBJECT:
        LocalObject = g_malloc(sizeof(QuadricStruct));
        {
          QuadricStruct *QQuadric = (QuadricStruct *)LocalObject;
          QuadricStruct *OrgQuadric = (QuadricStruct *)OrgObject;

          QQuadric->A = OrgQuadric->A;
          QQuadric->B = OrgQuadric->B;
          QQuadric->C = OrgQuadric->C;
          QQuadric->D = OrgQuadric->D;
          QQuadric->E = OrgQuadric->E;
          QQuadric->F = OrgQuadric->F;
          QQuadric->G = OrgQuadric->G;
          QQuadric->H = OrgQuadric->H;
          QQuadric->I = OrgQuadric->I;
          QQuadric->J = OrgQuadric->J;
        }
        break;
      case QUARTIC_OBJECT:
        LocalObject = g_malloc(sizeof(QuarticStruct));
        {
          QuarticStruct *QQuartic = (QuarticStruct *)LocalObject;
          QuarticStruct *OrgQuartic = (QuarticStruct *)OrgObject;
          int i;

          for (i=0 ; i<35 ; i++)
            QQuartic->T[i] = OrgQuartic->T[i];
        }
        break;
      case SOR_OBJECT:
        LocalObject = g_malloc(sizeof(SorStruct));
        {
          SorStruct *SSor = (SorStruct *)LocalObject;
          SorStruct *OrgSor = (SorStruct *)OrgObject;
          int i;

          SSor->NumberOfPoints = OrgSor->NumberOfPoints;
          SSor->Point = g_malloc(SSor->NumberOfPoints*sizeof(Vector));
          for (i=0 ; i<SSor->NumberOfPoints ; i++)
          {
            SSor->Point[i][0] = OrgSor->Point[i][0];
            SSor->Point[i][1] = OrgSor->Point[i][1];
          }
        }
        break;
      case SPHERE_OBJECT:
        LocalObject = g_malloc(sizeof(SphereStruct));
        {
          SphereStruct *SSphere = (SphereStruct *)LocalObject;
          SphereStruct *OrgSphere = (SphereStruct *)OrgObject;

          SSphere->Center[0] = OrgSphere->Center[0];
          SSphere->Center[1] = OrgSphere->Center[1];
          SSphere->Center[2] = OrgSphere->Center[2];
          SSphere->Radius = OrgSphere->Radius;
        }
        break;
      case SUPERELLIPSOID_OBJECT:
        LocalObject = g_malloc(sizeof(SuperEllipsoidStruct));
        {
          SuperEllipsoidStruct *SSuperEllipsoid = (SuperEllipsoidStruct *)LocalObject;
          SuperEllipsoidStruct *OrgSuperEllipsoid = (SuperEllipsoidStruct *)OrgObject;

          SSuperEllipsoid->E = OrgSuperEllipsoid->E;
          SSuperEllipsoid->N = OrgSuperEllipsoid->N;
        }
        break;
      case TEXT_OBJECT:
        LocalObject = g_malloc(sizeof(TextStruct));
        {
/*          TextStruct *TText = (TextStruct *)LocalObject;
FIXME          TextStruct *OrgText = (TextStruct *)OrgObject;
*/
        }
        break;
      case TORUS_OBJECT:
        LocalObject = g_malloc(sizeof(TorusStruct));
        {
          TorusStruct *TTorus = (TorusStruct *)LocalObject;
          TorusStruct *OrgTorus = (TorusStruct *)OrgObject;

          TTorus->Major = OrgTorus->Major;
          TTorus->Minor = OrgTorus->Minor;
        }
        break;
      case TRIANGLE_OBJECT:
        LocalObject = g_malloc(sizeof(TriangleStruct));
        {
          TriangleStruct *TTriangle = (TriangleStruct *)LocalObject;
          TriangleStruct *OrgTriangle = (TriangleStruct *)OrgObject;

          TTriangle->P1[0] = OrgTriangle->P1[0];
          TTriangle->P1[1] = OrgTriangle->P1[1];
          TTriangle->P1[2] = OrgTriangle->P1[2];
          TTriangle->P2[0] = OrgTriangle->P2[0];
          TTriangle->P2[1] = OrgTriangle->P2[1];
          TTriangle->P2[2] = OrgTriangle->P2[2];
          TTriangle->P3[0] = OrgTriangle->P3[0];
          TTriangle->P3[1] = OrgTriangle->P3[1];
          TTriangle->P3[2] = OrgTriangle->P3[2];
          TTriangle->N1[0] = OrgTriangle->N1[0];
          TTriangle->N1[1] = OrgTriangle->N1[1];
          TTriangle->N1[2] = OrgTriangle->N1[2];
          TTriangle->N2[0] = OrgTriangle->N2[0];
          TTriangle->N2[1] = OrgTriangle->N2[1];
          TTriangle->N2[2] = OrgTriangle->N2[2];
          TTriangle->N3[0] = OrgTriangle->N3[0];
          TTriangle->N3[1] = OrgTriangle->N3[1];
          TTriangle->N3[2] = OrgTriangle->N3[2];
        }
        break;
      default:
        LocalObject = NULL;
        printf("Severe bug in CopyObject\n");
        exit(1);
    }
    LocalObject->Type = OrgObject->Type;
    LocalObject->FirstTriangle = NULL;
    LocalObject->Next = NULL;
    LocalObject->Trans = CopyTransform(OrgObject->Trans);
    LocalObject->BoundedBy = CopyObject(OrgObject->BoundedBy);
    LocalObject->ClippedBy = CopyObject(OrgObject->ClippedBy);
    LocalObject->Texture = CopyTexture(OrgObject->Texture);
    LocalObject->Inverse = OrgObject->Inverse;
    LocalObject->Sturm = OrgObject->Sturm;
    LocalObject->WaterLevel = OrgObject->WaterLevel;
    LocalObject->Smooth = OrgObject->Smooth;
    LocalObject->NoShadow = OrgObject->NoShadow;
    LocalObject->Hierarchy = OrgObject->Hierarchy;
    LocalObject->Hollow = OrgObject->Hollow;
    LocalObject->Selected = OrgObject->Selected;
    LocalObject->Visible = OrgObject->Visible;
    ComputeObjectTriangleMesh(LocalObject);
    return LocalObject;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyTexture
******************************************************************************/
TextureStruct *CopyTexture(TextureStruct *OrgTexture)
{
  TextureStruct *LocalTexture;

  if (OrgTexture)
  {
    LocalTexture = g_malloc(sizeof(TextureStruct));
    LocalTexture->Pigment = CopyPigment(OrgTexture->Pigment);
    LocalTexture->Normal = CopyNormal(OrgTexture->Normal);
    LocalTexture->Finish = CopyFinish(OrgTexture->Finish);
    LocalTexture->Halo = CopyHalo(OrgTexture->Halo);
    LocalTexture->Trans = CopyTransform(OrgTexture->Trans);
    LocalTexture->NextLayer = CopyTexture(OrgTexture->NextLayer);
    return LocalTexture;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyPigment
******************************************************************************/
PigmentStruct *CopyPigment(PigmentStruct *OrgPigment)
{
  PigmentStruct *LocalPigment;

  if (OrgPigment)
  {
    LocalPigment = g_malloc(sizeof(PigmentStruct));
    LocalPigment->Type = OrgPigment->Type;
    LocalPigment->Color[0] = OrgPigment->Color[0];
    LocalPigment->Color[1] = OrgPigment->Color[1];
    LocalPigment->Color[2] = OrgPigment->Color[2];
    LocalPigment->Color[3] = OrgPigment->Color[3];
    LocalPigment->Color[4] = OrgPigment->Color[4];
    LocalPigment->Trans = CopyTransform(OrgPigment->Trans);
    /* FIXME PigmentListStruct *PigmentList;*/
    LocalPigment->BrickSize[0] = OrgPigment->BrickSize[0];
    LocalPigment->BrickSize[1] = OrgPigment->BrickSize[1];
    LocalPigment->BrickSize[2] = OrgPigment->BrickSize[2];
    LocalPigment->Mortar = OrgPigment->Mortar;
    LocalPigment->AgateTurb = OrgPigment->AgateTurb;
    LocalPigment->Gradient[0] = OrgPigment->Gradient[0];
    LocalPigment->Gradient[1] = OrgPigment->Gradient[1];
    LocalPigment->Gradient[2] = OrgPigment->Gradient[2];
    LocalPigment->Mandel = OrgPigment->Mandel;
    LocalPigment->Control0 = OrgPigment->Control0;
    LocalPigment->Control1 = OrgPigment->Control1;
    LocalPigment->NumberOfArms = OrgPigment->NumberOfArms;
    LocalPigment->Frequency = OrgPigment->Frequency;
    LocalPigment->Phase = OrgPigment->Phase;
    LocalPigment->WaveForm = OrgPigment->WaveForm;
    LocalPigment->Turbulence[0] = OrgPigment->Turbulence[0];
    LocalPigment->Turbulence[1] = OrgPigment->Turbulence[1];
    LocalPigment->Turbulence[2] = OrgPigment->Turbulence[2];
    LocalPigment->Octaves = OrgPigment->Octaves;
    LocalPigment->Lambda = OrgPigment->Lambda;
    LocalPigment->Omega = OrgPigment->Omega;
    /* FIXME WarpStruct *Warp; */
    LocalPigment->ColorMap = CopyColorMap(OrgPigment->ColorMap);
    return LocalPigment;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyNormal
******************************************************************************/
NormalStruct *CopyNormal(NormalStruct *OrgNormal)
{
  NormalStruct *LocalNormal;

  if (OrgNormal)
  {
    LocalNormal = g_malloc(sizeof(NormalStruct));
    LocalNormal->Type = OrgNormal->Type;
    LocalNormal->AgateTurb = OrgNormal->AgateTurb;
    LocalNormal->BrickSize[0] = OrgNormal->BrickSize[0];
    LocalNormal->BrickSize[1] = OrgNormal->BrickSize[1];
    LocalNormal->BrickSize[2] = OrgNormal->BrickSize[2];
    LocalNormal->Mortar = OrgNormal->Mortar;
    LocalNormal->Normal1 = CopyNormal(OrgNormal->Normal1);
    LocalNormal->Normal2 = CopyNormal(OrgNormal->Normal2);
    LocalNormal->Normal3 = CopyNormal(OrgNormal->Normal3);
    LocalNormal->Gradient[0] = OrgNormal->Gradient[0];
    LocalNormal->Gradient[1] = OrgNormal->Gradient[1];
    LocalNormal->Gradient[2] = OrgNormal->Gradient[2];
    LocalNormal->Mandel = OrgNormal->Mandel;
    LocalNormal->Control0 = OrgNormal->Control0;
    LocalNormal->Control1 = OrgNormal->Control1;
    LocalNormal->NumberOfArms = OrgNormal->NumberOfArms;
    LocalNormal->Frequency = OrgNormal->Frequency;
    LocalNormal->Phase = OrgNormal->Phase;
    LocalNormal->WaveForm = OrgNormal->WaveForm;
    LocalNormal->Octaves = OrgNormal->Octaves;
    LocalNormal->Lambda = OrgNormal->Lambda;
    LocalNormal->Omega = OrgNormal->Omega;
    LocalNormal->Turbulence[0] = OrgNormal->Turbulence[0];
    LocalNormal->Turbulence[1] = OrgNormal->Turbulence[1];
    LocalNormal->Turbulence[2] = OrgNormal->Turbulence[2];
    LocalNormal->Amount = OrgNormal->Amount;
    LocalNormal->Trans = CopyTransform(OrgNormal->Trans);
    return LocalNormal;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyFinish
******************************************************************************/
FinishStruct *CopyFinish(FinishStruct *OrgFinish)
{
  FinishStruct *LocalFinish;

  if (OrgFinish)
  {
    LocalFinish = g_malloc(sizeof(FinishStruct));
    LocalFinish->Ambient[0] = OrgFinish->Ambient[0];
    LocalFinish->Ambient[1] = OrgFinish->Ambient[1];
    LocalFinish->Ambient[2] = OrgFinish->Ambient[2];
    LocalFinish->Ambient[3] = OrgFinish->Ambient[3];
    LocalFinish->Ambient[4] = OrgFinish->Ambient[4];
    LocalFinish->Diffuse = OrgFinish->Diffuse;
    LocalFinish->Brilliance = OrgFinish->Brilliance;
    LocalFinish->Phong = OrgFinish->Phong;
    LocalFinish->PhongSize = OrgFinish->PhongSize;
    LocalFinish->Specular = OrgFinish->Specular;
    LocalFinish->Roughness = OrgFinish->Roughness;
    LocalFinish->Metallic = OrgFinish->Metallic;
    LocalFinish->Reflection[0] = OrgFinish->Reflection[0];
    LocalFinish->Reflection[1] = OrgFinish->Reflection[1];
    LocalFinish->Reflection[2] = OrgFinish->Reflection[2];
    LocalFinish->Reflection[3] = OrgFinish->Reflection[3];
    LocalFinish->Reflection[4] = OrgFinish->Reflection[4];
    LocalFinish->Refraction = OrgFinish->Refraction;
    LocalFinish->Ior = OrgFinish->Ior;
    LocalFinish->Caustics = OrgFinish->Caustics;
    LocalFinish->FadeDistance = OrgFinish->FadeDistance;
    LocalFinish->FadePower = OrgFinish->FadePower;
    LocalFinish->Irid.Amount = OrgFinish->Irid.Amount;
    LocalFinish->Irid.Thickness = OrgFinish->Irid.Thickness;
    LocalFinish->Irid.Turbulence[0] = OrgFinish->Irid.Turbulence[0];
    LocalFinish->Irid.Turbulence[1] = OrgFinish->Irid.Turbulence[1];
    LocalFinish->Irid.Turbulence[2] = OrgFinish->Irid.Turbulence[2];
    LocalFinish->Crand = OrgFinish->Crand;
    return LocalFinish;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyHalo
******************************************************************************/
HaloStruct *CopyHalo(HaloStruct *OrgHalo)
{
  HaloStruct *LocalHalo;

  if (OrgHalo)
  {
    LocalHalo = g_malloc(sizeof(HaloStruct));
    LocalHalo->Type = OrgHalo->Type;
    LocalHalo->Density = OrgHalo->Density;
    LocalHalo->Mapping = OrgHalo->Mapping;
    LocalHalo->DustType = OrgHalo->DustType;
    LocalHalo->Eccentricity = OrgHalo->Eccentricity;
    LocalHalo->MaxValue = OrgHalo->MaxValue;
    LocalHalo->Exponent = OrgHalo->Exponent;
    LocalHalo->Samples = OrgHalo->Samples;
    LocalHalo->AaLevel = OrgHalo->AaLevel;
    LocalHalo->AaThreshold = OrgHalo->AaThreshold;
    LocalHalo->Jitter = OrgHalo->Jitter;
    LocalHalo->Turbulence[0] = OrgHalo->Turbulence[0];
    LocalHalo->Turbulence[1] = OrgHalo->Turbulence[1];
    LocalHalo->Turbulence[2] = OrgHalo->Turbulence[2];
    LocalHalo->Octaves = OrgHalo->Octaves;
    LocalHalo->Omega = OrgHalo->Omega;
    LocalHalo->Lambda = OrgHalo->Lambda;
    LocalHalo->ColorMap = CopyColorMap(OrgHalo->ColorMap);
    LocalHalo->Frequency = OrgHalo->Frequency;
    LocalHalo->Phase = OrgHalo->Phase;
    LocalHalo->Trans = CopyTransform(OrgHalo->Trans);
    LocalHalo->Next = NULL;
    return LocalHalo;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyColorMap
******************************************************************************/
ColorMapStruct *CopyColorMap(ColorMapStruct *OrgColorMap)
{
  ColorMapStruct *LocalColorMap;
  int i;

  if (OrgColorMap)
  {
    LocalColorMap = g_malloc(sizeof(ColorMapStruct));
    LocalColorMap->NbMapEntries = OrgColorMap->NbMapEntries;
    LocalColorMap->Entries = g_malloc(LocalColorMap->NbMapEntries *
                                      sizeof(ColorMapEntry));
    for (i=0 ; i<LocalColorMap->NbMapEntries ; i++)
    {
      LocalColorMap->Entries[i].Value = OrgColorMap->Entries[i].Value;
      LocalColorMap->Entries[i].Color[0] = OrgColorMap->Entries[i].Color[0];
      LocalColorMap->Entries[i].Color[1] = OrgColorMap->Entries[i].Color[1];
      LocalColorMap->Entries[i].Color[2] = OrgColorMap->Entries[i].Color[2];
      LocalColorMap->Entries[i].Color[3] = OrgColorMap->Entries[i].Color[3];
      LocalColorMap->Entries[i].Color[4] = OrgColorMap->Entries[i].Color[4];
    }
    return LocalColorMap;
  } else
    return NULL;
}
 
/*****************************************************************************
*  CopyPigmentMap
******************************************************************************/
PigmentMapStruct *CopyPigmentMap(PigmentMapStruct *OrgPigmentMap)
{
  PigmentMapStruct *LocalPigmentMap;
  int i;

  if (OrgPigmentMap)
  {
    LocalPigmentMap = g_malloc(sizeof(PigmentMapStruct));
    LocalPigmentMap->NbMapEntries = OrgPigmentMap->NbMapEntries;
    LocalPigmentMap->Entries = g_malloc(LocalPigmentMap->NbMapEntries *
                                        sizeof(PigmentMapEntry));
    for (i=0; i<LocalPigmentMap->NbMapEntries ; i++)
    {
      LocalPigmentMap->Entries[i].Value = OrgPigmentMap->Entries[i].Value;
      LocalPigmentMap->Entries[i].Pigment = CopyPigment(OrgPigmentMap->Entries[i].Pigment);
    }
    return LocalPigmentMap;
  } else
    return NULL;
}

/*****************************************************************************
*  CopySlopeMap
******************************************************************************/
SlopeMapStruct *CopySlopeMap(SlopeMapStruct *OrgSlopeMap)
{
  SlopeMapStruct *LocalSlopeMap;
  int i;

  if (OrgSlopeMap)
  {
    LocalSlopeMap = g_malloc(sizeof(SlopeMapStruct));
    LocalSlopeMap->NbMapEntries = OrgSlopeMap->NbMapEntries;
    LocalSlopeMap->Entries = g_malloc(LocalSlopeMap->NbMapEntries *
                                      sizeof(SlopeMapEntry));
    for (i=0 ; i<LocalSlopeMap->NbMapEntries ; i++)
    {
      LocalSlopeMap->Entries[i].Value = OrgSlopeMap->Entries[i].Value;
      LocalSlopeMap->Entries[i].Slope[0] = OrgSlopeMap->Entries[i].Slope[0];
      LocalSlopeMap->Entries[i].Slope[1] = OrgSlopeMap->Entries[i].Slope[1];
      LocalSlopeMap->Entries[i].Slope[2] = OrgSlopeMap->Entries[i].Slope[2];
      LocalSlopeMap->Entries[i].Slope[3] = OrgSlopeMap->Entries[i].Slope[3];
      LocalSlopeMap->Entries[i].Slope[4] = OrgSlopeMap->Entries[i].Slope[4];
    }
    return LocalSlopeMap;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyNormalMap
******************************************************************************/
NormalMapStruct *CopyNormalMap(NormalMapStruct *OrgNormalMap)
{
  NormalMapStruct *LocalNormalMap;
  int i;

  if (OrgNormalMap)
  {
    LocalNormalMap = g_malloc(sizeof(NormalMapStruct));
    LocalNormalMap->NbMapEntries = OrgNormalMap->NbMapEntries;
    LocalNormalMap->Entries = g_malloc(LocalNormalMap->NbMapEntries *
                                       sizeof(NormalMapEntry));
    for (i=0; i<LocalNormalMap->NbMapEntries ; i++)
    {
      LocalNormalMap->Entries[i].Value = OrgNormalMap->Entries[i].Value;
      LocalNormalMap->Entries[i].Normal = CopyNormal(OrgNormalMap->Entries[i].Normal);
    }
    return LocalNormalMap;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyCamera
******************************************************************************/
CameraStruct *CopyCamera(CameraStruct *OrgCamera)
{
  CameraStruct *LocalCamera;

  if (OrgCamera)
  {
    LocalCamera = g_malloc(sizeof(CameraStruct));
    LocalCamera->Type = OrgCamera->Type;
    LocalCamera->CylinderType = OrgCamera->CylinderType;
    LocalCamera->Location[0] = OrgCamera->Location[0];
    LocalCamera->Location[1] = OrgCamera->Location[1];
    LocalCamera->Location[2] = OrgCamera->Location[2];
    LocalCamera->LookAt[0] = OrgCamera->LookAt[0];
    LocalCamera->LookAt[1] = OrgCamera->LookAt[1];
    LocalCamera->LookAt[2] = OrgCamera->LookAt[2];
    LocalCamera->Right[0] = OrgCamera->Right[0];
    LocalCamera->Right[1] = OrgCamera->Right[1];
    LocalCamera->Right[2] = OrgCamera->Right[2];
    LocalCamera->Up[0] = OrgCamera->Up[0];
    LocalCamera->Up[1] = OrgCamera->Up[1];
    LocalCamera->Up[2] = OrgCamera->Up[2];
    LocalCamera->Direction[0] = OrgCamera->Direction[0];
    LocalCamera->Direction[1] = OrgCamera->Direction[1];
    LocalCamera->Direction[2] = OrgCamera->Direction[2];
    LocalCamera->Sky[0] = OrgCamera->Sky[0];
    LocalCamera->Sky[1] = OrgCamera->Sky[1];
    LocalCamera->Sky[2] = OrgCamera->Sky[2];
    LocalCamera->Angle = OrgCamera->Angle;
    LocalCamera->BlurSamples = OrgCamera->BlurSamples;
    LocalCamera->Aperture = OrgCamera->Aperture;
    LocalCamera->FocalPoint[0] = OrgCamera->FocalPoint[0];
    LocalCamera->FocalPoint[1] = OrgCamera->FocalPoint[1];
    LocalCamera->FocalPoint[2] = OrgCamera->FocalPoint[2];
    LocalCamera->Confidence = OrgCamera->Confidence;
    LocalCamera->Variance = OrgCamera->Variance;
    LocalCamera->Normal = CopyNormal(LocalCamera->Normal);
    return LocalCamera;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyLightSource
******************************************************************************/
LightSourceStruct *CopyLightSource(LightSourceStruct *OrgLightSource)
{
  LightSourceStruct *LocalLightSource;

  if (OrgLightSource)
  {
    LocalLightSource = g_malloc(sizeof(LightSourceStruct));
    LocalLightSource->Location[0] = OrgLightSource->Location[0];
    LocalLightSource->Location[1] = OrgLightSource->Location[1];
    LocalLightSource->Location[2] = OrgLightSource->Location[2];
    LocalLightSource->Color[0] = OrgLightSource->Color[0];
    LocalLightSource->Color[1] = OrgLightSource->Color[1];
    LocalLightSource->Color[2] = OrgLightSource->Color[2];
    LocalLightSource->Color[3] = OrgLightSource->Color[3];
    LocalLightSource->Color[4] = OrgLightSource->Color[4];
    LocalLightSource->Type = OrgLightSource->Type;
    LocalLightSource->PointAt[0] = OrgLightSource->PointAt[0];
    LocalLightSource->PointAt[1] = OrgLightSource->PointAt[1];
    LocalLightSource->PointAt[2] = OrgLightSource->PointAt[2];
    LocalLightSource->Radius = OrgLightSource->Radius;
    LocalLightSource->FallOff = OrgLightSource->FallOff;
    LocalLightSource->Tightness = OrgLightSource->Tightness;
    LocalLightSource->AreaLight = OrgLightSource->AreaLight;
    LocalLightSource->Area1[0] = OrgLightSource->Area1[0];
    LocalLightSource->Area1[1] = OrgLightSource->Area1[1];
    LocalLightSource->Area1[2] = OrgLightSource->Area1[2];
    LocalLightSource->Area2[0] = OrgLightSource->Area2[0];
    LocalLightSource->Area2[1] = OrgLightSource->Area2[1];
    LocalLightSource->Area2[2] = OrgLightSource->Area2[2];
    LocalLightSource->AreaSize1 = OrgLightSource->AreaSize1;
    LocalLightSource->AreaSize2 = OrgLightSource->AreaSize2;
    LocalLightSource->Adaptive = OrgLightSource->Adaptive;
    LocalLightSource->Jitter = OrgLightSource->Jitter;
    LocalLightSource->LooksLike = CopyObject(OrgLightSource->LooksLike);
    LocalLightSource->FadeDistance = OrgLightSource->FadeDistance;
    LocalLightSource->FadePower = OrgLightSource->FadePower;
    LocalLightSource->AtmosphericAttenuation = OrgLightSource->AtmosphericAttenuation;
    LocalLightSource->Atmosphere = LocalLightSource->Atmosphere;
    LocalLightSource->Next = NULL;
    return LocalLightSource;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyAtmosphere
******************************************************************************/
AtmosphereStruct *CopyAtmosphere(AtmosphereStruct *OrgAtmosphere)
{
  AtmosphereStruct *LocalAtmosphere;

  if (OrgAtmosphere)
  {
    LocalAtmosphere = g_malloc(sizeof(AtmosphereStruct));
    LocalAtmosphere->Type = OrgAtmosphere->Type;
    LocalAtmosphere->Distance = OrgAtmosphere->Distance;
    LocalAtmosphere->Scattering = OrgAtmosphere->Scattering;
    LocalAtmosphere->Eccentricity = OrgAtmosphere->Eccentricity;
    LocalAtmosphere->Samples = OrgAtmosphere->Samples;
    LocalAtmosphere->Jitter = OrgAtmosphere->Jitter;
    LocalAtmosphere->AaThreshold = OrgAtmosphere->AaThreshold;
    LocalAtmosphere->AaLevel = OrgAtmosphere->AaLevel;
    LocalAtmosphere->Color[0] = OrgAtmosphere->Color[0];
    LocalAtmosphere->Color[1] = OrgAtmosphere->Color[1];
    LocalAtmosphere->Color[2] = OrgAtmosphere->Color[2];
    LocalAtmosphere->Color[3] = OrgAtmosphere->Color[3];
    LocalAtmosphere->Color[4] = OrgAtmosphere->Color[4];
    return LocalAtmosphere;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyFog
******************************************************************************/
FogStruct *CopyFog(FogStruct *OrgFog)
{
  FogStruct *LocalFog;

  if (OrgFog)
  {
    LocalFog = g_malloc(sizeof(FogStruct));
    LocalFog->FogType = OrgFog->FogType;
    LocalFog->Distance = OrgFog->Distance;
    LocalFog->Color[0] = OrgFog->Color[0];
    LocalFog->Color[1] = OrgFog->Color[1];
    LocalFog->Color[2] = OrgFog->Color[2];
    LocalFog->Color[3] = OrgFog->Color[3];
    LocalFog->Color[4] = OrgFog->Color[4];
    LocalFog->Turbulence[0] = OrgFog->Turbulence[0];
    LocalFog->Turbulence[1] = OrgFog->Turbulence[1];
    LocalFog->Turbulence[2] = OrgFog->Turbulence[2];
    LocalFog->TurbDepth = OrgFog->TurbDepth;
    LocalFog->Omega = OrgFog->Omega;
    LocalFog->Lambda = OrgFog->Lambda;
    LocalFog->Octaves = OrgFog->Octaves;
    LocalFog->FogOffset = OrgFog->FogOffset;
    LocalFog->FogAlt = OrgFog->FogAlt;
    LocalFog->Up[0] = OrgFog->Up[0];
    LocalFog->Up[1] = OrgFog->Up[1];
    LocalFog->Up[2] = OrgFog->Up[2];
    LocalFog->Trans = CopyTransform(OrgFog->Trans);
    LocalFog->Next = NULL;
    return LocalFog;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyRainbow
******************************************************************************/
RainbowStruct *CopyRainbow(RainbowStruct *OrgRainbow)
{
  RainbowStruct *LocalRainbow;

  if (OrgRainbow)
  {
    LocalRainbow = g_malloc(sizeof(RainbowStruct));
    LocalRainbow->Direction[0] = OrgRainbow->Direction[0];
    LocalRainbow->Direction[1] = OrgRainbow->Direction[1];
    LocalRainbow->Direction[2] = OrgRainbow->Direction[2];
    LocalRainbow->Angle = OrgRainbow->Angle;
    LocalRainbow->Width = OrgRainbow->Width;
    LocalRainbow->Distance = OrgRainbow->Distance;
    LocalRainbow->ColorMap = CopyColorMap(OrgRainbow->ColorMap);
    LocalRainbow->Jitter = OrgRainbow->Jitter;
    LocalRainbow->Up[0] = LocalRainbow->Up[0];
    LocalRainbow->Up[1] = LocalRainbow->Up[1];
    LocalRainbow->Up[2] = LocalRainbow->Up[2];
    LocalRainbow->ArcAngle = OrgRainbow->ArcAngle;
    LocalRainbow->FalloffAngle = OrgRainbow->FalloffAngle;
    LocalRainbow->Next = NULL;
    return LocalRainbow;
  } else
    return NULL;
}

/*****************************************************************************
*  CopySkySphere
******************************************************************************/
SkySphereStruct *CopySkySphere(SkySphereStruct *OrgSkySphere)
{
  SkySphereStruct *LocalSkySphere;
  int i;

  if (OrgSkySphere)
  {
    LocalSkySphere = g_malloc(sizeof(SkySphereStruct));
    LocalSkySphere->NbPigment = OrgSkySphere->NbPigment;
    LocalSkySphere->pPigmentList = g_malloc(sizeof(PigmentStruct *)*LocalSkySphere->NbPigment);
    for (i=0; i < LocalSkySphere->NbPigment ; i++)
      LocalSkySphere->pPigmentList[i] = CopyPigment(OrgSkySphere->pPigmentList[i]);
    LocalSkySphere->Trans = CopyTransform(OrgSkySphere->Trans);
    return LocalSkySphere;
  } else
  return NULL;
}

/*****************************************************************************
*  CopyTransform
******************************************************************************/
TransformStruct *CopyTransform(TransformStruct *OrgTransform)
{
  TransformStruct *LocalTransform;
  int i,j;

  if (OrgTransform)
  {
    LocalTransform = g_malloc(sizeof(TransformStruct));
    for (i=0 ; i<4 ; i++)
      for (j=0 ; j<4 ; j++)
      {
        LocalTransform->Direct[i][j] = OrgTransform->Direct[i][j];
        LocalTransform->Inverse[i][j] = OrgTransform->Inverse[i][j];
      }
    return LocalTransform;
  } else
    return NULL;
}

